/***************************************************************************
    file	         : kb_dbdlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qiconset.h>

#include	"kb_classes.h"
#include	"kb_location.h"
#include	"kb_listitem.h"
#include	"kb_script.h"
#include	"kb_scriptlist.h"
#include	"kb_viewer.h"
#include	"kb_notifier.h"
#include	"kb_dom.h"
#include	"kb_build.h"
#include	"kb_prompt.h"

#include	"kb_svrchooser.h"
#include	"kb_dbdociter.h"
#include	"kb_ascii.h"

#if		__KB_AUTOMOC
#include	"kb_dbdlg.moc"
#else
#include	"kb_dbdlg.h"
#endif

#include	"rk_vbox.h"
#include	"rk_tabwidget.h"

#include	"tk_print.h"
#include	"tk_icons.h"



/*  KBDBaseDlg								*/
/*  KBDBaseDlg	: Constructor for main database dialog			*/
/*  parent	: QWidget *	  : Parent widget			*/
/*  app		: KBaseApp *	  : Application root			*/
/*  dbName	: const QString&  : Database name			*/
/*  create	: bool		  : True to create new database		*/
/*  gui		: KBaseGUI *	  : GUI for this viewer			*/
/*  (returns)	: KBDBaseDlg	  :					*/

KBDBaseDlg::KBDBaseDlg
	(	QWidget		*parent,
		KBaseApp	*app,
		const QString	&dname,
		bool		create,
		KBaseGUI	*gui
	)
	:
	RKHBox		(parent),
	m_kbaseApp	(app),
	m_dirWatch	(2000),
	m_gui		(gui)
{
	setTracking ()	;

	m_window	= new KBListWidget (this) ;

	m_fileDict  .setAutoDelete (true) ;
	m_scriptDict.setAutoDelete (true) ;

	/* Create a database information structure appropriate to the	*/
	/* database, and then create the server dialog, which always	*/
	/* appears under the first tab. Then load the rest of the parts	*/
	m_dbInfo    = new KBDBInfoData   (dname) ;
	m_dbInfo->init (create) ;

	m_serverTab = new KBServerTab    (this, m_dbInfo, gui) ;

	addPage   (m_serverTab, TR("Servers"), QString::null, false) ;
	loadParts () ;

	m_allParts.append (TR("Graphics")) ;
	m_allParts.append (TR("Printers")) ;

	KBLocation::registerType
	(	"print",
		"prn",
		"Printer",
		0
	)	;
	KBLocation::registerType
	(	"graphic",
		QString::null,
		"Graphic",
		0
	)	;


	connect
	(	&m_dirWatch,
		SIGNAL (dirty   (const QString &)),
		SLOT   (dirDirty(const QString &))
	)	;
	connect
	(	m_window,
		SIGNAL(currentChanged(QWidget *)),
		SLOT  (pageChanged   (QWidget *))
	)	;

	m_dirWatch.addDir (m_dbInfo->getDBPath ()) ;
}

/*  KBDBaseDlg								*/
/*  ~KBDBaseDlg	: Destructor for main database dialog			*/
/*  (returns)	:		:					*/

KBDBaseDlg::~KBDBaseDlg ()
{
	m_fileDict  .clear() ;
	m_scriptDict.clear() ;

	delete	m_serverTab  ;
	delete	m_dbInfo     ;
}

/*  KBDBaseDlg								*/
/*  dirDirty	: Signal that directory has changed			*/
/*  dname	: const QString & : Directory name			*/
/*  (returns)	: void		  :					*/

void	KBDBaseDlg::dirDirty
	(	const QString	&
	)
{
	KBLocation location (m_dbInfo, "all", KBLocation::m_pFile, "") ;

	DITER
	(	KBFileList,
		m_fileDict,
		list,
		list->objChange(location)
	)
	DITER
	(	KBScriptList,
		m_scriptDict,
		lang,
		lang->objChange(location)
	)
}

/*  KBDBaseDlg								*/
/*  autoStart	: Check for auto-start forms				*/
/*  autoLoc	: KBLocation &	: Return started location if any	*/
/*  (returns)	: bool		:					*/

bool	KBDBaseDlg::autoStart
	(	KBLocation	&autoLoc
	)
{
	return	m_serverTab->autoStart (autoLoc) ;
}

/*  KBDBaseDlg								*/
/*  addPage	: Add page to tabber					*/
/*  page	: QWidget *	  : Page to add				*/
/*  label	: const QString & : Tab text				*/
/*  icon	: const QString & : Icon name				*/
/*  addToList	: bool		  : Add to allPages list		*/
/*  (returns)	: void		  :					*/

void	KBDBaseDlg::addPage
	(	QWidget		*page,
		const QString	&label,
		const QString	&icon,
		bool		addToList
	)
{
	m_window->addPage
	(	page,
		label,
		getSmallIcon (icon.isEmpty() ? QString("rekall") : icon)
	)	;

	if (addToList)
		m_allParts.append	(label) ;
}

void	KBDBaseDlg::pageChanged
	(	QWidget		*page
	)
{
	m_gui->setEnabled ("KB_showRawSQL", page == m_serverTab) ;
}

#if	! __KB_RUNTIME

/*  KBDBaseDlg								*/
/*  showOneObject							*/
/*		: Show quick objects dialog				*/
/*  (returns)	: bool		: Repeat				*/

bool	KBDBaseDlg::showOneObject ()
{
	KBQuickDlg	   qDlg	  (m_dbInfo, m_allParts, m_fileDict, m_scriptDict) ;
	KBQuickDlg::Option option = (KBQuickDlg::Option)qDlg.exec() ;

	if (option == KBQuickDlg::Cancelled)
		return	false	;

	QString		type		;
	QString		server		;
	QString		object		;
	QString		extn		;
	KBFileList	*objList	= 0    ;
	qDlg.getResults (type, server, object) ;

//	fprintf
//	(	stderr,
//		"showObjects(%s,%s,%s)\n",
//		(cchar *)type,
//		(cchar *)server,
//		(cchar *)object
//	)	;

	/* First checl on the immediate results to sort out the object	*/
	/* type and extension, and where possible the part lists.	*/

	if (type == TR("Printers"))
	{
		/* Printers are handled specially, since there is no	*/
		/* associated part.					*/
		type	= "print" ;
		extn	= ""	  ;
	}
	else if (type == TR("Graphics"))
	{
		/* Graphics are also handled specially, again there is	*/
		/* no part associated with them ....			*/
		int dotp = object.findRev ('.') ;

		if (dotp >= 0)
		{	extn	= object.mid  (dotp + 1) ;
			object	= object.left (dotp) ;
		}
		else	extn	= ""  ;

		type	= "graphic"   ;
	}
	else
	{
		/* ... while all other objects types are handled via	*/
		/* their part with just a slight difference for		*/
		/* scripts.						*/
		KBFileList	*fList	= m_fileDict  .find (type) ;
		KBScriptList	*sList	= m_scriptDict.find (type) ;

		if	(fList != 0) objList = fList ;
		else if (sList != 0) objList = sList ;
		else		     objList = 0     ;

		if (objList == 0)
		{
			fprintf
			(	stderr,
				"KBDBDlg::showObjects type [%s] not found\n",
				(cchar *)type
			)	;
			return	true	;
		}

		type	= objList->getTabType     () ;
		extn	= objList->getDocExtension() ;
	}


	/* Construct the location for the object. If the object		*/
	/* selection is "all" the object name will actually be "(All)"	*/
	/* translated.							*/
	KBLocation location
		(	m_dbInfo,
			type,
			server,
			object,
			extn
		)	;

	switch (option)
	{
		case KBQuickDlg::Copy	:
			KBQuickDlg::copyObjects  (location) ;
			return	true	;

		case KBQuickDlg::Delete	:
			KBQuickDlg::deleteObject (location) ;
			return	true	;

		case KBQuickDlg::Rename	:
			KBQuickDlg::renameObject (location) ;
			return	true	;

		case KBQuickDlg::SaveToFile	:
			if (object == TR("(All)"))
				KBQuickDlg::saveAllToFile (location) ;
			else	KBQuickDlg::saveObjToFile (location) ;
			return	true	;

		case KBQuickDlg::SaveToWeb	:
			if (object == TR("(All)"))
				KBQuickDlg::saveAllToWeb  (location, objList) ;
			else	KBQuickDlg::saveObjToWeb  (location, objList) ;
			return	true	;

		default	:
			break	;
	}

	KBError	   	error	;
	QDict<QString>	dummy	;

	/* Create and open are handled specially for printers ....	*/
	if (type == "print")
	{
		showPrinters (option, location) ;
		return	true ;
	}

	/* ... and similarly for graphics.				*/
	if (type == "graphic")
	{
		showGraphics (option, location) ;
		return	true ;
	}

	if (option == KBQuickDlg::Create)
	{
		if (!m_kbaseApp->newObject (location, error))
		{	error.DISPLAY() ;
			return	true	;
		}

		return	false ;
	}

	if (m_kbaseApp->openObject
		(	0,
			location,
			KB::ShowAsData,
			dummy,
			error,
			KBValue(),
			0
		) == KB::ShowRCError)
	{
		error.DISPLAY () ;
		return	true	 ;
	}

	return	false	;
}

/*  KBDBaseDlg								*/
/*  showObjects	: Show quick objects dialog				*/
/*  (returns)	: void		:					*/

void	KBDBaseDlg::showObjects ()
{
	while (showOneObject())
		;
}

void	KBDBaseDlg::showRawSQL ()
{
	m_serverTab->showRawSQL (m_kbaseApp) ;
}


void	KBDBaseDlg::dumpDatabase ()
{
	m_serverTab->dumpDatabase (m_kbaseApp) ;
}

void	KBDBaseDlg::loadDatabase ()
{
	m_serverTab->loadDatabase (m_kbaseApp) ;
}


/*  KBDBaseDlg								*/
/*  showGraphics: Handle graphics configuration				*/
/*  option	: KBQuickDlg::Option	: Quick dialog option		*/
/*  location	: KBLocation &		: Graphic location		*/
/*  (returns)	: void			:				*/

void	KBDBaseDlg::showGraphics
	(	KBQuickDlg::Option	option,
		KBLocation		&location
	)
{
	KBError	error	;

	/* First case is request to create a new graphic. This is just	*/
	/* treated as a load into the system.				*/
	if (option == KBQuickDlg::Create)
	{
		extern	bool	importImages (KBDBInfo *, const QString &, KBError &) ;

		if (!importImages
			(	location.dbInfo(),
				location.server(),
				error
			))
		{
			error.DISPLAY() ;
			return	;
		}

		return	;
	}

	TKMessageBox::sorry
	(	0,
		QString(TR("Edit/Open option not available for graphics")),
		QString(TR("Edit/Open graphic ..."))
	)	;
}
#endif

#if	! __KB_RUNTIME

/*  KBDBaseDlg								*/
/*  showPrinters: Handle printer configuration				*/
/*  option	: KBQuickDlg::Option	: Quick dialog option		*/
/*  location	: KBLocation &		: Printer location		*/
/*  (returns)	: void			:				*/

void	KBDBaseDlg::showPrinters
	(	KBQuickDlg::Option	option,
		KBLocation		&location
	)
{
	KBError	error ;

	/* First case is request to create a new printer definition.	*/
	/* Run the printer setup, and if the user OKs then prompt for	*/
	/* a location and save.						*/
	if (option == KBQuickDlg::Create)
	{
		TKPrinter printer (TKPrinter::ResPrinter) ;
		if (!printer.setup()) return ;

		QString	name 	;
		QString	locn	= location.server() ;

		if (!doPromptSave
			(	TR("Save printer as ..."),
				TR("Enter printer name" ),
				name,
				locn,
				m_dbInfo,
				true
			))
			return	;

		QDomDocument	pXML	("printer") ;
		QDomElement	root	;

		pXML.appendChild
		(	pXML.createProcessingInstruction
			(	"xml",
				"version=\"1.0\" encoding=\"UTF-8\""
		)	)	;

		pXML   .appendChild (root = pXML.createElement ("printer")) ;
		printer.saveSettings(root) ;

		if (!location.save (locn, name, pXML.toString(), error))
			error.DISPLAY() ;

		return	;
	}

	/* Other case must be edit. In this case load the specified	*/
	/* printer definition into a printer object.			*/
	QString		_doc	= location.contents (error) ;
	if (_doc.isNull())
	{	error.DISPLAY() ;
		return	;
	}

	QDomDocument	pXML	;
	pXML.setContent	(_doc)	;
	QDomElement	root	= pXML.documentElement () ;
	if (root.isNull())
	{
		KBError::EError
		(	TR("Printer definition has no root element"),
			QString::null,
			__ERRLOCN
		)	;
		return	;
	}

	TKPrinter printer (TKPrinter::ResPrinter) ;
	printer.loadSettings  (root) ;

	/* Run the printer setup dialog, and give up if the used does	*/
	/* not OK it. Otherwise, we can just save the settings back	*/
	/* into the XML document and save it as above.			*/
	if (!printer.setup()) return ;
	printer.saveSettings  (root) ;

	QString	name 	= location.name  () ;
	QString	locn	= location.server() ;

	if (!doPromptSave
		(	TR("Save printer as ..."),
			TR("Enter printer name" ),
			name,
			locn,
			m_dbInfo,
			true
		))
		return	;

	if ((name != location.name()) || (locn != location.server()))
	{
		KBLocation check (m_dbInfo, "print", locn, name, "") ;
		if (check.exists())
			if (TKMessageBox::questionYesNo
				(	0,
					QString(TR("Printer %1 on server %2 exists, overwrite?"))
						.arg(name)
						.arg(locn),
					TR("Overwrite printer ...")
				)
				!= TKMessageBox::Yes) return ;
	}

	if (!location.save (locn, name, pXML.toString(), error))
		error.DISPLAY() ;
}
#endif

/*  ------------------------------------------------------------------  */

static	QDict<QString>	identStrings	;

void	KBDBaseDlg::addIdentString
	(	const QString	&part,
		cchar		*ident
	)
{
	identStrings.setAutoDelete (true) ;
	identStrings.replace	   (part, new QString(ident)) ;
}

QString	KBDBaseDlg::getIdentStrings ()
{
	QString			res  ;
	QDictIterator<QString>	iter (identStrings) ;

	while (iter.current() != 0)
	{
		res	+= QString("<tr><td>Part</td><td><b>%1</b></td><td><nobr>%2</nobr></td></tr>")
				  .arg( iter.currentKey())
				  .arg(*iter.current   ()) ;

		iter	+= 1 ;
	}

	return	res	;
}


