/***************************************************************************
    file	         : kb_location.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_LOCATION_H
#define	__KB_LOCATION_H

#include "kb_classes.h"

#include "libcommon_exports.h"


class 	KBLocnTypeMap	;
class	KBServerInfo	;


/*  KBLocation								*/
/*  ----------								*/
/*  The location of a document (a form, query, script, ...) is		*/
/*  specified by three things						*/
/*	(a) the database it is located in				*/
/*	(b) location in that database 					*/
/*	(c) the name within that location				*/
/*  The location object also provides other information, such as the	*/
/*  part factory for the object.					*/

class	LIBCOMMON_API	KBLocation
{
	KBDBInfo		*m_dbInfo	;
	QString			m_type		;
	QString			m_server	;
	QString			m_name		;
	QString			m_extension	;

	bool			getData 	(cchar *, KBError &, QByteArray &) const ;

	bool			saveToFile	(const QString &, const QString &, cchar *, uint, KBError &) ;
	bool			saveToDB	(const QString &, const QString &, cchar *, uint, KBError &) ;
	bool			renameFile	(const QString &, KBError &) ;
	bool			renameDB	(const QString &, KBError &) ;
	bool			removeFile	(KBError &) ;
	bool			removeDB	(KBError &) ;

static	KBLocnTypeMap		*findByType	(const QString &) ;
static	KBLocnTypeMap		*findByExtn	(const QString &) ;

public	:

	static	const char* 	m_pFile		;
	static	const char* 	m_pInline	;
	static	const char* 	m_pStock	;


	KBLocation		(KBDBInfo *, cchar *, const QString &, const QString & = "", const QString & = "") ;
	KBLocation		()	;

	bool		operator== 	(const KBLocation &) ;

	QString		path		(const QString & = QString::null) const ;
	QString		filename	(const QString & = QString::null) const ;

	inline	KBDBInfo	*dbInfo	  () const { return	m_dbInfo	; }
	inline	const QString	&type	  () const { return	m_type		; }
	inline	const QString	&server	  () const { return	m_server	; }
	inline	const QString	&name	  () const { return	m_name		; }
	inline	const QString	&extension() const { return	m_extension	; }

	inline	void	setServer
		(	const QString	&server
		)
	{
		m_server= server;
	}
	inline	void	setName
		(	const QString	&name
		)
	{
		m_name	= name	;
	}

	QString		ident		()	const ;
	QString		title		()	const ;
	bool		exists		()	const ;

	bool		contents	(QByteArray &, KBError &) 	const ;
	QString		contents	(KBError    &) 			const ;
	QString		timestamp 	(KBError    &) 			const ;

	bool		save		(const QString &, const QString &, const QString &, KBError &) ;
	bool		save		(const QString &, const QString &, cchar *,   uint, KBError &) ;
	bool		rename		(const QString &, KBError &) ;
	bool		remove		(KBError    &) ;

	inline	bool	isFile  () const { return m_server == m_pFile	   ; }
	inline	bool	isInline() const { return m_server == m_pInline	   ; }
	inline	bool	isStock () const { return m_server == m_pStock	   ; }

	inline	bool	isInDB  () const { return !isFile() && !isInline() ; }

	KBServerInfo	*getServerInfo	() const ;
	KBPartFactory	*getFactory	() const ;

	static	void	registerType	(cchar *, cchar *, const QString &, KBPartFactory *) ;
	static	QString	extnForType	(KBDBInfo *,       const QString &, const QString &) ;
	static	void	setCacheSize	(uint)	;
	static	uint	getCacheUsed	()	;
}	;

#endif	// __KB_LOCATION_H
