/***************************************************************************
    file	         : kb_button.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#if		__KB_AUTOMOC
#include	"kb_button.moc"
#else
#include 	"kb_button.h"
#endif

#include	"kb_nodereg.h"
#include	"kb_navigator.h"
#include	"kb_docroot.h"
#include	"kb_options.h"

#include        "tk_icons.h"


#if	! __KB_RUNTIME
static	cchar	*imageLegends[] =
{
	"Normal",
	"Rollover",
	"Disable",
	0
}	;
#endif

/*  KBButton								*/
/*  KBButton	: Consructor for button node				*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  ok		: bool *	: Success				*/
/*  (returns)	: KBButton	:					*/

KBButton::KBButton
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*ok
	)
	:
	KBObject 	(parent, "KBButton",	aList),
	m_fgcolor	(this,   "fgcolor",	aList),
	m_bgcolor	(this,   "bgcolor",	aList),
	m_font		(this,	 "font",	aList),
	m_text	 	(this,   "text",     	aList),
	m_image		(this,	 "image",	aList, 	KAF_CLEAR),
	m_tabOrd	(this,	 "taborder", 	aList),
	m_toggle	(this,	 "toggle", 	aList),
	m_onClick	(this,   "onclick",  	"onButton", aList),
	m_onToggle	(this,   "ontoggle",  	"onButton", aList)

{
	m_button	= 0 	;
	m_code		= 0 	;
	m_disable	= false	;


#if	! __KB_RUNTIME
	m_image.setNumImages (3);
	m_image.setLegends   (imageLegends) ;

	if (ok != 0)
	{
		if (!KBNode::propertyDlg ("Button"))
		{	delete	this	;
			*ok	= false	;
			return	;
		}

		*ok	= true	;
	}
#endif
}

/*  KBButton								*/
/*  KBButton	: Consructor for button node				*/
/*  _parent	: KBNode *	: Parent node				*/
/*  _button	: KBButton *	: Extant button				*/
/*  (returns)	: KBButton	:					*/

KBButton::KBButton
	(	KBNode	 	*_parent,	
		KBButton 	*_button
	)
	:
	KBObject	(_parent, _button),
	m_fgcolor	(this,   "fgcolor",	_button),
	m_bgcolor	(this,   "bgcolor",	_button),
	m_font		(this,	 "font",	_button),
	m_text	 	(this,   "text",	_button),
	m_image		(this,	 "image",	_button, KAF_CLEAR),
	m_tabOrd	(this,	 "taborder", 	_button),
	m_toggle	(this,	 "toggle", 	_button),
	m_onClick	(this,   "onclick",	"onButton", _button),
	m_onToggle	(this,   "ontoggle",	"onButton", _button)
{
#if	! __KB_RUNTIME
	m_image.setNumImages (2);
	m_image.setLegends   (imageLegends) ;
#endif

	m_button	= 0	;
	m_code		= 0 	;
	m_disable	= false	;
}

/*  KBButton								*/
/*  ~KBButton	: Desructor for button node				*/
/*  (returns)	:		:					*/

KBButton::~KBButton ()
{
}

/*  KBButton								*/
/*  buildDisplay: Build controls for this node				*/
/*  display	: KBDisplay *	: Owning display			*/
/*  (returns)	: void		:					*/

void	KBButton::buildDisplay
	(	KBDisplay	*display
	)
{
	KBObject::buildDisplay (display) ;

	if (m_button == 0)
	{
		m_button = new KBCtrlButton (display, this) ;
		setControl (m_button) ;
	}

	setupProperties	() ;
}

QPixmap	KBButton::loadImage
	(	const QString	&image
	)
{
	QStringList	iparts	= QStringList::split ('.', image) ;

//	fprintf
//	(	stderr,
//		"KBButton::loadImage: [%s][%s]\n",
//		(cchar *)iparts[0],
//		(cchar *)iparts[1]
//	)	;

	KBLocation location
	(	getDocRoot()->getDBInfo(),
		"graphic",
		getDocRoot()->getLocation().server(),
		iparts[0],
		iparts[1]
	)	;

	KBError	   error	;
	QByteArray data		;

	if (!location.contents (data, error))
	{
		setError (error)   ;
		return	QPixmap	() ;
	}

	return	QPixmap	(data)	;
}

bool	KBButton::setPixmaps
	(	const QString	&name1,
		const QString	&name2
	)
{
	QPixmap	pm1 = name1.isEmpty() ? QPixmap() : loadImage (name1) ;
	QPixmap	pm2 = name2.isEmpty() ? QPixmap() : loadImage (name2) ;

	if (!pm1.isNull() && !KBOptions::getNoButtonImages())
	{
		m_button->setPixmaps (pm1, pm2) ;
		return	true	;
	}

	m_button->setPixmaps (QPixmap(), QPixmap()) ; 
	return	false	;
}

/*  KBButton								*/
/*  loadPixmaps	: Load image into button				*/
/*  (returns)	: bool		: Image loaded				*/

bool	KBButton::loadPixmaps ()
{
	QStringList images = QStringList::split (';', m_image.getValue()) ;
	return	setPixmaps (images[0], images[1]) ;
}

/*  KBButton								*/
/*  showAs	: Switch in or out of design mode			*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBButton::showAs
	(	KB::ShowAs	mode
	)
{
	KBObject::showAs (mode) ;

	if (!loadPixmaps ()) 
		setText	  (m_text.getValue()) ;

	m_button->setToggleButton (m_toggle.getBoolValue()) ;
	m_button->setOn (false) ;
}

/*  KBButton								*/
/*  clicked	: Handle button click					*/
/*  (returns)	: void		:					*/

void	KBButton::clicked ()
{
	bool	evRc ;
	eventHook (m_onClick, 0, 0, evRc) ;
}

/*  KBButton								*/
/*  toggled	: Handle button toggle					*/
/*  (returns)	: void		:					*/

void	KBButton::toggled
	(	bool		on
	)
{
	bool	evRc	;
	KBValue	arg	= on	;

	eventHook (m_onToggle, 1, &arg, evRc) ;
}

/*  KBButton								*/
/*  keyStroke	: Handle keystroke					*/
/*  e		: QKeyEvent *	: Event to be handled			*/
/*  (returns)	: bool		: True if event consumed		*/

bool	KBButton::keyStroke
	(	QKeyEvent	*k
	)
{
//	fprintf
//	(	stderr,
//		"KBButton::keyStroke: %d\n",
//		k->key()
//	)	;
	KBNavigator *navi = getNavigator() ;
	bool	    shift = (k->state () & Qt::ShiftButton) != 0 ;

	if (m_button != 0)
		switch (k->key())
		{
			case Qt::Key_Enter	:
			case Qt::Key_Return	:
				((QButton *)m_button->topWidget())->animateClick () ;
				return	true	;

			case Qt::Key_Tab	:
				if (navi != 0)
				{
					if (shift)
						navi->goPrevious (this, true) ;
					else	navi->goNext 	 (this, true) ;
				}
				return	true	;

			case Qt::Key_Backtab	:
				if (navi != 0)
					navi->goPrevious (this, true) ;
				return	true	;
			
			default	:
				break	;
		}

	return	false	;
}

/*  KBButton								*/
/*  setTabOrder	: Return tab order value				*/
/*  order	: int		: Tab order				*/
/*  (returns)	: void		:					*/

void	KBButton::setTabOrder
	(	int	order
	)
{
	m_tabOrd.setValue (order) ;
}

/*  KBButton								*/
/*  getTabOrder	: Return tab order value				*/
/*  (returns)	: int		: Tab order				*/

int	KBButton::getTabOrder ()
{
	return	m_tabOrd.getIntValue () ;
}

/*  KBButton								*/
/*  replicate	: Replicate this button					*/
/*  _parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New button node			*/

KBNode	*KBButton::replicate
	(	KBNode	*_parent
	)
{
	return	new KBButton (_parent, this) ;
}

/*  KBButton								*/
/*  setupProperties							*/
/*		: Update to current properies.				*/
/*  (returns)	: void		:					*/

void	KBButton::setupProperties ()
{
	KBObject::setupProperties () ;

	if (!loadPixmaps ())
		setText (m_text.getValue ()) ;

	m_button->setVisible	  (true) ;
	m_button->setToggleButton (m_toggle.getBoolValue()) ;
}

/*  KBButton								*/
/*  enumKBProperty							*/
/*		: Get list of properties				*/
/*  list	: QStringList &	: Property list				*/
/*  (returns)	: void		:					*/

void	KBButton::enumKBProperty
	(	QStringList	&list
	)
{
	list.append ("text") ;
	KBObject::enumKBProperty (list) ;
}

/*  KBButton								*/
/*  getKBProperty: Get a property rekall value				*/
/*  name	 : cchar *	: Proprty name				*/
/*  value	 : KBValue &	: Return property value			*/
/*  (returns)	 : bool		: Property exists			*/

bool	KBButton::getKBProperty
	(	cchar		*name,
		KBValue		&value
	)
{
	if (qstrcmp (name, "text") == 0)
	{
		value	= getText() ;
		return	true ;
	}

	return	KBObject::getKBProperty (name, value) ;
}

/*  KBButton								*/
/*  setKBProperty: Set a rekall property value				*/
/*  name	 : cchar *	: Proprty name				*/
/*  value	 : KBValue	: Property value			*/
/*  (returns)	 : bool		: Success				*/

bool	KBButton::setKBProperty
	(	cchar		*name,
		const KBValue	&value
	)
{
	if (qstrcmp (name, "text") == 0)
	{
		setText	(value.getRawText()) ;
		return	true	;
	}

	return	KBObject::setKBProperty (name, value) ;
}

#if	! __KB_RUNTIME

/*  KBButton								*/
/*  propertyDlg	: Show property dialog					*/
/*  iniAttr	: cchar *	: Initial property			*/
/*  (returns)	: bool		: Success				*/

bool	KBButton::propertyDlg
	(	cchar	*iniAttr
	)
{
	if (KBNode::propertyDlg ("Button", iniAttr))
	{
		setupProperties () ;
		return	true ;
	}

	return	false	;
}
#endif


NEWNODE(Button, __TR("New &Button"), 0, KNF_FORM|KNF_BLOCK|KNF_STATIC)
