/***************************************************************************
    file	         : kb_button.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	_KB_BUTTON_H
#define	_KB_BUTTON_H

#include 	"libkbase_exports.h"

#include	"kb_object.h"
#include	"kb_qtbutton.h"
#include	"kb_attrimage.h"

/*  KBButton								*/
/*  ------------							*/
/*  This class is used for XML parse tree elements corresponding to	*/
/*  simple push-buttons.						*/

class	LIBKBASE_API	KBButton : public KBObject
{
	Q_OBJECT

	KBAttrStr	m_fgcolor	;	/* Text colour			*/
	KBAttrStr	m_bgcolor	;	/* Background colour		*/
	KBAttrStr	m_font		;	/* Font				*/
	KBAttrStr	m_text		;
	KBAttrImage	m_image		;
	KBAttrUInt	m_tabOrd 	;	/* Tab ordering			*/
	KBAttrBool	m_toggle	;
	KBEvent		m_onClick	;
	KBEvent		m_onToggle	;

	KBCtrlButton	*m_button	;
	bool		m_disable	;
	KBScriptCode	*m_code		;

	QPixmap		loadImage	(const QString &) ;
	bool		loadPixmaps	()	;


private slots :

	void	clicked ()	;
	void	toggled (bool)	;

public	:

	KBButton	(KBNode  *, const QDict<QString> &, bool * = 0) ;
	KBButton	(KBNode  *, KBButton *) ;
virtual~KBButton	() ;

#if	! __KB_RUNTIME
	virtual	bool	propertyDlg	(cchar *) ;
#endif
	virtual	void	setupProperties	() 	;
	virtual	void	buildDisplay	(KBDisplay *) ;
	virtual	KBNode	*replicate	(KBNode    *) ;
	virtual	void	showAs		(KB::ShowAs ) ;

	virtual	bool	keyStroke	(QKeyEvent *) ;
	virtual	void	setTabOrder	(int )	;
	virtual	int	getTabOrder	()     	;

	bool		setPixmaps	(const QString &, const QString &) ;

	inline	bool	isOn	()
	{
		return	m_button->isOn	() ;
	}

	inline	void	setText
		(	const QString	&text
		)
	{
		m_button->setValue (KBValue (text, &_kbString)) ;
	}

	inline	QString	getText ()
	{
		return	m_button->getValue().getRawText() ;
	}

	inline	void	setOn
		(	bool		on
		)
	{
		m_button->setOn	(on) ;
	}

	virtual	void	enumKBProperty	(QStringList	  	&) ;
	virtual	bool	getKBProperty	(cchar *,	KBValue &) ;
	virtual	bool	setKBProperty	(cchar *, const KBValue &) ;
}	;

#endif	// _KB_BUTTON_H
