/***************************************************************************
    file		 : kb_display.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	_KB_DISPLAY_H
#define	_KB_DISPLAY_H

#include	<qsize.h>
#include	<qguardedptr.h>


#include 	"libkbase_exports.h"
#include	"kb_classes.h"
#include	"kb_attrgeom.h"

class		KBDispWidget	;
class		KBDispScroller	;
class		KBGridSetup	;

class		QWidget		;
class		QScrollView	;


/*  KBDisplay								*/
/*  ---------								*/
/*  This class is effectively the widget onto which forms and the like	*/
/*  place their controls. However, since the top-most block must be	*/
/*  shows in a scroller (becuase the form can be very large, especially	*/
/*  the width of a table data display), KBDisplay provides an interface	*/
/*  which hides the various differences.				*/

class	LIBKBASE_API	KBDisplay : public QObject
{
	Q_OBJECT


	QGuardedPtr<KBDispScroller>	m_scroller	;
	QGuardedPtr<KBDispWidget>	m_widget	;
	QGuardedPtr<QWidget>		m_canvas	;

	bool		m_marking	;
	int		m_startMX	;
	int		m_startMY	;
	QPoint		m_startCell	;
	int		m_markX	 	;
	int		m_markY	 	;
	int		m_markW	 	;
	int		m_markH	 	;

	QSize		m_bounds	; /* Marking area bounds	*/
	bool		m_shown	 	;

	QRect		m_rubber	;
	int		m_lMargin	;
	int		m_rMargin	;
	int		m_tMargin	;
	int		m_bMargin	;

	int		m_left	 	;
	int		m_top	 	;

	QPtrList<KBItem>	m_morphList	;

#if	! __KB_RUNTIME
	void		moveRubberRect	(QPainter *, const QRect &) ;
	void		endRubberRect	()		;
#endif

protected :

	KBDisplay	*m_parent ;	/* Parent display if any	*/
	KBObject	*m_object  ;	/* Owning object		*/

protected slots :

	void		morphDestroyed		()	;

public	:

	KBDisplay (QWidget   *, KBObject *, uint, bool, bool = false) ;
	KBDisplay (KBDisplay *, KBObject *, uint) ;
	virtual	~KBDisplay () ;

	void		setRowColSetup
			(	const KBGridSetupList	&,
				const KBGridSetupList	&
			)	;

	void		addChild   		(QWidget     *, const QRect &, Qt::AlignmentFlags) ;
	void		moveChild  		(QWidget     *, const QRect &, Qt::AlignmentFlags) ;
	void		setGeometry		(const QRect &)	;
	void		resizeContents		(const QSize &)	;

	void		addMorph		(KBItem      *) ;
	void		remMorph		(KBItem      *) ;
	void		updateMorph		(KBItem      *, const QRect &) ;
	void		updateMorph		(KBItem      *, uint) ;
	void		repaintMorphs		(QPainter    &, const QRect &) ;


	void		showAs			(KB::ShowAs)	;
	void		setStretchable		(bool)		;
	void		setShowbar		(uint)		;

	void		setTitle		(const QString	  &) ;
	void		setPalette 		(const QPalette   *) ;
	void		setFont    		(const QFont      *) ;
	void		updateDynamic		() ;

	void		show	   		() ;
	int		width	   		() ;
	int		height	   		() ;

	QSize		getBaseSize		() ;
	QSize		sizeHint		() ;
	QSize		showBarUsed		() ;

	void		displayIsGone		() ;
	QWidget		*getTopWidget		() ;
	QWidget		*getDisplayWidget	() ;
	QSize		effectiveSize		() ;
	void		makeVisible		(const QRect &, QFocusEvent::Reason) ;
	void		cvtViewToCtrl		(      QRect &) ;
	void		cvtCtrlToView		(      QRect &) ;

	void		setFrame		(int, int)	;

	const QPalette  palette			() ;
	const QFont     font			() ;
	QSize	  	getTopSize 		() ;
	QRect	  	getTopRect 		() ;

#if	! __KB_RUNTIME
	bool		contextEvent		(QRect) ;
	bool		markStartEvent		(QMouseEvent *, QPoint) ;
	void		markMoveEvent		(QMouseEvent *) ;
	bool		markEndEvent		(QMouseEvent *, QPoint) ;
	void		drawDisplay		(QPainter    *, QRect ) ;
#endif

	bool		mousePressEvent		(QMouseEvent *, QPoint)	;
	bool		mouseMoveEvent		(QMouseEvent *)	;
	bool		mouseReleaseEvent	(QMouseEvent *, QPoint)	;

	void		setRowRange		(uint, uint, uint, uint, uint ) ;	
	void		scrollToRow		(uint)			;
	void		scrollBy		( int)			;
	void		doOperation		(KB::Action, uint)	;
	bool		keyStroke		(QKeyEvent *)		;
	void		resizeEvent		()			;

	void		setTagLabel		(const QString &)	;
	void		setBackgroundPixmap	(const QPixmap &, int) 	;

	inline	KBDisplay *getParent 	()
	{
		return	m_parent   ;
	}
	inline	KBObject  *getObject	()
	{
		return	m_object   ;
	}
}	;

#endif	// _KB_DISPLAY_H
