/***************************************************************************
    file	         : kb_docroot.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef		_KB_DOCROOT_H
#define		_KB_DOCROOT_H

#include	"libkbase_exports.h"

#include 	<qptrlist.h>
#include 	<qstringlist.h>
#include 	<qguardedptr.h>

#include	"kb_classes.h"
#include	"kb_location.h"
#include	"kb_attrdict.h"
#include	"kb_qrybase.h"


class	KBMimeSourceFactory	;
class	KBProgressBox		;
class	QLabel			;

/*  ------------------------------------------------------------------  */

/*  KBDocRoot								*/
/*  ---------								*/
/*  This class is used at the top-most node in parsed documents, such	*/
/*  as forms and reports. It holds information, such as the document	*/
/*  location, that is common to all such documents.			*/

class	LIBKBASE_API	KBDocRoot : public QObject
{
	Q_OBJECT

	KBNode		*m_parent	;	/* Embedding node ...	*/
	QPtrList<KBNode>&m_children	;	/* ... and its children	*/
	bool		m_donePDlg	;	/* Parameter dialog run	*/
	bool		m_gotExecError	;	/* Execution error flag	*/

	QStringList	m_scripts	;	/* List of scripts	*/
	QStringList	m_imports	;	/* List of imports	*/

	bool				m_loading	;
	QGuardedPtr<QLabel>		m_message	;
	QGuardedPtr<QLabel>		m_locking	;
	QGuardedPtr<KBProgressBox>	m_progress	;

	KBMimeSourceFactory		*m_mimeSource	;

signals	:

	void		execError 	()		;
	void		requestClose	(int)		;
	void		layoutChanged	()		;

protected :

	KBLocation	m_location	;	/* Database location	*/
	KBServerInfo	*m_serverInfo	;	/* Server information	*/
	KBScriptIF	*m_scrIface[2]	;	/* Scripting interfaces	*/
	KBAttrDict	*m_paramDict	;	/* Parameter dictionary	*/

public	:

	KBDocRoot	(KBNode *, QPtrList<KBNode> &, const KBLocation &) ;
       ~KBDocRoot	() ;

	inline		KBDBInfo	 	*getDBInfo	() { return m_location.dbInfo() ; }
	inline		const KBLocation 	&getLocation	() { return m_location		; }
	inline		const KBServerInfo 	*getServerInfo	() { return m_serverInfo	; }

	void		reset			() ;

	KBScriptIF	*getScriptIF		(bool, bool &, KBError &) ;
	KBScriptIF	*loadScripting		(bool, bool &, KBError &) ;

	KB::ShowRC	setParamDict		(const QDict<QString> &, KBError &) ;
	QString		getParamValue		(cchar *) ;
	void		setParamValue		(cchar *, const QString &) ;

	const QString	&appFont		() ;

	void		setStatusBar		(QLabel *, QLabel *, KBProgressBox *) ;
	KBMimeSourceFactory *mimeSource		() ;

	inline	const QStringList &getScripts	() { return m_scripts ; }
	inline	const QStringList &getImports	() { return m_imports ; }

	void		loadingStart		() ;
	void		loadingDone		() ;
	bool		loadingProgress		(uint, uint, QString = QString::null) ;

	inline	bool	isLoading ()
	{
		return	m_loading ;
	}

	void		doExecError		() ;
	void		doRequestClose		(int) ;
	void		doSetMessage		(const QString    &) ;
	void		doSetLocking		(KBQryBase::Locking) ;
	void		doLayoutChanged		() ;

	/* The following method indicates whether an execution error	*/
	/* has occurred. This is returned as a (const) reference to the	*/
	/* error flag for reasons of efficiency on the scripting	*/
	/* interfaces, viz:						*/
	/*								*/
	/*	const bool &e = xxx->gotExecError() ;			*/
	/*	if (e) return ....	;				*/
	/* 	make_api_call ()	;				*/
	/*	if (e) return ....	;				*/
	/*								*/
	/* Hence, error occurance can be checked before and after the	*/
	/* API call at minimum cost.					*/

	const bool	&gotExecError		() ;
}	;


#endif		// _KB_DOCROOT_H
