/***************************************************************************
    file	         : kb_formdlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qwidgetstack.h>

#if		__KB_AUTOMOC
#include	"kb_formdlg.moc"
#else
#include	"kb_formdlg.h"
#endif

#include	"kb_form.h"
#include	"kb_param.h"
#include	"kb_paramdlg.h"
#include	"kb_script.h"
#include	"kb_import.h"

#include	"tk_messagebox.h"
#include	"rk_combobox.h"


/*  KBFormPropDlg							*/
/*  KBFormPropDlg: Constructor for root properties dialog		*/
/*  root	 : KBForm *	   : Root node				*/
/*  caption	 : cchar *	   : Caption text			*/
/*  attribs	 : QPtrList<KBAttr  >&:Attributes list			*/
/*  scripts	 : QPtrList<KBModule>&: List of script modules		*/
/*  imports	 : QPtrList<KBModule>&: List of import modules		*/
/*  params	 : QPtrList<KBParam >&: List of parameters modules		*/
/*  iniAttr	 : cchar *	   : Initial attribute			*/
/*  (returns)	 : KBFormPropDlg   :					*/

KBFormPropDlg::KBFormPropDlg
	(	KBForm		*root,
		cchar		*caption,
		QPtrList<KBAttr  >	&attribs,
		QPtrList<KBModule>	&scripts,
		QPtrList<KBModule>	&imports,
		QPtrList<KBParam > &params, 
		cchar		*iniAttr
	)
	:
	KBPropDlg (root, caption, attribs, iniAttr),
	m_root	  (root)
{
	m_scriptDlg = new KBScriptDlg (m_propStack, root, scripts) ;
	m_importDlg = new KBImportDlg (m_propStack, root, imports) ;
	m_paramDlg  = new KBParamDlg  (m_propStack, root, params ) ;

	m_scriptDlg->hide() ;
	m_importDlg->hide() ;
	m_paramDlg ->hide() ;
}

/*  KBFormPropDlg							*/
/*  ~KBFormPropDlg: Constructor for root properties dialog		*/
/*  (returns)	  :		:					*/

KBFormPropDlg::~KBFormPropDlg ()
{
}

/*  KBFormPropDlg							*/
/*  hideProperty : Check whether to hide attribute property		*/
/*  attr	 : KBAttr *	: Attribute in question			*/
/*  (returns)	 : bool		: True to hide				*/

bool	KBFormPropDlg::hideProperty
	(	KBAttr	*attr
	)
{
	/* Since KBForm is derived from KBBlock, there are a whole lot	*/
	/* of properties which we must ignore; so check just for those	*/
	/* of our own.							*/
	if (qstrcmp (attr->getOwnerName(), "KBForm") == 0)
		return	false	;

	return	true	;
}


/*  KBFormPropDlg							*/
/*  showProperty: Show property						*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Property on show			*/

bool	KBFormPropDlg::showProperty
	(	KBAttrItem	*item
	)
{
	QString	aName	= item->attr()->getName () ;

	/* The script module list is handled entirely locally, with	*/
	/* none of the default KBPropDlg controls.			*/
	if (aName == "modlist")
	{
		setUserWidget (m_scriptDlg) ;
		return	true	 ;
	}
	if (aName == "implist")
	{
		setUserWidget (m_importDlg) ;
		return	true	 ;
	}
	if (aName == "paramlist")
	{
		setUserWidget (m_paramDlg ) ;
		return	true	 ;
	}

//	/* Block type options are shown using the KBPropDlg combobox,	*/
//	/* initialised with the set of possibilities. Also, the first	*/
//	/* time round, make a note of the current form setting.		*/
//	if (aName == "blktype")
//	{
//		initToplevelCombo (m_comboBox, item->value()) ;
//		m_comboBox->show () ;
//
//		if (m_ibtype.isNull()) m_ibtype = item->value() ;
//		return	true	 ;
//	}

	return	KBPropDlg::showProperty (item) ;
}

/*  KBFormPropDlg								*/
/*  saveProperty: Check and save attribute				*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Attribute OK				*/

bool	KBFormPropDlg::saveProperty
	(	KBAttrItem	*item
	)
{
	QString	aName	= item->attr()->getName() ;

	/* As for showing the script module list, saving the results is	*/
	/* handled locally.						*/
	if (aName == "modlist"  )
	{
		setProperty  ("modlist",   m_scriptDlg->getText()) ;
		return	true ;
	}
	if (aName == "implist"  )
	{
		setProperty  ("implist",   m_importDlg->getText()) ;
		return	true ;
	}
	if (aName == "paramlist")
	{
		setProperty  ("paramlist", m_paramDlg ->getText()) ;
		return	true ;
	}

//	/* Similarly handle the block type here ...			*/
//	if (aName == "blktype")
//	{
//		static	cchar	*vals[] =
//		{	"null",
//			"table",
//			"sql",
//			"query"
//		}	;
//
//		/* Get the selected type. If this is a change from the	*/
//		/* current block type then warn the user that a change	*/
//		/* will trash the form contents.			*/
//		cchar	*newTyp	= vals[m_comboBox->currentItem()] ;
//
//		if (!m_ibtype.isEmpty() && (newTyp != m_ibtype))
//			if (TKMessageBox::questionYesNo
//			   	(	0,
//					TR("Changing block type will reset form contents; continue?"),
//					TR("Change block type")
//				)
//				!= TKMessageBox::Yes) return false ;
//
//		setProperty (item, newTyp) ;
//		return	true ;
//	}

	return	KBPropDlg::saveProperty (item) ;
}

/*  KBFormPropDlg:							*/
/*  clickOK	 : Handle OK button click				*/
/*  (returns)	 : void		:					*/

void	KBFormPropDlg::clickOK ()
{
	m_scriptDlg->fixUp () ;
	m_importDlg->fixUp () ;
	m_paramDlg ->fixUp () ;

	KBPropDlg::clickOK () ;
}

/*  KBFormPropDlg:							*/
/*  preExec	  : Called immediately before KBDialog::exec		*/
/*  (returns)	  : void	:					*/

void	KBFormPropDlg::preExec ()
{
	/* Trap this and load the notional hidden attribute. We have to	*/
	/* do this here since it is not until "KBPropDlg::exec" has	*/
	/* been called that "setProperty" will work.			*/
	setProperty  ("modlist",   m_scriptDlg->getText()) ;
	setProperty  ("implist",   m_importDlg->getText()) ;
	setProperty  ("paramlist", m_paramDlg ->getText()) ;
}


void	KBFormPropDlg::initToplevelCombo
	(	RKComboBox	*combo,
		const QString	&value
	)
{
	combo->clear	() ;
	combo->insertItem  (TR("Menu-only block" )) ;
	combo->insertItem  (TR("Table block"     )) ;
	combo->insertItem  (TR("SQL block"       )) ;
	combo->insertItem  (TR("Query block"     )) ;

	if	(value == "null" ) combo->setCurrentItem (0) ;
	else if (value == "sql"  ) combo->setCurrentItem (2) ;
	else if (value == "query") combo->setCurrentItem (3) ;
	else			   combo->setCurrentItem (1) ;
}


/*  ------------------------------------------------------------------	*/

/*  formPropDlg	: Run the root properties dialog box			*/
/*  form	: KBForm *	  : Form node				*/
/*  caption	: cchar *	  : Caption text			*/
/*  attribs	: QPtrList<KBAttr >& :Attributes list			*/
/*  scripts	: QPtrList<KBModule>&: List of script modules		*/
/*  imports	: QPtrList<KBModule>&: List of import modules		*/
/*  params	: QPtrList<KBParam >&: List of parameters modules		*/
/*  iniAttr	: cchar *	  : Initial attribute			*/
/*  (returns)	: bool		  : Success				*/

bool LIBKBASE_API formPropDlg
	(	KBForm		*form,
		cchar		*caption,
		QPtrList<KBAttr  >	&attribs,
		QPtrList<KBModule>	&scripts,
		QPtrList<KBModule>	&imports,
		QPtrList<KBParam > &params, 
		cchar		*iniAttr
	)
{
	KBFormPropDlg fDlg (form, caption, attribs, scripts, imports, params, iniAttr) ;
	return	fDlg.exec  () ;
}
