/***************************************************************************
    file	         : kb_framer.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>

#include	<qapplication.h>
#include	<qptrlist.h>
#include	<qstring.h>
#include	<qframe.h>
#include	<qpopupmenu.h>
#include	<qpainter.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_attrdict.h"

#include	"kb_sizer.h"
#include	"kb_display.h"
#include	"kb_ctrl.h"
#include	"kb_formblock.h"
#include	"kb_reportblock.h"
#include	"kb_rowmark.h"

#include	"kb_form.h"
#include	"kb_report.h"
#include	"kb_component.h"
#include	"kb_designpopup.h"

#if		__KB_AUTOMOC
#include 	"kb_framer.moc"
#else
#include 	"kb_framer.h"
#endif

#include	"kb_tabber.h"
#include	"kb_nodereg.h"
#include	"kb_report.h"
#include	"kb_layout.h"
#include	"kb_options.h"
#include	"kb_qrybase.h"
#include	"kb_writer.h"
#include	"kb_ascii.h"

#if		! __KB_RUNTIME
#include	"kb_framerdlg.h"
#include	"kb_formcopier.h"
#include	"kb_popupmenu.h"
#endif

#include	"tk_messagebox.h"

#define		__DEBUG_FRAMER		(0)

/*  KBFramer								*/
/*  KBFramer	: Constructor for framer node				*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  element	: cchar *	: Element name				*/
/*  ok		: bool *	: Success				*/
/*  (returns)	: KBFramer	:					*/

KBFramer::KBFramer
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		cchar			*element,
		bool			*
	)
	:
	KBObject 	(parent, element,	aList),
	KBNavigator	(this,   getBlock(),	m_children),
	m_bgcolor	(this,   "bgcolor",	aList),
	m_title		(this,   "title",	aList, KAF_FORM),
	m_frame		(this,   "frame",	aList, KAF_FORM),
	m_showbar	(this,   "showbar",	aList, KAF_FORM),
	m_tabOrd	(this,   "taborder",	aList, KAF_FORM)
{
	m_frmDisp	= 0 ;
	m_query		= 0 ;
	m_qryLvl	= 0 ;
	m_numRows	= 0 ;
	m_blkInfo	= 0 ;

	m_attribs.remove (&m_showbar) ;
}

/*  KBFramer								*/
/*  KBFramer	: Consructor for framer node				*/
/*  parent	: KBNode *	: Parent node				*/
/*  frame	: KBFramer *	: Extant framer				*/
/*  (returns)	: KBFramer	:					*/

KBFramer::KBFramer
	(	KBNode	  *parent,
		KBFramer  *framer
	)
	:
	KBObject 	(parent, 			framer),
	KBNavigator	(this,	parent == 0 ?
				0 :
				parent->isObject()->getBlock(),
							m_children),
	m_bgcolor 	(this,	"bgcolor",		framer),
	m_title		(this,  "title",		framer, KAF_FORM),
	m_frame		(this,  "frame",		framer, KAF_FORM),
	m_showbar	(this,  "showbar",	  	framer, KAF_FORM),
	m_tabOrd	(this,  "taborder",	  	framer, KAF_FORM)
{
	m_frmDisp	= 0 ;
	m_query		= 0 ;
	m_qryLvl	= 0 ;
	m_numRows	= 0 ;
	m_blkInfo	= 0 ;

	m_attribs.remove (&m_showbar) ;
}

/*  KBFramer								*/
/*  ~KBFramer	: Destructor for framer node				*/
/*  (returns)	:		:					*/

KBFramer::~KBFramer ()
{
	DELOBJ	(m_frmDisp) ;
}

/*  KBNode								*/
/*  endParse	: End parsing for this node				*/
/*  (returns)	: KBNode *	: Parent element			*/

KBNode	*KBFramer::endParse ()
{
	/* This is a hack, in case the number of rows/columns in a	*/
	/* dynamic grid is smaller than the range required by the	*/
	/* objects inside the grid.					*/
	if (isDynamic())
		CITER
		(	Object,
			obj,

			const KBAttrGeom &geom = obj->attrGeom() ;
			m_geom.extendGrid (geom.x() + geom.w(), geom.y() + geom.h()) ;
		)

	return	KBObject::endParse () ;
}

/*  KBFramer								*/
/*  remChild	: Remove a child node					*/
/*  child	: KBNode *	: Child in question			*/
/*  (returns)	: void		:					*/

void	KBFramer::remChild
	(	KBNode	*child
	)
{
	/* Intercept this call since, if the child is an item, then we	*/
	/* must notify our query.					*/
	if ((m_query != 0) && (child->isItem () != 0))
		m_query->remItem (m_qryLvl, child->isItem()) ;

	KBObject::remChild (child) ;
}

/*  KBFramer								*/
/*  framerSetup	: Set up prior to execution				*/
/*  query	: KBQryBase *	: Associated query			*/
/*  qryLvl	: uint		: Level in query			*/
/*  blkInfo	: KBBlockInfo *	: Block information			*/
/*  (returns)	: bool		: Success				*/

bool	KBFramer::framerSetup
	(	KBQryBase	*query,
		uint		qryLvl,
		KBBlockInfo	*blkInfo
	)
{
	m_query		= query  ;
	m_qryLvl	= qryLvl ;
	m_blkInfo	= blkInfo;

	/* Scan to see if there is a rowmark, so that row marks can be	*/
	/* placed in containers as well as blocks ...			*/
	CITER
	(	RowMark,
		r,
		m_blkInfo->rowmark = r ;
	)

	if (m_showbar.getBoolValue())
		m_blkInfo->scroll = m_frmDisp ;

	fixGridLayout () ;

	/* Finally, recurse down to all nested blocks and framers.	*/
	/* This process will traverse the entire tree of blocks and	*/
	/* header, footers, etc.					*/
	CITER
	(	Block,
		b,
		if (!b->blockSetup())
		{	setError (b->lastError ()) ;
			return false ;
		}
	)
	CITER
	(	Framer,
		f,
		if (!f->framerSetup (m_query, qryLvl, m_blkInfo))
		{	setError (f->lastError ()) ;
			return false ;
		}
	)

	return	true	;
}

/*  KBFramer								*/
/*  addAllItems	: Add all items to the query				*/
/*  (returns)	: bool		: Any item fetches from database	*/

bool	KBFramer::addAllItems ()
{
	bool	fetch	= false	;

	CITER
	(	Item,
		i,
		if (m_query->addItem (m_qryLvl, i)) fetch = true ;
	)

	/* We recurse down into nested framers. Note that items in	*/
	/* framers are at the same query level as the block in which	*/
	/* they are embedded.						*/
	CITER
	(
		Framer,
		f,
		if ( f->addAllItems ()) fetch = true  ;
	)

	/* Also recurse down into nested blocks ...			*/
	CITER
	(
		Block,
		b,
		if (!b->addAllItems ()) fetch = false ;
	)

	return	fetch	;
}

/*  KBFramer								*/
/*  replicate	: Replicate this framer					*/
/*  _parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New framer node			*/

KBNode	*KBFramer::replicate
	(	KBNode	*parent
	)
{
	return	replicateBelow (new KBFramer (parent, this)) ;
}

/*  KBFramer								*/
/*  newNode	: Add a new node					*/
/*  id		: int		: Actually NodeSpec pointer for node	*/
/*  (returns)	: KBNode *	: The new node				*/

KBNode	*KBFramer::newNode
	(	int		id
	)
{
#if	! __KB_RUNTIME
	return	KBNavigator::newNode
		(	(NodeSpec *)id,
			newCtrlRect (false),
			getBlock    (),
			m_frmDisp,
			m_bState
		)	;
#else
	return	0 ;
#endif
}

#if	! __KB_RUNTIME

/*  KBFramer								*/
/*  makeNewPopup: Make a new-thingy popup menu for a framer		*/
/*  parent	: KBPopupMenu *	: Parent popup menu			*/
/*  rect	: QRect		: New object area			*/
/*  (returns)	: void		:					*/

KBPopupMenu
	*KBFramer::makeNewPopup
	(	KBPopupMenu	*parent,
		QRect		rect
	)
{
	setCtrlRect (rect)	;

	/* The new-thing popup depends on whether the framer is		*/
	/* embedded in a form, a report or a container. Detect which of	*/
	/* these obtains, and call the corresponding root object to do	*/
	/* the business.						*/
	if (getRoot()->isForm     () != 0)
		return	makeFormNewPopup   (parent, rect, this, &m_bState) ;

	if (getRoot()->isReport   () != 0)
		return	makeReportNewPopup (parent, rect, this, &m_bState) ;

	if (getRoot()->isComponent() != 0)
		switch (getRoot()->isComponent()->objType())
		{
			case KB::ObjForm   :
				return	makeFormNewPopup   (parent, rect, this, &m_bState) ;
			case KB::ObjReport :
				return	makeReportNewPopup (parent, rect, this, &m_bState) ;
			default	:
				break	;
		}

	return	0 ;
}

/*  KBFramer								*/
/*  designPopup	: Handle design menu request				*/
/*  parent	: KBPopupMenu *	: Possible parent menu			*/
/*  cell	: QRect		: Position information			*/
/*  (returns)	: KBPopupMenu *	: Popup menu				*/

KBPopupMenu
	*KBFramer::designPopup
	(	KBPopupMenu	*parent,
		QRect		cell
	)
{
	QString	cText	;
	QString	sText	;

	if	(isHeader    () != 0)
	{	cText = TR("Header")     	;
		sText = TR("header")      	;
	}
	else if (isFooter    () != 0)
	{	cText = TR("Footer")		;
		sText = TR("footer")		;
	}
	else if (isTabberPage() != 0)
	{	cText = TR("Tabber page")	;
		sText = TR("tabber page")	;
	}
	else
	{	cText = TR("Container")		;
		sText = TR("container")		;
	}

	KBPopupMenu  *popupMain	= new KBPopupMenu (parent, &m_bState) ;
	KBPopupMenu  *popupEdit	= 0 ;

	if (parent == 0)
		popupEdit = makeContainerEditPopup
				(	popupMain,
					this,
					sText,
					false
				)	;

	makeContainerMainPopup
	(	popupMain,
		this,
		cText,
		!isDynamic() || (objectInCell (cell) == 0) ?
			makeNewPopup (popupMain, cell) :
			0,
		popupEdit
	)	;

	return	popupMain ;
}

#endif

/*  KBFramer								*/
/*  getPosition	: Get associated control position			*/
/*  (returns)	: QRect		:					*/

QRect	KBFramer::getPosition ()
{
	QRect	pRect	= m_frmDisp->getDisplayWidget()->geometry() ;
	getDisplay()->cvtViewToCtrl (pRect) ;
	return	pRect	;
}

/*  KBFramer								*/
/*  setPalette	: Set palette						*/
/*  (returns)	: void		:					*/

void	KBFramer::setPalette ()
{
#if	__DEBUG_FRAMER
	fprintf
	(	stderr,
		"KBFramer::setPalette: this=%p m_frmDisp=%p\n",
		(void *)this,
		(void *)m_frmDisp
	)	;
#endif
	m_frmDisp->setPalette(getPalette (false)) ;
	KBObject::setPalette () ;
}

/*  KBFramer								*/
/*  setFont	: Set font						*/
/*  (returns)	: void		:					*/

void	KBFramer::setFont ()
{
	m_frmDisp->setFont (getFont (false)) ;
}


/*  KBFramer								*/
/*  buildDisplay: Build display for this node				*/
/*  display	: KBDisplay *	: Parent display			*/
/*  (returns)	: void		:					*/

void	KBFramer::buildDisplay
	(	KBDisplay	*display
	)
{
	m_frmDisp = new KBDisplay
			(	display,
				this,
				m_showbar.getBoolValue()
			)	;

	KBObject::buildDisplay (display) ;

	CITER
	(	Object,
		o,
		o->buildDisplay (m_frmDisp)
	) ;

	setPalette	() ;
	setFont		() ;

	m_frmDisp ->updateDynamic() ;
	m_frmDisp ->setTitle     (m_title.getValue()) ;

	QString	fs	= m_frame.getValue() ;
	int	coff	= fs.find (',')	;

	if (coff >= 0)
		m_frmDisp->setFrame
		(	fs.left(coff    ).toInt(),
			fs.mid (coff + 1).toInt()
		)	;
	else	m_frmDisp->setFrame (0, 0) ;
}

/*  KBFramer								*/
/*  setGeometry	: Object moves in parent				*/
/*  rect	: const QRect &	: New position				*/
/*  (returns)	: void		:					*/

void	KBFramer::setGeometry
	(	const QRect	&rect
	)
{
	KBObject::setGeometry (rect) ;
	if (m_frmDisp != 0) m_frmDisp->setGeometry (rect) ;
}

/*  KBFramer								*/
/*  resizeEvent	: Framer has been resized in its parent			*/
/*  (returns)	: void		:					*/

void	KBFramer::resizeEvent ()
{
#if	__DEBUG_FRAMER
	fprintf
	(	stderr,
		"KBFramer::resizeEvent: %s\n",
		_TEXT(m_frmDisp->getTopWidget()->size())
	)	;
#endif
	getBlock()->resizeEvent() ;
	fixGridLayout () ;
}

/*  KBFramer								*/
/*  updateDisplay							*/
/*		: Owning display geometry has changed			*/
/*  (returns)	: void		:					*/

void	KBFramer::updateDisplay ()
{
#if	__DEBUG_FRAMER
	fprintf
	(	stderr,
		"KBFramer::updateDisplay: %s: %s: %p\n",
		(cchar *)getElement(),
		_TEXT(geometry()),
		(void  *)m_frmDisp
	)	;
#endif
	if (m_frmDisp != 0)
		m_frmDisp->setGeometry (geometry()) ;
}

/*  KBFramer								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBFramer::showAs
	(	KB::ShowAs	mode
	)
{
	m_frmDisp->showAs (mode) ;

#if	! __KB_RUNTIME
	if ((mode == KB::ShowAsDesign) && (getSizer() == 0))
		setSizer (new KBSizer (this, getDisplay(), m_frmDisp->getDisplayWidget())) ;

	if ((mode == KB::ShowAsData  ) && (getSizer() != 0))
		setSizer (0) ;
#endif

	if ((mode == KB::ShowAsData  ))
	{
		fixTabOrder   () ;
		fixGridLayout () ;
	}

	/* Clear all items. This is mainly intended when switching into	*/
	/* design mode, to clear field contents.			*/
	CITER
	(	Item,
		i, 
		i->clearBelow (1) ;
	)

	/* Pass the request on down to the base class, which will in	*/
	/* turn recurse down to any children.				*/
	KBObject::showAs (mode) ;
	updateDisplay () ;

	/* Update the display widget, so that we correctly add or	*/
	/* remove the grid and such like.				*/
	m_frmDisp->getDisplayWidget()->update() ;
}

/*  KBFramer								*/
/*  writeData	 : Display framer contents				*/
/*  prior	 : bool		: Write prior value flag		*/
/*  (returns)	 : bool		: Success				*/

bool	KBFramer::writeData
	(	bool		prior
	)
{
	QRect	 rect    (QPoint(0, 0), geometry().size()) ;
	KBWriter *writer = getReport()->getWriter() ;
	int	 extra	 ;

	new KBWriterBG (writer, rect, m_bgcolor.getValue()) ;

	CITER
	(	Object,
		o,
		if (!o->write (writer, QPoint(0,0), true, extra, prior))
			return false ;
	)

	return	true ;
}

/*  KBFramer								*/
/*  height	: Get header or footer height				*/
/*  (returns)	: int		: Height				*/

int	KBFramer::height ()
{
	return	geometry().height() ;
}

/*  KBFramer								*/
/*  write	: Write block 						*/
/*  writer	: KBWriter *	: Output writer				*/
/*  offset	: QPoint	: Offset in writer			*/
/*  first	: bool		: First value flag			*/
/*  extra	: int &		: Return extra space			*/
/*  prior	: bool		: Write prior value flag		*/
/*  (returns)	: bool		: Success				*/

bool	KBFramer::write
	(	KBWriter	*writer,
		QPoint		offset,
		bool		first,
		int		&extra,
		bool		prior
	)
{
	QString	bgcolor	;
	bgcolor.sprintf	("0x%06x", m_frmDisp->getDisplayWidget()->backgroundColor().rgb() & 0x00ffffff) ;

	new KBWriterBG  (writer, geometry(offset), bgcolor) ;
#if	! __KB_RUNTIME
	if (showingDesign())
		new KBWriterBox (writer, geometry(offset)) ;
#endif

	QPoint	s = writer->setOffset (false, QPoint (geometry().topLeft())) ;
	KBNode::write (writer, offset, first, extra, prior) ;
	writer->setOffset (true, s)  ;
	return	true	;
}

/*  KBFramer								*/
/*  newTabOrder	: Invoke tab order dialog				*/
/*  (returns)	: void		:					*/

void	KBFramer::newTabOrder ()
{
#if	! __KB_RUNTIME
	if (KBNavigator::newTabOrder()) getLayout()->setChanged() ;
#endif
}

/*  KBFramer								*/
/*  newNullBlock: Add a new menu block					*/
/*  (returns)	: void		:					*/

void	KBFramer::newNullBlock ()
{
#if	! __KB_RUNTIME
	newFormBlock (m_frmDisp, KBBlock::BTNull) ;
#endif
}

/*  KBFramer								*/
/*  newTableBlock: Add a new table block				*/
/*  (returns)	 : void		:					*/

void	KBFramer::newTableBlock ()
{
#if	! __KB_RUNTIME
	newFormBlock (m_frmDisp, KBBlock::BTTable) ;
#endif
}

/*  KBFramer								*/
/*  newQueryBlock: Add a new query block				*/
/*  (returns)	 : void		:					*/

void	KBFramer::newQueryBlock ()
{
#if	! __KB_RUNTIME
	newFormBlock (m_frmDisp, KBBlock::BTQuery) ;
#endif
}

/*  KBFramer								*/
/*  newSQLBlock	: Add a new SQL block					*/
/*  (returns)	: void		:					*/

void	KBFramer::newSQLBlock ()
{
#if	! __KB_RUNTIME
	newFormBlock (m_frmDisp, KBBlock::BTSQL) ;
#endif
}

/*  KBFramer								*/
/*  newContainer : Add a new container					*/
/*  (returns)	 : void		:					*/

void	KBFramer::newContainer ()
{
#if	! __KB_RUNTIME
	KBObject::newContainer (m_frmDisp) ;
#endif
}

/*  KBFramer								*/
/*  rowsInBlock	: Calculate number of rows that can be displayed	*/
/*  dx		: int		: X-delta				*/
/*  dy		: int		: Y-delta				*/
/*  (returns)	: uint		: Number of rows			*/

uint	KBFramer::rowsInFrame
	(	int		dx,
		int		dy
	)
{
	if (m_frmDisp != 0)
	{
		QSize	s	= m_frmDisp->effectiveSize() ;
		uint	fw	= s.width  () ;
		uint	fh	= s.height () ;

		uint	nr	= 999	;

		CITER
		(	Item,
			i,

			uint _nr = i->calcNumRows (fw, fh, dx, dy) ;
			if (_nr < nr) nr = _nr ;
		)

		CITER
		(	Framer,
			f,

			uint _nr = f->rowsInFrame (dx, dy) ;
			if (_nr < nr) nr = _nr ;
		)

		if (nr <=   0) nr = 1 ;
		return	nr ;
	}

	return	999	;
}

/*  KBFramer								*/
/*  hideBelow	: Hide all controls below specified row			*/
/*  qRow	: uint		: Query row from which to hide		*/
/*  (returns)	: void		:					*/

void	KBFramer::hideBelow
	(	uint	qRow
	)
{
	CITER
	(	Item,
		i,
		i->hideBelow (qRow)
	)
}

/*  KBFramer								*/
/*  setupControls							*/
/* 		: Set up controls					*/
/*  (returns)	: void		:					*/

void	KBFramer::setupControls ()
{
	CITER
	(	Item,
		i,
		i->setupControls ()
	)
	CITER
	(	Framer,
		f,
		f->setupControls ()
	)
}

/*  KBFramer								*/
/*  clearFields	: Clear values in fields in row				*/
/*  qrow	: uint		: Query row				*/
/*  query	: bool		: Clearing for query			*/
/*  (returns)	: void		:					*/

void	KBFramer::clearFields
	(	uint	qrow,
		bool	query
	)
{
	CITER
	(	Item,
		i,
		i->clearValue  (qrow, query)
	)	;
	CITER
	(	Framer,
		f,
		f->clearFields (qrow, query)
	)	;
}

/*  KBFramer								*/
/*  changed	: See of any item has changed				*/
/*  curQRow	: uint		: Row to check				*/
/*  (returns)	: bool		: True if changed			*/

bool	KBFramer::changed
	(	uint	curQRow
	)
{
	CITER
	(	Item,
		i,
		if (i->isUpdateVal () && i->changed(curQRow))
		{
//			fprintf
//			(	stderr,
//				"KBFramer::changed: [%s]\n",
//				(cchar *)i->getName()
//			)	;
			return	true ;
		}
	)

	CITER
	(	Framer,
		f,
		if (f->changed(curQRow)) return true ;
	)

	return	false	;
}

/*  KBFramer								*/
/*  getNumRows	: Get number of rows in query				*/
/*  (returns)	: uint		: Number of rows			*/

uint	KBFramer::getNumRows ()
{
	return	m_query == 0 ? 0 : m_query->getNumRows (m_qryLvl) ;
}

/*  KBFramer								*/
/*  getCurQRow	: Get current query row					*/
/*  (returns)	: uint		: Query row number			*/

uint	KBFramer::getCurQRow ()
{
	return	getBlock()->getCurQRow() ;
}


/*  KBFramer								*/
/*  getRowValue	: Get value from query					*/
/*  name	: const QString & : Query column name			*/
/*  qrow	: uint		  : Query row				*/
/*  (returns)	: KBValue	  : Value				*/

KBValue	KBFramer::getRowValue
	(	const QString	&name,
		uint		qrow
	)
{
	if (qrow <= getNumRows())
		CITER
		(	Item,
			i,
			if (i->getName() == name)
				return	m_query->getField (m_qryLvl, qrow, i->getQueryIdx()) ;
		)

	return	KBValue() ;
}

/*  KBFramer								*/
/*  setRowValue	: Set value						*/
/*  name	: const QString & : Query column name			*/
/*  qrow	: uint		  : Query row				*/
/*  value	: const KBValye & : Value to set			*/
/*  (returns)	: void		  :					*/

void	KBFramer::setRowValue
	(	const QString	&name,
		uint		qrow,
		const KBValue	&value
	)
{
	if (qrow <= getNumRows())
		CITER
		(	Item,
			i,
			if (i->getName() == name)
			{	m_query->setField (m_qryLvl, qrow, i->getQueryIdx(), value) ;
				return	;
			}
		)
}

/*  KBFramer								*/
/*  setEnabled	: Set control enabled/disabled				*/
/*  enabled	: bool		: Enabled/disabled			*/
/*  (returns)	: void		:					*/

void	KBFramer::setEnabled
	(	bool	enabled
	)
{
	m_frmDisp->getDisplayWidget()->setEnabled (enabled) ;
}

/*  KBFramer								*/
/*  setVisible	: Set framer visibilty					*/
/*  (returns)	: void		  :					*/

void	KBFramer::setVisible
	(	bool		visible
	)
{
	if (visible)
		m_frmDisp->getDisplayWidget()->show() ;
	else	m_frmDisp->getDisplayWidget()->hide() ;
}

/*  KBFramer								*/
/*  isEnabled	: Get control enabled/disabled				*/
/*  (returns)	: bool		: Enabled/disabled			*/

bool	KBFramer::isEnabled ()
{
	return	m_frmDisp->getDisplayWidget()->isEnabled () ;
}

/*  KBFramer								*/
/*  isVisible	: Get control visibility				*/
/*  (returns)	: bool		: Visibility				*/

bool	KBFramer::isVisible ()
{
	return	m_frmDisp->getDisplayWidget()->isVisible () ;
}

/*  KBFramer								*/
/*  showData	: Show data in framer					*/
/*  (returns)	: bool		: Success				*/

bool	KBFramer::showData ()
{
	CITER
	(	FormBlock,
		b,
		if (!b->requery () || !b->showData (KBBlock::SDBackToTop|KBBlock::SDRedisplay))
		{	setError (b->lastError()) ;
			return   false ;
		}
	) ;
	CITER
	(	Framer,
		f,
		if (!f->showData ())
		{	setError (f->lastError()) ;
			return   false ;
		}
	)

	return	true	;
}

/*  KBFramer								*/
/*  pasteObjects: Paste in cut or copied objects			*/
/*  (returns)	: void		:					*/

void	KBFramer::pasteObjects ()
{
#if	! __KB_RUNTIME
	insertObjects (m_frmDisp) ;
#endif
}

/*  KBFramer								*/
/*  pasteHere	: Paste in cut or copied objects at specified position	*/
/*  (returns)	: void		:					*/

void	KBFramer::pasteHere ()
{
#if	! __KB_RUNTIME
	insertHere
	(	KBFormCopier::self()->getCopied (objType()),
		m_frmDisp,
		KBObject::newCtrlRect(false)
	)	;
#endif
}

/*  KBFramer								*/
/*  pasteComponent							*/
/*		: Paste component object				*/
/*  (returns)	: void		:					*/

void	KBFramer::pasteComponent ()
{
#if	! __KB_RUNTIME
	insertComponent
	(	m_frmDisp,
		newCtrlRect(true),
		true
	)	;
#endif
}

/*  KBFramer								*/
/*  linkComponent							*/
/*		: Link component object					*/
/*  (returns)	: void		:					*/

void	KBFramer::linkComponent ()
{
#if	! __KB_RUNTIME
	insertComponent
	(	m_frmDisp,
		newCtrlRect(true),
		false
	)	;
#endif
}


/*  KBFramer								*/
/*  setRowMarked: Row marking operation					*/
/*  qrow1	: uint		: First row				*/
/*  qrow2	: uint		: Upper limit				*/
/*  (returns)	: void		:					*/

void	KBFramer::setRowMarked
	(	uint		qrow1,
		uint		qrow2
	)
{
	for (uint qrow = qrow1 ; qrow < qrow2 ; qrow += 1)
	{
		bool	marked	= m_query->getRowMarked (m_qryLvl, qrow) ;

		CITER
		(	Item,
			i,
			i->setMarked    (qrow, marked)
		)
	}

	CITER
	(	Framer,
		f,
		f->setRowMarked (qrow1, qrow2)
	)
}

/*  KBFramer								*/
/*  doAction	: Propogate action down to nested blocks		*/
/*  action	: KB::Action	: Action in question			*/
/*  (returns)	: bool		: Success				*/

bool	KBFramer::doAction
	(	KB::Action	action
	)
{
	/* NOTE: Calls to this method only make sense for actions like	*/
	/*	 save....						*/
	CITER
	(	FormBlock,
		b,
		if (!b->doAction (action))
		{	setError (b->lastError()) ;
			return   false ;
		}
	)
	CITER
	(	Framer,
		f,
		if (!f->doAction (action))
		{	setError (f->lastError()) ;
			return   false ;
		}
	)

	return	true	;
}

/*  KBFramer								*/
/*  setQryLevel	: Pass query level information into sub-blocks		*/
/*  qryLvl	: uint		: Query level				*/
/*  query	: KBQryBase *	: Query					*/
/*  (returns)	: void		:					*/

void	KBFramer::setQryLevel
	(	uint		qryLvl,
		KBQryBase	*query
	)
{
	/* Scan for framers and pass down the query into them in case	*/
	/* they contain sub-blocks ....					*/
	CITER
	(	Framer,
		f,
		f->setQryLevel (qryLvl, query) ;
	)

	/* ... and then scan for any directly nested sub-blocks.	*/
	CITER
	(	Block,
		b,
		if (b->getBlkType() == KBBlock::BTSubBlock)
			b->setQryLevel (qryLvl + 1, query) ;
	)
}

/*  KBFramer								*/
/*  anyChildBlock: See if there are any child blocks.			*/
/*  (returns)	 : bool		: True if child blocks exist		*/

bool	KBFramer::anyChildBlock	()
{
	/* Scan through all child objects. Descend into any framers,	*/
	/* and return if we find a block _other_ than null blocks.	*/
	CITER
	(	Object,
		o,

		if ((o->isFramer() != 0) && o->isFramer()->anyChildBlock())
			return	true	;

		if ((o->isBlock () != 0) && (o->isBlock()->getBlkType() != KBBlock::BTNull))
			return	true	;
	)

	return	false	;
}

void	KBFramer::getResults
	(	const QString	&path,
		QDict<QString>	&dict
	)
{
	QString	prefix	= QString("%1%2").arg(path).arg(path.isEmpty() ? "" : ".") ;
	uint	qrow	= getBlock()->getCurQRow() ;

	CITER
	(
		Item,
		item,

		if (item->isBlock() == 0)
			dict.insert
			(	QString	("%1%3")
					.arg(prefix)
					.arg(item->getName()),
				new QString(item->getValue(qrow).getRawText())
			)	;
	)

	CITER
	(
		Framer,
		framer,
		framer->getResults
		(	QString	("%1%2%3").arg(prefix).arg(framer->getName()),
			dict
		)	;
	)

	CITER
	(
		FormBlock,
		block,
		block->getResults
		(	QString	("%1%2%3").arg(prefix).arg(block->getName()),
			dict
		)	;
	)
}

/*  KBFramer								*/
/*  invalidControls							*/
/* 		: Scan for any controls which are not valid		*/
/*  row		: uint		     : Row number			*/
/*  errors	: QPtrList<KBItem> & : Return list of invalid items	*/
/*  recurse	: bool		     : Recurse into framers		*/
/*  (returns)	: bool		     : True unless row is empty		*/

void	KBFramer::invalidControls
	(	uint			row,
		QPtrList<KBItem>	&errors,
		bool			recurse
	)
{
	CITER
	(
		Item,
		item,
		if (item->isUpdateVal() && !item->isValid (row, false))
			errors.append (item) ;
	)

	if (recurse)
		CITER
		(
			Framer,
			framer,
			framer->invalidControls (row, errors, true) ;
		)
}

/*  KBFramer								*/
/*  setTabOrder	: Return tab order value				*/
/*  order	: int		: Tab order				*/
/*  (returns)	: void		:					*/

void	KBFramer::setTabOrder
	(	int	order
	)
{
	m_tabOrd.setValue (order) ;
}

/*  KBFramer								*/
/*  getTabOrder	: Return tab order value				*/
/*  (returns)	: int		: Tab order				*/

int	KBFramer::getTabOrder ()
{
	return	m_tabOrd.getIntValue () ;
}

/*  KBFramer								*/
/*  minimumSize	: Get minimum size					*/
/*  (returns)	: QSize		: Minimum size				*/

QSize	KBFramer::minimumSize ()
{
//	extern	int	_indent	;
//	fprintf
//	(	stderr,
//		"%*sKBFramer::minimumSize: %s: isd=%d dsp=%p\n",
//		_indent,
//		"",
//		(cchar *)getName(),
//		isDynamic(),
//		(void  *)m_frmDisp
//	)	;

	/* If we are dynamic and have a display then get the size hint	*/
	/* via that (hence via QT's layout management). Otherwise, work	*/
	/* directly from the configuration.				*/
	if (isDynamic() && (m_frmDisp != 0))
		return	m_frmDisp->sizeHint() ;

	return	KBObject::minimumSize () ;
}


/*  ------------------------------------------------------------------  */

/*  KBHeader								*/
/*  KBHeader	: Constructor for block header node			*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  element	: cchar *	: Element name				*/
/*  ok		: bool *	: Success				*/
/*  (returns)	: KBFramer	:					*/

KBHeader::KBHeader
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		cchar			*element,
		bool			*ok
	)
	:
	KBFramer (parent, aList, element, ok)
{
	m_geom.set	(0, 0, 0, KBAttrGeom::XYNoChange) ;
	m_geom.set	(KBAttrGeom::FMStretch, KBAttrGeom::FMFixed) ;
	m_geom.setMask	(KBAttrGeom::HideX|KBAttrGeom::HideY|KBAttrGeom::HideW|KBAttrGeom::HideXMode|KBAttrGeom::HideYMode) ;

#if	! __KB_RUNTIME
	if (ok != 0)
	{
		if (!::framerPropDlg (this, m_attribs, 0))
		{	delete	this	;
			*ok	= false	;
			return	;
		}

		*ok	= true	;
	}
#endif
}

/*  KBHeader								*/
/*  KBHeader	: Consructor for header node				*/
/*  parent	: KBNode *	: Parent node				*/
/*  frame	: KBFramer *	: Extant framer				*/
/*  (returns)	: KBFramer	:					*/

KBHeader::KBHeader
	(	KBNode	  *parent,
		KBHeader  *header
	)
	:
	KBFramer  (parent, header)
{
	m_geom.set	(0, 0, 0, KBAttrGeom::XYNoChange) ;
	m_geom.set	(KBAttrGeom::FMStretch, KBAttrGeom::FMFixed) ;
	m_geom.setMask	(KBAttrGeom::HideX|KBAttrGeom::HideY|KBAttrGeom::HideW|KBAttrGeom::HideXMode|KBAttrGeom::HideYMode) ;
}

/*  KBHeader								*/
/*  ~KBHeader	: Destructor for framer node				*/
/*  (returns)	:		:					*/

KBHeader::~KBHeader ()
{
}

/*  KBHeader								*/
/*  replicate	: Replicate this header					*/
/*  _parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New framer node			*/

KBNode	*KBHeader::replicate
	(	KBNode	*_parent
	)
{
	return	replicateBelow (new KBHeader (_parent, this)) ;
}


/*  KBHeader								*/
/*  buildDisplay: Build display for this node				*/
/*  display	: KBDisplay *	: Parent display			*/
/*  (returns)	: void		:					*/

void	KBHeader::buildDisplay
	(	KBDisplay	*display
	)
{
	KBFramer::buildDisplay (display) ;

	m_frmDisp->setTagLabel
	(	QString(TR("Header: %1")).arg(getBlock()->getQueryComment())
	)	;
}

/*  KBHeader								*/
/*  setGeometry	: Object moves in parent				*/
/*  rect	: const QRect &	: New position				*/
/*  (returns)	: void		:					*/

void	KBHeader::setGeometry
	(	const QRect	&rect
	)
{
	QRect	pr = parentObject()->geometry() ;

	KBFramer::setGeometry
	(	QRect
		(	0,
			0,
			pr  .width (),
			rect.height()
	)	)	;
}

/*  KBHeader								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBHeader::showAs
	(	KB::ShowAs	mode
	)
{
#if	! __KB_RUNTIME
	if ((mode == KB::ShowAsDesign) && (getSizer() == 0))
	{
		static	QCursor	cSizeVer  (SizeVerCursor) ;

		KBSizerInfoSet	infoSet	;
		KBSizer::defaultInfoSet (infoSet) ;

		infoSet.m_tl.m_proxy  = parentObject () ;
		infoSet.m_tr.m_proxy  = parentObject () ;
		infoSet.m_bl.m_flags  = SZF_Y		;
		infoSet.m_bl.m_cursor = &cSizeVer	;
		infoSet.m_br.m_flags  = SZF_Y		;
		infoSet.m_br.m_cursor = &cSizeVer	;


		setSizer
		(	new KBSizer
			(	this,
				getDisplay(),
				m_frmDisp->getDisplayWidget(),
				&infoSet
		)	)	;
	}
#endif
	KBFramer::showAs (mode) ;
}


/*  ------------------------------------------------------------------  */

/*  KBFooter								*/
/*  KBFooter	: Constructor for block footer node			*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  element	: cchar *	: Element name				*/
/*  ok		: bool *	: Success				*/
/*  (returns)	: KBFramer	:					*/

KBFooter::KBFooter
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		cchar			*element,
		bool			*ok
	)
	:
	KBFramer (parent, aList, element, ok)
{
	m_geom.set	(0, KBAttrGeom::XYNoChange, 0, KBAttrGeom::XYNoChange) ;
	m_geom.set	(KBAttrGeom::FMStretch, KBAttrGeom::FMFloat) ;
	m_geom.setMask	(KBAttrGeom::HideX|KBAttrGeom::HideY|KBAttrGeom::HideW|KBAttrGeom::HideXMode|KBAttrGeom::HideYMode) ;

#if	! __KB_RUNTIME
	if (ok != 0)
	{
		if (!::framerPropDlg (this, m_attribs, 0))
		{	delete	this	;
			*ok	= false	;
			return	;
		}

		*ok	= true	;
	}
#endif
}

/*  KBFooter								*/
/*  KBFooter	: Consructor for footer node				*/
/*  parent	: KBNode *	: Parent node				*/
/*  frame	: KBFramer *	: Extant framer				*/
/*  (returns)	: KBFramer	:					*/

KBFooter::KBFooter
	(	KBNode	  *parent,
		KBFooter  *footer
	)
	:
	KBFramer  (parent, footer)
{
	m_geom.set	(0, KBAttrGeom::XYNoChange, 0, KBAttrGeom::XYNoChange) ;
	m_geom.set	(KBAttrGeom::FMStretch, KBAttrGeom::FMFloat) ;
	m_geom.setMask	(KBAttrGeom::HideX|KBAttrGeom::HideY|KBAttrGeom::HideW|KBAttrGeom::HideXMode|KBAttrGeom::HideYMode) ;
}

/*  KBFooter								*/
/*  ~KBFooter	: Destructor for footer node				*/
/*  (returns)	:		:					*/

KBFooter::~KBFooter ()
{
}

/*  KBFooter								*/
/*  replicate	: Replicate this footer					*/
/*  _parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New framer node			*/

KBNode	*KBFooter::replicate
	(	KBNode	*_parent
	)
{
	return	replicateBelow (new KBFooter (_parent, this)) ;
}

/*  KBFooter								*/
/*  buildDisplay: Build display for this node				*/
/*  display	: KBDisplay *	: Parent display			*/
/*  (returns)	: void		:					*/

void	KBFooter::buildDisplay
	(	KBDisplay	*display
	)
{
	KBFramer::buildDisplay (display) ;

	m_frmDisp->setTagLabel
	(	QString(TR("Footer: %1")).arg(getBlock()->getQueryComment())
	)	;
}

/*  KBFooter								*/
/*  setGeometry	: Object moves in parent				*/
/*  rect	: const QRect &	: New position				*/
/*  (returns)	: void		:					*/

void	KBFooter::setGeometry
	(	const QRect	&rect
	)
{
	QRect	pr = parentObject()->geometry() ;

	fprintf
	(	stderr,
		"KBFooter::setGeometry: pr=%s me=%s\n",
		_TEXT(pr),
		_TEXT(rect)
	)	;

	KBFramer::setGeometry
	(	QRect
		(	0,
			pr  .height() - rect.height(),
			pr  .width (),
			rect.height()
	)	)	;
}

/*  KBFooter								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBFooter::showAs
	(	KB::ShowAs	mode
	)
{
#if	! __KB_RUNTIME
	if ((mode == KB::ShowAsDesign) && (getSizer() == 0))
	{
		static	QCursor	cSizeVer  (SizeVerCursor) ;

		KBSizerInfoSet	infoSet	;
		KBSizer::defaultInfoSet (infoSet) ;

		infoSet.m_bl.m_proxy  = parentObject () ;
		infoSet.m_br.m_proxy  = parentObject () ;
		infoSet.m_tl.m_flags  = SZF_Y		;
		infoSet.m_tl.m_cursor = &cSizeVer	;
		infoSet.m_tr.m_flags  = SZF_Y		;
		infoSet.m_tr.m_cursor = &cSizeVer	;

		setSizer
		(	new KBSizer
			(	this,
				getDisplay(),
				m_frmDisp->getDisplayWidget(),
				&infoSet
		)	)	;
	}
#endif
	KBFramer::showAs (mode) ;
}

/*  ------------------------------------------------------------------  */

/*  KBContainer								*/
/*  KBContainer	: Constructor for container node			*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  element	: cchar *	: Element name				*/
/*  ok		: bool *	: Success				*/
/*  (returns)	: KBFramer	:					*/

KBContainer::KBContainer
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		cchar			*element,
		bool			*ok
	)
	:
	KBFramer 	(parent, aList, element, ok),
	m_image		(this,	 "image",	aList,	KAF_GRPDATA),
	m_autosize	(this,	 "autosize",	aList,	KAF_GRPDATA)
{
#if	! __KB_RUNTIME
	if (ok != 0)
	{
		if (parentIsDynamic())
			m_geom.set (KBAttrGeom::MgmtDynamic, 2, 2) ;

		if (!::framerPropDlg (this, m_attribs, 0))
		{	delete	this	;
			*ok	= false	;
			return	;
		}

		*ok	= true	;
	}
#endif
}

/*  KBContainer								*/
/*  KBContainer	: Consructor for container node				*/
/*  parent	: KBNode *	: Parent node				*/
/*  container	: KBContainer *	: Extant framer				*/
/*  (returns)	: KBContainer	:					*/

KBContainer::KBContainer
	(	KBNode	     *parent,
		KBContainer  *container
	)
	:
	KBFramer  	(parent, container),
	m_image		(this,	 "image",	container,	KAF_GRPDATA),
	m_autosize	(this,	 "autosize",	container,	KAF_GRPDATA)
{
}

/*  KBContainer								*/
/*  ~KBContainer: Destructor for container node				*/
/*  (returns)	:		:					*/

KBContainer::~KBContainer ()
{
}

/*  KBContainer								*/
/*  replicate	: Replicate this container				*/
/*  _parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New framer node			*/

KBNode	*KBContainer::replicate
	(	KBNode	*_parent
	)
{
	return	replicateBelow (new KBContainer (_parent, this)) ;
}

/*  KBContainer								*/
/*  setGeometry	: Set position in parent				*/
/*  rect	: const QRect &	: Position				*/
/*  (returns)	: void		:					*/

void	KBContainer::setGeometry
	(	const QRect	&rect
	)
{
	KBFramer::setGeometry (rect) ;
}

/*  KBContainer								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBContainer::showAs
	(	KB::ShowAs	mode
	)
{
#if	! __KB_RUNTIME
	if ((mode == KB::ShowAsDesign) && (getSizer() == 0))
	{
		setSizer
		(	new KBSizer
			(	this,
				getDisplay(),
				m_frmDisp->getDisplayWidget()
		)	)	;
	}
#endif
	setupProperties	 ()	;
	KBFramer::showAs (mode) ;
}


/*  ------------------------------------------------------------------  */

#if	! __KB_RUNTIME
static	cchar	*imageLegends[] =
{
	"Normal",
	"Rollover",
	"Active",
	"Disable",
	0
}	;
#endif

/*  KBTabberPage							*/
/*  KBTabberPage: Constructor for tabber page node			*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  element	: cchar *	: Element name				*/
/*  ok		: bool *	: Success				*/
/*  (returns)	: KBFramer	:					*/

KBTabberPage::KBTabberPage
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		cchar			*element,
		bool			*ok
	)
	:
	KBFramer 	(parent, aList, element, ok),
	m_tabText	(this,   "tabtext",	aList, KAF_FORM),
	m_image		(this,	 "image",	aList, KAF_GRPWEB)
{
	m_geom.set	(0, 0, 0, 0) ;
	m_geom.set	(KBAttrGeom::FMStretch, KBAttrGeom::FMStretch) ;
	m_geom.setMask	(KBAttrGeom::HideX|KBAttrGeom::HideY|KBAttrGeom::HideW|KBAttrGeom::HideH|KBAttrGeom::HideXMode|KBAttrGeom::HideYMode) ;

#if	! __KB_RUNTIME
	m_image.setNumImages (4);
	m_image.setLegends   (imageLegends) ;

	if (ok != 0)
	{
		if (!::framerPropDlg (this, m_attribs, 0))
		{	delete	this	;
			*ok	= false	;
			return	;
		}

		*ok	= true	;
	}
#endif
}

/*  KBTabberPage							*/
/*  KBTabberPage: Consructor for tabber page node			*/
/*  parent	: KBNode *	: Parent node				*/
/*  tabberpage	: KBTabberPage *: Extant page				*/
/*  (returns)	: KBTabberPage	:					*/

KBTabberPage::KBTabberPage
	(	KBNode	  	*parent,
		KBTabberPage 	*page
	)
	:
	KBFramer	(parent, page),
	m_tabText	(this,   "tabtext", 	page, KAF_FORM),
	m_image		(this,	 "image",	page, KAF_GRPWEB)
{
#if	! __KB_RUNTIME
	m_image.setNumImages (4);
	m_image.setLegends   (imageLegends) ;
#endif

	m_geom.set	(0, 0, 0, 0) ;
	m_geom.set	(KBAttrGeom::FMStretch, KBAttrGeom::FMStretch) ;
	m_geom.setMask	(KBAttrGeom::HideX|KBAttrGeom::HideY|KBAttrGeom::HideW|KBAttrGeom::HideH|KBAttrGeom::HideXMode|KBAttrGeom::HideYMode) ;
}

/*  KBTabberPage							*/
/*  ~KBTabberPage: Destructor for tabber page node			*/
/*  (returns)	:		:					*/

KBTabberPage::~KBTabberPage ()
{
}

/*  KBTabberPage							*/
/*  replicate	: Replicate this tabber page				*/
/*  parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New framer node			*/

KBNode	*KBTabberPage::replicate
	(	KBNode	*parent
	)
{
	return	replicateBelow (new KBTabberPage (parent, this)) ;
}


/*  KBTabberPage							*/
/*  setGeometry	: Set position in parent				*/
/*  rect	: const QRect &	: Position				*/
/*  (returns)	: void		:					*/

void	KBTabberPage::setGeometry
	(	const QRect	&
	)
{
	/* Totally ignore this. The page geometry is completely locked	*/
	/* to the parent.						*/
}

/*  KBTabberPage							*/
/*  geometry	: Get page geometry					*/
/*  (returns)	: QRect		: Geometry				*/

QRect	KBTabberPage::geometry ()
{
	m_geom.set (0, getParent()->isTabber()->barHeight(), 0, 0) ;
	return	KBFramer::geometry() ;
}

/*  KBtabberPage							*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBTabberPage::showAs
	(	KB::ShowAs	mode
	)
{
#if	! __KB_RUNTIME
	if ((mode == KB::ShowAsDesign) && (getSizer() == 0))
	{
		static	QCursor	cSizeVer  (SizeVerCursor) ;
		extern	QCursor	cNoCursor ;

		KBSizerInfoSet	infoSet	;
		KBSizer::defaultInfoSet (infoSet) ;

		infoSet.m_bl.m_proxy  = parentObject () ;
		infoSet.m_br.m_proxy  = parentObject () ;
		infoSet.m_tl.m_flags  = 0		;
		infoSet.m_tl.m_cursor = &cNoCursor	;
		infoSet.m_tr.m_flags  = 0		;
		infoSet.m_tr.m_cursor = &cNoCursor	;

		setSizer
		(	new KBSizer
			(	this,
				getDisplay(),
				m_frmDisp->getDisplayWidget(),
				&infoSet
		)	)	;
	}
#endif
	KBFramer::showAs (mode) ;
}

/*  KBtabberPage							*/
/*  propertyDlg	: Show tabber page property dialog			*/
/*  iniattr	: cchar *	: Initial attribute			*/ 
/*  (returns)	: bool		: Success				*/

bool	KBTabberPage::propertyDlg
	(	cchar		*iniAttr
	)
{
	if (!KBFramer::propertyDlg (iniAttr))
		return false ;

	getParent()->isTabber()->setTabText (this, m_tabText.getValue()) ;
	return	true	;
}

void	KBTabberPage::setEnabled
	(	bool	enabled
	)
{
	if (getParent()->isTabber() != 0)
		getParent()->isTabber()->setPageEnabled (this, enabled) ;

	if (m_frmDisp != 0)
		m_frmDisp->getDisplayWidget()->setEnabled (enabled) ;
}

void	KBTabberPage::setCurrent ()
{
	if (getParent()->isTabber() != 0)
		getParent()->isTabber()->setCurrentPage (this) ;
}

/*  KBTabberPage							*/
/*  printNode	: Output node XML					*/
/*  text	: QString &	: Result string				*/
/*  indent	: int		: Indent level of this node		*/
/*  flat	: bool		: Generate flatened version		*/
/*  (returns)	: void		:					*/

void	KBTabberPage::printNode
	(	QString		&text,
		int		indent,
		bool		flat
	)
{
	KBAttrUInt *tw	= 0 ;

	if (flat)
	{
		uint	w = getParent()->isTabber()->getTabRect(this).width() ;
		tw	= new KBAttrUInt (this, "tabwidth", w) ;
	}
	
	KBFramer::printNode (text, indent, flat) ;
	if (tw != 0) delete tw	;
}


#if	! __KB_RUNTIME

/*  KBFramer								*/
/*  setupProperties							*/
/*		: Set current properties				*/
/*  (returns)	: void		:					*/

void	KBFramer::setupProperties ()
{
	KBObject::setupProperties () ;

	m_frmDisp ->updateDynamic() ;
	m_frmDisp ->setTitle     (m_title.getValue()) ;

	QString	fs	= m_frame.getValue() ;
	int	coff	= fs.find (',')	;

	if (coff >= 0)
		m_frmDisp->setFrame
		(	fs.left(coff    ).toInt(),
			fs.mid (coff + 1).toInt()
		)	;
	else	m_frmDisp->setFrame (0, 0) ;

	getBlock()->resizeEvent  () ;
}

/*  KBFramer								*/
/*  propertyDlg	: Show framer property dialog				*/
/*  iniattr	: cchar *	: Initial attribute			*/ 
/*  (returns)	: bool		: Success				*/

bool	KBFramer::propertyDlg
	(	cchar		*iniAttr
	)
{
	if (::framerPropDlg (this, m_attribs, iniAttr))
	{
		setupProperties () ;
		return	true ;
	}

	return	false	;
}


void	KBContainer::setupProperties ()
{
	KBFramer::setupProperties () ;

	if (!m_image.getValue().isEmpty())
	{
		QStringList	iparts	= QStringList::split ('.', m_image.getValue()) ;

		KBLocation location
		(	getDocRoot()->getDBInfo(),
			"graphic",
			getDocRoot()->getLocation().server(),
			iparts[0],
			iparts[1]
		)	;

		KBError	   error	;
		QByteArray image	;

		if (location.contents (image, error))
			m_frmDisp->setBackgroundPixmap
			(	QPixmap(image),
				m_autosize.getIntValue()
			)	;
		else
			error.DISPLAY()	;
	}
	else	m_frmDisp->setBackgroundPixmap (QPixmap(), 0) ;
}

#endif


NEWNODENAMED(Header,     (cchar *)0, 0, KNF_REPORT, BlockHeader)
NEWNODENAMED(Footer,     (cchar *)0, 0, KNF_REPORT, BlockFooter)
NEWNODENAMED(Container,  (cchar *)0, 0, KNF_FORM,   Container  )
NEWNODENAMED(TabberPage, (cchar *)0, 0, KNF_FORM,   TabberPage )
