/***************************************************************************
    file	         : kb_geometry.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_GEOMETRY_H
#define	__KB_GEOMETRY_H

#include 	<qvaluelist.h>
#include 	<qptrdict.h>
#include	<qpoint.h>
#include	<qobject.h>

class	KBDisplay	;
class	KBGridPosition	;

class	QGridLayout	;
class	QSize		;
class	QRect		;
class	QPainter	;



/*  KBGeometry								*/
/*  ----------								*/
/*  Geometry management class. Runs in either "static" mode which is	*/
/*  provides original fixed/stretch/float operation, or "dynamic" in	*/
/*  which case a "QGridLyaout" manager is used.				*/

class	KBGeometry : public QObject
{
	Q_OBJECT

	QWidget			*m_widget	;
	KBDisplay		*m_display	;
	QGridLayout		*m_grid		;
	QPtrDict<KBGridPosition>m_widgetMap	;
	bool			m_reactivate	;

	void		initGrid		(const KBAttrGeom &) ;
	void		insertIntoGrid		(QWidget *, const QRect &, Qt::AlignmentFlags) ;

protected slots :

	void		slotWidgetDestroyed	(QObject *)	;

public	:

	KBGeometry
	(	QWidget		*,
		KBDisplay	*
	)	;

	void			addChild	(QWidget *, const QRect &, Qt::AlignmentFlags) ;
	void			moveChild	(QWidget *, const QRect &, Qt::AlignmentFlags) ;

	void			setRowColSetup
				(	const KBGridSetupList	&,
					const KBGridSetupList	&
				)	;

	QSize			sizeHint	() ;

	void			resize		(const QSize &) ;
	void			outlineCells	(QPainter    &) ;
	QPoint			getCell		(QPoint) ;

	bool			isDynamic 	() ;

	void			updateDynamic	(const KBAttrGeom &) ;

	/* Static method used to draw cell outlines so that it can be	*/
	/* used from the row/column setup dialog.			*/
	static	void		outlineCells	(QGridLayout *, QPainter &) ;

}	;

#endif	// __KB_GEOMETRY_H
