/***************************************************************************
    file	         : kb_hidden.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<ctype.h>

#include	<qapplication.h>
#include	<qptrlist.h>
#include	<qstring.h>
#include	<qcursor.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"

#include	"kb_sizer.h"
#include	"kb_ctrl.h"
#include	"kb_itemdlg.h"
#include	"kb_block.h"
#include	"kb_hidden.h"
#include	"kb_nodereg.h"


#define		__DEBUG_HIDDEN	(0)

/*  KBHidden								*/
/*  KBHidden	: Constructor for hidden form field			*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*		: bool *	:					*/
/*  (returns)	: KBNode	:					*/

KBHidden::KBHidden
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*
	)
	:
	KBItem	(parent, "KBHidden",	"expr",  aList)
{
	m_values = 0 ;
	m_nRows	 = 0 ;
	m_tabOrd.setValue (0) ;
	init	()  ;
}

/*  KBHidden								*/
/*  KBHidden	: Constructor for hidden form field			*/
/*  parent	: KBObject *	: Parent node				*/
/*  name	: const QString&: Control name				*/
/*  expr	: const QString&: Expression				*/
/*  (returns)	: KBHidden	:					*/

KBHidden::KBHidden
	(	KBObject	*parent,
		bool		&ok
	)
	:
	KBItem	(parent, "KBHidden",	QRect(), "expr", "", 0)
{
	m_tabOrd.setValue (0) ;
	m_values = 0 ;
	m_nRows	 = 0 ;
	init	() ;

#if	! __KB_RUNTIME
	KBItemPropDlg hDlg (this, "Hidden", m_attribs) ;
	if (!hDlg.exec())
	{	ok	= false	;
		return	;
	}
#endif
	ok	= true	;
}

/*  KBHidden								*/
/*  KBHidden	: Constructor for hidden form field			*/
/*  _parent	: KBNode *	: Parent node				*/
/*  _hidden	: KBHidden *	: Extant hidden field			*/
/*  (returns)	: KBNode	:					*/

KBHidden::KBHidden
	(	KBNode		*_parent,
		KBHidden	*_hidden
	)
	:
	KBItem	(_parent, "expr", _hidden)
{
	m_tabOrd.setValue (0) ;
	m_values = 0 ;
	m_nRows	 = 0 ;
	init	() ;
}

/*  KBHidden								*/
/*  ~KBHidden	: Destructor for simple form field			*/
/*  (returns)	:		:					*/

KBHidden::~KBHidden ()
{
	if (m_values != 0) delete []m_values ;
}

/*  KBHidden								*/
/*  init	: Initialise						*/
/*  (returns)	: void		:					*/

void	KBHidden::init ()
{
	m_geom.set	(0, 0, 0, 0) ;
	m_geom.setMask	(KBAttrGeom::HideAll) ;

	/* We detach all the attributes that are not relevent, so that	*/
	/* they will not show up in the properties dialog.		*/
	m_attribs.remove (&m_tabOrd    ) ;
	m_attribs.remove (&m_defVal    ) ;
	m_attribs.remove (&m_rdonly    ) ;
	m_attribs.remove (&m_onEnter   ) ;
	m_attribs.remove (&m_onLeave   ) ;
	m_attribs.remove (&m_onDblClick) ;
}

/*  KBHidden								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBHidden::showAs
	(	KB::ShowAs	mode
	)
{
//	if (m_values != 0)
//	{	delete []m_values ;
//		m_values = 0	  ;
//	}

	KBItem::showAs (mode) ;
}

/*  KBHidden								*/
/*  calcNumRows	: Calculate number of rows which can be show		*/
/*  bw		: uint		: Block display width			*/
/*  bh		: uint		: Block display height			*/
/*  dx		: int		: Control X-spacing			*/
/*  dy		: int		: Control Y-spacing			*/
/*  (returns)	: uint		: Number of rows that can be displayed	*/

uint	KBHidden::calcNumRows
	(	uint	,
		uint	,
		int	,
		int
	)
{
	/* There is no limit since hidden controls do not actually	*/
	/* appear on the display.					*/
	return	999	;
}

/*  KBHidden								*/
/*  dx		: int		: Control X-spacing			*/
/*  dy		: int		: Control Y-spacing			*/
/*  limit	: uint		: Upper limit on number of rows		*/
/*  (returns)	: uint		: Number of rows that can be displayed	*/

uint	KBHidden::calcNumRows
	(	int	,
		int	,
		uint	limit
	)
{
	return	limit	;
}

/*  KBHidden								*/
/*  setupControls							*/
/*		: Extend (or decrease) the number of controls		*/
/*  (returns)	: void		:					*/

void	KBHidden::setupControls ()
{
	uint	numrows	= getBlock()->getDisplayRows() ;

#if	__DEBUG_HIDDEN
	fprintf
	(	stderr,
		"KBHidden::setupControls: %s: m_values=%p numrows=%d m_nRows=%d\n",
		getPath().latin1(),
		(void *)m_values,
		numrows,
		m_nRows
	)	;
#endif
	if ((m_values == 0) || (numrows != m_nRows))
	{
		KBValue	*newvals = new KBValue[numrows]  ;

		if (m_values != 0)
		{
			uint	copyto   = m_nRows < numrows ? m_nRows : numrows ;
			for (uint idx = 0 ; idx < copyto ; idx += 1)
				newvals[idx] = m_values[idx] ;
			delete	 [] m_values	;
		}

		m_values =  newvals	;
		m_nRows	 =  numrows	;
	}
}

/*  KBItem								*/
/*  valuesAtQRow: Get value pair at specified row			*/
/*  qrow	: uint		: Specific row				*/
/*  (returns)	: KBValue *	: Value or none				*/

KBValue	*KBHidden::valueAtQRow
	(	uint	qrow
	)
{
	uint	curDRow  = getBlock()->getCurDRow () ;

	if ((m_values == 0) || (qrow >= curDRow + m_nRows))
	{
#if	__DEBUG_HIDDEN
		fprintf
		(	stderr,
			"KBHidden::valueAtQRow: *** calling setupControls ***\n"
		)	;
#endif
		setupControls () ;
	}

#if	__DEBUG_HIDDEN
	fprintf
	(	stderr,
		"KBHidden::valueAtQRow: %s: qrow=%d: showing [%p] %d for %d\n",
		getPath().latin1(),
		qrow,
		(void *)m_values,
		curDRow,
		m_nRows
	)	;
#endif

	if (m_values != 0)
		if ((qrow >= curDRow) && (qrow < curDRow + m_nRows))
			return	&m_values[qrow - curDRow] ;

	setError
	(	KBError::Error,
		TR("Row in query not currently displayed"),
		QString	(TR("Hidden %1, query row %2: showing %3 for %4"))
			.arg(getName())
			.arg(qrow)
			.arg(curDRow)
			.arg(m_nRows),
		__ERRLOCN
	)	;

	return	0 ;
}

/*  KBHidden								*/
/*  getValue	: Get value						*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBHidden::getValue
	(	uint	qrow
	)
{
	KBValue	*value	= valueAtQRow (qrow)	;

#if	__DEBUG_HIDDEN
	fprintf
	(	stderr,
		"KBHidden::getValue: %s: %d: %p\n",
		getPath().latin1(),
		qrow,
		value
	)	;
#endif
	return	value == 0 ? KBValue() : *value ;
}

/*  KBHidden								*/
/*  setValue	: Set value display row					*/
/*  qrow	: uint		 : Query row				*/
/*  value	: const KBValue &: Value to set				*/
/*  (returns)	: bool		 : Success				*/

bool	KBHidden::setValue
	(	uint		qrow,
		const KBValue	&value
	)
{
	KBScriptError	*eRC	;
	bool		subs	;
	KBValue		v	= m_expr.evaluate (value, eRC, subs) ;

	if (eRC != 0)
	{
		KBScriptError::processError (eRC, KBScriptError::Hidden) ;
		return	false	;
	}

	KBValue	*vp	= valueAtQRow (qrow) ;

#if	__DEBUG_HIDDEN
	fprintf
	(	stderr,
		"KBHidden::setValue: %s: qrow=%d vp=%p v=[%s]\n",
		getPath().latin1(),
		qrow,
		(void  *)vp,
		(cchar *)v.getRawText()
	)	;
#endif
	if (vp != 0) *vp = v ;
	return	true	;
}

/*  KBHidden								*/
/*  clearValue	: Clear value for specified query and display row	*/
/*  qrow	: uint		: Starting row in query			*/
/*  query	: bool		: Clearing for query			*/
/*  (returns)	: void		:					*/

void	KBHidden::clearValue
	(	uint	qrow,
		bool
	)
{
	KBValue	*value = valueAtQRow (qrow) ;
	if (value != 0) *value = KBValue("", &_kbString) ;
}

/*  KBHidden								*/
/*  changed	: See if item has changed				*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: bool		: Item changed				*/

bool	KBHidden::changed
	(	uint	qrow
	)
{
	KBValue	*value = valueAtQRow (qrow) ;
	return	value == 0 ? false : *value != getIniValue (qrow) ;
}

/*  KBHidden								*/
/*  isValid	: Test if value is valid to be saved			*/
/*  qrow	: uint		: Query row				*/
/*  alowNull	: bool		: Ignore not-null check			*/
/*  (returns)	: bool		: Value can be saved			*/

bool	KBHidden::isValid
	(	uint	qrow,
		bool
	)
{
	return	valueAtQRow (qrow) != 0 ;
}

/*  KBHidden								*/
/*  isEmpty	: See if value is empty					*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: bool		: Empty					*/

bool	KBHidden::isEmpty
	(	uint	qrow
	)
{
	KBValue	*value	= valueAtQRow (qrow) ;
	return	value == 0 ? true : value->getRawText().isEmpty() ;
}

/*  KBHidden								*/
/*  replicate	: Replicate this hidden field				*/
/*  _parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New hidden node			*/

KBNode	*KBHidden::replicate
	(	KBNode	*_parent
	)
{
	return	new KBHidden (_parent, this) ;
}

#if	! __KB_RUNTIME
bool	KBHidden::propertyDlg
	(	cchar		*iniAttr
	)
{
	KBItemPropDlg hDlg (this, "Hidden", m_attribs) ;
	return	hDlg.exec() ;
}
#endif


NEWNODE(Hidden, (cchar *)0, 0, KNF_FORM)
