/***************************************************************************
    file	         : kb_item.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qapplication.h>
#include	<qregexp.h>

#if		__KB_AUTOMOC
#include	"kb_item.moc"
#else
#include	"kb_item.h"
#endif

#include	"kb_ctrl.h"
#include	"kb_formblock.h"
#include	"kb_qrybase.h"
#include	"kb_nodemonitor.h"
#include	"kb_writer.h"
#include	"kb_layout.h"
#include	"kb_display.h"
#include	"kb_sizer.h"
#include	"kb_debugging.h"
#include	"kb_script.h"


QPalette *getMarkedPalette ()
{
	static	QPalette *markedPal ;

	if (markedPal == 0)
	{
		const QColor fg = QColor (255, 255, 255) ;
		const QColor bg = QColor (  0,   0,   0) ;
	
		markedPal = new QPalette (QApplication::palette()) ;
		markedPal->setColor  (QColorGroup::Text,       fg) ;
		markedPal->setColor  (QColorGroup::Foreground, fg) ;
		markedPal->setColor  (QColorGroup::Base,       bg) ;
		markedPal->setColor  (QColorGroup::Background, bg) ;
	}

	return	markedPal ;
}

/*  KBItem								*/
/*  KBItem	: Constructor for form item class			*/
/*  node	: KBNode *	: Parent node				*/
/*  name	: cchar *	: Name of actual node			*/
/*  ename	: cchar *	: Name of expression attribute		*/
/*  aList	: const QDict<QString> &				*/
/*				: Attribute list			*/
/*  (returns)	: KBItem	:					*/

KBItem::KBItem
	(	KBNode			*node,
		cchar 			*name,
		cchar			*ename,
		const QDict<QString>	&aList
	)
	:
	KBObject	(node, name,       		aList),
	m_expr		(this, ename,      		aList),
	m_rdonly	(this, "rdonly",   		aList, KAF_FORM),
	m_noupdate	(this, "noupdate",   		aList, KAF_FORM),
	m_tabOrd	(this,	"taborder", 		aList, KAF_FORM),
	m_defVal	(this, "default",  		aList),
	m_errtext	(this, "errtext",		aList),
	m_onEnter	(this, "onenter",    "onItem",	aList, KAF_FORM),
	m_onLeave	(this, "onleave",    "onItem",	aList, KAF_FORM),
	m_onSet		(this, "onset",      "onItem",	aList),
	m_onDblClick	(this, "ondblclick", "onItem",	aList)
{
	m_nCtrls	= 0	;
	m_ctrls		= 0	;
	m_ctrlMon	= 0	;
	m_table		= 0	;
	m_flags		= 0	;
	m_allEnabled	= true	;
	m_allVisible	= true	;
	m_isUpdateVal	= -1	;
	m_type		= 0	;
}

/*  KBItem								*/
/*  KBItem	: Constructor for form item class			*/
/*  parent	: KBObject *	: Parent object				*/
/*  name	: cchar *	: Name of actual node			*/
/*  rect	: const QRect &	: Size and position			*/
/*  ename	: cchar *	: Name of expression attribute		*/
/*  value	: cchar *	: Value for exprssion attribute		*/
/*  tabOrd	: uint		: Tab order				*/
/*  (returns)	: KBItem	:					*/

KBItem::KBItem
	(	KBObject	*parent,
		cchar 		*name,
		const QRect &	rect,
		cchar 		*ename,
		cchar		*value,
		uint		tabOrd
	)
	:
	KBObject	(parent, name, rect),
	m_expr		(this,   ename,       		 value),
	m_rdonly	(this,   "rdonly",    		 false,		KAF_FORM),
	m_noupdate 	(this,   "noupdate",  		 false,		KAF_FORM),
	m_tabOrd	(this,	  "taborder",  		 tabOrd + 1,	KAF_FORM),
	m_defVal	(this,   "default",   		 ""),
	m_errtext	(this,   "errtext",		 ""),
	m_onEnter	(this,   "onenter",    "onItem", "",		KAF_FORM),
	m_onLeave	(this,   "onleave",    "onItem", "",		KAF_FORM),
	m_onSet		(this,   "onset",      "onItem", ""),
	m_onDblClick	(this,   "ondblclick", "onItem", "")
{
	m_nCtrls	= 0	;
	m_ctrls		= 0	;
	m_ctrlMon	= 0	;
	m_table		= 0	;
	m_flags		= 0	;
	m_allEnabled	= true	;
	m_allVisible	= true	;
	m_isUpdateVal	= -1	;

	m_type		= 0	;
}

/*  KBItem								*/
/*  KBItem	: Constructor for form item class			*/
/*  _parent	: KBNode *	: Parent node				*/
/*  _ename	: cchar *	: Name of expression attribute		*/
/*  _item	: KBItem *	: Extant item				*/
/*  (returns)	: KBItem	:					*/

KBItem::KBItem
	(	KBNode		*_parent,
		cchar		*_ename,
		KBItem		*_item
	)
	:
	KBObject	(_parent, _item),
	m_expr		(this, _ename,     		_item),
	m_rdonly	(this, "rdonly",   		_item, KAF_FORM),
	m_noupdate	(this, "noupdate",   		_item, KAF_FORM),
	m_tabOrd	(this,	"taborder", 		_item, KAF_FORM),
	m_defVal	(this, "default",  		_item),
	m_errtext	(this, "errtext",		_item),
	m_onEnter	(this, "onenter",    "onItem", 	_item, KAF_FORM),
	m_onLeave	(this, "onleave",    "onItem", 	_item, KAF_FORM),
	m_onSet		(this, "onset",      "onItem", 	_item),
	m_onDblClick	(this, "ondblclick", "onItem",  _item)
{
	m_nCtrls	= 0	;
	m_ctrls		= 0	;
	m_ctrlMon	= 0	;
	m_table		= 0	;
	m_flags		= 0	;
	m_allEnabled	= true	;
	m_allVisible	= true	;
	m_isUpdateVal	= -1	;

	/* See comment in constructor above ....			*/
	m_type		= 0	;
}

/*  KBItem								*/
/*  ~KBItem	: Destructor for form item class			*/
/*  (returns)	:		:					*/

KBItem::~KBItem ()
{
	if (m_ctrls != 0)
	{
		for (uint idx = 0 ; idx < m_nCtrls ; idx += 1)
			DELOBJ (m_ctrls[idx]) ;

		delete	[] m_ctrls ;
	}

	if (m_type != 0) m_type->deref() ;
}

/*  KBItem								*/
/*  moveFocusOK	: Check whether focus can be set			*/
/*  drow	: uint		: Display row				*/
/*  (returns)	: bool		: True if focus can be set		*/

bool	KBItem::moveFocusOK
	(	uint	drow
	)
{
#if	__KB_DEBUG_FOCUS
	fprintf
	(	stderr,
		"KBItem::moveFocusOK: %s: row=%d\n",
		(cchar *)getName(),
		drow
	)	;
#endif
	/* Unless in design mode, or there is no parent block (and	*/
	/* there should be!), check with the block whether focus is	*/
	/* allowed into this item (really, out of a current item).	*/
	return	showingDesign()	     ? true :
		getFormBlock () == 0 ? true :
				     getFormBlock()->moveFocusOK (this, drow) ;
}

/*  KBItem								*/
/*  focusInEvent: Handle focus in					*/
/*  drow	: uint		: Display row				*/
/*  reason	: Reason	: Reason focus arrives			*/
/*  (returns)	: void		:					*/

void	KBItem::focusInEvent
	(	uint			drow,
		QFocusEvent::Reason	reason
	)
{
#if	__KB_DEBUG_FOCUS
	fprintf
	(	stderr,
		"KBItem::focusInEvent: %s:row=%d r=%d\n",
		(cchar *)getName(),
		drow,
		reason
	)	;
#endif
	if (getFormBlock() != 0)
		getFormBlock()->focusInEvent (this, drow, reason) ;
}

/*  KBItem								*/
/*  doEnter	: Handle focus entry					*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: void		:					*/

void	KBItem::doEnter
	(	uint	qrow
	)
{
#if	__KB_DEBUG_FOCUS
	fprintf
	(	stderr,
		"KBItem::doEnter: %s: qrow=%d\n",
		(cchar *)getName(),
		qrow
	)	;
#endif

	KBValue	arg	= (int)qrow ;
	bool	evRc	;
	eventHook (m_onEnter, 1, &arg, evRc) ;
}

/*  KBItem								*/
/*  doLeave	: Handle focus exit					*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: bool		: OK to leave				*/

bool	KBItem::doLeave
	(	uint	qrow
	)
{
#if	__KB_DEBUG_FOCUS
	fprintf
	(	stderr,
		"KBItem::doLeave: %s: %d\n",
		(cchar *)getName(),
		qrow
	)	;
#endif
	KBValue	arg	= (int)qrow ;
	bool	evRc	;
	bool	rc	= eventHook (m_onLeave, 1, &arg, evRc) ? evRc : false ;

//	fprintf
//	(	stderr,
//		"KBItem::doLeave [%s][%s] -> %d\n",
//		(cchar *)getElement(),
//		(cchar *)getName(),
//		rc
//	)	;

	/* If execution OK then return the exection return code. On	*/
	/* error return false to prevent moving focus (and thence maybe	*/
	/* getting further errors).					*/
	return	rc ? evRc : false ;
}

/*  KBItem								*/
/*  doDblClick	: Handle double click					*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBItem::doDblClick
	(	uint	qrow
	)
{
	KBValue	arg	= (int)qrow ;
	bool	evRc	;
	bool	rc	= eventHook (m_onDblClick, 1, &arg, evRc, false) ? evRc : false ;

	fprintf
	(	stderr,
		"KBItem::doDblClick [%s][%s] -> rc=%d\n",
		(cchar *)getElement(),
		(cchar *)getName(),
		rc
	)	;

	return	rc ;
}

/*  KBItem								*/
/*  setupControls							*/
/*		: Setup controls					*/
/*  (returns)	: void		:					*/

void	KBItem::setupControls ()
{
//	if (getDisplay() == 0) return ;

	KBBlock	*block	= getBlock()		  ;
	uint	nRows	= block->getDisplayRows() ;
	int	dx	= block->getDisplayDX  () ;
	int	dy	= block->getDisplayDY  () ;
	QRect	r	= geometry()		  ;
	int	ax	= r.x()			  ;
	int	ay	= r.y()			  ;

//	fprintf
//	(	stderr,
//		"KBItem::setupControls: %s: %s: %d->%d\n",
//		(cchar *)getElement(),
//		(cchar *)getName(),
//		m_nCtrls,
//		nRows
//	)	;

	/* If the number of controls is the same or reduced, then	*/
	/* reposition those that remain and delete the reset.		*/
	if (nRows <= m_nCtrls)
	{
		for (uint idx1 = 0 ; idx1 < nRows ; idx1 += 1)
		{
			m_ctrls[idx1]->setGeometry (QRect(ax, ay, r.width(), r.height())) ;
			ax += dx ;
			ay += dy ;
		}

		for (uint idx2 = nRows ; idx2 < m_nCtrls ; idx2 += 1)
			DELOBJ (m_ctrls[idx2]) ;

		m_nCtrls = nRows ;

		if (m_nCtrls == 0) setControl (0) ;
		return	;
	}

	/* Increasing the number of controls. Start by allocating a	*/
	/* new controls vector, clearing to null pointer and copying	*/
	/* pointers at the extant controls.				*/
	KBControl **save = m_ctrls ;

	m_ctrls = new KBControl * [nRows] ;
	memset	(m_ctrls, 0, sizeof(KBControl *) * nRows) ;

	for (uint idx1 = 0 ; idx1 < m_nCtrls ; idx1 += 1)
		m_ctrls[idx1] = save[idx1] ;

	if (save != 0) delete [] save ;

	/* Now allocate as many more controls as are needed. These	*/
	/* are stored in the controls array, set up and positioned.	*/
	/* Extant controls are simple repositioned.			*/

	for (uint idx2 = 0 ; idx2 < nRows ; idx2 += 1)
	{
		KBControl *ctrl   = m_ctrls[idx2] ;
		QRect	  rect	  (ax, ay, r.width(), r.height()) ;

		if (ctrl == 0)
		{
			ctrl	= makeCtrl (idx2) ;

			ctrl->showAs  	  (isShowing ()) ;
			ctrl->setVisible  (m_allVisible) ;
			ctrl->setEnabled  (m_allEnabled) ;

			if (m_ctrlMon != 0) ctrl->showMonitor (m_ctrlMon) ;
			if (showingData ()) ctrl->setMorphed  (isMorphing()) ;

			KBObject::setupProperties (ctrl, rect) ;
			m_ctrls[idx2] = ctrl ;
		}
		else	ctrl->setGeometry (rect) ;

		ax += dx ;
		ay += dy ;

	}


	/* Note the number of controls now present, and pass the zeroth	*/
	/* control down to the object base class.			*/
	m_nCtrls = nRows ;
	setControl (m_nCtrls == 0 ? 0 : m_ctrls[0]) ;
}

/*  KBItem								*/
/*  ctrlGone	: Note death of control					*/
/*  ctrl	: KBControl *	: The control				*/
/*  (returns)	: void		:					*/

void	KBItem::ctrlGone
	(	KBControl	*ctrl
	)
{
	for (uint idx = 0 ; idx < m_nCtrls ; idx += 1)
		if (m_ctrls[idx] == ctrl)
		{	m_ctrls[idx] = 0 ;
			break ;
		}

	KBObject::ctrlGone (ctrl) ;
}

/*  KBItem								*/
/*  setGeometry	: Set new geometry					*/
/*  rect	: const QRect &	: Position				*/
/*  (returns)	: void		:					*/

void	KBItem::setGeometry
	(	const QRect	&rect
	)
{
	KBObject::setGeometry (rect) ;
	if (getDisplay() != 0) setupControls () ;
}

/*  KBItem								*/
/*  setPalette	: Set palette						*/
/*  (returns)	: void		:					*/

void	KBItem::setPalette ()
{
	KBObject::setPalette () ;

	QPalette  *pal 	= getPalette (false) ;
	for (uint idx = 0 ; idx < m_nCtrls ; idx += 1)
		m_ctrls[idx]->setPalette (pal) ;
}

/*  KBItem								*/
/*  setFont	: Set font						*/
/*  (returns)	: void		:					*/

void	KBItem::setFont ()
{
	KBObject::setFont () ;

	QFont	*font 	= getFont (false) ;
	for (uint idx = 0 ; idx < m_nCtrls ; idx += 1)
		m_ctrls[idx]->setFont (font) ;
}

/*  KBItem								*/
/*  getVTrans	: Get value translation mode				*/
/*  (returns)	: KBValue::VTrans : Translation mode			*/

KBValue::VTrans	KBItem::getVTrans ()
{
	/* Default is text, since this is true for almost all items,	*/
	/* the obvious exception being pixmaps.				*/
	return	KBValue::VText	;
}

/*  KBItem								*/
/*  makeCtrl	: Make specific control					*/
/*  drow	: uint		: Display row number			*/
/*  (returns)	: KBControl *	: New control				*/

KBControl
	*KBItem::makeCtrl
	(	uint
	)
{
	KBError::EFault
	(	QString	(TR("KBItem::makeCtrl called for \"%1::%2\""))
			.arg (getElement())
			.arg (getName   ()),
		QString::null,
		__ERRLOCN
	)	;
	return	0 ;
}

/*  KBItem								*/
/*  getExpr	: Get item expression and set query column index	*/
/*  (returns)	: QString	: Expression				*/

QString	KBItem::getExpr ()
{
	return	m_expr.getValue ()   ;
}


/*  KBItem								*/
/*  setFieldType: Set database field type for the item			*/
/*  _type	: KBType *	: Type					*/
/*  (returns)	: void		:					*/

void	KBItem::setFieldType
	(	KBType	*_type
	)
{
	if (m_type != 0) m_type->deref() ;
	m_type	= _type	;
	if (m_type != 0) m_type->ref  () ;
}

/*  KBItem								*/
/*  isUpdateVal	: Test if item is used when updating database		*/
/*  (returns)	: bool		: TRUE if so				*/

bool	KBItem::isUpdateVal ()
{
	if (m_isUpdateVal < 0)
	{
		/* NOTE : Qt2.2.1: Setting the case sensitive argument	*/
		/* false does not seem to work, hence [a-zA-Z]		*/

		/* FIXME: This should be handled in the drivers ...	*/
		static	QRegExp e1	("^\\s*[_a-zA-Z0-9]*[_a-zA-Z][_a-zA-Z0-9]*\\s*$", false) ;
		static	QRegExp e2	("^\\s*[_a-zA-Z0-9]*[_a-zA-Z][_a-zA-Z0-9]*\\s*\\.\\s*[_a-zA-Z0-9]*[_a-zA-Z][_a-zA-Z0-9]*\\s*$", false) ;
		const 	QString	&e	= m_expr.getValue () ;

		m_isUpdateVal = (e1.match (e) >= 0) || (e2.match (e) >= 0) ;

//		fprintf
//		(	stderr,
//			"KBItem::isUpdateVal: [%s] update [%d]\n",
//			(cchar *)e,
//			m_isUpdateVal
//		)	;
	}

	return	m_noupdate.getBoolValue() ? false : m_isUpdateVal ;
}

/*  KBItem								*/
/*  getValue	: Get value						*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBItem::getValue
	(	uint	qrow
	)
{
	static	KBValue	nullVal	;
	KBControl *ctrl = ctrlAtQRow (qrow) ;
	return	ctrl == 0 ? nullVal : ctrl->getValue()  ;
}

/*  KBItem								*/
/*  getRowValue	: Get value for row from query				*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBItem::getRowValue
	(	uint	qrow
	)
{
	return	getBlock()->getQuery()->getField
				(	m_qryIdx.qryLvl(),
					qrow,
					m_qryIdx.qryIdx()
				)	;
}

/*  KBItem								*/
/*  getOrderType: Get ordering type					*/
/*  (returns)	: KB::IType	: Type					*/

KB::IType KBItem::getOrderType()
{
	/* Return the ordering type for the item given a raw value.	*/
	/* The default is the value itself, but the method may be	*/
	/* overridden, for instance by a link control.			*/
	return	m_type == 0 ? KB::ITUnknown : m_type->getIType() ;
}

/*  KBItem								*/
/*  getOrderValue							*/
/*		: Get ordering value					*/
/*  value	: const KBValue & : Value				*/
/*  (returns)	: QString	  : Ordering value			*/

QString	KBItem::getOrderValue
	(	const KBValue	&value
	)
{
	/* Return the ordering value for the item given a raw value.	*/
	/* The default is the value itself, but the method may be	*/
	/* overridden, for instance by a link control.			*/
	return	value.getRawText() ;
}

/*  KBItem								*/
/*  setData	: Set control-specific data				*/
/*  qrow	: uint		: Query row				*/
/*  data	: void *	: Data					*/
/*  (returns)	: void		:					*/

void	KBItem::setData
	(	uint	qrow,
		void	*data
	)
{
	KBControl *ctrl = ctrlAtQRow (qrow) ;
	if (ctrl != 0) ctrl->setData (data) ;
}

/*  KBItem								*/
/*  setValue	: Set value display row					*/
/*  qrow	: uint		 : Query row				*/
/*  value	: const KBValue &: Value to set				*/
/*  (returns)	: bool		 : Success				*/

bool	KBItem::setValue
	(	uint		qrow,
		const KBValue	&value
	)
{
	KBScriptError	*eRC	;

	m_curVal = m_expr.evaluate (value, eRC, m_fSubs) ;
	if (eRC != 0)
	{
		KBScriptError::processError (eRC, KBScriptError::Normal) ;
		return	false	;
	}

	KBControl *ctrl = ctrlAtQRow  (qrow)   ;

//	fprintf
//	(	stderr,
//		"KBItem::setValue: [%d/%d][%s][%s]->[%p]\n",
//		qrow,
//		getBlock()->getCurQRow(),
//		(cchar *)value .getRawText(),
//		(cchar *)m_curVal.getRawText(),
//		(void  *)ctrl
//	)	;

	if (ctrl != 0) ctrl->setValue (m_curVal) ;

	/* Normally the type will be set from the query as soon as the	*/
	/* select query has been executed, but in case we set a value	*/
	/* before the query (eg., in the block "PreQuery" event) then	*/
	/* use the value type if the current type is unknown.		*/
	if (m_type->getIType() == KB::ITUnknown)
	{
		m_type->deref() ;
		m_type	= m_curVal.getType() ;
		m_type->ref  () ;
	}

	KBValue	args[2]	    ;
	bool	evRc	    ;

	args[0] = (int)qrow ;
	args[1] = m_curVal  ;

	return	eventHook (m_onSet, 2, args, evRc) ;
}

/*  KBItem								*/
/*  clearValue	: Clear value for specified query and display row	*/
/*  qrow	: uint		: Starting row in query			*/
/*  query	: bool		: Clearing for query			*/
/*  (returns)	: void		:					*/

void	KBItem::clearValue
	(	uint	qrow,
		bool	query
	)
{
	KBControl *ctrl = ctrlAtQRow (qrow) ;
	if (ctrl != 0) ctrl->clearValue (query) ;
}

/*  KBItem								*/
/*  changed	: See if user has changed an item			*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: bool		: Item changed				*/

bool	KBItem::changed
	(	uint	qrow
	)
{
	if (isUpdateVal())
	{
		KBControl *ctrl = ctrlAtQRow (qrow) ;
		return	ctrl == 0 ? false : ctrl->changed () ;
	}

	return	false	;
}

/*  KBItem								*/
/*  isEmpty	: See if control is empty				*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: bool		: Empty					*/

bool	KBItem::isEmpty
	(	uint	qrow
	)
{
	KBControl *ctrl = ctrlAtQRow (qrow) ;
	return	ctrl == 0 ? true : ctrl->isEmpty () ;
}

/*  KBItem								*/
/*  isValid	: Test if value is valid to be saved			*/
/*  qrow	: uint		: Query row				*/
/*  allowNull	: bool		: Ignore not-null checks		*/
/*  (returns)	: bool		: Value can be saved			*/

bool	KBItem::isValid
	(	uint	qrow,
		bool	allowNull
	)
{
	KBControl *ctrl = ctrlAtQRow (qrow) ;
	if (ctrl == 0) return true	    ;

	/* At this point check for an empty value along with a default	*/
	/* value or expression, in which case evaluate the default	*/
	/* expression: note that we pass the default itself as the	*/
	/* 'default' for the evaluation, so that writing a simple	*/
	/* default value has the obvious effect. On return check for	*/
	/* errors.							*/
	KBValue	  value	= ctrl->getValue () ;
	if (value.isEmpty() && !m_defVal.getValue().isEmpty())
	{
		KBScriptError	*eRC	;
		bool		_fSubs	;
		KBValue		value2	= m_defVal.evaluate
					  (	KBValue(m_defVal.getValue()),
						eRC,
						_fSubs
					  )	;

		if (eRC != 0)
		{
			KBScriptError::processError (eRC, KBScriptError::Normal) ;
			return	false	;
		}

		/* Got a default, so pass this down to the control,	*/
		/* whence it can be retrieved and checked ...		*/
		if (!setValue (qrow, KBValue(value2.getRawText(), m_type)))
			return	false	;
	}

	if (!ctrl->isValid (allowNull))
	{	setError (ctrl->lastError ()) ;
		return	 false ;
	}

	return	true ;
}

/*  KBItem								*/
/*  setupProperties							*/
/*		: Set current properties				*/
/*  (returns)	: void		:					*/

void	KBItem::setupProperties ()
{
	KBObject::setupProperties () ;
	setupControls () ;
	if (getNavigator() != 0) getNavigator()->fixTabOrder () ;
}

/*  KBItem								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBItem::showAs
	(	KB::ShowAs	mode
	)
{
	m_allEnabled	= true	;
	m_allVisible	= true	;

	if (mode != KB::ShowAsData) m_isUpdateVal = -1 ;

	/* Note: If the expression is empty then set the type to string	*/
	/* so that anything is acceptable. This is needed since without	*/
	/* an expression, the item will not be set from the database	*/
	/* and hence we would not get a type.				*/
	if (m_type != 0) m_type->deref() ;

	m_type = m_expr.getValue().isEmpty() ? &_kbString : &_kbUnknown ;
	m_type->ref  ()	   ;

//	fprintf
//	(	stderr,
//		"KBItem::showAs: [%s] [0 ... %d]\n",
//		(cchar *)getName(),
//		nCtrls
//	)	;

	if (isHidden() == 0)
	{
		for (uint idx = 0 ; idx < m_nCtrls ; idx += 1)
		{
			m_ctrls[idx]->showAs	(mode) ;
			m_ctrls[idx]->setMorphed(isMorphing() && (mode == KB::ShowAsData)) ;
		}

		if (isMorphing() && (mode == KB::ShowAsData))
			getDisplay()->addMorph (this) ;
		else	getDisplay()->remMorph (this) ;
	}

	KBObject::showAs (mode) ;
}

/*  KBItem								*/
/*  hideBelow	: Hide controls from specified row downwards		*/
/*  qrow	: uint		: Specific row				*/
/*  (returns)	: void		:					*/

void	KBItem::hideBelow
	(	uint	qrow
	)
{
	bool	hide	= false ;

#if	! __KB_RUNTIME
	if (!showingData ())
	{
		for (uint idx = 0 ; idx < m_nCtrls ; idx += 1)
		{	m_ctrls[idx]->setVisible (true ) ;
			m_ctrls[idx]->setHidden  (false) ;
		}
		return	;
	}
#endif
	for (uint idx = 0 ; idx < m_nCtrls ; idx += 1)
	{
		if (!hide && (qrow - getBlock()->getCurDRow() == idx))
			hide	= true ;

		m_ctrls[idx]->setHidden (hide) ;
	}
}

/*  KBItem								*/
/*  clearBelow	: Clear controls from specified row downwards		*/
/*  qrow	: uint		: Specific row				*/
/*  (returns)	: void		:					*/

void	KBItem::clearBelow
	(	uint	qrow
	)
{
	bool	clear	= false ;

	for (uint idx = 0 ; idx < m_nCtrls ; idx += 1)
	{
		if (!clear && (qrow - getBlock()->getCurDRow() == idx))
			clear	= true ;

		m_ctrls[idx]->clearValue (false) ;
	}
}

/*  KBItem								*/
/*  ctrlAtQRow	: Get control at specified query row if any		*/
/*  qrow	: uint		: Specific row				*/
/*  (returns)	: KBControl *&	: Control or none			*/

KBControl *&KBItem::ctrlAtQRow
	(	uint	qrow
	)
{
	static	KBControl  *noCtrl = 0 ;

	uint	curDRow  = getBlock()->getCurDRow    () ;

	if (m_ctrls != 0)
		if ((qrow >= curDRow) && (qrow < curDRow + m_nCtrls))
			return	m_ctrls[qrow - curDRow] ;

	setError
	(	KBError::Error,
		TR("Row in query not currently displayed"),
		QString	(TR("%1 %2, query row %3: showing %4 for %5"))
			.arg(getElement())
			.arg(getName   ())
			.arg(qrow    )
			.arg(curDRow )
			.arg(m_nCtrls),
		__ERRLOCN
	)	;
	return	noCtrl	;
}

/*  KBItem								*/
/*  setMarked	: Set or clear marked state				*/
/*  qrow	: uint		: Specific row				*/
/*  enable	: bool		: Enable				*/
/*  (returns)	: void		:					*/

void	KBItem::setMarked
	(	uint	qrow,
		bool	marked
	)
{
	KBControl *ctrl = ctrlAtQRow (qrow) ;
	if (ctrl == 0) return ;

//	static	QPalette *markedPal ;
//	if (markedPal == 0)
//	{
//		const QColor fg = QColor (255, 255, 255) ;
//		const QColor bg = QColor (  0,   0,   0) ;
//	
//		markedPal = new QPalette (QApplication::palette()) ;
//		markedPal->setColor  (QColorGroup::Text,       fg) ;
//		markedPal->setColor  (QColorGroup::Foreground, fg) ;
//		markedPal->setColor  (QColorGroup::Base,       bg) ;
//		markedPal->setColor  (QColorGroup::Background, bg) ;
//	}

	QPalette *pal = marked ? getMarkedPalette() : getPalette(true) ;
	ctrl->setPalette (pal) ;
}

/*  KBItem								*/
/*  setVisible	: Set control visibility				*/
/*  qrow	: uint		: Specific row				*/
/*  visible	: bool		: Visibility				*/
/*  (returns)	: void		:					*/

void	KBItem::setVisible
	(	uint	qrow,
		bool	visible
	)
{
	KBControl *ctrl	= ctrlAtQRow (qrow) ;
	if (ctrl != 0) ctrl->setVisible (visible) ;
}

/*  KBItem								*/
/*  setEnabled	: Enable or disable a control				*/
/*  qrow	: uint		: Specific row				*/
/*  enable	: bool		: Enable				*/
/*  (returns)	: void		:					*/

void	KBItem::setEnabled
	(	uint	qrow,
		bool	enabled
	)
{
	KBControl *ctrl	= ctrlAtQRow (qrow) ;
	if (ctrl != 0) ctrl->setEnabled (enabled) ;
}

/*  KBItem								*/
/*  qrow	: uint		: Specific row				*/
/*  isEnabled	: Get control enabled/disabled				*/
/*  (returns)	: bool		: Enabled/disabled			*/

bool	KBItem::isEnabled
	(	uint	qrow
	)

{
	KBControl *ctrl	= ctrlAtQRow (qrow) ;
	return	ctrl == 0 ? false : ctrl->isEnabled () ;
}

/*  KBItem								*/
/*  isVisible	: Get control visibility				*/
/*  qrow	: uint		: Specific row				*/
/*  (returns)	: bool		: Visibility				*/

bool	KBItem::isVisible 
	(	uint	qrow
	)
{
	KBControl *ctrl	= ctrlAtQRow (qrow) ;
	return	ctrl == 0 ? false : ctrl->isVisible () ;
}

/*  KBItem								*/
/*  setAllVisible: Set control visibility of all controls		*/
/*  visible	 : bool		: Visibility				*/
/*  (returns)	 : void		:					*/

void	KBItem::setAllVisible
	(	bool	visible
	)
{
	m_allVisible = visible ;
	for (uint idx = 0 ; idx < m_nCtrls ; idx += 1)
		if (m_ctrls[idx] != 0)
			m_ctrls[idx]->setVisible (m_allVisible) ;
}

/*  KBItem								*/
/*  setAllEnabled: Enable or disable all controls			*/
/*  enabled	 : bool		: Enabled				*/
/*  (returns)	 : void		:					*/

void	KBItem::setAllEnabled
	(	bool	enabled
	)
{
	m_allEnabled = enabled ;
	for (uint idx = 0 ; idx < m_nCtrls ; idx += 1)
		if (m_ctrls[idx] != 0)
			m_ctrls[idx]->setEnabled (m_allEnabled) ;
}

/*  KBItem								*/
/*  reMorphItem	: Convert control back to morph on losing focus		*/
/*  qrow	: uint		: Query row number			*/
/*  (returns)	: void		:					*/

void	KBItem::reMorphItem
	(	uint	drow
	)
{
	/* If this item is morphing, and the control at this row is not	*/
	/* a morph control, then convert it.				*/
	if (isMorphing() && (drow < m_nCtrls) && !m_ctrls[drow]->isMorphed())
		m_ctrls[drow]->setMorphed (true) ;
}

/*  KBItem								*/
/*  giveFocus	: Give focus to specified row				*/
/*  qrow	: uint		: Query row number			*/
/*  (returns)	: void		:					*/

void	KBItem::giveFocus
	(	uint	qrow
	)
{
#if	__KB_DEBUG_FOCUS
	fprintf
	(	stderr,
		"KBItem::giveFocus: %s: qrow=%d\n",
		(cchar *)getName(),
		qrow
	)	;
#endif
	KBControl *&ctrl = ctrlAtQRow (qrow) ;
	if (ctrl == 0) return ;

	getLayout()->setUnMorphedItem (0, 0) ;

	/* If this is a morphing item and the control is currently	*/
	/* morphed, then convert it to the real control.		*/
	if (isMorphing() && ctrl->isMorphed())
	{	ctrl->setMorphed (false) ;
		getLayout()->setUnMorphedItem (this, ctrl->getDRow())  ;
	}

	ctrl->giveFocus() ;
}

/*  KBItem								*/
/*  setTabOrder	: Return tab order value				*/
/*  order	: int		: Tab order				*/
/*  (returns)	: void		:					*/

void	KBItem::setTabOrder
	(	int	order
	)
{
	m_tabOrd.setValue (order) ;
}

/*  KBItem								*/
/*  getTabOrder	: Return tab order value				*/
/*  (returns)	: int		: Tab order				*/

int	KBItem::getTabOrder ()
{
	return	m_tabOrd.getIntValue () ;
}

/*  KBItem								*/
/*  ctrlGeometry: Get control geometry					*/
/*  qrow	: uint		: Query row number			*/
/*  rCtrl	: QRect &	: Result				*/
/*  (returns)	: bool		: Success				*/

bool	KBItem::ctrlGeometry
	(	uint	qrow,
		QRect	&rCtrl
	)
{
	rCtrl	= m_ctrls[qrow - getBlock()->getCurDRow()]->geometry() ;
	return	true ;
}


/*  KBItem								*/
/*  keyStroke	: Control has received keystroke			*/
/*  k		: QKeyEvent *	: Key event in question			*/
/*  (returns)	: bool		: Key event consumed			*/

bool	KBItem::keyStroke
	(	QKeyEvent *k
	)
{
	KBNavigator *navi = getNavigator() ;
	return	navi == 0 ? false : navi->keyStroke (this, k) ;
}

/*  KBItem								*/
/*  isReadOnly	: See if item is read only				*/
/*  (returns)	: bool		: True for read only			*/

bool	KBItem::isReadOnly ()
{
	/* Check the embedding form block. Fields are never read-only	*/
	/* if we are in a query; fields are always read-only if the	*/
	/* block is marked read-only.					*/
	KBFormBlock *fb	  = getBlock()->isFormBlock()  ;
	if (fb != 0)
	{	if (fb->isInQuery    ()) return false ;
		if (fb->isBlkReadOnly()) return true  ;
	}

	if (m_rdonly.getBoolValue ()) return true ;
	if ((m_flags & KBFieldSpec::ReadOnly) != 0) return true ;

	return	false ;
}

/*  KBItem								*/
/*  getReportValue							*/
/*		: Get value for report writing				*/
/*  first	: bool		: First output flag			*/
/*  prior	: bool		: Return prior value flag		*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBItem::getReportValue
	(	bool		,
		bool
	)
{
	return	m_curVal ;
}

/*  KBItem								*/
/*  write	: Write control values					*/
/*  writer	: KBWriter *	: Writer				*/
/*  offset	: QPoint	: Writer offset				*/
/*  first	: bool		: First on page flag			*/
/*  extra	: init &	: Return extra space			*/
/*  prior	: bool		: Write prior value flag		*/
/*  (returns)	: bool		: Success				*/

bool	KBItem::write
	(	KBWriter	*writer,
		QPoint		offset,
		bool		first,
		int		&extra,
		bool		prior
	)
{
	if (!writer->asReport())
	{
		/* This method will be called in design mode and when	*/
		/* printing forms. The work is all done at the control	*/
		/* level, but we iterate through each row of controls.	*/
		QRect	posn	= geometry(offset) ;
		int	dx	= getBlock()->getAttrVal("dx").toInt() ;
		int	dy	= getBlock()->getAttrVal("dy").toInt() ;

		for (uint idx = 0 ; idx < m_nCtrls ; idx += 1)
		{
			if (showingDesign() || m_ctrls[idx]->isVisible ())
				m_ctrls[idx]->write (writer, posn, KBValue(), false, extra) ;
			posn.moveBy (dx, dy) ;
		}

		return	true	;
	}

	fprintf
	(	stderr,
		"KBItem::write: %s: %s: %p/%p\n",
		(cchar *)getElement(),
		(cchar *)getName   (),
		(void  *)m_ctrls,
		(void  *)(m_ctrls == 0 ? 0 : m_ctrls[0])
	)	;

	if (m_ctrls == 0)
		setupControls () ;

	return	m_ctrls[0]->write
		(	writer,
			geometry (offset),
			getReportValue (first, prior),
			m_fSubs,
			extra
		)	;
}

/*  KBItem								*/
/*  setMonitor	: Set monitor values for this node			*/
/*  myitem	: KBNodeMonitor * : Monitor item for this node		*/
/*  (returns)	: void		  :					*/

void	KBItem::setMonitor
	(	KBNodeMonitor	*myitem
	)
{
	KBNode::setMonitor (myitem) ;

	if (myitem != 0)
	{
		m_ctrlMon = new KBNodeMonitor (0, myitem) ;
		m_ctrlMon->setText (0, "Controls") ;
		m_ctrlMon->setSelectable (false) ;
	}
	else	m_ctrlMon	= 0 ;

	for (uint idx = 0 ; idx < m_nCtrls ; idx += 1)
		if (m_ctrls[idx] != 0)
			m_ctrls[idx]->showMonitor (m_ctrlMon) ;
}

/*  KBItem								*/
/*  property	: Get QT widget property				*/
/*  qrow	: uint		 : Query row				*/
/*  name	: cchar *	: Property name				*/
/*  (returns)	: QVariant	: Property value			*/

QVariant
	KBItem::property
	(	uint		qrow,
		cchar		*name
	)
{
	KBControl *ctrl = ctrlAtQRow  (qrow)   ;
	return	ctrl == 0 ? QVariant() : ctrl->property (name) ;
}

/*  KBItem								*/
/*  setProperty	: Set QT widget property				*/
/*  qrow	: uint		 : Query row				*/
/*  name	: cchar *	   : Property name			*/
/*  value	: const QVariant & : Value				*/
/*  (returns)	: bool		   : Success				*/

bool	KBItem::setProperty
	(	uint		qrow,
		cchar		*name,
		const QVariant	&value
	)
{
	KBControl *ctrl = ctrlAtQRow  (qrow)   ;
	return	ctrl == 0 ? false : ctrl->setProperty (name, value) ;
}

/*  KBItem								*/
/*  repaintMorph: Repaint morphed control				*/
/*  p		: QPainter *	: Painter				*/
/*  drow	: uint		: Display row				*/
/*  (returns)	: void							*/

void	KBItem::repaintMorph
	(	QPainter	*p,
		uint		drow
	)
{
	if (drow < m_nCtrls) m_ctrls[drow]->repaintMorph (p) ;
}

/*  KBItem								*/
/*  repaintMorph: Repaint morphed controls				*/
/*  p		: QPainter *	: Painter				*/
/*  area	: const QRect &	: Area to be repainted			*/
/*  (returns)	: void							*/

void	KBItem::repaintMorph
	(	QPainter	*p,
		const QRect	&area
	)
{
//	fprintf	(stderr, "repaintMorph [%d,%d,%d,%d]\n",
//			 area.x	    (),
//			 area.y     (),
//			 area.width (),
//			 area.height()
//		)	;

	for (uint drow = 0 ; drow < m_nCtrls ; drow += 1)
	{
		KBControl	*ctrl	= m_ctrls[drow]	   ;
		QRect		cRect	= ctrl->geometry() ;

		if (cRect.intersects(area)) ctrl->repaintMorph (p) ;
			ctrl->repaintMorph (p) ;
	}
}

bool	KBItem::mouseClickHit
	(	const QPoint	&pos
	)
{
	KBBlock	*block	= getBlock () ;
	uint	curDRow	= block->getCurDRow () ;
	uint	numRows	= block->getNumRows () ;

	for (uint drow = 0 ; drow < m_nCtrls ; drow += 1)
	{
		KBControl *ctrl = m_ctrls[drow] ;

		if (drow + curDRow > numRows + 1)
			break	 ;

		if (!ctrl->isVisible ()) continue ;
		if (!ctrl->isEnabled ()) continue ;

		if (ctrl->geometry().contains(pos))
		{
			if (moveFocusOK (drow))
			{	giveFocus (curDRow + drow) ;
				if (showingData())
					focusInEvent (drow, QFocusEvent::Mouse) ;
			}

			return	  true	;
		}
	}

	return	false	;
}

/*  KBItem								*/
/*  isMorphing	: Test if item is morphed				*/
/*  (returns)	: bool		: Morphed				*/

bool	KBItem::isMorphing ()
{
	/* This is always false. Specific data controls like fields and	*/
	/* links may override this.					*/
	return	false	;
}

/*  KBItem								*/
/*  getIniValue	: Get initial value					*/
/*  qrow	: uint		: Row number				*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBItem::getIniValue
	(	uint	qrow
	)
{
	KBQryBase	*query	= getBlock()->getQuery() ;

	return	query->getField (getQueryLvl(), qrow, getQueryIdx(), true) ;
}

/*  KBItem								*/
/*  userChange	: User has changed value				*/
/*  qrow	: uint		: Query row number			*/
/*  (returns)	: void		:					*/

void	KBItem::userChange
	(	uint	qrow
	)
{
	KBFormBlock *fBlk = getFormBlock() ;

//	fprintf
//	(	stderr,
//		"KBItem::userChange: !iuv=%d fb=%p iiq=%d\n",
//		!isUpdateVal(),
//		(void *)fBlk,
//		fBlk == 0 ? false : fBlk->isInQuery()
//	)	;

	/* Check if this item is updatable (should not get called	*/
	/* unless it is, but ...), and that it is in a form (ditto ...)	*/
	/* and ignore otherwise; also, more interestingly, ignore if we	*/
	/* are in a query.						*/
	if (!isUpdateVal() || (fBlk == 0) || fBlk->isInQuery())
		return	;

	/* Also ignore changes if the associated query is the null	*/
	/* (menu-only) query.						*/
	if (fBlk->getQuery()->isQryNull() != 0)
		return	;

	/* Notify the layout object so that the toolbar gets updated,	*/
	/* and the form block so that the block-level change event can	*/
	/* be triggered.						*/
	fprintf
	(	stderr,
		"KBItem::userChange: notifying\n"
	)	;
	getLayout()->setChanged  (true, errorText()) ;
	fBlk       ->dataChanged (qrow) ;
}

/*  KBItem								*/
/*  isInQuery	: See if user has started a query			*/
/*  (returns)	: bool		: In query				*/

bool	KBItem::isInQuery ()
{
	KBFormBlock *fBlk = getBlock()->isFormBlock() ;
	return	fBlk == 0 ? false : fBlk->isInQuery() ;
}

/*  KBItem								*/
/*  startUpdate	: Start locked update					*/
/*  qrow	: uint		: Row to be updated			*/
/*  (returns)	: bool		: Locked OK				*/

bool	KBItem::startUpdate
	(	uint		qrow
	)
{
	/* Paranoia check that we are in data view, in case of events	*/
	/* triggered by design changes in design view. Code in the	*/
	/* KBControl class should trap these, but better safe than	*/
	/* sorry. Also check in case we are in a query.			*/
	if (showingData() && !isInQuery())
	{
		KBFormBlock *fBlk = getFormBlock() ;
		if ((fBlk != 0) && !fBlk->startUpdate (qrow))
		{
			fBlk->lastError().DISPLAY() ;
			return	false ;
		}
	}

	return	true	;
}

/*  KBItem								*/
/*  doSearch	: Do record search					*/
/*  (returns)	: void		:					*/

void	KBItem::doSearch ()
{
	/* Default is to do nothing. Controls that can actually do a	*/
	/* record search override this.					*/
}

/*  KBItem								*/
/*  getSearchText: Get sarch text for record search			*/
/*  qrow	 : uint		: Query row				*/
/*  (returns)	 : QString	: Text					*/

QString	KBItem::getSearchText
	(	uint	qrow
	)
{
	return	getBlock()->getRowValue(this, qrow).getRawText() ;
}

/*  KBItem								*/
/*  errorText	: Get error message text				*/
/*  (returns)	: QString	: Text					*/

QString	KBItem::errorText ()
{
	if (!m_errtext.getValue().isEmpty())
		return m_errtext.getValue() ;

	QStringList bits ;
	if (!getName().isEmpty()) bits.append (getName()) ;
	if (!getExpr().isEmpty()) bits.append (getExpr()) ;
	if (bits.count() == 1) return bits[0] ;
	if (bits.count() == 2) return bits.join (": ") ;

	return	TR("unidentified control") ;
}

/*  KBItem								*/
/*  enumKBProperty							*/
/*		: Get list of properties				*/
/*  list	: QStringList &	: Property list				*/
/*  (returns)	: void		:					*/

void	KBItem::enumKBProperty
	(	QStringList	&list
	)
{
	/* Note: Don't include "visible" and "enabled" as these are	*/
	/*	 present at the KBObject level as well.			*/
	list.append ("value") ;
	KBObject::enumKBProperty (list) ;
}

/*  KBItem								*/
/*  getKBProperty: Get a rekall property value				*/
/*  value	 : KBValue &	: Return property value			*/
/*  (returns)	 : bool		: Property exists			*/

bool	KBItem::getKBProperty
	(	cchar		*name,
		KBValue		&value
	)
{
//	fprintf
//	(	stderr,
//		"KBItem::getKBProperty: for [%s]\n",
//		name
//	)	;

	if (qstrcmp (name, "value") == 0)
	{
		if (getBlock() != 0)
			value	= getValue (getBlock()->getCurQRow()) ;
		else	value	= KBValue  () ;

		return	true ;
	}

	if (qstrcmp (name, "visible") == 0)
	{
		if (getBlock() != 0)
			value	= isVisible(getBlock()->getCurQRow()) ;
		else	value	= KBValue  () ;

		return	true ;
	}

	if (qstrcmp (name, "enabled") == 0)
	{
		if (getBlock() != 0)
			value	= isEnabled(getBlock()->getCurQRow()) ;
		else	value	= KBValue  () ;

		return	true ;
	}

	return	KBObject::getKBProperty (name, value) ;
}

/*  KBItem								*/
/*  setKBProperty: Set a rekall property value				*/
/*  name	 : cchar *	: Proprty name				*/
/*  value	 : KBValue	: Property value			*/
/*  (returns)	 : bool		: Success				*/

bool	KBItem::setKBProperty
	(	cchar		*name,
		const KBValue	&value
	)
{
	if (qstrcmp (name, "value") == 0)
	{
		if (getBlock() != 0)
			setValue   (getBlock()->getCurQRow(), value) ;
		return	true ;

	}

	if (qstrcmp (name, "visible") == 0)
	{
		if (getBlock() != 0)
			setVisible (getBlock()->getCurQRow(), value.isTrue()) ;
		return	true ;

	}

	if (qstrcmp (name, "enabled") == 0)
	{
		if (getBlock() != 0)
			setEnabled (getBlock()->getCurQRow(), value.isTrue()) ;
		return	true ;

	}

	return	KBObject::setKBProperty (name, value) ;
}

#if	! __KB_RUNTIME

/*  KBItem								*/
/*  whatsThisExtra							*/
/*		: Extra stuff for pseudo-what's this popup		*/
/*  info	: QValueList<KBWhatsThisPair> &	: List of items		*/
/*  (returns)	: void				:			*/

void	KBItem::whatsThisExtra
	(	QValueList<KBWhatsThisPair>	&info
	)
{
	QString	expr	= getExpr() ;
	if (expr.isEmpty()) expr = TR("<i>None</i>") ;
	info.append (KBWhatsThisPair(TR("Expression"), expr)) ;
}

#endif
