/***************************************************************************
    file	         : kb_listbox.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_formblock.h"

#if		__KB_AUTOMOC
#include	"kb_listbox.moc"
#else
#include	"kb_listbox.h"
#endif

#include	"kb_qtlistbox.h"
#include	"kb_nodereg.h"
#include	"kb_layout.h"
#include	"kb_finddlg.h"


#if		! __KB_RUNTIME
#include	"kb_propdlg.h"
#endif

#include 	<qstringlist.h>


/*  KBListBox								*/
/*  KBListBox	: Constructor for simple listbox field			*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  ok		: bool *	: Success				*/
/*  (returns)	: KBNode	:					*/

KBListBox::KBListBox
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*ok
	)
	:
	KBItem		(parent, "KBListBox",	"master", 	aList),
	m_values	(this,   "values",			aList),
	m_nullval	(this,	 "nullval",			aList),
	m_nullOK	(this,	 "nullok",			aList),
	m_fgcolor	(this,   "fgcolor",			aList),
	m_bgcolor	(this,   "bgcolor",			aList),
	m_font		(this,	 "font",			aList),
	m_onChange	(this,	 "onchange",	"onListBox",	aList)
{
#if	! __KB_RUNTIME
	if (ok != 0)
	{
		if (!::listBoxPropDlg (this, "Listbox", m_attribs))
		{	delete	this	;
			*ok	= false	;
			return	;
		}

		*ok	= true		;
	}
#endif
}

/*  KBListBox								*/
/*  KBListBox	: Constructor for simple listbox field			*/
/*  parent	: KBNode *	: Parent node				*/
/*  listbox	: KBListBox *	: Extant listbox				*/
/*  (returns)	: KBNode	:					*/

KBListBox::KBListBox
	(	KBNode		*parent,
		KBListBox	*listbox
	)
	:
	KBItem		(parent,  "master", 			listbox),
	m_values	(this,    "values",			listbox),
	m_nullval	(this,	  "nullval",			listbox),
	m_nullOK	(this,	  "nullok",			listbox),
	m_fgcolor	(this,    "fgcolor",			listbox),
	m_bgcolor	(this,    "bgcolor",			listbox),
	m_font		(this,	  "font",			listbox),
	m_onChange	(this,	  "onchange",	"onListBox",	listbox)
{
}

/*  KBListBox								*/
/*  ~KBListBox	: Destructor for list box control			*/
/*  (returns)	:		:					*/

KBListBox::~KBListBox ()
{
}

/*  KBListBox								*/
/*  setData	: Set a new set of values				*/
/*  qrow	: uint		: Query row				*/
/*  vals	: void *	: Set of values				*/
/*  (returns)	: void		:					*/

void	KBListBox::setData
	(	uint	,
		void	*vals
	)
{
	m_values.setValue (QString::fromUtf8((cchar *)vals)) ;
	loadValues	  () ;

	/* Pass the data to the controls. Actually, these will use the	*/
	/* KBListBox::getValues() method to get the list of values.	*/
	for (uint idx = 0 ; idx < m_nCtrls ; idx += 1)
		m_ctrls[idx]->setData (0) ;
}

/*  KBListBox								*/
/*  setValues	: Set a new set of values				*/
/*  vals	: QStringList &	: Set of values				*/
/*  (returns)	: void		:					*/

void	KBListBox::setValues
	(	const QStringList	&vals
	)
{
	m_valset	= vals ;
	m_valset.insert (m_valset.begin(), m_nullval.getValue()) ;

	for (uint idx2 = 0 ; idx2 < m_nCtrls ; idx2 += 1)
		m_ctrls[idx2]->setData ((void *)&m_valset) ;
}

/*  KBListBox								*/
/*  loadValues	: Load value set into combo boxes			*/
/*  (returns)	: void		:					*/

void	KBListBox::loadValues ()
{
	m_valset  = QStringList::split
		  (	'|',
			m_nullval.getValue() + "|" + m_values.getValue(),
			true
		  )	;
}

/*  KBListBox								*/
/*  userChange	: Selection changed notification			*/
/*  qrow	: uint		  : Query row number			*/
/*  text	: const QString & : New text				*/
/*  (returns)	: void		  :					*/

void	KBListBox::userChange
	(	uint		qrow,
		const QString	&text
	)
{
	KBValue	args[2]	;
	bool	evRc	;

	args[0] = (int)qrow	 ;
	args[1] = KBValue (text) ;
	eventHook (m_onChange, 2, args, evRc) ;

	KBItem::userChange (qrow) ;
}

/*  KBListBox								*/
/*  keyStroke	: Control has received keystroke			*/
/*  k		: QKeyEvent *	: Key event in question			*/
/*  (returns)	: bool		: Key event consumed			*/

bool	KBListBox::keyStroke
	(	QKeyEvent	*k
	)
{
	/* Override for key up and key down so that we can change	*/
	/* selection, otherwise these keys will navigate between	*/
	/* records.							*/
	if (k->type() == QEvent::KeyPress)
		if ((k->key() == Qt::Key_Up) || (k->key() == Qt::Key_Down))
			return	false	;

	return	KBItem::keyStroke (k) ;
}

/*  KBListBox								*/
/*  showAs	: Switch in and out of design mode			*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBListBox::showAs
	(	KB::ShowAs	mode
	)
{
	if (mode == KB::ShowAsData) loadValues  () ;
	KBItem::showAs (mode) ;
}

/*  KBListBox								*/
/*  makeCtrl	: Make a listbox control					*/
/*  drow	: uint		: Display row number			*/
/*  (returns)	: KBControl *	: Associated control			*/

KBControl *KBListBox::makeCtrl
	  (	uint		drow
	  )
{
	return	new KBCtrlListBox (getDisplay(), this, drow) ;
}

/*  KBListBox								*/
/*  checkValid	: Check whether value can be saved in database		*/
/*  value	: const QString & : Text value				*/
/*  alowNull	: bool		  : Ignore not-null check		*/
/*  (returns)	: bool		  : True if OK				*/

bool	KBListBox::checkValid
	(	const QString	&value,
		bool		allowNull
	)
{
	KBError		err	;

	fprintf
	(	stderr,
		"KBListBox::checkValid: [%s]: an=%d no=%d ve=%d\n",
		value.latin1 (),
		allowNull,
		m_nullOK.getBoolValue(),
		value.isEmpty()
	)	;

	if (!allowNull)
		if (!m_nullOK.getBoolValue() && value.isEmpty())
		{
			setError
			(	KBError::Error,
				QString(TR("Value must be selected from list for %1")).arg(errorText()),
				QString::null,
				__ERRLOCN
			)	;
			return	false ;
		}

	if (!m_type->isValid (value, err, getName()))
	{
		setError (err) ;
		return	 false ;
	}

	return	true	;
}


void	KBListBox::setCurrentItem
	(	uint		qrow,
		uint		item
	)
{
	KBControl *ctrl = ctrlAtQRow (qrow) ;
	if (ctrl != 0) ((KBCtrlListBox *)ctrl)->setCurrentItem (item) ;
}

uint	KBListBox::currentItem
	(	uint		qrow
	)
{
	KBControl *ctrl = ctrlAtQRow (qrow) ;
	return	ctrl == 0 ? -1 : ((KBCtrlListBox *)ctrl)->currentItem () ;
}

/*  KBListBox								*/
/*  replicate	: Replicate this listbox					*/
/*  parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New listbox node			*/

KBNode	*KBListBox::replicate
	(	KBNode	*parent
	)
{
	return	new KBListBox (parent, this) ;
}

/*  KBListBox								*/
/*  doSearch	: Do record search					*/
/*  (returns)	: void		:					*/

void	KBListBox::doSearch ()
{
//	KBFindListBoxDlg findDlg (getFormBlock(), this, m_valset, m_valset) ;
//	findDlg.exec () ;
}

#if	! __KB_RUNTIME
/*  KBListBox								*/
/*  propertyDlg	: Show property dialog					*/
/*  iniAttr	: cchar *	: Initial attribute			*/
/*  (returns)	: bool		: Success				*/

bool	KBListBox::propertyDlg
	(	cchar	*iniAttr
	)
{
	if (::listBoxPropDlg (this, "Listbox", m_attribs, iniAttr))
	{
		setupProperties () ;
		return	true ;
	}

	return	false	;
}
#endif


NEWNODE(ListBox, __TR("New &ListBox"), 0, KNF_FORM|KNF_BLOCK|KNF_DATA)
