/***************************************************************************
    file	         : kb_noobjdlg.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_NOOBJDLG_H
#define	__KB_NOOBJDLG_H

#if	! __KB_RUNTIME

#include	"libkbase_exports.h"

#include	<qptrlist.h>
#include	<qlabel.h>


#include	"kb_classes.h"
#include	"kb_attr.h"
#include	"kb_dialog.h"

class		RKComboBox	;
class		RKListView	;
class		RKPushButton	;
class		RKHBox		;


class	LIBKBASE_API	KBObjTreeObject : public QListViewItem
{
	KBObject	*m_object	;
	bool		m_expanded	;
	bool		m_start		;

public	:

	KBObjTreeObject (RKListView 	*, KBObject *)	;
	KBObjTreeObject (QListViewItem	*, KBObject *)	;

	virtual	void	paintCell	(QPainter *, const QColorGroup &, int, int, int) ;
	virtual	void	setOpen 	(bool)	;

	inline	KBObject *object 	() { return m_object   ; }
	inline	bool	expanded 	() { return m_expanded ; }

	inline	void	setExpanded	() { m_expanded = true ; }
	inline	void	setStart 	() { m_start	= true ; }

	static	KBObjTreeObject	*expandToObject	(RKListView *, KBObject *, KBObject * = 0) ;
}	;


/*  KBNoObjDlg								*/
/*  ----------								*/
/*  This dialog is used when KBObject::getNamedObject fails to locate	*/
/*  an object and the show-error option is set. It displays a tree	*/
/*  view showing objects and their names, and has the option to allow	*/
/*  the user to pick an object and continue with it. Useful for		*/
/*  debugging.								*/

class	LIBKBASE_API	KBNoObjDlg : public KBDialog
{
	Q_OBJECT

protected	:

	RKListView	*m_objTree	;
	KBObject	*m_starting	;
	KBObject	*m_selected	;
	bool		*m_abort	;

	RKHBox		*m_layButt	;
	RKPushButton	*m_bFail	;
	RKPushButton	*m_bAbort	;
	RKPushButton	*m_bContinue	;

protected slots :

	virtual	void	slotObjClicked	()	;
	void		slotObjSelected	()	;
	void		clickAbort	()	;
	void		clickFail	()	;

public	:

	KBNoObjDlg
	(	KBObject	*,
		QString		,
		bool		*,
		const QString	& = QString::null,
		cchar		* = "noobjdlg"
	)	;

	inline	KBObject *selectedObj	()
	{
		return	 m_selected	;
	}
}	;

/*  KBRouteToObjDlg							*/
/*  ---------------							*/
/*  Variant of KBNoObjDlg used to allow quick location (and automatic	*/
/*  entry of the path) of objects with event script editing.		*/

class	LIBKBASE_API	KBRouteToObjDlg	: public KBNoObjDlg
{
	Q_OBJECT

	RKPushButton	*m_bAsPath	;

protected slots	:

	virtual	void	slotObjClicked	() ;
	void		slotObjAsPath	() ;

public	:

	KBRouteToObjDlg	(KBObject *, QString, const QString & = QString::null) ;

	QString	routeToObject		(bool)	;
}	;

/*  KBSlotFindDlg							*/
/*  -------------							*/
/*  This dialog is used to aid location of objects and their events	*/
/*  in the slot dialogs.						*/

class	LIBKBASE_API	KBSlotFindDlg : public KBDialog
{
	Q_OBJECT

	RKListView		*m_objTree	;
	RKComboBox		*m_events	;
	RKPushButton		*m_bOK		;
	RKPushButton		*m_bCancel	;

	QPtrList<KBAttr>		m_eventList	;
	KBObject		*m_selected	;



protected slots :

	void		slotObjClicked	(QListViewItem *) ;

public	:

	KBSlotFindDlg	(KBObject *, KBObject *, const QString &) ;

	KBObject	*selectedObj	(QString &) ;
}	;

/*  KBConfigFindDlg							*/
/*  ---------------							*/
/*  This dialog is used to aid location of objects and their child	*/
/*  configuration objects in the component override dialog.		*/

class	LIBKBASE_API	KBConfigFindDlg : public KBDialog
{
	Q_OBJECT

	RKListView		*m_objTree	;
	RKComboBox		*m_configs	;
	KBObject		*m_start	;
	RKPushButton		*m_bOK		;
	KBObject		*m_selected	;



protected slots :

	void		slotObjClicked	(QListViewItem *) ;

public	:

	KBConfigFindDlg	(KBObject *, KBObject *, const QString &) ;

	KBObject	*selectedConfig	(QString &, QString &) ;
}	;

#endif
#endif	// __KB_NOOBJDLG_H
