/***************************************************************************
    file	         : kb_popupprompt.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qlabel.h>
#include	<qfontmetrics.h>

#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_lineedit.h"
#include	"rk_pushbutton.h"
#include	"rk_modalfilter.h"
#include	"tk_icons.h"

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_object.h"
#include	"kb_slot.h"

#if		__KB_AUTOMOC
#include	"kb_popupprompt.moc"
#else
#include	"kb_popupprompt.h"
#endif


/*  KBPopupPrompt							*/
/*  KBPopupPrompt							*/
/*		: Constructor for popup prompt				*/
/*  caption	: const QString & : Dialog caption			*/
/*  message	: const QString & : Dialog prompt message		*/
/*  value	: const QString & : Initial value			*/
/*  target	: KBObject *	  : Object to notify on success		*/
/*  slotName	: const QString & : Target slot to invoke		*/
/*  (returns)	: KBPopupPrompt  :					*/

KBPopupPrompt::KBPopupPrompt
	(	const QString		&caption,
		const QString		&message,
		const QString		&value,
		KBObject		*target,
		const QString		&slotName
	)
	:
	QWidget
	(	0,
		"kbpoppuprompt",
		WStyle_Customize|WStyle_Title|WStyle_NormalBorder|WStyle_StaysOnTop|Qt::WDestructiveClose
//		WStyle_Customize|WStyle_Title|WStyle_NormalBorder|WStyle_Tool|WStyle_StaysOnTop|Qt::WDestructiveClose
	),
	m_target(target)
{
	RKVBox	*layMain	= new RKVBox (this) ;
	layMain->setTracking () ;

	m_slot	 = 0	;
	const QPtrList<KBSlot> &slist = target->getSlots() ;
	LITER
	(	KBSlot,
		slist,
		slot,
		if (slot->name() == slotName)
		{	m_slot	 = slot ;
			break	 ;
		}
	)

	new QLabel  (message, layMain) ;

	m_tValue = new RKLineEdit (layMain) ;
	m_tValue->setText (value) ;

	RKHBox	*buttons = new RKHBox (layMain) ;
	buttons->addFiller () ;
	RKPushButton *o  = new RKPushButton (TR("OK"),     buttons, "ok"    ) ;
	RKPushButton *c  = new RKPushButton (TR("Cancel"), buttons, "cancel") ;

	connect (o, SIGNAL(clicked()), SLOT(accept())) ;
	connect (c, SIGNAL(clicked()), SLOT(reject())) ;

	KBDialog::sameSize	(o, c, 0) ;
	KBDialog::setupLayout	(this) ;

	m_tValue->setFocus	() ;
	m_tValue->setSelection	(0, value.length()) ;

	setMinimumSize (285, 110) ;

	if (m_slot   == 0)
		KBError::EError
		(	QString(TR("Slot %1 not found")).arg(slotName),
			QString::null,
			__ERRLOCN
		)	;

	RKModalFilter::self()->push (this) ;
	m_modal	= true	;

	connect
	(	m_target,
		SIGNAL	(destroyed()),
		SLOT	(reject	  ())
	)	;

	setIcon		(getSmallIcon ("rekall")) ;
	setCaption	(caption) ;

	int	wc	= QFontMetrics(font()).width(caption) ;
	int	wm	= QFontMetrics(font()).width(message) ;

	if ((wc > 200) || (wm > 200))
		m_tValue->setMinimumWidth (QMAX(wc,wm) + 100) ;
}

/*  KBPopupPrompt							*/
/*  ~KBPopupPrompt: Destructor for popup prompt				*/
/*  (returns)	   : void		:				*/

KBPopupPrompt::~KBPopupPrompt ()
{
	if (m_modal)
		RKModalFilter::self()->pop () ;
}

/*  KBPopupPrompt							*/
/*  ok		: Check helper is good to go				*/
/*  (returns)	: bool		: True if OK				*/

bool	KBPopupPrompt::ok ()
{
	return	m_slot != 0 ;
}

/*  KBPopupPrompt							*/
/*  accept	: Handle OK button click				*/
/*  (returns)	: void		:					*/

void	KBPopupPrompt::accept ()
{
	RKModalFilter::self()->pop () ;
	m_modal	= false	;
	hide () ;

	if (m_slot != 0)
	{
		KBScriptError	*rc	= 0 ;
		KBValue		resval	;
		KBValue		arg	(m_tValue->text()) ;

		m_slot->eventSignal (m_target, "prompt", 1, &arg, resval, rc) ;

		if (rc != 0) 
			KBScriptError::processError (rc, KBScriptError::Normal) ;
	}

	deleteLater () ;
}

/*  KBPopupPrompt							*/
/*  reject	: Handle cancel button click				*/
/*  (returns)	: void		:					*/

void	KBPopupPrompt::reject ()
{
	RKModalFilter::self()->pop () ;
	m_modal	= false	;
	hide () ;

	deleteLater  () ;
}
