/***************************************************************************
    file	         : kb_qrysqldlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	"kb_classes.h"
#include	"kb_propdlg.h"
#include	"kb_docroot.h"

#include	"kb_select.h"
#include	"kb_qrysql.h"

#if		__KB_AUTOMOC
#include 	"kb_qrysqldlg.moc"
#else
#include 	"kb_qrysqldlg.h"
#endif

#include	"tk_messagebox.h"
#include	"rk_combobox.h"
#include	"rk_pushbutton.h"


#include	"kb_textedit.h"
#include	"kb_manual.h"


/*  KBQrySQLPropDlg							*/
/*  KBQrySQLPropDlg							*/
/*		: Constructor for query-query properties dialog		*/
/*  link	: KBLink *	 : Parent link				*/ 
/*  caption	: const char *   : Dialog box caption			*/
/*  attribs	: QPtrList<KBAttr>& : List of attributes			*/
/*  (returns)	: KBQrySQLPropDlg :					*/

KBQrySQLPropDlg::KBQrySQLPropDlg
	(	KBQrySQL	*query,
		cchar		*caption,
		QPtrList<KBAttr>	&attribs
	)
	:
	KBPropDlg	(query, caption, attribs),
	m_query		(query)
{
	m_bHelp->setEnabled (true) ;
}


/*  KBQrySQLPropDlg							*/
/*  ~KBQrySQLPropDlg							*/
/*		: Destructor for block properties dialog		*/
/*  (returns)	: void		:					*/

KBQrySQLPropDlg::~KBQrySQLPropDlg ()
{
}

/*  KBQrySQLPropDlg							*/
/*  showProperty: Show property						*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Property on show			*/

bool	KBQrySQLPropDlg::showProperty
	(	KBAttrItem	*item
	)
{
	const QString	&aName = item->attr()->getName () ;

	/* Query							*/
	/* Use the tkEditor and enable the verify button so that the	*/
	/* user can check their SQL.					*/
	if (aName == "query")
	{
		m_textEdit->setHighlight("Sql") ;
		m_textEdit->setText	(item->value()) ;
		m_textEdit->show   	() ;

		m_bVerify->setEnabled (true) ;
		return	true	;
	}

	if (aName == "toptable")
	{
		QString	  sql	  = getProperty ("query") ;
		KBSelect  select  ;

		m_comboBox->clear      ()   ;
		m_comboBox->insertItem ("") ;

		if (select.parseQuery (sql, 0))
			m_comboBox->insertStringList (select.tableList()) ;

		for (int idx = 0 ; idx < m_comboBox->count() ; idx += 1)
			if (m_comboBox->text(idx) == item->value())
			{	m_comboBox->setCurrentItem (idx) ;
				break	;
			}

		m_comboBox->show() ;
		return	true ;
	}

	return	KBPropDlg::showProperty (item) ;
}

/*  KBPropDlg								*/
/*  saveProperty: Check and save attribute				*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Attribute OK				*/

bool	KBQrySQLPropDlg::saveProperty
	(	KBAttrItem	*item
	)
{
	const QString	&aName = item->attr()->getName () ;

	/* Query							*/
	/* We parse the query to see if it is valid, and if not then	*/
	/* ask the user whether to accept it ...			*/
	if (aName == "query")
	{
		QString	  sql	 = m_textEdit->text () ;
		KBSelect  select ;
		KBDBLink  dbLink ;

		if (!dbLink.connect (m_query->getDocRoot()->getLocation(), getProperty ("server")))
			if (TKMessageBox::questionYesNo
				(	0,
					QString(TR("Cannot parse SQL: %1\nSave anyway?"))
						.arg(dbLink.lastError().getDetails()),
					TR("SQL error")
				)
				!= TKMessageBox::Yes) return false ;

		if (!select.parseQuery (sql, &dbLink))
			if (TKMessageBox::questionYesNo
				(	0,
					QString(TR("Cannot parse SQL: %1\nSave anyway?"))
						.arg(select.lastError().getDetails()),
					TR("SQL error")
				)
				!= TKMessageBox::Yes) return false ;

		setProperty (item, sql) ;
		return	true  ;
	}

	if (aName == "toptable")
	{
		setProperty (item, m_comboBox->currentText()) ;
		return	true  ;
	}

	return	KBPropDlg::saveProperty (item) ;
}

/*  KBQrySQLPropDlg								*/
/*  clickVerify : Handle click of the verify button			*/
/*  (returns)	: void		:					*/

void	KBQrySQLPropDlg::clickVerify  ()
{
	QString	  sql	 = m_textEdit->text () ;
	KBSelect  select ;
	KBDBLink  dbLink ;

	if (!dbLink.connect (m_query->getDocRoot()->getLocation(), getProperty ("server")))
	{
		dbLink.lastError().DISPLAY() ;
		return	;
	}

	if (!select.parseQuery (sql, &dbLink))
	{
		select.lastError().DISPLAY() ;
		return	;
	}
}

/*  qrySQLPropDlg							*/
/*		: Run the link properties dialog			*/
/*  query	: KBQrySQL *	: Parent link				*/ 
/*  caption	: cchar * 	: Dialog box caption			*/
/*  attribs	: QPtrList<KBAttr>&: List of attributes			*/
/*  (returns)	: bool		: Success				*/

bool	qrySQLPropDlg
	(	KBQrySQL	*query,
		cchar		*caption,
		QPtrList<KBAttr>	&attribs
	)
{
	KBQrySQLPropDlg qDlg (query, caption, attribs) ;
	return	qDlg.exec () ;
}
