/***************************************************************************
    file	         : kb_qtbutton.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2000,2001,2002,2003 by theKompany.com
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	"kb_qtbutton.h"

#include	"kb_button.h"
#include	"kb_display.h"
#include	"kb_writer.h"


/*  KBCtrlButton							*/
/*  KBCtrlButton: Constructor for pushbutton wrapper			*/
/*  display	: KBDisplay *	: Parent display			*/
/*  button	: KBButton *	: Parent button				*/
/*  design	: bool		: Design mode				*/
/*  (returns)	: KBCtrlButton	:					*/

KBCtrlButton::KBCtrlButton
	(	KBDisplay	*display,
		KBButton	*button
	)
	:
	RKPushButton	(display->getDisplayWidget()),
	KBControl	(this, display, button),
	m_button	(button)
{
	RKPushButton::connect
	(	(RKPushButton *)this,	SIGNAL(clicked()),
		m_button, 		SLOT  (clicked())
	)	;
	RKPushButton::connect
	(	(RKPushButton *)this,	SIGNAL(toggled(bool)),
		m_button, 		SLOT  (toggled(bool))
	)	;
}

/*  KBCtrlButton							*/
/*  ~KBCtrlButton: Destructor for button widget				*/
/*  (returns)	 :		:					*/

KBCtrlButton::~KBCtrlButton ()
{
}

/*  KBCtrlButton							*/
/*  setPixmaps	: Set normal and rollover images			*/
/*  pm1		: const QPixmap & : Normal image			*/
/*  pm2		: const QPixmap & : Rollover image			*/
/*  (returns)	: void		  :					*/

void	KBCtrlButton::setPixmaps
	(	const QPixmap	&pm1,
		const QPixmap	&pm2
	)
{
	m_pm1	= pm1	;
	m_pm2	= pm2	;

	setPixmap (pm1)	;
}

/*  KBCtrlButton							*/
/*  eventFilter	: Local event filter					*/
/*  o		: QObject *	: Object receiving event		*/
/*  e		: QEvent *	: Event in question			*/
/*  (returns)	: void		  :					*/

bool	KBCtrlButton::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	if (o == (RKPushButton *)this)
		switch (e->type())
		{
			case QEvent::Enter	:
				if (!m_pm2.isNull())
				{	setPixmap (m_pm2) ;
					return	true	  ;
				}
				break	;

			case QEvent::Leave	:
				if (!m_pm1.isNull())
				{	setPixmap (m_pm1) ;
					return	true	  ;
				}
				break	;

			default	:
				break	;
		}

	return	KBControl::eventFilter (o, e) ;
}

/*  KBCtrlButton							*/
/*  setValue	: Set button text					*/
/*  value	: const KBValue &: Value for text			*/
/*  (returns)	: void		 :					*/

void	KBCtrlButton::setValue
	(	const KBValue	&value
	)
{
	setText (value.getRawText()) ;
}

/*  KBCtrlButton							*/
/*  getValue	: Get button text					*/
/*  (returns)	: KBValue	: Button text				*/

KBValue	KBCtrlButton::getValue ()
{
	return	KBValue(text()) ;
}

/*  KBCtrlButton							*/
/*  giveFocus	: Give focus to the control				*/
/*  (returns)	: void							*/

void	KBCtrlButton::giveFocus ()
{
	QButton::setFocus () ;
}

/*  KBCtrlButton							*/
/*  write	: Write field 						*/
/*  writer	: KBWriter *	: Output writer				*/
/*  rect	: QRect		: Field area				*/
/*  value	: KBValue &	: Value for reports			*/
/*  fSubs	: bool		: Substitution flag			*/
/*  extra	: int &		: Return extra space			*/
/*  (returns)	: bool		: Success				*/

bool	KBCtrlButton::write
	(	KBWriter	*writer,
		QRect		rect,
		const KBValue	&value,
		bool		fSubs,
		int		&extra
	)
{
	if (writer->asReport())
	{
		extra	= 0	;
		return	true	;
	}

	return	KBControl::write (writer, rect, value, fSubs, extra) ;
}
