/***************************************************************************
    file	         : kb_qttree.h
    copyright            : (C) 1999,2000,2001,2002,2003,2004 by Mike Richardson
			   (C) 2001,2002,2003,2004 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	_KB_QTTREE_H
#define	_KB_QTTREE_H


#include	<qstringlist.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_ctrl.h"
#include	"kb_combobox.h"

#include	"libkbase_exports.h"

#include	"rk_listview.h"

class	KBCtrlTreeItem	;
class	KBTree		;

/*  KBCtrlTree								*/
/*  ----------								*/
/*  This class provides a tree-link control where displayed values can	*/
/*  be grouped up into a tree for compact and organised display.	*/

class	LIBKBASE_API	KBCtrlTree : public RKListView, public KBControl
{
	Q_OBJECT

	KBTree			*m_tree		;

	QString			m_userFilter	;
	QString			m_userSorting	;
	QStringList		*m_keyset	;

	QFocusEvent		*m_deferFocus	;
	QMouseEvent		*m_deferMouse	;

	void		loadDataTree
			(	KBCtrlTreeItem		*parent,
				const KBLTValueSet	&valset,
				uint			fromRow,
				uint			forRows,
				uint			offset
			)	;
	void		loadDataFlat
			(	const KBLTValueSet	&valset
			)	;
	void		loadDataValues
			(	const KBLTValueSet	&valset
			)	;

	void		closeAllTree		(QListViewItem       *) ;
	bool		setCurrent		(int, KBCtrlTreeItem *) ;

private slots :

	void		slotCurrentChanged	(QListViewItem *) ;
	void		passFocus		() ;

protected :

        virtual void    giveFocus		() ;

public	:

	KBCtrlTree (KBDisplay *, KBTree *, uint) ;
virtual~KBCtrlTree () ;

	virtual	void	showName	() ;
	virtual	void	setupProperties	() ;
	virtual	void	setData		(void *) ;
	virtual	void	showAs		(KB::ShowAs) ;
	virtual	void	setValue   	(const KBValue &) ;
	virtual	KBValue	getValue   	() ;
	virtual	void	clearValue	(bool) ;
	virtual	bool	changed    	() ;
	virtual	bool	isEmpty		() ;
	virtual	bool	isValid		(bool) ;
	virtual	bool	write		(KBWriter *, QRect, const KBValue &, bool, int &) ;

	virtual	bool	event		(QEvent      *) ;
	virtual	void	focusInEvent	(QFocusEvent *) ;

	void		setUserFilter	(const QString &) ;
	void		setUserSorting	(const QString &) ;
	void		reload		() ;
	KBValue		getExtra	(bool, uint) ;

	SETFOCUS
//	SIZEHINT

	friend	class	KBCtrlTreeItem	;
}	;

#endif	// _KB_QTTREE_H
