/***************************************************************************
    file	         : kb_slotnotifier.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_AUTOMOC
#include	"kb_slotnotifier.moc"
#else
#include	"kb_slotnotifier.h"
#endif


#include	"kb_object.h"
#include	"kb_slot.h"


/*  KBSlotNotifierEntry							*/
/*  KBSlotNotifierEntry							*/
/*		: Null constructor for slot notifier entry		*/
/*  (returns)	: KBSlotNotifierEntry	:				*/

KBSlotNotifierEntry::KBSlotNotifierEntry ()
	:
	m_object	(0),
	m_slot		(0)
{
}

/*  KBSlotNotifierEntry							*/
/*  KBSlotNotifierEntry							*/
/*		: Constructor for slot notifier entry			*/
/*  object	: KBObject *		: Slot parent object		*/
/*  slot	: KBSlot *		: Slot				*/
/*  name	: const QString &	: Notifier name			*/
/*  (returns)	: KBSlotNotifierEntry	:				*/

KBSlotNotifierEntry::KBSlotNotifierEntry
	(	KBObject	*object,
		KBSlot		*slot,
		const QString	&name
	)
	:
	m_object	(object),
	m_slot		(slot),
	m_name		(name)
{
}


/*  ------------------------------------------------------------------	*/


/*  KBSlotNotifier							*/
/*  slotDestroyed: Slot has been destroyed				*/
/*  slot	 : QObject *	: Actually the slot			*/
/*  (returns)	 : void		:					*/

void	KBSlotNotifier::slotDestroyed
	(	QObject		*slot
	)
{
//	fprintf
//	(	stderr,
//		"KBSlotNotifier::slotDestroyed: called: %p\n",
//		(void *)slot
//	)	;

	QValueList<KBSlotNotifierEntry>::iterator iter ;
	iter = m_entries.begin() ;
	while (iter != m_entries.end())
		if ((*iter).m_slot == slot)
			iter	= m_entries.remove(iter) ;
		else	++iter	;
}

/*  KBSlotNotifier							*/
/*  add		: Add slot to notifier					*/
/*  object	: KBObject *		: Slot parent object		*/
/*  slot	: KBSlot *		: Slot				*/
/*  name	: const QString &	: Notifier name			*/
/*  (returns)	: void			:				*/

void	KBSlotNotifier::add
	(	KBObject	*object,
		KBSlot		*slot,
		const QString	&name
	)
{
//	fprintf
//	(	stderr,
//		"KBSlotNotifier::add: %s: adding %s.%s:%s: %p\n",
//		name.latin1(),
//		object->getElement().latin1(),
//		object->getName   ().latin1(),
//		slot  ->name      ().latin1(),
//		(void *)slot
//	)	;

	m_entries.append (KBSlotNotifierEntry (object, slot, name)) ;

	connect
	(	slot,
		SIGNAL	(destroyed	(QObject *)),
		SLOT	(slotDestroyed	(QObject *))
	)	;
}

/*  KBSlotNotifier							*/
/*  remove	: Remove the slot					*/
/*  slot	: KBSlot *	: Slot					*/
/*  (returns)	: void		:					*/

void	KBSlotNotifier::remove
	(	KBSlot		*slot
	)
{
	slotDestroyed (slot) ;
}

/*  KBSlotNotifier							*/
/*  fire	: Fire off notification					*/
/*  source	: KBObject *	   : Source of notification		*/
/*  name	: const QString &  : Notification name			*/
/*  argc	: uint		   : Number of arguments		*/
/*  argv	: KBValue *	   : Vector of arguments		*/
/*  rc		: KBScriptError *& : Error or null if OK		*/
/*  (returns)	: void		   :					*/

void	KBSlotNotifier::fire
	(	KBObject	*source,
		const QString	&name,
		uint		argc,
		KBValue		*argv,
		KBScriptError	*&rc
	)
{
//	fprintf
//	(	stderr,
//		"KBSlotNotifier::fire: name=[%s]\n",
//		name.latin1()
//	)	;

	rc	= 0 ;

	for
	(	QValueList<KBSlotNotifierEntry>::iterator iter = m_entries.begin() ;
		iter != m_entries.end() ;
		++iter
	)
		if ((*iter).m_name == name)
		{
//			fprintf
//			(	stderr,
//				"KBSlotNotifier::fire: matched .....\n"
//			)	;

			(*iter).m_slot->eventSignal (source, name, argc, argv, rc) ;
		}
}


KBSlotNotifier
	*KBSlotNotifier::self()
{
	static	KBSlotNotifier	*notifier = 0 ;
	if (notifier == 0) notifier = new KBSlotNotifier ;
	return	notifier ;
}

