/***************************************************************************
    file	         : kb_treedlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



#include	"kb_tree.h"
#include	"rk_lineedit.h"

#if		__KB_AUTOMOC
#include	"kb_treedlg.moc"
#else
#include	"kb_treedlg.h"
#endif


static	IntChoice	choiceTreeType[] =
{
	{	0,			"Expanded tree"			},
	{	1,			"Flattened Tree"		},
	{	-1,			0				}
}	;

/*  KBTreePropDlg							*/
/*  KBTreePropDlg: Constructor for tree properties dialog		*/
/*  tree	 : KBTree *	 : Parent tree				*/ 
/*  caption	 : const char *  : Dialog box caption			*/
/*  attribs	 : QPtrList<KBAttr>&: List of attributes			*/
/*  (returns)	 : KBTreePropDlg :					*/

KBTreePropDlg::KBTreePropDlg
	(	KBTree		*tree,
		cchar		*caption,
		QPtrList<KBAttr>	&attribs
	)
	:
	KBLinkTreePropDlg(tree, caption, attribs),
	m_tree		 (tree)
{
}

/*  KBTreePropDlg							*/
/*  ~KBTreePropDlg: Destructor for block properties dialog		*/
/*  (returns)	   : void	:					*/

KBTreePropDlg::~KBTreePropDlg ()
{
}

/*  KBTreePropDlg							*/
/*  getAttrItem	: Get attribute item for attribute			*/
/*  attr	: KBAttr *	: Attribute				*/
/*  (returns)	: KBAttrItem *	: Associated attribute item		*/

KBAttrItem
	*KBTreePropDlg::getAttrItem
	(	KBAttr	*attr
	)
{
	if (attr->getName() == "treetype")
		return	new KBAttrIntChoice (attr, choiceTreeType) ;

	return	KBLinkTreePropDlg::getAttrItem (attr) ;
}

/*  KBTreePropDlg							*/
/*  showProperty: Show property						*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Property on show			*/

bool	KBTreePropDlg::showProperty
	(	KBAttrItem	*item
	)
{
	const QString	&aName = item->attr()->getName () ;

	/* Group							*/
	/* This is the tree grouping query. Show all the fields		*/
	/* associated with that query.					*/
	if (aName == "group")
	{
		KBQryBase *query ;

		if ((query = m_tree->getQuery()) == 0)
			return	warning ("Tree lacks a query") ;

		return	showQueryField (query, 0, item->value()) ;
	}

	if (aName == "treetype")
	{
		showChoices (item, choiceTreeType, item->value()) ;
		return	true ;
	}

	return	KBLinkTreePropDlg::showProperty (item) ;
}

/*  KBTreePropDlg							*/
/*  saveProperty: Check and save attribute				*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Attribute OK				*/

bool	KBTreePropDlg::saveProperty
	(	KBAttrItem	*item
	)
{
	const QString	&aName = item->attr()->getName () ;

	/* Group							*/
	/* Copy the result back ....					*/
	if (aName == "group")
	{
		if (strcmp (m_lineEdit->text(), item->value()) != 0)
			setProperty (aName, m_lineEdit->text()) ;

		return	true ;
	}

	if (aName == "treetype")
	{
		saveChoices (item, choiceTreeType) ;
		return	true	;
	}

	return	KBLinkTreePropDlg::saveProperty (item) ;
}


/*  treePropDlg	: Run the link properties dialog			*/
/*  tree	: KBTree *	: Parent tree				*/ 
/*  caption	: cchar * 	: Dialog box caption			*/
/*  attribs	: QPtrList<KBAttr>&: List of attributes			*/
/*  (returns)	: bool		: Success				*/

bool	treePropDlg
	(	KBTree		*tree,
		cchar		*caption,
		QPtrList<KBAttr>	&attribs
	)
{
	KBTreePropDlg lDlg (tree, caption, attribs) ;
	return	lDlg.exec () ;
}
