/***************************************************************************
    file	         : kb_dispwidget.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_DISPWIDGET
#define	__KB_DISPWIDGET

#include 	"libkbase_exports.h"

#include	<qtimer.h>
#include	<qframe.h>
#include	<qscrollbar.h>
#include	<qptrlist.h>
#include	<qlabel.h>
#include	<qvaluelist.h>
#include	<qpixmap.h>

#include	"kb_classes.h"
#include	"kb_recordnav.h"
#include	"kb_attrnav.h"
#include	"kb_attrgeom.h"


class		KBGeometry	;
class		KBAttrGeom	;
class		KBGridSetup	;


/*  KBDispWidget							*/
/*  ------------							*/
/*  This is a wrapper for QWidget so that we can pick up mouse and	*/
/*  paint events and pass them up to the owning KBDisplay for possible	*/
/*  processing.								*/

class 	LIBKBASE_API	KBDispWidget : public QFrame
{
	Q_OBJECT

	KBDisplay	*m_display	;
	uint		m_showBar	;
	bool		m_inSetRange	;

	QWidget		*m_canvas	;
	QSize		m_topSize	;

	QScrollBar	*m_vBar		;
	KBRecordNav	*m_rLabel	;	/* Record count label	*/

	KB::ShowAs	m_showing	;
	QTimer		m_scrollTimer	;

	QPoint		m_mouseDownAt	;

	QString		m_title		;
	QPixmap		m_bgPixmap	;
	int		m_bgScale	;
	QString		m_tagText	;
	QLabel		*m_tagLabel	;

	KBGeometry	*m_geometry	;

signals	  :

	void		backgroundChanged	() ;

protected :

	virtual void	mousePressEvent		(QMouseEvent  *) ;
	virtual void	mouseMoveEvent		(QMouseEvent  *) ;
	virtual	void	mouseReleaseEvent	(QMouseEvent  *) ;
	virtual	void	contextMenuEvent	(QContextMenuEvent *) ;
	virtual	void	paintEvent 		(QPaintEvent  *) ;
	virtual	void	resizeEvent		(QResizeEvent *) ;
	virtual	bool	event			(QEvent       *) ;
	virtual	bool	eventFilter		(QObject      *, QEvent *) ;
	virtual	void	mouseDoubleClickEvent	(QMouseEvent  *) ;

protected slots :

	void		vbarMoved		()	;
	void		scrollTimeout		()	;
	void		slotOperation		(KB::Action, uint) ;

public	:

	KBDispWidget		(QWidget *, KBDisplay *, uint)	;
	virtual	~KBDispWidget	() ;

	void		setRowColSetup
			(	const KBGridSetupList	&,
				const KBGridSetupList	&
			)	;

	void		showAs			(KB::ShowAs)	;
	void		setShowbar		(uint)		;
	void		setFrame		(int, int)	;
	void		setRowRange		(uint, uint, uint, uint, uint) ;
	QSize		effectiveSize		() ;
	void		setTitle		(const QString    &) ;
	void		updateDynamic		(const KBAttrGeom &) ;

	void		addChild		(QWidget *, const QRect &, Qt::AlignmentFlags) ;
	void		moveChild		(QWidget *, const QRect &, Qt::AlignmentFlags) ;
	void		resizeContents		(const QSize &)	  ;
	void		setTopSize		(const QSize &)	  ;

	void		updateMorph		(KBItem  *, const QRect &) ;
	void		updateMorph		(KBItem  *, uint) ;

	void		setTagLabel		(const QString &) ;
	QLabel		*getTagLabel		(QWidget *)	  ;
	QSize		showBarUsed		() 		  ;

	void		setBackgroundPixmap	(const QPixmap &, int) ;

	inline	QWidget		*getCanvas	()
	{
		return	m_canvas	;
	}

	inline	const QSize	&getTopSize	()
	{
		return	m_topSize	;
	}

	virtual	QSize	sizeHint		() const ;
}	;

#endif	// __KB_DISPWIDGET
