/***************************************************************************
    file	         : kb_event.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qregexp.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_database.h"
#include	"kb_location.h"
#include	"kb_attr.h"
#include	"kb_script.h"
#include	"kb_object.h"
#include	"kb_event.h"
#include	"kb_callback.h"
#include	"kb_location.h"
#include	"kb_callback.h"
#include	"kb_appptr.h"
#include	"kb_docroot.h"
#include	"kb_eventdlg.h"

#define		EFLAGS	(KAF_GRPEVENT|KAF_EDITOR|KAF_CLEAR|KAF_CUSTOM)

/*  KBEvent								*/
/*  KBEvent	: Constructor for event class				*/
/*  node	: KBNode *	: Owning node				*/
/*  name	: cchar *	: Attribute name			*/
/*  base	: cchar *	: Event base				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  nflags	: uint		: Required node flags			*/
/*  (returns)	: KBEvent	:					*/

KBEvent::KBEvent
	(	KBNode			*node,
		cchar			*name,
		cchar			*base,
		const QDict<QString>	&aList,
		uint			nFlags
	)
	:
	KBAttrStr (node, name, aList, nFlags | EFLAGS),
	m_base	  (base)
{
	QString	  *l2 = aList.find (QString("%1_l2").arg(name)) ;
	m_value2  =  l2 == 0 ? QString::null : *l2 ;

	init	  () ;

	QString	*bpt = aList.find (QString("%1_bpt").arg(name)) ;
	if (bpt != 0)
	{	QStringList l = QStringList::split (',', *bpt) ;
		for (uint idx = 0 ; idx < l.count() ; idx += 1)
			m_breakpoints.append (l[idx].toInt()) ;
	}
}

/*  KBEvent								*/
/*  KBEvent	: Constructor for event class				*/
/*  node	: KBNode *	: Owning node				*/
/*  name	: cchar *	: Attribute name			*/
/*  base	: cchar *	: Event base				*/
/*  text	: cchar *	: Initial text value			*/
/*  nflags	: uint		: Required node flags			*/
/*  (returns)	: KBEvent	:					*/

KBEvent::KBEvent
	(	KBNode	*node,
		cchar	*name,
		cchar	*base,
		cchar	*text,
		uint	nFlags
	)
	:
	KBAttrStr (node, name, text, nFlags | EFLAGS),
	m_base	  (base)
{
	init	  () ;
}

/*  KBEvent								*/
/*  KBEvent	: Constructor for event class				*/
/*  node	: KBNode *	: Owning node				*/
/*  name	: cchar *	: Attribute name			*/
/*  base	: cchar *	: Event base				*/
/*  srce	: KBNode *	: Extant source				*/
/*  nflags	: uint		: Required node flags			*/
/*  (returns)	: KBEvent	:					*/

KBEvent::KBEvent
	(	KBNode	*node,
		cchar	*name,
		cchar	*base,
		KBNode	*srce,
		uint	nFlags
	)
	:
	KBAttrStr (node, name, srce, nFlags | EFLAGS),
	m_base	  (base)
{
	init	  () ;

	/* See if we have an extant event and if such an event has a	*/
	/* macro, in which case replicate the macro.			*/
	KBAttr	*attr	= srce->getAttr(name) ;
	if (attr  == 0) return ;

	KBEvent	*event	= attr->isEvent() ;
	if (event == 0) return ;

	m_value2 = event->m_value2 ;

	KBMacroExec *macro = event->getMacro() ;
	if (macro != 0) m_macro	= new KBMacroExec(macro) ;
}

/*  KBEvent								*/
/*  ~KBEvent	: Destructor for event class				*/
/*  (returns)	:		:					*/

KBEvent::~KBEvent ()
{
	clearOverride () ;

	if (m_code    != 0) delete m_code    ;
	if (m_code2   != 0) delete m_code2   ;
	if (m_emitter != 0) delete m_emitter ;
	if (m_macro   != 0) delete m_macro   ;
}

void	KBEvent::init ()
{
	m_disable	= false	;
	m_code		= 0	;
	m_code2		= 0	;
	m_emitter	= 0	;
	m_cppFunc	= 0	;
	m_macro		= 0	;
	m_override	= 0	;
	m_inherit	= 0	;
}

/*  KBEvent								*/
/*  clearOverride:							*/
/*  (returns)	 : void		  :					*/

void	KBEvent::clearOverride ()
{
	/* Clear the override/inherit chain. Note that such a chain	*/
	/* ends up back at this event so careful with the loop		*/
	/* termiantion.							*/
	/* NOTE: The m_override pointer is only set in the original	*/
	/*	 event so we don't need to worry about enexpected	*/
	/*	 recursion.						*/
	KBEvent	*over	;
	KBEvent	*next	;

	if ((over = m_override) != 0)
		while ((over != 0) && (over != this))
		{	next	= over->m_inherit ;
			delete	over	;
			over	= next	;
		}

	m_override = 0 ;
	m_inherit  = 0 ;
}

/*  KBEvent								*/
/*  showAs	: Set display mode					*/
/*  showAs	: KB::ShowAs	: New mode				*/
/*  (returns)	: bool		: Value has been changed		*/

bool	KBEvent::showAs
	(	KB::ShowAs	showAs
	)
{
	/* Always clear the override/inherit chain. This only matters	*/
	/* if the user goes data->design->data but just to it on all	*/
	/* changes.							*/
	if (showAs != m_showing) clearOverride () ;

	return	KBAttrStr::showAs (showAs) ;
}

/*  KBEvent								*/
/*  setOverride	: Set override event function				*/
/*  over	: const QString & : Override function			*/
/*  (returns)	: void		  :					*/

void	KBEvent::setOverride
	(	const QString	&over
	)
{
	/* Create a new event attribute object, set the override and	*/
	/* link into this event. The new evet is chained such that	*/
	/* if goes to the head of a list which will chain back to this	*/
	/* event (which is effectively the base method).		*/
	QDict<QString> dummy ;

	KBEvent	*oe = new KBEvent
			  (	m_owner,
				m_name,
				m_base,
				dummy,
				m_nFlags|EFLAGS
			  )	;

	oe->setValue  (over)	;
	oe->m_inherit = m_override == 0 ? this : m_override ;
	m_override    = oe	;
}

/*  KBEvent								*/
/*  setValue	: Set event text value					*/
/*  value	: const QString & : Value				*/
/*  (returns)	: void		  :					*/

void	KBEvent::setValue
	(	const QString	&value
	)
{
	/* This method is overridden so that we can clear any code	*/
	/* compiled for an earlier value, and clear the disabled flag	*/
	DELOBJ	(m_code)  ;
	m_disable = false ;

	KBAttrStr::setValue (value) ;
}


void	KBEvent::setValue
	(	KBAttrItem	*item
	)
{
#if	! __KB_RUNTIME
	setValue (item->value()) ;
#endif
}

void	KBEvent::setValue2
	(	const QString	&value2
	)
{
	DELOBJ	(m_code2) ;
	m_value2 = value2 ;
}

QString	KBEvent::getValue2 ()
{
	return	m_value2 ;
}


void	KBEvent::setCPPFunc
	(	CPPFunc		cppFunc
	)
{
	m_disable = false 	;
	m_cppFunc = cppFunc	;
}

bool	KBEvent::isEmpty ()
{
	if (m_macro	   != 0) return false ;
	if (m_cppFunc	   != 0) return false ;
	if (!m_value2.isEmpty()) return false ;

	return	getValue().isEmpty () ;
}

/*  KBEvent								*/
/*  execCode	: Execute script code					*/
/*  scrIface	: KBScriptIF *	  : Scripting interface			*/
/*  code	: KBScriptCode *  : Precompiled script code		*/
/*  event	: const QString & : Event code				*/
/*  resval	: KBValue &	  : Return value 			*/
/*  argc	: uint		  : Argument count			*/
/*  argv	: KBValue *	  : Argument vector			*/
/*  (returns)	: KBScriptError * : Error or null if ok			*/

KBScriptError
	*KBEvent::execCode
	(	KBScriptIF	*scrIface,
		KBScriptCode	*&code,
		const QString	&event,
		KBValue		&resval,
		uint		argc,
		KBValue		*argv
	)
{
	/* Handle explicit events, where we compile the code and keep a	*/
	/* copy. First see if the code has already been compiled and	*/
	/* if not then so so now.					*/
	if (code == 0)
	{
		KBError		error	;
		QString		eText	;
		QString		ePatt	;

#if	__KB_DEBUG_EVENT
		fprintf
		(	stderr,
			"KBEvent::execCode: compiling code .......\n"
		)	;
#endif
		if ((code = scrIface->compileFunc
			    (	event,
				m_owner->isObject  ()->getPath   (),
				"eventFunc",
				eText,
				ePatt,
				m_owner->getDocRoot()->getImports(),
				m_inherit,
				error
			    )	) == 0)
		{
			/* The event is disabled, and we return false	*/
			/* so that the caller will bring up the		*/
			/* required property dialog.			*/
			m_disable = true ;
			return	new KBScriptError (error, this) ;
		}

		code->setBreakpoints (m_breakpoints) ;
	}

	/* Code was or has been compiled successfully, in which case	*/
	/* execute it. Check for an execution time error ....		*/
	KBScript::ExeRC rc = code->execute (m_owner, argc, argv, resval) ;

	switch (rc)
	{
		case KBScript::ExeAbort	:
			return	new KBScriptError () ;

		case KBScript::ExeError	:
		case KBScript::ExeFail	:
			{
			QString	   errMsg   ;
			uint	   errLno   ;
			QString	   errText  ;
			KBLocation errLocn  = scrIface->exeError (errMsg, errLno, errText) ;

			m_disable = true    ;

			fprintf
			(	stderr,
				"KBEvent::execCode: locn=[%s]\n",
				errLocn.name().latin1()
			)	;

			if (errLocn.name() != KBLocation::m_pInline)
				return	new KBScriptError
					(	KBError
						(	KBError::Error,
							errMsg,
							errText,
							__ERRLOCN
						),
						m_owner->isObject(),
						errLocn,
						errText,
						errLno,
						rc == KBScript::ExeFail
			  		)	;

			return	new KBScriptError
				(	KBError
					(	KBError::Error,
						errMsg,
						errText,
						__ERRLOCN
					),
					this
				)	;
			}

		default	:
			break	;
	}

	/* Return true; success here means that there were no execution	*/
	/* errors, not that the event script itself returned true.	*/
	return	0 ;
}

/*  KBEvent								*/
/*  doExecute	: Execute event						*/
/*  resval	: KBValue &	  : Return value 			*/
/*  argc	: uint		  : Argument count			*/
/*  argv	: KBValue *	  : Argument vector			*/
/*  defval	: bool		  : Default execution return code	*/
/*  (returns)	: KBScriptError * : Error or null if OK			*/

KBScriptError
	*KBEvent::doExecute
	(	KBValue		&resval,
		uint		argc,
		KBValue		*argv,
		bool		defval
	)
{
#if	__KB_DEBUG_EVENT
	fprintf
	(	stderr,
		"KBEvent::doExecute: [%s] e=[%p] m=[%p] [%s]\n",
		(cchar *)m_name,
		(void  *)m_emitter,
		(void  *)m_macro,
		(cchar *)getValue()
	)	;
	for (uint idx = 0 ; idx < argc ; idx += 1)
		fprintf
		(	stderr,
			"..... %d: %s\n",
			idx,
			(cchar *)argv[idx].getRawText()
		)	;
#endif


	/* SIGNAL CODE							*/
	/* -----------							*/
	/* If this event has been wired up to any slots then signal	*/
	/* these first.							*/
	if (m_emitter != 0)
	{
		KBScriptError *rc = m_emitter->doSignal (argc, argv) ;
		if (rc != 0) return rc ;
	}


	/* MACRO							*/
	/* -----							*/
	/* Event may be a macro, in which case all the work is done	*/
	/* inside the macro itself.					*/
	if (m_macro != 0)
	{
		KBScriptError *rc = m_macro->execute (m_owner->getRoot()) ;
		if (rc != 0) return rc ;

		if (defval)
			resval.setTrue	() ;
		else	resval.setFalse	() ;

		return	0 ;
	}


	/* EVENT CODE							*/
	/* ----------							*/
	/* C++-level function. Used internally, should really never	*/
	/* fail!							*/
	if (m_cppFunc != 0)
	{
		KBError		error	;
		KBScript::ExeRC rc	= (*m_cppFunc) (this, resval, argc, argv, error) ;

		switch (rc)
		{
			case KBScript::ExeAbort	:
				return	new KBScriptError () ;

			case KBScript::ExeError	:
			case KBScript::ExeFail	:
				return	new KBScriptError (error) ;

			default	:
				break	;
		}

		return	0 ;
	}


	/* Next case, there is no text associated with the event, in	*/
	/* which case do nothing. This is considered successful.	*/
	QString	 event	= getValue () ;
	if (event.isEmpty())
	{
		if (defval)
			resval.setTrue	() ;
		else	resval.setFalse	() ;

		return	0 ;
	}

	/* Set the result value false until proven otherwise. This is	*/
	/* the catch-all case.						*/
	resval.setFalse ()   ;

	/* A script interface is needed to be able to execute the event	*/
	/* whether the code is implicit or explicit ....		*/
	bool		ok	  ;
	KBError		error	  ;
	KBDocRoot	*root	  = m_owner->getDocRoot   () ;
	KBScriptIF	*scrIface = root  ->loadScripting (false, ok, error) ;

	if (!ok)
		return	new KBScriptError (error, this) ;

	if (scrIface == 0)
		return	new KBScriptError
			(	KBError
				(	KBError::Error,
					TR("No scripting language specified"),
					QString(TR("Trying to execute event %1")).arg(getLegend()),
					__ERRLOCN
				),
				this
			)	;

	/* SHORTCUT							*/
	/* --------							*/
	/* If the event is marked with #[A-Za-z]* then this is an	*/
	/* implicit event, and we execute a routine named for the	*/
	/* event base (eg., "onClick" for a button)  and the reset of	*/
	/* the event text.						*/
	if ((event.at(0) == '#') && event.at(1).isLetter())
	{
		QString		name	= m_base + event.mid(1).stripWhiteSpace() ;
		KBScript::ExeRC	rc 	  ;

		rc	= scrIface->execute
			  (	m_owner->getDocRoot()->getScripts(),
				name,
				m_owner,
				argc,
				argv,
				resval
			  )	;

		switch (rc)
		{
			case KBScript::ExeAbort	:
				return	new KBScriptError () ;

			case KBScript::ExeError	:
			case KBScript::ExeFail	:
				{
				/* Ho-hum, got an error which		*/
				/* executing the code.			*/
				QString	   errMsg   ;
				uint	   errLno   ;
				QString	   errText  ;
				KBLocation errLocn  = scrIface->exeError (errMsg, errLno, errText) ;

				m_disable = true    ;

				if (errLocn.name() != KBLocation::m_pInline)
					return	new KBScriptError
						(	KBError
							(	KBError::Error,
								errMsg,
								errText,
								__ERRLOCN
							),
							m_owner->isObject(),
							errLocn,
							errText,
							errLno,
							rc == KBScript::ExeFail
				  		)	;

				return	new KBScriptError
					(	KBError
						(	KBError::Error,
							errMsg,
							errText,
							__ERRLOCN
						),
						this
					)	;
				}

			default	:
				break	;
		}

		return	0 ;
	}


	/* EVENT CODE							*/
	/* ----------							*/
	/* This is the real thing!!!					*/
	return	execCode (scrIface, m_code, event, resval, argc, argv) ;
}

/*  KBEvent								*/
/*  execute	: Execute event						*/
/*  resval	: KBValue &	  : Return value 			*/
/*  argc	: uint		  : Argument count			*/
/*  argv	: KBValue *	  : Argument vector			*/
/*  defval	: bool		  : Default execution return code	*/
/*  (returns)	: KBScriptError * : Error or null if OK			*/

KBScriptError
	*KBEvent::execute
	(	KBValue		&resval,
		uint		argc,
		KBValue		*argv,
		bool		defval
	)
{
#if	__KB_DEBUG_EVENT
	fprintf
	(	stderr,
		"KBEvent::execute: [%p] [%s] o=[%p] i=[%p] e=[%p]\n",
		(void  *)this,
		(cchar *)m_name,
		(void  *)m_override,
		(void  *)m_inherit,
		(void  *)m_emitter
	)	;
	for (uint idx = 0 ; idx < argc ; idx += 1)
		fprintf
		(	stderr,
			".... %d: %s\n",
			idx,
			(cchar *)argv[idx].getRawText()
		)	;
#endif
	KBError		error	;
	KBScriptError	*rc	;
	KBCallback	*cb 	= KBAppPtr::getCallback() ;

	void		*logid	= cb == 0 ?
					0 :
					cb->logEvent
					(	"Event",
						m_ownerName,
						getOwner()->getAttrVal("name"),
						m_name,
						argc,
						argv
					)	;

	/* If the event is flagged as disabled then there has been an	*/
	/* earlier error ...						*/
	if (m_disable)
		return	new KBScriptError
			(	KBError
				(	KBError::Error,
					TR("Event has been disabled due to earlier error"),
					QString::null,
					__ERRLOCN
				),
				this
			)	;

	/* See if a second language has been set up _and_ there is a	*/
	/* second language (ie., client side) script. If so this is	*/
	/* executed. Whether control reaches a first-language script	*/
	/* depends on the outcome.					*/
	if
	(	!m_owner->getRoot()->getAttrVal("language2").isEmpty() &&
		!m_value2.isEmpty()
	)
	{
		bool		ok	  ;
		KBDocRoot	*root	  = m_owner->getDocRoot   () ;
		KBScriptIF	*scrIface = root  ->loadScripting (true, ok, error) ;

		if (!ok)
			return	new KBScriptError (error, this) ;

		if (scrIface == 0)
			return	new KBScriptError
				(	KBError
					(	KBError::Error,
						TR("No second scripting language specified"),
						QString(TR("Trying to execute event %1")).arg(getLegend()),
						__ERRLOCN
					),
					this
				)	;

		rc	= execCode (scrIface, m_code2, m_value2, resval, argc, argv) ;
		if (cb != 0)
			if (rc == 0)
				cb->logEventResult (resval, true, logid) ;
			else	cb->logEventResult (KBValue(rc->error().getMessage()), true, logid) ;

		fprintf
		(	stderr,
			"KBEvent::execute: l2: rc_ok=%d rv_true=%d\n",
			rc == 0,
			resval.isTrue()
		)	;

		/* If the script execution failed we terminate the	*/
		/* event. If it succeeded then still terminate if the	*/
		/* result was not "true"; this allows a client-side	*/
		/* script to determine whether a server-side script	*/
		/* executes.						*/
		if (rc 		!= 0) return rc	;
		if (!resval.isTrue()) return 0  ;
	}


	/* If the override pointer is set then there is a chain of	*/
	/* overriding (and inheriting) events starting here and leading	*/
	/* back to this event, so start following the chain.		*/
	if (m_override != 0)
		rc	= m_override->doExecute (resval, argc, argv, defval) ;
	else	rc	= doExecute		(resval, argc, argv, defval) ;

	if (cb != 0)
		if (rc == 0)
			cb->logEventResult (resval, true, logid) ;
		else	cb->logEventResult (KBValue(rc->error().getMessage()), true, logid) ;

#if	__KB_DEBUG_EVENT
	fprintf
	(	stderr,
		"KBEvent::execute: returns [%d]\n",
		rc == 0
	)	;
#endif
	return	rc	;
}

/*  KBEvent								*/
/*  getDescription							*/
/*		: Get description for property dialogs			*/
/*  (returns)	: QString	: Legend				*/


QString	KBEvent::getDescription () const
{
	QString	descr = KBAttr::getDescription () ;
	if (descr.find ('%') >= 0) descr = descr.arg(m_base) ;
	return	descr ;
}

/*  KBEvent								*/
/*  getEmitter	: Get emitter object					*/
/*  (returns)	: KBEmitter *	: Emitter				*/

KBEmitter
	*KBEvent::getEmitter ()
{
	/* The emitter object is created as needed, so that we do not	*/
	/* waste space by needing KBEvent to embed or be derived from	*/
	/* QObject.							*/
	if (m_emitter == 0)
		if (m_owner->isObject() != 0)
		{
			m_emitter = new KBEmitter (m_owner->isObject(), this) ;

#if	__KB_DEBUG_EVENT
			fprintf
			(	stderr,
				"KBEvent::getEmitter: event=[%p] emit=[%p]\n",
				(void *)this,
				(void *)m_emitter
			)	;
#endif
		}

	return	m_emitter	;
}

/*  KBEvent								*/
/*  clearEmitter: Clear emitter object if any				*/
/*  (returns)	: void		:					*/

void	KBEvent::clearEmitter ()
{
	/* Having a dynamically allocated emitter also gives an easy	*/
	/* way to clear down all signals ...				*/
	DELOBJ	(m_emitter) ;
}


/*  KBEvent								*/
/*  isEvent	: Checkes whether attribute is an event			*/
/*  (returns)	: KBEvent *	: Always the object			*/

KBEvent	*KBEvent::isEvent ()
{
	return	this ;
}

/*  KBEvent								*/
/*  setCode	: Set code associated with event			*/
/*  code	: const QString & : Code				*/
/*  append	: bool		  : Append code else replace		*/
/*  (returns)	: void		  :					*/

void	KBEvent::setCode
	(	const QString	&code,
		bool		append
	)
{
	if (append)
	{
		setValue (getValue() + code) ;
		return	;
	}

	setValue (code) ;
}

/*  KBEvent								*/
/*  setCode2	: Set second code associated with event			*/
/*  code	: const QString & : Code				*/
/*  append	: bool		  : Append code else replace		*/
/*  (returns)	: void		  :					*/

void	KBEvent::setCode2
	(	const QString	&code,
		bool		append
	)
{
	if (append)
	{
		setValue2 (getValue2() + code) ;
		return	;
	}

	setValue2 (code) ;
}

/*  KBEvent								*/
/*  tidy	: Tidy up after loading					*/
/*  (returns)	: void		:					*/

void	KBEvent::tidy ()
{
	/* This is just so we can output the code with a leading 	*/
	/* newline in the XML document.					*/
	QString	v  = getValue ().stripWhiteSpace() ;
	if (!v .isEmpty()) v  += "\n" ;
	setValue  (v ) ;

	QString	v2 = getValue2().stripWhiteSpace() ;
	if (!v2.isEmpty()) v2 += "\n" ;
	setValue2 (v2) ;
}

/*  KBEvent								*/
/*  setMacro	: Set a macro						*/
/*  macro	: KBMacro *	: Macro in question			*/
/*  (returns)	: void		:					*/

void	KBEvent::setMacro
	(	KBMacroExec	*macro
	)
{
	if (m_macro != 0) delete m_macro ;
	m_macro	= macro	;
}

/*  KBEvent								*/
/*  getMacro	: Get macro if any					*/
/*  (returns)	: KBMacro *	: Macro or null of none			*/

KBMacroExec *KBEvent::getMacro ()
{
	return	m_macro	;
}

/*  KBEvent								*/
/*  printAttr	: Print attribute					*/
/*  attrText	: QString &	: Result text for attribute output	*/
/*  nodeText	: QString &	: Result text for node output		*/
/*  indent	: int		: Indent depth for node output		*/
/*  flat	: bool		: Generate flatened version		*/
/*  (returns)	: void		:					*/

void	KBEvent::printAttr
	(	QString		&attrText,
		QString		&nodeText,
		int		indent,
		bool		flat
	)

{
	if ((getFlags() & (KAF_HIDDEN|KAF_USER)) != 0)
		return	;

	if (m_macro != 0)
	{
		nodeText += QString("%1<macro name=\"%2\">\n")
				   .arg("", indent)
				   .arg(m_name) ;

		m_macro->save (nodeText, indent + 2) ;

		nodeText += QString("%1</macro>\n")
				   .arg("", indent) ;
		return	;
	}

	static int asText = -1 ;
	if (asText < 0)
	{
		cchar *e = getenv("REKALL_EVENTS_ASTEXT") ;

		if (e == 0)
			asText	= 0 ;
		else	asText	= atoi(getenv("REKALL_EVENTS_ASTEXT")) ;

		fprintf
		(	stderr,
			"KBEvent::printAttr: asText=%d [%s]\n",
			asText,
			e == 0 ? "NULL" : e
		)	;
	}


	if (asText)
	{
		if (!getValue().isEmpty())
			nodeText += QString("%1<event name=\"%2\">\n%3%4</event>\n")
					   .arg("", indent)
					   .arg(m_name)
					   .arg(escapeText(getValue(), false))
				   	   .arg("", indent) ;
	}
	else	KBAttrStr::printAttr (attrText, nodeText, indent, flat) ;


	if (!m_value2.isEmpty())
	{
		if (asText)
			nodeText += QString("%1<event2 name=\"%2\">\n%3%4</event2>\n")
					   .arg("", indent)
					   .arg(m_name)
					   .arg(escapeText(m_value2, false))
				   	   .arg("", indent) ;
		else	addAttrText (attrText, QString("%1_l2").arg(m_name), m_value2) ;
	}

	if (m_breakpoints.count() != 0)
	{
		QString	list	;
		for (uint idx = 0 ; idx < m_breakpoints.count(); idx += 1)
		{	if (idx > 0) list += ',' ;
			list += QString::number(m_breakpoints[idx]) ;
		}

		attrText += QString (" %1_bpt=\"%2\"").arg(m_name).arg(list) ;
	}

}

void	KBEvent::setBreakpoints
	(	QValueList<int>	&breakpoints
	)
{
	m_breakpoints = breakpoints ;

#if	__KB_DEBUG_EVENT
	for (uint idx = 0 ; idx < m_breakpoints.count() ; idx += 1)
		fprintf
		(	stderr,
			"KBEvent::setBreakpoints: breakpoint %d\n",
			m_breakpoints[idx]
		)	;
#endif
}

const QValueList<int>
	&KBEvent::breakpoints ()
{
#if	__KB_DEBUG_EVENT
	for (uint idx = 0 ; idx < m_breakpoints.count() ; idx += 1)
		fprintf
		(	stderr,
			"KBEvent::breakpoints: breakpoint %d\n",
			m_breakpoints[idx]
		)	;
#endif
	return	m_breakpoints ;
}

#if	! __KB_RUNTIME

/*  KBEvent	:							*/
/*  trimEvent	: Trim event text					*/
/*  event	: QString	: User text				*/
/*  (returns)	: QString	: Trimmed text				*/

QString KBEvent::trimEvent
	(	QString		event
	)
{
	int	offset	;

	/* If this is a shortcut then strip leading whitespace and	*/
	/* anything from the first non-leading whitespace onwards.	*/
	/* This leaves just the shortcut itself.			*/
	if ((offset = QRegExp("^\\s*#[A-Za-z]").match (event)) >= 0)
	{
		event	= event.stripWhiteSpace() ;

		if ((offset = QRegExp("\\s").match (event)) >= 0)
			event	= event.left (offset) ;

		return	event	;
	}

	/* Otherwise, it is script code. In this case, strip leading	*/
	/* and trailing	whitespace and add a newline *unless* the text	*/
	/* is empty.							*/
	QString	trimmed	= event.stripWhiteSpace() ;
	return	trimmed.isEmpty() ? QString::null : trimmed + "\n" ;
}

/*  KBEvent								*/
/*  getAttrItem	: Get item for property dialogs				*/
/*  (returns)	: KBAttrItem *	: Actually KBEventItem			*/

KBAttrItem
	*KBEvent::getAttrItem ()
{
	return	new KBAttrEventItem (this) ;
}

/*  KBEvent								*/
/*  getAttrDlg	: Get dialog component for frame			*/
/*  parent	: QWidget *		: Parent widget			*/
/*  item	: KBAttrItem *		: Associated item		*/
/*  attrDict	: QDict<KBAttrItem> &	: Attribute dictionary		*/
/*  (returns)	: KBAttrDlg *		: Actually KBAttrFrameDlg	*/

KBAttrDlg
	*KBEvent::getAttrDlg
	(	QWidget			*parent,
		KBAttrItem		*,
		QDict<KBAttrItem>	&attrDict
	)
{
	return	new KBEventDlg (parent, this, 0, attrDict) ;
}

#endif
