/***************************************************************************
    file	         : kb_field.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	_KB_FIELD_H
#define	_KB_FIELD_H

#include	"libkbase_exports.h"

#include 	<qregexp.h>

#include	"kb_classes.h"
#include	"kb_item.h"
#include	"kb_qtfield.h"
#include	"kb_attralign.h"


/*  KBField								*/
/*  -----------								*/
/*  This class is used to represent a KBaseField element in the XML	*/
/*  form description. Note that it may map onto more than one actual	*/
/*  field.								*/

class	KBField : public KBItem
{
	Q_OBJECT

	KBAttrStr	m_fgcolor   ;	/* Foreground (text) colour	*/
	KBAttrStr	m_bgcolor   ;	/* Background colour		*/
	KBAttrStr	m_font	    ;	/* Report font			*/
	KBAttrBool	m_passwd    ;	/* Treat as a password field	*/
	KBAttrBool	m_nullOK    ;	/* Is empty field OK		*/
	KBAttrBool	m_emptyNull ;	/* Treat empty fields as null	*/
	KBAttrStr	m_eValid    ;	/* Validity regular expression 	*/
	KBAttrBool	m_ignCase   ;	/* Ignore case			*/
	KBAttrStr	m_mask	    ;	/* Input mask			*/
	KBAttrStr	m_format    ;	/* Format string		*/
	KBAttrBool	m_deformat  ;	/* Deformat on save		*/
	KBAttrAlign	m_align	    ;	/* Alignment			*/
	KBAttrBool	m_supress   ;	/* Supress repeats		*/
	KBAttrStr	m_helper    ;	/* Input helper			*/
	KBAttrBool	m_morph	    ;

	KBEvent		m_onChange  ;
	KBEvent		m_onReturn  ;
	KBEvent		m_onHelper  ;

	QRegExp		m_rValid    ;	/* Compiled validity pattern	*/
	
	KBReport	*m_report   ;	/* Embedding report		*/
	KBValue		m_lastVal   ;	/* Last row value		*/

	bool			doCheckValid	(const QString &, bool) ;

protected :

	virtual	KBControl	*makeCtrl	(uint)	  ;
	virtual	bool		isMorphing	()	  ;

#if	! __KB_RUNTIME
	virtual	void		whatsThisExtra	(QValueList<KBWhatsThisPair> &) ;
#endif

public	:

	KBField	(KBNode  *, const QDict<QString> &, bool * = 0) ;
	KBField (KBNode  *, KBField *) ;
virtual~KBField	() ;

#if	! __KB_RUNTIME
	virtual	bool		propertyDlg	(cchar *) ;
#endif
	virtual	KBNode		*replicate	(KBNode *) ;
	virtual	KBValue		getReportValue	(bool, bool = false) ;
	virtual	KBValue		getValue	(uint) ;

	virtual	void		showAs		(KB::ShowAs) ;

	bool			checkValid	(const QString &, bool) ;
	void			userChange	(uint) ;
	void			returnPressed	(uint, const QString &) ;
	void			onHelper	(uint, const QString &) ;

	inline	QString	getFormat ()
	{
		return	m_format.getValue    () ;
	}
	inline	int	getAlign  ()
	{
		return	m_align .getIntValue () ;
	}
	inline	QString	getHelper ()
	{
		return	m_helper.getValue    () ;
	}
	inline	bool	getDeFormat ()
	{
		return	m_deformat.getBoolValue() ;
	}
	inline	bool	isPasswd  ()
	{
		return	m_passwd.getBoolValue() ;
	}

	virtual	bool		doLeave		(uint) ;
	virtual	void		doSearch	() ;
}	;


#if	! __KB_RUNTIME
extern	LIBKBASE_API	bool	fieldPropDlg
			(	KBField		*,
				cchar		*,
				QPtrList<KBAttr>	&,
				cchar		* = 0
			)	;
#endif
#endif	// _KB_FIELD_H
