/***************************************************************************
    file	         : kb_linktree.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_formblock.h"
#include	"kb_select.h"
#include	"kb_ctrl.h"
#include	"kb_popupmenu.h"
#include	"kb_designpopup.h"

#if		__KB_AUTOMOC
#include	"kb_linktree.moc"
#else
#include	"kb_linktree.h"
#endif

#include	"kb_qrytable.h"
#include	"kb_qryquery.h"
#include	"kb_qrysql.h"
#include	"kb_queryset.h"
#include	"kb_layout.h"
#include	"kb_finddlg.h"


/*  KBLinkTreeDummy							*/
/*  ---------------							*/
/*  The KBLinkTree class works by creating some dummy items. These are	*/
/*  of class KBLinkTreeDummy, which overrides the various methods ....	*/

class	KBLinkTreeDummy : public KBItem
{
public	:

	inline	KBLinkTreeDummy
		(	KBObject 	*parent,
			const QString	&eName,
			const QString	show
		)
		:
		KBItem	(parent, "KBLinkTreeDummy", QRect(), eName, show, 0)
	{
	}

	virtual	void	buildGUI
		(	uint	,
			int	,
			int
		)
	{
	}
	virtual	void	showAs
		(	KB::ShowAs
		)
	{
		/* Switching in and out of design mode should not do	*/
		/* anything.						*/
	}
	virtual	void	printNode
		(	QString		&,
			int		,
			bool
		)
	{
		/* Dont want to print anything for this node, it is	*/
		/* always built on the fly.				*/
	}
	virtual	bool	isUpdateVal ()
	{
		/* Don't get a value from here, is should come from the	*/
		/* read link node.					*/
		return	false	;
	}
}	;


/*  KBLinkTree								*/
/*  KBLinkTree	: Constructor for link control				*/
/*  parent	: KBNode *	  : Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				  : List of attributes			*/
/*  element	: const QString & : Element name			*/
/*  (returns)	: KBNode	  :					*/

KBLinkTree::KBLinkTree
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		const QString		&element
	)
	:
	KBItem		(parent, element,
				 "master", 	aList),
	m_child		(this,   "child",	aList,    KAF_REQD),
	m_show		(this,   "show",	aList,    KAF_REQD),
	m_fgcolor	(this,   "fgcolor",	aList),
	m_bgcolor	(this,   "bgcolor",	aList),
	m_font		(this,	 "font",	aList),
	m_nullval	(this,	 "nullval",	aList),
	m_nullOK	(this,	 "nullok",	aList,    KAF_FORM),
	m_dynamic	(this,	 "dynamic",	aList),
	m_onChange	(this,	 "onchange",	"onLink", aList),
	m_extra		(this,	 "extra",	aList,	  KAF_FORM)
{
	m_query	 	= 0	;
	m_iKey	 	= 0	;
	m_loaded 	= false ;
	m_exprCount	= 0	;
	m_extraCount	= 0	;

	m_iShow .setAutoDelete (true) ;
}

/*  KBLinkTree								*/
/*  KBLinkTree	: Constructor for link control				*/
/*  parent	: KBNode *	: Parent node				*/
/*  link	: KBLinkTree *	: Extant link				*/
/*  (returns)	: KBNode	:					*/

KBLinkTree::KBLinkTree
	(	KBNode		*parent,
		KBLinkTree	*link
	)
	:
	KBItem		(parent,  "master",	 		link),
	m_child		(this,    "child",			link, KAF_REQD),
	m_show		(this,    "show",			link, KAF_REQD),
	m_fgcolor	(this,    "fgcolor",			link),
	m_bgcolor	(this,    "bgcolor",			link),
	m_font		(this,	  "font",			link),
	m_nullval	(this,	  "nullval",			link),
	m_nullOK	(this,	  "nullok",			link),
	m_dynamic	(this,	  "dynamic",			link),
	m_onChange	(this,	  "onchange",	"onChoice",	link),
	m_extra		(this,	  "extra",			link)
{
	m_iKey	 	= 0 	;
	m_loaded 	= false	;
	m_query	 	= 0	;
	m_exprCount	= 0	;
	m_extraCount	= 0	;

	m_iShow .setAutoDelete (true) ;
}

/*  KBLinkTree								*/
/*  ~KBLinkTree	: Destructor for simple form field			*/
/*  (returns)	:		:					*/

KBLinkTree::~KBLinkTree ()
{
	DELOBJ	(m_iKey) ;
	m_iShow .clear() ;
}

/*  KBLinkTree								*/
/*  keyStroke	: Control has received keystroke			*/
/*  k		: QKeyEvent *	: Key event in question			*/
/*  (returns)	: bool		: Key event consumed			*/

bool	KBLinkTree::keyStroke
	(	QKeyEvent	*k
	)
{
	/* Override for key up and key down so that we can change	*/
	/* selection, otherwise these keys will navigate between	*/
	/* records.							*/
	if (k->type() == QEvent::KeyPress)
		if ((k->key() == Qt::Key_Up) || (k->key() == Qt::Key_Down))
			return	false	;

	return	KBItem::keyStroke (k) ;
}

/*  KBLinkTree								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBLinkTree::showAs
	(	KB::ShowAs	mode
	)
{
#if	! __KB_RUNTIME
	if (mode == KB::ShowAsDesign)
	{
		m_keyset.clear ()	;
		m_valset.clear ()	;
		remDummyItems  () 	;
		m_loaded = false  	;
	}
#endif
	m_query		= 0		;

	CITER
	(	QryBase,
		q,
		m_query = q
	)	;

	if (m_query == 0)
		KBError::EFault
		(	"Link/Tree control lacks a query",
			QString::null,
			__ERRLOCN
		)	;

	KBItem::showAs (mode) ;
}

/*  KBLinkTree								*/
/*  remDummyItems: Remove dummy items for query				*/
/*  (returns)	 : void		:					*/

void	KBLinkTree::remDummyItems ()
{
	if (m_iKey  != 0)
	{
		if (m_query != 0) m_query->remItem (0, m_iKey ) ;
		DELOBJ	(m_iKey)	;
	}

	LITER
	(	KBItem,
		m_iShow,
		item,
		m_query->remItem (0, item)
	)

	m_iShow .clear () ;
}

/*  KBLinkTree								*/
/*  addExprItems: Add expression item(s) for query			*/
/*  exprs	: QString	: Expressions				*/
/*  (returns)	: uint		: Number of expressions added		*/

uint	KBLinkTree::addExprItems
	(	QString		exprs
	)
{
	/* Strip leading and trailing white space; if the expression(s)	*/
	/* string is then empty, return. This handles the case where	*/
	/* there are no "extra" expressions.				*/
	exprs	= exprs.stripWhiteSpace () ;
	if (exprs.isEmpty()) return 0	   ;

	/* Split the expressions into separate expressions and add a	*/
	/* dummy item for each. We use the select parser to do this;	*/
	/* if it fails to parse then assume that we have a single	*/
	/* expression and hope!						*/
	KBSelect select ;
	if (select.parseExprList (exprs))
	{
		const QValueList<KBSelectExpr> exprList = select.getExprList () ;
		QValueList<KBSelectExpr>::ConstIterator iter ;
		uint idx = 0 ;

		for (iter = exprList.begin() ; iter != exprList.end() ; iter++, idx++)
		{
			KBItem	*item = new KBLinkTreeDummy
					(	this,
						QString("__show_%1").arg(idx),
						(*iter).exprText(0)
					)	;

			m_iShow.append   (item)	   ;
			m_query->addItem (0, item) ;

//			fprintf
//			(	stderr,
//				"KBLinkTree::addDummyItems: add %d [%s]\n",
//				idx,
//				(cchar *)(*iter).exprText(0)
//			)	;
		}

		return	exprList.count() ;
	}

	KBItem	*item = new KBLinkTreeDummy
			(	this,
				"__show_0",
				m_show.getValue().stripWhiteSpace()
			)	;

	m_iShow.append   (item)	   ;
	m_query->addItem (0, item) ;
	return	1 ;
}

/*  KBLinkTree								*/
/*  addDummyItems: Add dummy items for query				*/
/*  (returns)	 : uint		: Number of items added excluding key	*/

uint	KBLinkTree::addDummyItems ()
{
	/* The link works by adding two items, one to contain the key	*/
	/* from the query (ie., the child value), and the other for	*/
	/* the values from the query (ie., the displayed expressions).	*/

	/* If any dummy items already exist then remove them. This is	*/
	/* needed since this routine is called after the link		*/
	/* properties have been changed.				*/
	remDummyItems	  	() ;

	m_query->addItem	(0, 0) ;
	m_query->addItem	(0, m_iKey = new KBLinkTreeDummy (this, "_key",  m_child.getValue())) ;

	m_exprCount  = addExprItems (m_show .getValue()) ;
	m_extraCount = addExprItems (m_extra.getValue()) ;

	return	m_exprCount + m_extraCount ;
}

/*  KBLinkTree								*/
/*  checkValid	: Check whether value can be saved in database		*/
/*  idx		: uint		: Value set index			*/
/*  alowNull	: bool		: Ignore not-null check			*/
/*  (returns)	: bool		: True if OK				*/

bool	KBLinkTree::checkValid
	(	uint		idx,
		bool		allowNull
	)
{
	if (!allowNull)
		if (!m_nullOK.getBoolValue () && (idx == 0))
		{
			setError
			(	KBError::Error,
				QString(TR("Value must be selected from list for %1")).arg(errorText()),
				QString::null,
				__ERRLOCN
			)	;
			return	false ;
		}

	return	true	;
}

/*  KBLinkTree								*/
/*  filter	: const QString & : User filter expression		*/
/*  sorting	: const QString & : User sorting expression		*/
/*  keyset	: QStringList &	  : Return list of keys			*/
/*  valset	: KBLTValueSet &  : Return list of value lists		*/
/*  (returns)	: void		  :					*/

void	KBLinkTree::loadValues
	(	const QString		&filter,
		const QString		&sorting,
		QStringList		&keyset,
		KBLTValueSet		&valset
	)
{
	fprintf
	(	stderr,
		"KBLinkTree::loadValues: f=[%s] s=[%s]\n",
		(cchar *)filter,
		(cchar *)sorting
	)	;

	keyset.clear   () ;
	valset.clear   () ;
	keyset.append  ("")  ;
	valset.append  (QStringList(m_nullval.getValue())) ;

	if (! m_query->doSelect
		(	0,
			(KBValue *)0,
			QString::null,
			filter,
			sorting
	   ))
	{
		m_query->lastError().DISPLAY () ;
		return	;
	}

	for (uint row = 0 ; row < m_query->getNumRows(0) ; row += 1)
	{
		QString     key = m_query->getField(0, row, m_iKey->getQueryIdx()).getRawText() ;
		QStringList qsl ;

		LITER
		(	KBItem,
			m_iShow,
			item,

			/* FIX: Including a null value in a string	*/
			/* list which is then loaded into a combobox	*/
			/* seems to terminate the combo list at that	*/
			/* entry.					*/
			QString	show = m_query->getField(0, row, item->getQueryIdx()).getRawText() ;
			if (show.isNull()) show = "" ;
			qsl.append (show) ;
		)

		keyset.append (key) ;
		valset.append (qsl) ;
	}
}

/*  KBLinkTree								*/
/*  loadValues	: Load keys and values					*/
/*  (returns)	: void		:					*/

void	KBLinkTree::loadValues ()
{
	loadValues (QString::null, QString::null, m_keyset, m_valset) ;
}

/*  KBLinkTree								*/
/*  prepare	: Prepare link prior to display				*/
/*  (returns)	: void		:					*/

void	KBLinkTree::prepare ()
{
	if (!m_loaded)
	{
		addDummyItems	() ;
		loadValues	() ;
		m_loaded = true	   ;
	}

	KBItem::prepare() ;

	KBLinkTreeData ltd (m_keyset, m_valset) ;
	for (uint idx = 0 ; idx < m_nCtrls ; idx += 1)
		if (m_ctrls[idx] != 0)
			m_ctrls[idx]->setData ((void *)&ltd) ;
}

/*  KBLinkTree								*/
/*  setupControls							*/
/*		: Set up controls					*/
/*  (returns)	: void		:					*/

void	KBLinkTree::setupControls ()
{
	uint	nCtrls	= m_nCtrls ;
	KBItem::setupControls ()   ;

	if (!showingData()) return ;

	KBLinkTreeData ltd (m_keyset, m_valset) ;
	for (uint idx = nCtrls ; idx < m_nCtrls ; idx += 1)
		if (m_ctrls[idx] != 0)
			m_ctrls[idx]->setData ((void *)&ltd) ;
}


/*  KBLinkTree								*/
/*  itemToValue	: Map item number to value				*/
/*  item	: uint		: Item number				*/
/*  keyset	: QStringList *	: Optional keyset			*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBLinkTree::itemToValue
	(	uint			item,
		const QStringList	*keyset
	)
{
	if (keyset == 0) keyset = &m_keyset ;

	return	item == 0 ?
			KBValue (m_type) :
			KBValue	((*keyset)[item], m_type) ;
}

/*  KBLinkTree								*/
/*  valueToItem	: Map value to item number				*/
/*  value	: const KBValue &: Value to map				*/
/*  keyset	: QStringList *	 : Optional keyset			*/
/*  (returns)	: uint		 : Item number				*/

uint	KBLinkTree::valueToItem
	(	const KBValue		&value,
		const QStringList	*keyset
	)
{
	if (keyset == 0) keyset = &m_keyset ;

//	fprintf
//	(	stderr,
//		"KBLinkTree::valueToItem: [%s] in (%s)[%s]\n",
//		(cchar *)value.getRawText(),
//		keyset == &m_keyset ? "global" : "local",
//		(cchar *)keyset->join(" ,")
//	)	;

	int	idx = keyset->findIndex (value.getRawText()) ;
	return	idx < 0 ? 0 : idx ;
}

/*  KBLinkTree								*/
/*  valueToText	: Map value to display text				*/
/*  value	: const KBValue &: Value to map				*/
/*  (returns)	: QString	 : Display text				*/

QString	KBLinkTree::valueToText
	(	const KBValue	&value
	)
{
	int	idx = m_keyset.findIndex (value.getRawText()) ;
	return	idx < 0 ? QString::null :
			  m_valset[idx].join(" ") ;
}

/*  KBLinkTree								*/
/*  userChange	: Selection changed notification			*/
/*  qrow	: uint		: Query row number			*/
/*  idx		: uint		: New selection index			*/
/*  (returns)	: void		:					*/

void	KBLinkTree::userChange
	(	uint		qrow,
		uint		idx
	)
{
	KBValue	args[2]	;
	bool	evRc	;

	args[0] = (int)qrow		    ;
	args[1] = KBValue (m_keyset[idx])   ;
	eventHook (m_onChange, 2, args, evRc) ;

	KBItem::userChange (qrow) ;
}

/*  KBLinkTree								*/
/*  getReportValue							*/
/*		: Get value for report writing				*/
/*  first	: bool		: First output flag			*/
/*  prior	: bool		: Return prior value flag		*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBLinkTree::getReportValue
	(	bool		,
		bool
	)
{
	int	slot	;

	if ((slot = m_keyset.findIndex (m_curVal.getRawText())) >= 0)
		return	KBValue (m_valset[slot].join(" ")) ;

	return	KBValue	() ;
}

/*  KBLinkTree								*/
/*  getOrderType: Get ordering type					*/
/*  (returns)	: KB::IType	  : Ordering type			*/

KB::IType KBLinkTree::getOrderType()
{
	return	KB::ITString	;
}

/*  KBLinkTree								*/
/*  getOrderValue							*/
/*		: Get ordering value					*/
/*  value	: const KBValue & : Value				*/
/*  (returns)	: QString	  : Ordering value			*/

QString	KBLinkTree::getOrderValue
	(	const KBValue	&value
	)
{
	return	valueToText (value) ;
}

/*  KBLinkTree								*/
/*  getQuery	: Get associated query					*/
/*  (returns)	: KBQryBase *	: Associated query			*/

KBQryBase *KBLinkTree::getQuery ()
{
	return	m_query	;
}

/*  KBLinkTree								*/
/*  doRefresh	: Possibly refresh selections				*/
/*  drow	: uint		: Display row to refresh		*/
/*  (returns)	: void		:					*/

void	KBLinkTree::doRefresh
	(	uint		drow
	)
{
	loadValues () ;
	KBLinkTreeData ltd (m_keyset, m_valset) ;
	m_ctrls[drow]->setData ((void *)&ltd) ;
}

/*  KBLinkTree								*/
/*  doSearch	: Do record search					*/
/*  (returns)	: void		:					*/

void	KBLinkTree::doSearch ()
{
	QStringList list ;

	for (uint idx = 0 ; idx < m_valset.count() ; idx += 1)
		list.append (m_valset[idx].join(" ")) ;

	KBFindChoiceDlg findDlg (getFormBlock(), this, list, m_keyset) ;
	findDlg.exec () ;
}

/*  KBLinkTree								*/
/*  itemToValue	: Map item number to value				*/
/*  item	: uint		: Item number				*/
/*  which	: bool		: Display/extra				*/
/*  extra	: uint		: Extra index				*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBLinkTree::itemToExtra
	(	uint			item,
		bool			which,
		uint			extra
	)
{
	if (item  == 0)
		return KBValue() ;

	if (which)
		if (extra < m_exprCount)
			return	m_valset[item][extra] ;
		else	return	KBValue() ;

	if (extra < m_extraCount)
		return	m_valset[item][m_exprCount + extra] ;

	return KBValue() ;
}

KBValue	KBLinkTree::getRowExtra
	(	uint			qrow,
		bool			which,
		uint			index
	)
{
	KBValue	value	= KBItem::getRowValue (qrow) ;
	int	item	= m_keyset.findIndex  (value.getRawText()) ;

	fprintf
	(	stderr,
		"KBLinkTree::getRowExtra: qr=%d w=%d idx=%d: [%s]->[%d]->[%s]\n",
		qrow,
		which,
		index,
		value.getRawText().latin1(),
		item,
		itemToExtra (item, which, index).getRawText().latin1()
	)	;

	return	itemToExtra (item, which, index) ;
}

/*  KBLinkTree								*/
/*  showQuery	: Show associated query					*/
/*  (returns)	: void		:					*/

void	KBLinkTree::showQuery ()
{
#if	! __KB_RUNTIME
	if (m_query != 0)
	{
		addDummyItems  () ;

		QString	sql	= m_query->getSQLText   (0, true) ;
		KBQryDisplay(sql, QString::null).exec() ;

		remDummyItems  () ;
	}
#endif
}

#if	! __KB_RUNTIME

/*  KBLinkTree								*/
/*  whatsThisExtra							*/
/*		: Extra stuff for pseudo-what's this popup		*/
/*  info	: QValueList<KBWhatsThisPair> &	: List of items		*/
/*  (returns)	: void				:			*/

void	KBLinkTree::whatsThisExtra
	(	QValueList<KBWhatsThisPair>	&info
	)
{
	KBItem::whatsThisExtra (info) ;

	if (m_query != 0) m_query->whatsThisExtra (info) ;

	info.append (KBWhatsThisPair(TR("Link"),   m_child.getValue())) ;
	info.append (KBWhatsThisPair(TR("Display"),m_show .getValue())) ;
}


/*  KBLinkTree								*/
/*  designPopup	: Display design menu					*/
/*  parent	: KBPopupMenu *	: Possible parent menu			*/
/*  cell	: QRect		: Cell for dynamic layouts		*/
/*  (returns)	: KBPopupMenu *	: Popup menu				*/

KBPopupMenu
	*KBLinkTree::designPopup
	(	KBPopupMenu	*parent,
		QRect		cell
	)
{
	KBPopupMenu *popup = KBObject::designPopup (parent, cell) ;
	popup->insertItem (TR("&Show query"), this, SLOT(showQuery())) ;
	return	popup	;
}
#endif


