/***************************************************************************
    file	         : kb_linktree.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	_KB_LINKTREE_H
#define	_KB_LINKTREE_H

#include	"libkbase_exports.h"

#include	<qdict.h>
#include	<qstringlist.h>

#include	"kb_classes.h"
#include	"kb_item.h"

class		KBLink	;
class		KBTree	;

typedef		QValueList<const QStringList>	KBLTValueSet	;

class	KBLinkTreeData
{
public	:

	const QStringList	&m_keyset	;
	const KBLTValueSet	&m_valset	;

	inline	KBLinkTreeData
		(	const QStringList	&keyset,
			const KBLTValueSet	&valset
		)
		:
		m_keyset	(keyset),
		m_valset	(valset)
	{
	}
}	;

/*  KBLinkTree								*/
/*  ----------								*/
/*  This class is a base class for Link and Tree controls, which have	*/
/*  most of their functionality in common.				*/

class	LIBKBASE_API	KBLinkTree : public KBItem
{
	Q_OBJECT

	KBAttrStr	m_child		;
	KBAttrStr	m_show		;
	KBAttrStr	m_fgcolor	;
	KBAttrStr	m_bgcolor	;
	KBAttrStr	m_font		;
	KBAttrStr	m_nullval	;
	KBAttrBool	m_nullOK	;
	KBAttrBool	m_dynamic	;
	KBEvent		m_onChange	;
	KBAttrStr	m_extra		;


	KBItem	 	*m_iKey		;	/* Dummy for child key	*/
	QPtrList<KBItem>m_iShow		;	/* Dummy for display	*/

	bool		m_loaded	;	/* Query values loaded	*/

	QStringList	m_keyset	;	/* Key list from query	*/
	KBLTValueSet	m_valset	;	/* Query display list	*/

	uint		m_exprCount	;
	uint		m_extraCount	;

protected	:

	uint			addExprItems	(QString) ;
	virtual	uint		addDummyItems 	() ;
	virtual	void		remDummyItems	() ;
	void			loadValues	() ;

	KBQryBase 	*m_query	;


protected slots	:

	void		showQuery	() ;

public	:

	KBLinkTree	(KBNode  *, const QDict<QString> &, const QString &) ;
	KBLinkTree	(KBNode  *, KBLinkTree *) ;

virtual~KBLinkTree	()		 ;

	virtual	void		prepare		()			;
	virtual void		setupControls	()			;
	virtual	KBValue		getReportValue	(bool, bool = false)	;

#if	! __KB_RUNTIME
	virtual	void		whatsThisExtra	(QValueList<KBWhatsThisPair> &) ;
	virtual	KBPopupMenu	*designPopup	(KBPopupMenu *, QRect)	;
#endif
	virtual	QString		getOrderValue	(const KBValue &)	;
	virtual	KB::IType	getOrderType	() ;

	virtual	void		showAs		(KB::ShowAs)	;
	virtual	bool		keyStroke	(QKeyEvent   *) ;

	KBQryBase		*getQuery	() ;

	bool			checkValid	(uint, bool)	;
	void			userChange	(uint, uint)	;
	KBValue			itemToValue	(uint, 		  const QStringList *) ;
	uint			valueToItem	(const KBValue &, const QStringList *) ;
	QString			valueToText	(const KBValue &) ;
	void			doRefresh	(uint) ;

	void			loadValues
				(	const QString			&,
					const QString			&,
					QStringList			&,
					KBLTValueSet			&
				)	;

	virtual	void		doSearch	() ;

	virtual void		setUserFilter	(uint, const QString	&) = 0 ;
	virtual void		setUserSorting	(uint, const QString	&) = 0 ;
	virtual	void		reload		(uint)			   = 0 ;
	virtual	KBValue		getExtra	(uint, bool, uint)	   = 0 ;
	KBValue			getRowExtra	(uint, bool, uint)	;

	KBValue			itemToExtra	(uint, bool, uint) ;

	inline	bool	dynamic	()
	{
		return	m_dynamic.getBoolValue	() ;
	}
	inline	uint	exprCount	()
	{
		return	m_exprCount	;
	}
	inline	uint	extraCount	()
	{
		return	m_extraCount	;
	}
}	;

extern	LIBKBASE_API	bool	linkPropDlg (KBLink *, cchar *, QPtrList<KBAttr> &) ;
extern	LIBKBASE_API	bool	treePropDlg (KBTree *, cchar *, QPtrList<KBAttr> &) ;

#endif	// _KB_LINKTREE_H
