/***************************************************************************
    file	         : kb_object.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>

#include	<qapplication.h>
#include	<qpalette.h>
#include	<qwhatsthis.h>

#if		__KB_AUTOMOC
#include	"kb_object.moc"
#else
#include 	"kb_object.h"
#endif

#include	"kb_slot.h"
#include	"kb_ctrl.h"
#include	"kb_display.h"
#include	"kb_layout.h"
#include	"kb_formblock.h"
#include	"kb_reportblock.h"
#include	"kb_form.h"
#include	"kb_options.h"
#include	"kb_font.h"
#include	"kb_docroot.h"
#include	"kb_location.h"
#include	"kb_script.h"
#include	"kb_config.h"
#include	"kb_override.h"
#include	"kb_complink.h"
#include	"kb_quicktext.h"
#include	"kb_directprop.h"
#include	"kb_rowcoldialog.h"
#include	"kb_deleter.h"
#include	"kb_designpopup.h"
#include	"kb_ascii.h"
#include	"kb_debugging.h"


#if		! __KB_RUNTIME
#include	"kb_popupmenu.h"
#include	"kb_formcopier.h"
#include	"kb_sizer.h"
#include	"kb_noobjdlg.h"
#include	"kb_component.h"
#include	"kb_compaccessdlg.h"
#include	"kb_nodereg.h"
#include	"kb_toolbox.h"
#endif

#include	"tk_fontdialog.h"
#include	"tk_colordialog.h"
#include	"tk_messagebox.h"




/*  KBObject								*/
/*  KBObject	: Constructor for object class				*/
/*  parent	: KBNode *	: Parent node				*/
/*  element	: cchar *	: Element tag				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  (returns)	: KBObject	:					*/

KBObject::KBObject
	(	KBNode			*parent,
		cchar			*element,
		const QDict<QString>	&aList
	)
	:
	KBNode	(parent,   element, aList),
	m_geom	(this,		    aList)
{
	m_control	= 0	;
	m_block		= parent == 0 ? 0 : parent->isBlock () ;
	m_display	= 0	;
	m_curPal	= 0	;
	m_curFont	= 0	;

#if	! __KB_RUNTIME
	m_sizer		= 0	;
	m_quickText	= 0	;
#endif

	m_scriptObj	= 0 ;
	m_attrConfig	= new KBAttrStr (this, "configs", "", KAF_CONFIG) ;
	m_attrSlots	= new KBAttrStr	(this, "slots",   "", KAF_SLOTS ) ;
	m_slotList.setAutoDelete (true) ;
}

/*  KBObject								*/
/*  KBObject	: Constructor for new object				*/
/*  parent	: KBObject *	: Parent object				*/
/*  element	: cchar *	: Element tag				*/
/*  rect	: const QRect &	: Size and position			*/
/*  (returns)	: KBObject	:					*/

KBObject::KBObject
	(	KBObject	*parent,
		cchar		*element,
		const QRect	&rect
	)
	:
	KBNode	(parent,   element),
	m_geom
	(	this,
		rect.x     (),
		rect.y     (),
		rect.width (),
		rect.height(),
		KBAttrGeom::FMFixed, 
		KBAttrGeom::FMFixed
	) 
{
	m_control	= 0	;
	m_block		= parent == 0 ? 0 : parent->isBlock() ;
	m_display	= 0	;
	m_curPal	= 0	;
	m_curFont	= 0	;

#if	! __KB_RUNTIME
	m_sizer		= 0 	;
	m_quickText	= 0	;
#endif

	m_scriptObj	= 0 ;
	m_attrConfig	= new KBAttrStr (this, "configs", "", KAF_CONFIG) ;
	m_attrSlots	= new KBAttrStr	(this, "slots",   "", KAF_SLOTS ) ;
	m_slotList.setAutoDelete (true) ;
}

/*  KBObject								*/
/*  KBObject	: Constructor for new form object			*/
/*  parent	: KBNode *	: Parent				*/
/*  object	: KBObject *	: Extant object				*/
/*  (returns)	: KBObject	:					*/

KBObject::KBObject
	(	KBNode		*parent,
		KBObject	*object
	)
	:
	KBNode	(parent, 	object),
	m_geom	(this,		object)
{
	m_control	= 0	;
	m_display	= 0	;
	m_block		= parent == 0 ? 0 : parent->isBlock() ;
	m_curPal	= 0	;
	m_curFont	= 0	;

#if	! __KB_RUNTIME
	m_sizer		= 0	;
	m_quickText	= 0	;
#endif

	m_scriptObj	= 0 ;
	m_attrConfig	= new KBAttrStr (this, "configs", "", KAF_CONFIG) ;
	m_attrSlots	= new KBAttrStr	(this, "slots",   "", KAF_SLOTS ) ;
	m_slotList.setAutoDelete (true) ;

	LITER
	(	KBSlot,
		object->m_slotList,
		slot,
		new KBSlot (this, slot)
	)
}

/*  KBObject								*/
/*  ~KBObject	: Destructor for form object class			*/
/*  (returns)	 :		:					*/

KBObject::~KBObject ()
{
	if (m_scriptObj != 0)
	{
		for (uint code = 0 ; code < KBScriptIF::languageCount() ; code += 1)
			DELOBJ	(m_scriptObj[code]) ;
		delete	[] m_scriptObj ;
	}

	DELOBJ	(m_attrConfig) ;
	DELOBJ	(m_attrSlots ) ;
	DELOBJ	(m_control   ) ;
	DELOBJ	(m_curPal    ) ;
	DELOBJ	(m_curFont   ) ;

#if	! __KB_RUNTIME
	DELOBJ	(m_quickText ) ;
	DELOBJ	(m_sizer     ) ;
#endif
}

/*  KBObject								*/
/*  showAs	: Switch in or out of design mode			*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBObject::showAs
	(	KB::ShowAs	mode
	)
{
#if	__KB_RUNTIME
	if (m_control)
	{
		m_control->showAs (mode) ;
	}
#else	
	DELOBJ	(m_quickText ) ;

	/* If there is an associated control then create or destroy any	*/
	/* associated sizer, and show the control in the requested mode	*/
	if (m_control)
	{
		if ((mode == KB::ShowAsDesign) && (m_sizer == 0))
		{
			createSizer () ;
		}
		if ((mode == KB::ShowAsData  ) && (m_sizer != 0))
		{	getLayout()->dropSizer (m_sizer) ;
			DELOBJ	(m_sizer) ;
		}
		m_control->showAs (mode) ;
	}

	/* When toggling between design and data modes, destroy any	*/
	/* extant scripting object (so that when entering data mode we	*/
	/* start with a clean slate), and clear eny event emitters.	*/
	bool	toggle	= false	;
	if ((mode == KB::ShowAsDesign) && showingData  ()) toggle = true ;
	if ((mode == KB::ShowAsData  ) && showingDesign()) toggle = true ;

	if (toggle)
	{
		if (m_scriptObj != 0)
		{
			for (uint code = 0 ; code < KBScriptIF::languageCount() ; code += 1)
				DELOBJ	(m_scriptObj[code]) ;
			delete [] m_scriptObj ;
			m_scriptObj = 0	;
		}

		LITER
		(	KBAttr,
			m_attribs,
			attrib,
			if (attrib->isEvent() != 0)
				attrib->isEvent()->clearEmitter() ;
		)
	}
#endif

	KBNode::showAs (mode) ;
}

/*  KBObject								*/
/*  setChanged	: Set changed state after design operation		*/
/*  (returns)	: void		:					*/

void	KBObject::setChanged ()
{
	getLayout()->setChanged () ;
}

/*  KBObject								*/
/*  makeNewPopup: Make new object popup menu				*/
/*  parent	: KBPopupMenu *	: Parent popup menu			*/
/*  rect	: QRect		: New object area			*/
/*  (returns)	: KBPopupMenu *	: Menu					*/

KBPopupMenu
	*KBObject::makeNewPopup
	(	KBPopupMenu	*,
		QRect
	)
{
	return	0	;
}

/*  KBObject								*/
/*  setupProperties							*/
/*		: Set current properties on specified control		*/
/*  control	: KBControl *	: Control in question			*/
/*  rect	: const QRect & : Control geometry			*/
/*  (returns)	: void		:					*/

void	KBObject::setupProperties
	(	KBControl	*control,
		const QRect	&rect
	)
{
	if (control != 0)
	{
		if (parentIsDynamic())
		{
			QWidget	*w   = control->topWidget  () ;
			uint	minw = m_geom.minimumWidth () ;
			uint	maxw = m_geom.maximumWidth () ;
			uint	minh = m_geom.minimumHeight() ;
			uint	maxh = m_geom.maximumHeight() ;

			if (minw > 0) w->setMinimumWidth (minw) ;
			if (maxw > 0) w->setMaximumWidth (maxw) ;
			if (minh > 0) w->setMinimumHeight(minh) ;
			if (maxh > 0) w->setMaximumHeight(maxh) ;
		}
		else	control->setGeometry(rect) ;

		control->setPalette (getPalette (false)) ;
		control->setFont    (getFont    (false)) ;

		control->setupProperties () ;
	}
}

/*  KBObject								*/
/*  setupProperties							*/
/*		: Set current properties				*/
/*  (returns)	: void		:					*/

void	KBObject::setupProperties ()
{
	/* This method handles the basic common properties. Subclasses	*/
	/* will override it to set up control-specific properties.	*/

	/* (a)	Fix up the geometry in case this has changed		*/
	/* (b)	Clear cached palettes and fonts				*/
	/* (c)	Update all properties of the current control.		*/
	updateDisplay	() ;
	setPalette   	() ;
	setFont      	() ;

	setupProperties (m_control, geometry()) ;

#if	! __KB_RUNTIME
	if (showingDesign())
	{
		if (m_control != 0)
			m_control->showName () ;

		setChanged   () ;
		getLayout    ()->addSizer (getSizer(), false) ;
	}
#endif
}


/*  KBObject								*/
/*  minimumGeometry							*/
/*		: Calculate minimum geometry for the object		*/
/*  (returns)	: QSize		: Minimum size				*/

QSize	KBObject::minimumGeometry ()
{
	/* This routine calculates the minimum (x, y) position of the	*/
	/* bottom-right corner of the object in its parent, ie., the	*/
	/* smallest size that the parent can be shrunk down to.		*/

	/* Start by getting the mimumum size of this object if either	*/
	/* direction stretches (otherwise it is not needed).		*/
	QSize	minSize	;
	uint	x	;
	uint	y	;

//	fprintf
//	(	stderr,
//		"%*sKBObject::minimumGeom: .....\n",
//		_indent,
//		""
//	)	;
//	_indent	+= 4	;

	if
	(	(m_geom.xmode() == KBAttrGeom::FMStretch) ||
		(m_geom.ymode() == KBAttrGeom::FMStretch)
	)
		minSize	= minimumSize () ;

	/* The default "fixed" cases need to check whether the width	*/
	/* (or height) is zero, in which case use the corresponding	*/
	/* size hint. This is analagous to the code in ::geometry()	*/
	int	w	;
	int	h	;

	/* Now adjust for the geometry mode of this object within its	*/
	/* parent.							*/
	switch (m_geom.xmode())
	{
		case KBAttrGeom::FMFloat	:
			x = m_geom.x() ;
			break	;

		case KBAttrGeom::FMStretch	:
			x = m_geom.x() + minSize.width () ;
			break	;

		default	:
			w = m_geom.w()		;
			if ((w == 0) && (m_control != 0))
				w = m_control->topWidget()->sizeHint().width () ;
			x = m_geom.x() + w	;
			break	;
	}

	switch (m_geom.ymode())
	{
		case KBAttrGeom::FMFloat	:
			y = m_geom.y() ;
			break	;

		case KBAttrGeom::FMStretch	:
			y = m_geom.y() + minSize.height() ;
			break	;

		default	:
			h = m_geom.h()		;
			if ((h == 0) && (m_control != 0))
				h = m_control->topWidget()->sizeHint().height() ;
			y = m_geom.y() + h	;
			break	;
	}

//	_indent	-= 4	;
//	fprintf
//	(	stderr,
//		"%*sKBObject::minimumGeom: (%d,%d)\n",
//		_indent,
//		"",
//		x, y
//	)	;

	return	QSize(x, y) ;
}

/*  KBObject								*/
/*  minimumSize	: Get minimum size allowable				*/
/*  (returns)	: QSize		: Size					*/

QSize	KBObject::minimumSize()
{
	QSize	size	(0, 0) ;

	CITER
	(	Object,
		obj,
		size	= size.expandedTo (obj->minimumGeometry())
	)		

	/* If the parent is dynamic then (a) expand to the size hint of	*/
	/* the associated control if present and (b) expand to the	*/
	/* minimum geometry for this object.				*/
	if (parentIsDynamic())
	{
		if (m_control != 0)
			size	= size.expandedTo (m_control->topWidget()->sizeHint ()) ;
		size	= size.expandedTo (QSize(m_geom.minimumWidth(), m_geom.minimumHeight())) ;
	}

//	fprintf
//	(	stderr,
//		"KBObject::minimumSize: %s: %s\n",
//		(cchar *)getName(),
//		_TEXT(size)
//	)	;

	return	size	;
}

/*  KBObject								*/
/*  setPalette	: Set palette						*/
/*  (returns)	: void		:					*/

void	KBObject::setPalette ()
{
	/* This is more in the nature of a reset. The cached palette	*/
	/* will be recreated as soon as any attempt is made to use it.	*/
	DELOBJ	(m_curPal) ;

	CITER
	(	Object,
		o,
		o->setPalette()
	)
}

/*  KBObject								*/
/*  setFont	: Set font						*/
/*  (returns)	: void		:					*/

void	KBObject::setFont ()
{
	/* This is more in the nature of a reset. The cached font will	*/
	/* be recreated as soon as any attempt is made to use it ...	*/
	/* which is immediately as it it passed to the geometry attr.	*/
	DELOBJ	(m_curFont) ;

	m_geom.setFont (*getFont(false)) ;

	CITER
	(	Object,
		o,
		o->setFont   ()
	)
}

/*  KBObject								*/
/*  designPopup	: Display design menu					*/
/*  parent	: KBPopupMenu * : Possible parent menu			*/
/*  cell	: QRect		: Cell for dynamic layouts		*/
/*  (returns)	: void		:					*/

KBPopupMenu
	*KBObject::designPopup
	(	KBPopupMenu	*parent,
		QRect
	)
{
#if	! __KB_RUNTIME
	KBPopupMenu *popupMain = new KBPopupMenu (parent) ;

	popupMain->setTitle   (this) ;

	popupMain->insertItem (TR("C&ut"),		this, SLOT(cutObj         ())) ;
	popupMain->insertItem (TR("&Copy"),		this, SLOT(copyObj        ())) ;
	popupMain->insertItem (TR("&Delete"),	   	this, SLOT(deleteObj      ())) ;
	popupMain->insertItem (TR("Save as component"),	this, SLOT(saveAsComponent())) ;
	popupMain->insertItem (TR("&Properties"),	this, SLOT(propertyDlg    ())) ;

	/* The structure below lists attribute shortcuts. Scan through	*/
	/* looking for attributes that appear in the object, and	*/
	/* create a submenu for each one found. The code is passed to	*/
	/* the menu item so that it will be passed to the handler slot.	*/
	struct	DirectMap
	{
		const char	*attr	;
		const char	*menu	;
		int		code	;
	}	;

	static	DirectMap	dmap[]	=
	{
		{	"font",		"&Font",	DP_FONT		},
		{	"fgcolor",	"&Foreground",	DP_FOREGROUND	},
		{	"bgcolor",	"&Background",	DP_BACKGROUND	},
		{	"text",		"&Text",	DP_TEXT		},
		{	0,		0,		0		}
	}	;

	bool	direct	= false	;
	for (DirectMap *d = &dmap[0] ; d->attr != 0 ; d += 1)
		if (getAttr(d->attr) != 0)
		{	if (!direct)
			{	popupMain->insertSeparator() ;
				direct	= true	;
			}
			popupMain->insertItem (TR(d->menu), this, SLOT(setPropDirect(int)), 0, d->code) ;
		}

	if ((parent == 0) && (parentObject() != 0))
		makeAncestorPopup (popupMain, this) ;

	popupMain->insertSeparator() ;
	popupMain->insertItem(TR("&Information"), this, SLOT(whatsThis())) ;
	return	popupMain ;
#endif
#if	__KB_RUNTIME
	return	0 ;
#endif
}

/*  KBObject								*/
/*  setPropDirect: Handler for direct property setting			*/
/*  code	 : int		: Property code				*/
/*  (returns)	 : void		:					*/

void	KBObject::setPropDirect
	(	int		code
	)
{
#if	! __KB_RUNTIME
	KBAttr	*attr	= 0	;
	QString	value	;

	/* First step is to get the attribute. We should find it but	*/
	/* check for the sake if paranoia.				*/
	switch (code)
	{
		case DP_FONT		: attr	= getAttr("font"   ) ; break ;
		case DP_FOREGROUND	: attr	= getAttr("fgcolor") ; break ;
		case DP_BACKGROUND	: attr	= getAttr("bgcolor") ; break ;
		case DP_TEXT		: attr	= getAttr("text"   ) ; break ;
		default			: break	;
	}

	if (attr == 0)
	{
		fprintf
		(	stderr,
			"KBObject::setPropDirect: unknown for code %d\n",
			code
		)	;
		return	;
	}

	/* Got an attribute to pick up the current value and run the	*/
	/* appropriate dialog. If the user OK's then set the value back	*/
	/* and call the property update method.				*/
	value	= attr->getValue () ;

	switch (code)
	{
		case DP_FONT		:
			{
			TKFontDialog  fDialog (0, TR("Font"), false, true) ;
			fDialog.setFont (KBFont::specToFont (value)) ;
			if (!fDialog.exec()) return ;
			value	= KBFont::fontToSpec (fDialog.font()) ;
			}
			break	;

		case DP_FOREGROUND	:
		case DP_BACKGROUND	:
			{
			TKColorDialog cDialog (0, TR("Colour"), true) ;
			cDialog.setColor (QColor (value.toInt())) ;
			if (!cDialog.exec ()) return ;
			value.sprintf ("0x%06x", cDialog.color().rgb() & 0x00ffffff) ;
			}
			break	;

		case DP_TEXT	:
			{
			/* This is special. Create a quick text edit	*/
			/* and then leave it to run. The completion	*/
			/* event will be picked up in the event handler	*/
			/* below.					*/
			DELOBJ	(m_quickText) ;
			m_quickText = new KBQuickText
					  (	m_control->topWidget(),
						QRect
						(	0,
							0,
							m_control->topWidget()->width (),
							m_control->topWidget()->height()
						),
						value,
						this,
						(QEvent::Type)(QEvent::User + 1000)
					  )	;
			m_quickText->show     () ;
			m_quickText->setFocus () ;
			return	   ;
			}

		default	:
			return	;
	}

	attr->setValue	(value) ;
	setupProperties	()	;
#endif
}

/*  KBObject								*/
/*  event	: Local event handler					*/
/*  e		: QEvent *	: Event in question			*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBObject::event
	(	QEvent		*e
	)
{
#if	! __KB_RUNTIME
	/* Check for the quick text completion event. If it is *and*	*/
	/* there is a quick text widget (paranoia check) *and* there	*/
	/* is a text property (more paranoia) then update it.		*/
	if (e->type() == (QEvent::Type)(QEvent::User + 1000))
	{
		if (m_quickText != 0)
		{
			KBAttr	*attr	= getAttr("text") ;
			if (attr != 0)
			{	attr->setValue	(m_quickText->text()) ;
				setupProperties	() ;
			}

			DELOBJ	(m_quickText) ;
		}

		return	true	;
	}
#endif
	return	KBNode::event (e) ;
}

/*  KBObject								*/
/*  contextMenu	: Display context menu					*/
/*  e		: QMouseEvent *	: Triggering mouse event		*/
/*  drow	: uint		: Display row				*/
/*  (returns)	: void		:					*/

void	KBObject::contextMenu
	(	QMouseEvent	*,
		uint
	)
{
}

/*  KBObject								*/
/*  moveFocusOK	: Check if OK to set focus				*/
/*  drow	: uint		: Display row				*/
/*  (returns)	: bool		: OK					*/

bool	KBObject::moveFocusOK
	(	uint	
	)
{
#if	__KB_DEBUG_FOCUS
	fprintf
	(	stderr,
		"KBObject::moveFocusOK: %s:\n",
		(cchar *)getName()
	)	;
#endif
	return	showingDesign()	     ? true :
		getFormBlock () == 0 ? true :
				       getFormBlock()->focusOutOK (false) ;
}

void	KBObject::setFocus ()
{
#if	__KB_DEBUG_FOCUS
	fprintf
	(	stderr,
		"__KB_DEBUG_FOCUS: %s: m_control=%p\n",
		(cchar *)getName(),
		(void  *)m_control
	)	;
#endif
	if (m_control != 0) m_control->setFocus () ;
}

/*  KBObject								*/
/*  focusInEvent: Handle focus in event					*/
/*		: uint		: Display row				*/
/*		: Reason	: Reason focus arrives			*/
/*  (returns)	: void		:					*/

void	KBObject::focusInEvent
	(	uint		,
		QFocusEvent::Reason
	)
{
	/* Nothing to do here. Derived classes will implement this as	*/
	/* neccessary.							*/
#if	__KB_DEBUG_FOCUS
	fprintf
	(	stderr,
		"KBObject::focusInEvent: %s:\n",
		(cchar *)getName()
	)	;
#endif
}

/*  KBObject								*/
/*  geometry	: Get geometry of this object				*/
/*  (returns)	: QRect		: Extent				*/

QRect	KBObject::geometry ()
{
	QRect	g = m_geom.geometry () ;

	/* Zero widths or heights are mapped to the corresponding size	*/
	/* hint. Mostly useful for height, so that controls can adjust	*/
	/* to fonts when not in a dynamic parent.			*/
	if ((g.width () == 0) || (g.height() == 0))
		if (m_control != 0)
		{
			QSize hint = m_control->topWidget()->sizeHint() ;
			if (g.width () == 0) g.setWidth (hint.width ()) ;
			if (g.height() == 0) g.setHeight(hint.height()) ;
		}

	return	g ;
}

/*  KBObject								*/
/*  geometry	: Get geometry of this object with offset		*/
/*  offset	: const QPoint & : Offset to apply			*/
/*  (returns)	: QRect		 : Extent				*/

QRect	KBObject::geometry
	(	const QPoint	&offset
	)
{
	return	m_geom.geometry (offset) ;
}

QRect	KBObject::realGeometry ()
{
//	fprintf
//	(	stderr,
//		"KBObject::realGeometry: [%s.%s] cont=%p ctrl=%p\n",
//		(cchar *)getElement  (),
//		(cchar *)getName     (),
//		(void  *)getContainer(),
//		(void  *)m_control
//	)	;

	if (getContainer() != 0)
	{
		if (getParent() == 0)
			return	QRect(QPoint(0,0), getContainer()->getTopSize()) ;

		return	getContainer()->getTopRect() ;
	}

	if (m_control != 0)
	{
		QRect	r = m_control->topWidget()->geometry() ;
		getDisplay()->cvtViewToCtrl (r) ;
		return	r ;
	}

	return	QRect	(0,0,0,0) ;
}

/*  KBObject	:							*/
/*  sizeHint	: Get ideal size for object				*/
/*  (returns)	: QSize		: Size					*/

QSize	KBObject::sizeHint ()
{
	/* If the parent is dynamic then return the minimum size that	*/
	/* will completely display our content, otherwise use the	*/
	/* design geometry.						*/
	return	parentIsDynamic() ?
			minimumSize() :
			m_geom.geometry(true).size() ;
}

/*  KBObject								*/
/*  ctrlGone	: Note death of control					*/
/*  ctrl	: KBControl *	: The control				*/
/*  (returns)	: void		:					*/

void	KBObject::ctrlGone
	(	KBControl	*ctrl
	)
{
	if (ctrl == m_control)
		m_control = 0 ;
}

/*  KBObject								*/
/*  updateDisplay							*/
/*		: Owning display geometry has changed			*/
/*  (returns)	: void		:					*/

void	KBObject::updateDisplay ()
{
	if (m_control)
		m_control->setGeometry (geometry()) ;
}

/*  KBObject								*/
/*  resizeEvent	: Object has been resized in parent			*/
/*  (returns)	: void		:					*/

void	KBObject::resizeEvent ()
{
	/* Null operation. Derived classes override this method.	*/
}

/*  KBObject								*/
/*  propertyDlg	: Default property dialog				*/
/*  (returns)	: void		:					*/

bool	KBObject::propertyDlg ()
{
	return	propertyDlg (0) ;
}

/*  KBObject								*/
/*  propertyDlg	: Default property dialog				*/
/*  iniAttr	: cchar *	: Initial attribute to display		*/
/*  (returns)	: void		:					*/

bool	KBObject::propertyDlg
	(	cchar	*iniAttr
	)
{
#if	! __KB_RUNTIME
	if (KBNode::propertyDlg (TR("Properties"), iniAttr))
	{
		getLayout()->addSizer (getSizer(), false) ;
		return	true	;
	}
#endif
	return	false	;
}

/*  KBObject								*/
/*  deleteObj	: Delete the object					*/
/*  (returns)	: void		:					*/

void	KBObject::deleteObj ()
{
#if	! __KB_RUNTIME
	if (getParent() == 0)
	{
		TKMessageBox::sorry
		(	0,
			QString(TR("Cannot delete the top-most object")),
			TR("Error deleting object")
		)	;
		return	;
	}

	getLayout()->setChanged  () ;
	KBDeleter::addObject (this) ;
#endif
}

/*  KBObject								*/
/*  saveAsComponent							*/
/*		: Save object as component				*/
/*  (returns)	: void		:					*/

void	KBObject::saveAsComponent ()
{
#if	! __KB_RUNTIME

	const KBLocation  &locn1   = getRoot()->getDocRoot()->getLocation() ;
	QString		  svName   = locn1.server() ;
	QString		  objName  ;
	QString		  comment  ;

	/* Run the component save-as dialog, which will return the	*/
	/* desired object name, server, and optionally a comment.	*/
	bool		   toFile ;
	KBComponentSaveDlg csDlg
			   (	objName,
				svName,
				comment,
				locn1.dbInfo(),
				&toFile
			   )	;

	if (!csDlg.exec()) return ;


	/* We make a copy of the object, then adjust its position so	*/
	/* that the top-left one goes to (20,20). Note that this is	*/
	/* done to function correctly for stretching/floating objects.	*/
	KBObject *copy	= (KBObject *)replicate(0) ;
	QRect	 r	= copy->geometry() ;
	copy->setGeometry (QRect(QPoint(20, 20), r.size())) ;

	/* OK, got a name and stuff, so stitch together the text with	*/
	/* the top level component settings ....			*/
	extern	QString	kbXMLEncoding   () ;

	QString	 text	= QString
			  (	"<?xml version=\"1.0\" encoding=\"%1\"?>\n"
			  	"<KBComponent w=\"%2\" h=\"%3\" type=\"%4\" notes=\"%5\">\n" 
			  )
			  .arg	(kbXMLEncoding())
			  .arg	(r.width () + 40)
			  .arg	(r.height() + 40)
			  .arg	((uint)objType())
			  .arg	(comment) ;


	copy->printNode (text, 2, false)  ;
	text += "</KBComponent>\n" ;

	fprintf	  (stderr, "Component:[[[\n%s\n]]]\n", (cchar *)text) ;

	/*  .... and save the component where requested. The special	*/
	/* case is if the used specified save-to-file.			*/
	if (toFile)
	{
		saveComponentToFile (objName, text) ;
		return	;
	}

	KBLocation	locn2
			(	locn1.dbInfo(),
				"component",
				svName,
				objName
			)	;
	KBError		error	;

	if (!locn2.save (QString::null, QString::null, text, error))
		error.DISPLAY() ;
#endif
}

/*  KBObject								*/
/*  copyObj	: Copy the object					*/
/*  (returns)	: void		:					*/

void	KBObject::copyObj ()
{
#if	! __KB_RUNTIME
	KBFormCopier::self()->clearCopy () ;
	KBFormCopier::self()->addToCopy (replicate(0), objType()) ;
#endif
}

/*  KBObject								*/
/*  cutObj	: Cut the object					*/
/*  (returns)	: void		:					*/

void	KBObject::cutObj ()
{
#if	! __KB_RUNTIME
	if (getParent() == 0)
	{
		TKMessageBox::sorry
		(	0,
			QString(TR("Cannot cut the top-most object")),
			TR("Error cutting object")
		)	;
		return	;
	}

	copyObj   () ;
	deleteObj () ;

	getLayout ()->setChanged () ;
#endif
}


/*  KBObject								*/
/*  pasteObjects: Paste in cut or copied objects			*/
/*  (returns)	: void		:					*/

void	KBObject::pasteObjects ()
{
	/* Placeholder only. Overridden in KBBlock, KBFramer and	*/
	/* KBComponent.							*/
}

/*  KBObject								*/
/*  calcNumRows	: Calculate number of rows of controls			*/
/*  bw		: uint		: Block display width			*/
/*  bh		: uint		: Block display height			*/
/*  dx		: int		: Control X-spacing			*/
/*  dy		: int		: Control X-spacing			*/
/*  (returns)	: uint		: Number of rows that can be displayed	*/

uint	KBObject::calcNumRows
	(	uint	bw,
		uint	bh,
		int	dx,
		int	dy
	)
{
	if (getRoot()->isReport() != 0) return 1 ;

//	fprintf	(stderr, "calcNumRows: dy=%2d bh=%3d y=%3d\n",
//			 dy,
//			 bh,
//			 y.getIntValue()) ;

	QRect	g	= geometry() ;

	/* Calculate dynamically how many rows can be displayed (but	*/
	/* always display at least one whatever).			*/
	int	xmax	= dx <= 0 ? 0 : (bw - g.width () - m_geom.x()) / dx + 1 ;
	int	ymax	= dy <= 0 ? 0 : (bh - g.height() - m_geom.y()) / dy + 1 ;

	int	numrows	= xmax == 0    ? ymax :
			  ymax == 0    ? xmax :
			  xmax >  ymax ? ymax :
					 xmax ;

//	fprintf	(stderr, "KBObject::calcNumRows => %d\n", numrows) ;
	return	numrows	<= 0 ? 1 : numrows    ;
}

/*  KBObject								*/
/*  calcNumRows	: Calculate number of rows of controls			*/
/*  dx		: int		: Control X-spacing			*/
/*  dy		: int		: Control X-spacing			*/
/*  mRows	: uint		: Maximum number of rows		*/
/*  (returns)	: uint		: Number of rows that can be displayed	*/

uint	KBObject::calcNumRows
	(	int	dx,
		int	dy,
		uint	mRows
	)
{
	uint	nRows	= calcNumRows (m_display->width(), m_display->height(), dx, dy) ;
	return	nRows > mRows ? mRows : nRows ;
}

/*  KBObject								*/
/*  buildDisplay: Build display for this item				*/
/*  parent	: KBDisplay *	: Parent display			*/
/*  (returns)	: void		:					*/

void	KBObject::buildDisplay
	(	KBDisplay	*parent
	)
{
	m_display = parent  ;
}

/*  KBObject								*/
/*  setGeometry	: Set object geometry					*/
/*  rect	: const QRect &	: New geometry				*/
/*  (returns)	: void		:					*/

void	KBObject::setGeometry
	(	const QRect	&rect
	)
{
	m_geom.setGeometry (rect) ;
	updateDisplay	   ()	  ;
}

/*  KBObject								*/
/*  getPalette	: Get palette for this object				*/
/*  useParent	: bool		: Use parent's palette			*/
/*  (returns)	: QPalette *	: The palette				*/

QPalette *KBObject::getPalette
	 (	bool	useParent
	 )
{
	static	QPalette *appPalette	= 0 ;
	static	QPalette *whitePalette	= 0 ;

	if (appPalette == 0)
	{
		appPalette	= new QPalette (QApplication::palette()) ;
		whitePalette	= new QPalette (QApplication::palette()) ;

		whitePalette->setColor (QColorGroup::Background, Qt::white) ;
	}

	if (m_curPal == 0)
	{

		QString	 fgcolor = getAttrVal ("fgcolor") ;
		QString	 bgcolor = getAttrVal ("bgcolor") ;

		QPalette pal	 ;

		if	((getDisplay () != 0) && useParent)
			pal	= getDisplay()->getDisplayWidget()->palette() ;
		else if (getRoot()->isReport())
			pal	= *whitePalette ;
		else	pal	= *appPalette   ;

		if (!fgcolor.isEmpty())
		{	const QColor fg = QColor (QRgb(fgcolor.toInt(0,0))) ;
			pal.setColor (QColorGroup::Text,       fg) ;
			pal.setColor (QColorGroup::ButtonText, fg) ;
			pal.setColor (QColorGroup::Foreground, fg) ;
		}
		if (!bgcolor.isEmpty())
		{	const QColor bg = QColor (QRgb(bgcolor.toInt(0,0))) ;
			pal.setColor (QColorGroup::Base,       bg) ;
			pal.setColor (QColorGroup::Button,     bg) ;
			pal.setColor (QColorGroup::Background, bg) ;
		}

		m_curPal = new QPalette (pal) ;
	}

	return	m_curPal ;
}

/*  KBObject								*/
/*  getFont	: Get font for this object				*/
/*  useParent	: bool		: Use parent's font			*/
/*  (returns)	: QFont *	: The font				*/

QFont	*KBObject::getFont
	(	bool	useParent
	)
{
	if (m_curFont == 0)
	{
		QString	spec	= getAttrVal ("font") ;

		if (!spec.isEmpty())
			return	m_curFont = new QFont(KBFont::specToFont (spec)) ;
		if ((getDisplay () != 0) && useParent)
			return	m_curFont = new QFont(getDisplay()->getDisplayWidget()->font()) ;

		const QString &font = getDocRoot()->appFont() ;
		if (!font.isEmpty())
			return	m_curFont = new QFont(KBFont::specToFont (font)) ;

		return	m_curFont = new QFont(QApplication::font()) ;
	}

	return	m_curFont ;
}

/*  KBObject								*/
/*  setControl	: Set the control to be used for sizing			*/
/*  control	: KBControl *	: Control in question			*/
/*  (returns)	: void		:					*/

void	KBObject::setControl
	(	KBControl *control
	)
{
	if (control == m_control)
		return	;

	m_control = control ;

#if	! __KB_RUNTIME
	DELOBJ (m_sizer) ;
	if (!showingDesign() || (m_control == 0))
		return	;

	createSizer ()	;
#endif
}

/*  KBObject								*/
/*  getDisplayWidget							*/
/*		: Get widget used for display				*/
/*  (returns)	: QWidget *	: Display widget			*/

QWidget	*KBObject::getDisplayWidget ()
{
	return	m_display == 0 ? 0 : m_display->getDisplayWidget () ;
}

/*  KBObject								*/
/*  getTopWidget: Get top-most widget					*/
/*  (returns)	: QWidget *	: Top-most widget			*/

QWidget	*KBObject::getTopWidget ()
{
	return	m_display == 0 ? 0 : m_display->getTopWidget () ;
}

/*  KBObject								*/
/*  keyStroke	: Handle keystroke					*/
/*  e		: QKeyEvent *	: Event to be handled			*/
/*  (returns)	: bool		: True if event consumed		*/

bool	KBObject::keyStroke
	(	QKeyEvent	*
	)
{
	return	false	;
}

/*  KBObject								*/
/*  getBlock	: Get enclosing block for object			*/
/*  (returns)	: KBBlock *	: Block or null if none			*/

KBBlock	*KBObject::getBlock ()
{
	for (KBNode *p = getParent () ; p != 0 ; p = p->getParent())
		if (p->isBlock() != 0)
			return	p->isBlock() ;
	return	0 ;
}

/*  KBObject								*/
/*  getFormBlock: Get form block containing this object			*/
/*  (returns)	: KBFormBlock *	: Form block or null if none		*/

KBFormBlock *KBObject::getFormBlock ()
{
	for (KBNode *p = getParent () ; p != 0 ; p = p->getParent())
		if (p->isFormBlock() != 0)
			return	p->isFormBlock() ;
	return	0 ;
}

/*  KBObject								*/
/*  getReportBlock: Get report block containing this object		*/
/*  (returns)	  : KBReportBlock *	: Report block or null if none	*/

KBReportBlock *KBObject::getReportBlock ()
{
	for (KBNode *p = getParent () ; p != 0 ; p = p->getParent())
		if (p->isReportBlock() != 0)
			return	p->isReportBlock() ;
	return	0 ;
}

/*  KBObject								*/
/*  getNavigator: Get navigator containing this object			*/
/*  (returns)	: KBNavigator *	: Navigator or null if none		*/

KBNavigator *KBObject::getNavigator ()
{
	for (KBNode *p = getParent () ; p != 0 ; p = p->getParent())
		if (p->isNavigator() != 0)
			return	p->isNavigator() ;
	return	0 ;
}

/*  KBObject								*/
/*  write	: Write object to report writer				*/
/*  writer	: KBWriter *	: Writer object				*/
/*  offset	: QPoint	: Geometry offset			*/
/*  first	: bool		: First on page				*/
/*  extra	: init &	: Return extra space			*/
/*  prior	: bool		: Write prior value flag		*/
/*  (returns)	: void		:					*/

bool	KBObject::write
	(	KBWriter	*writer,
		QPoint		offset,
		bool		,
		int		&extra,
		bool		prior
	)
{
	/* The work is all done at the control level.			*/
	return	m_control->write
		(	writer,
			geometry(offset),
			KBValue(),
			false,
			extra
		)	;
}

/*  KBObject								*/
/*  setEnabled	: Set control enabled/disabled				*/
/*  enabled	: bool		: Enabled/disabled			*/
/*  (returns)	: void		:					*/

void	KBObject::setEnabled
	(	bool	enabled
	)
{
	if (m_control != 0) m_control->setEnabled (enabled) ;
}

/*  KBObject								*/
/*  setVisible	: Set control visibility				*/
/*  visible	: bool		: Visibility				*/
/*  (returns)	: void		:					*/

void	KBObject::setVisible
	(	bool	visible
	)
{
	if (m_control != 0) m_control->setVisible (visible) ;
}

/*  KBObject								*/
/*  isEnabled	: Get control enabled/disabled				*/
/*  (returns)	: bool		: Enabled/disabled			*/

bool	KBObject::isEnabled ()
{
	return	m_control == 0 ? false : m_control->isEnabled () ;
}

/*  KBObject								*/
/*  isVisible	: Get control visibility				*/
/*  (returns)	: bool		: Visibility				*/

bool	KBObject::isVisible ()
{
	return	m_control == 0 ? false : m_control->isVisible () ;
}

/*  KBObject								*/
/*  getMoveLimit: Get control movement limits				*/
/*  (returns)	: KBLimit	: Limit information			*/

KBLimit	KBObject::getMoveLimit ()
{
	if (getParent() == 0) return KBLimit () ;

	QSize	pSize	= getDisplay()->getTopSize() ;
	QRect	mRect	= geometry  () ;
 
	return	KBLimit
		(	-mRect.left(), pSize.width () - mRect.right (),
			-mRect.top (), pSize.height() - mRect.bottom()
		)	;
}

/*  KBObject								*/
/*  eventHook	 : Event execution hook					*/
/*  event	 : KBEvent &	: Event to execute			*/
/*  argc	 : uint		: Argument count			*/
/*  argv	 : KBValue *	: Argument vector			*/
/*  evRc	 : bool &	: Event return code			*/
/*  defres	 : bool	 	: Default return code			*/
/*  (returns)	 : bool		: No execution error			*/

bool	KBObject::eventHook
	(	KBEvent		&event,
		uint		argc,
		KBValue		*argv,
		bool		&evRc,
		bool		defval
	)
{
	KBValue		resval	;
	KBScriptError	*rc	;

#if	__KB_DEBUG_EVENT
	fprintf
	(	stderr,
		"KBObject::eventHook: [%s][%s][%s] executing event ....\n",
		(cchar *)getElement   (),
		(cchar *)getName      (),
		(cchar *)event.getName()
	)	;
#endif
	rc = event.execute (resval, argc, argv, defval) ;
	if (rc != 0)
	{
		KBScriptError::processError (rc, KBScriptError::Normal) ;
		return	false	;
	}

	/* No execution errors, so result is true with "evRc" set to	*/
	/* the value returned by the script.				*/
	evRc	= resval.isTrue() ;
	return	true	;
}

/*  KBObject								*/
/*  property	: Get QT widget property				*/
/*  name	: cchar *	: Property name				*/
/*  (returns)	: QVariant	: Property value			*/

QVariant
	KBObject::property
	(	cchar		*name
	)
{
	return	m_control == 0 ? QVariant() : m_control->property (name) ;
}

/*  KBObject								*/
/*  setProperty	: Set QT widget property				*/
/*  name	: cchar *	   : Property name			*/
/*  value	: const QVariant & : Value				*/
/*  (returns)	: bool		   : Success				*/

bool	KBObject::setProperty
	(	cchar		*name,
		const QVariant	&value
	)
{
	return	m_control == 0 ? false : m_control->setProperty (name, value) ;
}

/*  KBObject								*/
/*  getNamedObject							*/
/*		: Get named object relative to this one			*/
/*  name	: cchar *	: Name of object			*/
/*  showErr	: bool		: Show error dialog on error		*/
/*  abort	: bool *	: Abort on return flag			*/
/*  (returns)	: KBObject *	: Object or null if not found		*/

KBObject*KBObject::getNamedObject
	(	QString		string,
		bool		showErr,
		bool		*abort
	)
{
	QString	name	= string ;
	QString	rest	;
	int	idx	;

	/* Look for a separator. If this is the very first character	*/
	/* then work up to the root of the document, unless we are the	*/
	/* root.							*/
	if ((idx = name.find ('/')) == 0)
	{
		if (parentObject() != 0)
			return	parentObject()->getNamedObject (name, showErr, abort) ;

		name	= name.mid  (1)   ;
		idx	= name.find ('/') ;
	}

	/* If there is now a separator then split the string on it, so	*/
	/* that "name" is left with just the name at this level, and	*/
	/* "rest" contains everything after the separator.		*/
	if (idx > 0)
	{
		rest	= name.mid  (idx + 1)	;
		name	= name.left (idx)	;
	}

//	fprintf
//	(	stderr,
//		"KBObject::getNamedObject: [%s][%d][%s]\n",
//		(cchar *)name,
//		idx,
//		(cchar *)rest
//	)	;


	/* Scan for the named object. There are a number of special	*/
	/* cases to be accounted for ...				*/
	/*	.		: Self   (as in Unix path names)	*/
	/*	..		: Parent (as in Unix path names)	*/
	/*	getRoot ()	: Document root				*/
	/*	getBlock()	: Enclosing block			*/
	/* NOTE:							*/
	/* The latter are not method applications, though the format	*/
	/* leaves scope for this later.					*/
	KBObject *object = 0 ;

	if	(name == "." )
	{
		object	= this	;
	}
	else if (name == "..")
	{
		object	= parentObject() ;
	}
	else if (name == "getRoot()" )
	{
		object	= getRoot  ()->isObject() ;
	}
	else if (name == "getBlock()")
	{
		object	= getBlock () ;
	}
	else
	{
		CITER
		(	Object,
			o,
			if (o->getName() == name)
			{	object	= o ;
				break	;
			}
		)
	}

	/* If have found an object and there are more components in the	*/
	/* path then work on down through them.				*/
	if (object != 0)
		if (!rest.isNull())
			object	= object->getNamedObject (rest) ;


	/* If we have not found an object and the show-error option is	*/
	/* set then show the no-object error dialog. If the user OK's	*/
	/* this then continue with the selected object, otherwise	*/
	/* really return not-found.					*/
#if	! __KB_RUNTIME
	if ((object == 0) && showErr)
	{
		fprintf
		(	stderr,
			"KBObject::getNamedObject: failed: abort=%p\n",
			(void *)abort
		)	;

		KBNoObjDlg noObjDlg (this, string, abort) ;

		if (noObjDlg.exec ())
			object = noObjDlg.selectedObj() ;
	}
#endif

	return	object	;
}

/*  KBObject								*/
/*  getNamedObject							*/
/*		: Get named object relative to this one			*/
/*  name	: cchar *	: Name of object			*/
/*  pError	: KBError &	: Error message return			*/
/*  showErr	: bool		: Show error dialog on error		*/
/*  abort	: bool *	: Abort on return flag			*/
/*  (returns)	: KBObject *	: Object or null if not found		*/

KBObject*KBObject::getNamedObject
	(	QString		string,
		KBError		&pError,
		bool		showErr,
		bool		*abort
	)
{
	KBObject *object = getNamedObject (string, showErr, abort) ;

	if (object == 0)
		pError	= KBError
			  (	KBError::Error,
				"Cannot find named object",
				string,
				__ERRLOCN
			  )	;

	return	object	;
}

/*  KBObject								*/
/*  connectLinks: Connect event/slot links				*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBObject::connectLinks
	(	KBError		&pError
	)
{
//	fprintf
//	(	stderr,
//		"KBObject::connectLinks: [%s][%s]\n",
//		(cchar *)getElement(),
//		(cchar *)getName   ()
//	)	;
	
	LITER
	(	KBSlot,
		m_slotList,
		slot,
		if (!slot->connectLinks (pError)) return false ;
	)
	CITER
	(	Object,
		obj,
		if (!obj ->connectLinks (pError)) return false ;
	)

	return	true	;
}			

/*  KBObject								*/
/*  getEmitter	: Get signal emitter object for named event attribute	*/
/*  name	: const QString & : Event name				*/
/*  (returns)	: KBEmitter *	  : Emitter or null if none		*/

KBEmitter
	*KBObject::getEmitter
	(	const QString	&name
	)
{
//	fprintf
//	(	stderr,
//		"KBObject::getEmitter: called: [%s][%s][%s]\n",
//		(cchar *)getElement(),
//		(cchar *)getName   (),
//		(cchar *)name
//	)	;

	KBAttr	*attr	= getAttr (name) ;
	if (attr  == 0) return 0 ;

	KBEvent	*event	= attr->isEvent() ;
	if (event == 0) return 0 ;

//	fprintf
//	(	stderr,
//		"KBObject::getEmitter: got event\n"
//	)	;

	return	event->getEmitter () ;
}

/*  KBObject								*/
/*  addSlot	: Add a slot to this node				*/
/*  slot	: KBSlot *	: The slot in question			*/
/*  (returns)	: void		:					*/

void	KBObject::addSlot
	(	KBSlot		*slot
	)
{
	m_slotList.append (slot) ;

//	fprintf
//	(	stderr,
//		"Node [%s] adds slot [%s]\n",
//		(cchar *)getAttrVal("name"),
//		(cchar *)slot->name()
//	)	;
}

/*  KBObject								*/
/*  getSlots	: Get list of slots					*/
/*  (returns)	: const QPtrList<KBSlot> & :				*/

const QPtrList<KBSlot> &KBObject::getSlots ()
{
	return	m_slotList ;
}

/*  KBObject								*/
/*  clearSlots	: Clear current slot list				*/
/*  (returns)	: void		:					*/

void	KBObject::clearSlots ()
{
	m_slotList.clear () ;
}


void	KBObject::setTabOrder
	(	int
	)
{
}

int	KBObject::getTabOrder ()
{
	return	-1 ;
}


/*  KBObject								*/
/*  setScriptObject							*/
/*		: Set persistent script object				*/
/*  code	: uint			: Language code			*/
/*  scriptObj	: KBScriptObject *	: Persistent script object	*/
/*  (returns)	: void			:				*/

void	KBObject::setScriptObject
	(	uint		code,
		KBScriptObject	*scriptObj
	)
{
	/* Script objects are stored against the language code so that	*/
	/* we handle the case where we have multiple languages that	*/
	/* access the same object at the same time. Needed, say, when	*/
	/* running a mixed Python/KJS form.				*/
	if (m_scriptObj == 0)
	{
		m_scriptObj = new KBScriptObject * [KBScriptIF::languageCount()] ;
		memset	(m_scriptObj, 0, KBScriptIF::languageCount() * sizeof(KBScriptObject *)) ;
	}

	m_scriptObj[code] = scriptObj ;
}

/*  KBObject								*/
/*  scriptObject: Get persisistent script object			*/
/*  code	: uint			: Language code			*/
/*  (returns)	: KBScriptObject *	: Persistent script object	*/

KBScriptObject
	*KBObject::scriptObject
	(	uint		code
	)
{
	/* We assume that the language code is valid ....		*/
	return	m_scriptObj == 0 ? 0 : m_scriptObj[code] ;
}



/*  KBObject								*/
/*  parentObject: Return parent if parent is an object			*/
/*  (returns)	: KBObject *	: Parent if object			*/

KBObject*KBObject::parentObject()
{
	if (getParent() == 0) return false ;
	return	getParent()->isObject()	   ;
}
	
/*  KBObject								*/
/*  parentIsDynamic							*/
/*		: See if object is embedded in a dynamic layout parent	*/
/*  (returns)	: bool		: True if so				*/

bool	KBObject::parentIsDynamic ()
{
	if (parentObject() == 0) return false ;
	return	parentObject()->isDynamic()   ;
}

/*  KBObject								*/
/*  minPosition	: Get minimal x- and y- positions from a list of nodes	*/
/*  nodes	: const QPtrList<KBNode> &	: List of notes			*/
/*  minx	: int &			: Return minimum x-position	*/
/*  miny	: int &			: Return minimum y-position	*/
/*  (returns)	: void			:				*/

void	KBObject::minPosition
	(	const QPtrList<KBNode>	&nodes,
		int			&minx,
		int			&miny
	)
{
	KBObject *oObj	      ;
	minx	 = 0x7fffffff ;
	miny	 = 0x7fffffff ;

	LITER
	(	KBNode,
		nodes,
		oNode,

		if ((oObj = oNode->isObject()) == 0)
			continue ;

		if (oObj->isHidden() != 0)
			continue ;

		QRect	r = oObj->geometry() ;
		int 	x = r.x() ;
		int	y = r.y() ;

		if (x < minx) minx = x ;
		if (y < miny) miny = y ;
	)
}

/*  KBObject								*/
/*  newNode	: Add a new node					*/
/*  id		: int		: Actually NodeSpec pointer for node	*/
/*  (returns)	: KBNode *	: The new node				*/

KBNode	*KBObject::newNode
	(	int		
	)
{
	/* Stub implementation. Overridden in classes that need to	*/
	/* implement it.						*/
	return	0	;
}

/*  KBObject								*/
/*  newContainer: Virtual place holder					*/
/*  (returns)	:		:					*/

void	KBObject::newContainer ()
{
}

/*  KBObject								*/
/*  newTableBlock							*/
/*		: Virtual place holder					*/
/*  (returns)	:		:					*/

void	KBObject::newTableBlock ()
{
}

/*  KBObject								*/
/*  newQueryBlock							*/
/*		: Virtual place holder					*/
/*  (returns)	:		:					*/

void	KBObject::newQueryBlock ()
{
}

/*  KBObject								*/
/*  newSQLBlock	: Virtual place holder					*/
/*  (returns)	:		:					*/

void	KBObject::newSQLBlock ()
{
}

/*  KBObject								*/
/*  newNullBlock: Virtual place holder					*/
/*  (returns)	:		:					*/

void	KBObject::newNullBlock ()
{
}

/*  KBObject								*/
/*  pasteComponent							*/
/*		: Virtual place holder					*/
/*  (returns)	:		:					*/

void	KBObject::pasteComponent ()
{
}

/*  KBObject								*/
/*  linkComponent							*/
/*		: Virtual place holder					*/
/*  (returns)	:		:					*/

void	KBObject::linkComponent ()
{
}

/*  ------------------------------------------------------------------	*/

/*  KBObject								*/
/*  setCtrlFGColor							*/
/*		: Set control foreground colour				*/
/*  color	: const QString & : Color				*/
/*  (returns)	: void		  :					*/

void	KBObject::setCtrlFGColor
	(	const QString	&color
	)
{
	/* This call is used to allow scripts to conveniently change	*/
	/* the colour of non-data controls like buttons.		*/
	if (m_control != 0)
	{
		QPalette pal = m_control->mainWidget()->palette () ;
		pal.setColor (QColorGroup::Text,       color) ;
		pal.setColor (QColorGroup::ButtonText, color) ;
		pal.setColor (QColorGroup::Foreground, color) ;
		m_control->mainWidget()->setPalette (pal) ;
	}
}

/*  KBObject								*/
/*  setCtrlBGColor							*/
/*		: Set control background colour				*/
/*  color	: const QString & : Color				*/
/*  (returns)	: void		  :					*/

void	KBObject::setCtrlBGColor
	(	const QString	&color
	)
{
	if (m_control != 0)
	{
		QPalette pal = m_control->mainWidget()->palette () ;
		pal.setColor (QColorGroup::Base,       color) ;
		pal.setColor (QColorGroup::Button,     color) ;
		pal.setColor (QColorGroup::Background, color) ;
		m_control->mainWidget()->setPalette (pal) ;
	}
}

/*  KBObject								*/
/*  setCtrlFont	: Set control font					*/
/*  font	: const QString & : Font				*/
/*  (returns)	: void		  :					*/

void	KBObject::setCtrlFont
	(	const QString	&spec
	)
{
	/* .... similarly to the above for fonts			*/
	if (m_control != 0)
	{
		QFont	font	;

		if	(spec.isEmpty())
			font	= QApplication::font() ;
		else	font	= KBFont::specToFont (spec) ;

		m_control->mainWidget()->setFont (font) ;
	}
}

/*  ------------------------------------------------------------------	*/

/*  KBObject								*/
/*  enumKBProperty							*/
/*		: Get list of properties				*/
/*  list	: QStringList &	: Property list				*/
/*  (returns)	: void		:					*/

void	KBObject::enumKBProperty
	(	QStringList	&list
	)
{
	list.append ("visible"	 ) ;
	list.append ("enabled"	 ) ;
	list.append ("__parent__") ;
	list.append ("__block__" ) ;
	list.append ("__root__"  ) ;

	KBNode::enumKBProperty (list) ;

	LITER
	(	KBNode,
		getChildren(),
		node,
		list.append (node->getName()) ;
	)
}

/*  KBObject								*/
/*  getKBProperty: Get a rekall property value				*/
/*  value	 : KBValue &	: Return property value			*/
/*  (returns)	 : bool		: Property exists			*/

bool	KBObject::getKBProperty
	(	cchar		*name,
		KBValue		&value
	)
{
//	fprintf
//	(	stderr,
//		"KBObject::getKBProperty: for [%s]\n",
//		name
//	)	;

	if (qstrcmp (name, "visible") == 0)
	{
		value	= isVisible() ;
		return	true ;
	}

	if (qstrcmp (name, "enabled") == 0)
	{
		value	= isEnabled() ;
		return	true ;
	}

	/* The names "__parent__", "__block__" and "__root__" map to	*/
	/* the obvious ancestral objects. We uses __ as a prefixsuffix	*/
	/* to try to avoid clashes with names that people might		*/
	/* plausibly have used. It also distinguishes them from "real"	*/
	/* properties and likely child names (see below).		*/
	if (qstrcmp (name, "__parent__") == 0)
	{
		value	= KBValue(getParent()) ;
		return	true ;
	}

	if (qstrcmp (name, "__block__" ) == 0)
	{
		value	= KBValue(getBlock ()) ;
		return	true ;
	}

	if (qstrcmp (name, "__root__"  ) == 0)
	{
		value	= KBValue(getRoot  ()) ;
		return	true ;
	}

	/* Recurse down to the base class. Note that we do not fail	*/
	/* immediatley if this fails ...				*/
	if (KBNode::getKBProperty (name, value))
		return	true	;


	/* No match so far so see if the name identifies a child	*/
	/* object. This means that the user (as of Rekall 2.3.3) can	*/
	/* access other objects in the tree without needing to call	*/
	/* the "getNamedCtrl()" method.					*/
	LITER
	(	KBNode,
		getChildren(),
		node,

//		fprintf
//		(	stderr,
//			"KBObject::getKBProperty: ......[%s]\n",
//			(cchar *)node->getName()
//		)	;

		if (node->getName() == name)
		{
			value	= KBValue (node) ;
			return	true	;
		}
	)

	/* Bollox ...							*/
	return	false	;
}

/*  KBObject								*/
/*  setKBProperty: Set a rekall property value				*/
/*  name	 : cchar *	: Proprty name				*/
/*  value	 : KBValue	: Property value			*/
/*  (returns)	 : bool		: Success				*/

bool	KBObject::setKBProperty
	(	cchar		*name,
		const KBValue	&value
	)
{
	if (qstrcmp (name, "visible") == 0)
	{
		setVisible (value.isTrue()) ;
		return	true ;

	}

	if (qstrcmp (name, "enabled") == 0)
	{
		setEnabled (value.isTrue()) ;
		return	true ;

	}

	return	KBNode::setKBProperty (name, value) ;
}

/*  KBObject								*/
/*  insertDynamicRow							*/
/*		: Insert row into dynamic layout			*/
/*  (returns)	: void		:					*/

void	KBObject::insertDynamicRow ()
{
#if	! __KB_RUNTIME
	if (isDynamic())
	{
		int	from	= newCtrlRect(false).y() ;

		m_geom.insertRow (from) ;
		getContainer()->updateDynamic() ;

		CITER
		(	Object,
			o,

			QRect	rect	= o->geometry() ;
			if (rect.y() >= from)
			{
				rect.moveBy    (0, 1)	;
				o->setGeometry (rect)	;
			}
		)

		getLayout()->setChanged() ;
	}
#endif
}

/*  KBObject								*/
/*  deleteDynamicRow							*/
/*		: Delete row from dynamic layout			*/
/*  (returns)	: void		:					*/

void	KBObject::deleteDynamicRow ()
{
#if	! __KB_RUNTIME
	if (isDynamic())
	{
		uint	numCols	= m_geom.numCols(true)	 ;
		int	from	= newCtrlRect(false).y() ;

		if (checkOverlap (QRect (0, from, numCols, 1)))
		{
			TKMessageBox::sorry
			(	0,
				QString(TR("Row contains objects: please remove first")),
				TR("Cannot delete row")
			)	;
			return	;
		}

		CITER
		(	Object,
			o,

			QRect	rect	= o->geometry() ;
			if (rect.y() > from)
			{
				rect.moveBy    (0, -1)	;
				o->setGeometry (rect )	;
			}
		)

		m_geom.removeRow (from) ;
		getContainer()->updateDynamic() ;
		getLayout   ()->setChanged () ;
	}
#endif
}

/*  KBObject								*/
/*  insertDynamicColumn							*/
/*		: Insert column into dynamic layout			*/
/*  (returns)	: void		:					*/

void	KBObject::insertDynamicColumn ()
{
#if	! __KB_RUNTIME
	if (isDynamic())
	{
		int	from	= newCtrlRect(false).x() ;

		m_geom.insertCol (from) ;
		getContainer()->updateDynamic() ;

		CITER
		(	Object,
			o,

			QRect	rect	= o->geometry() ;
			if (rect.x() >= from)
			{
				rect.moveBy    (1, 0)	;
				o->setGeometry (rect)	;
			}
		)

		getLayout()->setChanged() ;
	}
#endif
}

/*  KBObject								*/
/*  deleteDynamicColumn							*/
/*		: Delete column from dynamic layout			*/
/*  (returns)	: void		:					*/

void	KBObject::deleteDynamicColumn ()
{
#if	! __KB_RUNTIME
	if (isDynamic())
	{
		uint	numRows	= m_geom.numRows(true)	 ;
		int	from	= newCtrlRect(false).x() ;

		if (checkOverlap (QRect (from, 0, 1, numRows)))
		{
			TKMessageBox::sorry
			(	0,
				QString(TR("Column contains objects: please remove first")),
				TR("Cannot delete column")
			)	;
			return	;
		}

		CITER
		(	Object,
			o,

			QRect	rect	= o->geometry() ;
			if (rect.x() > from)
			{
				rect.moveBy    (-1, 0)	;
				o->setGeometry (rect )	;
			}
		)

		m_geom.removeCol (from) ;
		getContainer()->updateDynamic() ;
		getLayout   ()->setChanged () ;
	}
#endif
}

/*  KBObject								*/
/*  gridSetup	: Run the grid setup dialog				*/
/*  (returns)	: void		:					*/

void	KBObject::gridSetup ()
{
#if	! __KB_RUNTIME
	/* If the user cancels the dialog then call the container's	*/	
	/* "setRowColSetup" method to restore the original settings.	*/
	if (!KBRowColDialog (m_geom, this, newCtrlRect(false).y(), newCtrlRect(false).x()).exec())
	{
		getContainer()->setRowColSetup
		(	m_geom.rowSetup(),
			m_geom.colSetup()
		)	;
		return	;
	}

	setChanged() ;
#endif
}

#if	! __KB_RUNTIME

/*  KBBlock								*/
/*  whatsThisExtra							*/
/*		: Extra stuff for pseudo-what's this popup		*/
/*  info	: QValueList<KBWhatsThisPair> &	: List of items		*/
/*  (returns)	: void				:			*/

void	KBObject::whatsThisExtra
	(	QValueList<KBWhatsThisPair>	&
	)
{
}

#endif

/*  KBBlock								*/
/*  whatsThis	: Display pseudo-what;s this popup			*/
/*  (returns)	: void		:					*/

void	KBObject::whatsThis ()
{
#if	! __KB_RUNTIME
	QPoint	showAt	= QCursor::pos() ;
	if (m_control != 0)
		showAt	= m_control->topWidget()->mapToGlobal(QPoint(0,0)) ;

	QValueList<KBWhatsThisPair> info ;
	info.append	(KBWhatsThisPair(TR("Element"), getElement().mid(2))) ;
	info.append	(KBWhatsThisPair(TR("Name"),    getName(),     true)) ;
	whatsThisExtra	(info)	;
	info.append	(KBWhatsThisPair(TR("Size Hint"), _TEXT(sizeHint()))) ;

	QString	text	;

	text	+= "<font size='-1'><table>" ;

	for
	(	QValueList<KBWhatsThisPair>::iterator it = info.begin() ;
		it != info.end() ;
		++it
	)
		if (!(*it).m_info.isEmpty())
			text += QString	("<tr><td><nobr><b>%1</b></nobr></td><td width='32'/><td><nobr>%2</nobr></td><td width='32'/></tr>")
					.arg((*it).m_legend)
					.arg((*it).m_info  )
				;
	text	+= "</table></font>" ;

	QWhatsThis::display (text, showAt) ;
#endif
}

