/***************************************************************************
    file	         : kb_qrysql.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	"kb_item.h"
#include	"kb_database.h"
#include	"kb_queryset.h"
#include	"kb_qrylevel.h"
#include	"kb_qrysql.h"
#include	"kb_table.h"
#include	"kb_nodereg.h"



/*  KBQrySQL								*/
/*  KBQrySQL	: Constructor for free-text SQL query node		*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*		: bool *	:					*/
/*  (returns)	: KBQrySQL	:					*/

KBQrySQL::KBQrySQL
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*
	)
	:
	KBQryData	(parent, aList,	     "KBQrySQL"),
	m_server	(this,	 "server",   aList, KAF_REQD),
	m_query		(this,   "query",    aList, KAF_REQD),
	m_topTable	(this,   "toptable", aList)
{
	m_qryTabPtr	= 0 ;
}

/*  KBQrySQL								*/
/*  KBQrySQL	: Constructor for free-text SQL query node		*/
/*  parent	: KBForm *	: Parent node				*/
/*  (returns)	: KBQrySQL	:					*/

KBQrySQL::KBQrySQL
	(	KBNode	*parent
	)
	:
	KBQryData	(parent, "KBQrySQL"  ),
	m_server	(this,	 "server",   "", KAF_REQD),
	m_query		(this,   "query",    "", KAF_REQD),
	m_topTable	(this,   "toptable", "")
{
	m_qryTabPtr	= 0 ;
}

/*  KBQrySQL								*/
/*  KBQrySQL	: Constructor for free-text SQL query node		*/
/*  _parent	: KBNode *	: Parent node				*/
/*  _query	: KBQrySQL *	: Extant form query			*/
/*  (returns)	: KBQrySQL	:					*/

KBQrySQL::KBQrySQL
	(	KBNode		*_parent,
		KBQrySQL	*_query
	)
	:
	KBQryData	(_parent, _query),
	m_server	(this,    "server",     _query, KAF_REQD),
	m_query		(this,    "query",      _query, KAF_REQD),
	m_topTable	(this,    "toptable",   _query)
{
	m_qryTabPtr	= 0 ;
}

/*  KBQrySQL								*/
/*  ~KBQrySQL	: Destructor for form query node			*/
/*  (returns)	:		:					*/

KBQrySQL::~KBQrySQL ()
{
	DELOBJ	(m_qryTabPtr) ;
}

/*  KBQrySQL								*/
/*  replicate	: Replicate this table query				*/
/*  _parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New table query node			*/

KBNode	*KBQrySQL::replicate
	(	KBNode	*_parent
	)
{
	return	new KBQrySQL (_parent, this) ;
}

/*  KBQrySQL								*/
/*  loadQuery	: Load query 						*/
/*  (returns)	: bool		: Success				*/

bool	KBQrySQL::loadQuery ()
{
	m_tabList  .clear  () ;
	m_qryLevels.clear  () ;
	DELOBJ	(m_qryTabPtr) ;

	if (!linkServer (m_server.getValue()))
		return	false	;

	if (!m_select.parseQuery
		(	m_query.getValue(),
			&dbLink
	   )	)
	{
		setError (m_select.lastError()) ;
		return	 false	;
	}

	if (m_limit.getIntValue() > 0)
		m_select.setLimit (m_limit.getIntValue(), 0) ;

	KBQryLevel *qryLevel  = m_select.makeQryLevel
				(	this,
					dbLink,
					m_topTable.getValue(),
					m_qryTabPtr
				)	;

	m_qryLevels.append (qryLevel) ;

	return	true ;
}

/*  KBQrySQL								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBQrySQL::showAs
	(	KB::ShowAs	mode
	)
{
#if	! __KB_RUNTIME
	if (mode == KB::ShowAsDesign)
	{	m_qryLevels.clear () ;
		DELOBJ (m_qryTabPtr) ;
	}
#endif
	KBNode::showAs (mode) ;
}

/*  KBQrySQL								*/
/*  printNode	: Output node XML					*/
/*  text	: QString &	: Result string				*/
/*  indent	: int		: Indent level of this node		*/
/*  flat	: bool		: Generate flatened version		*/
/*  (returns)	: void		:					*/

void	KBQrySQL::printNode
	(	QString		&text,
		int		indent,
		bool		flat
	)
{
	QString		nodeText ;

	/* We override this method since we don't want to print a child	*/
	/* KBTable node, which does exist in the document.		*/
	text	+= QString("%1<%2").arg("", indent).arg(getElement()) ;

	for (uint idx = 0 ; idx < m_attribs.count () ; idx += 1)
		m_attribs.at(idx)->printAttr(text, nodeText, indent + 2, flat) ;

	if (!nodeText.isEmpty())
	{
		text	+= ">\n" ;
		text	+= nodeText ;
		text	+= QString("%1</%2>\n").arg("", indent).arg(getElement()) ;
	}
	else	text	+= "/>\n" ;
}

/*  KBQrySQL								*/
/*  getFieldList: Get list of fields from database			*/
/*  qlvl	: uint		: Query level				*/
/*  fldList	: QPtrList<KBFieldSpec>&				*/
/*				: Field list result			*/
/*  pKey	: int &		: Primary key index			*/
/*  (returns)	: bool		: Success				*/

bool	KBQrySQL::getFieldList
	(	uint			qlvl,
		QPtrList<KBFieldSpec>	&fldList,
		int			&pKey
	)
{
	const QValueList<KBSelectExpr> eList = m_select.getExprList() ;
	for (uint idx = 0 ; idx < eList.count() ; idx += 1)
		fldList.append
		(	new KBFieldSpec
			(	0xffff0000,
				eList[idx].expr(),
				"",
				KB::ITUnknown,
				0,
				0,
				0
			)
		)	;

	return	KBQryData::getFieldList (qlvl, fldList, pKey) ;
}

/*  KBQrySQL								*/
/*  getComment	: Get brief descriptive comment				*/
/*  qryLvl	: uint		: Query level				*/
/*  (returns)	: QString	: Comment				*/

QString	KBQrySQL::getComment
	(	uint
	)
{
	return	m_select.getComment() ;
}

#if	! __KB_RUNTIME

/*  KBQrySQL								*/
/*  whatsThisExtra							*/
/*		: Extra stuff for pseudo-what's this popup		*/
/*  info	: QValueList<KBWhatsThisPair> &	: List of items		*/
/*  (returns)	: void				:			*/

void	KBQrySQL::whatsThisExtra
	(	QValueList<KBWhatsThisPair> 	&info
	)
{
	QString	sql	= m_query.getValue() ;
	if (sql.length() >= 32) sql = sql.left(31) + " ...." ;
	info.append (KBWhatsThisPair(TR("SQL"), sql)) ;
}


/*  KBQrySQL								*/
/*  propertyDlg	: Show property dialog					*/
/*  iniAttr	: cchar	*	: Initial attribute			*/
/*  (returns)	: bool		: Success				*/

bool	KBQrySQL::propertyDlg
	(	cchar		*iniAttr
	)
{
	extern	bool	qrySQLPropDlg (KBQrySQL *, cchar *, QPtrList<KBAttr> &) ;
	
	if (!qrySQLPropDlg (this, TR("SQL Query"), m_attribs))
		return false ;

	m_qryLevels.clear  () ;
	dropServer 	   () ;

	return true   ;
}
#endif

NEWNODE(QrySQL, (cchar *)0, 0, KNF_FORM|KNF_REPORT)
