/***************************************************************************
    file	         : kb_qtcheck.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_AUTOMOC
#include 	"kb_qtcheck.moc"
#else
#include 	"kb_qtcheck.h"
#endif

#include	"kb_check.h"
#include	"kb_block.h"
#include	"kb_display.h"
#include	"kb_writer.h"
#include	"kb_navigator.h"

#include	<qpixmap.h>
#include	<qstyle.h>


/*  KBCtrlCheck								*/
/*  KBCtrlCheck	: Constructor for check box wrapper			*/
/*  display	: KBDisplay *	: Parent display			*/
/*  check	: KBCheck *	: Parent check				*/
/*  drow	: uint		: Row number in field			*/
/*  (returns)	: KBCtrlCheck	:					*/

KBCtrlCheck::KBCtrlCheck
	(	KBDisplay	*display,
		KBCheck		*check,
		uint		drow
	)
	:
	RKCheckBox (display->getDisplayWidget()),
	KBControl  (this,  display, check, drow),
	m_check	   (check)
{
	connect	   (this, SIGNAL(clicked()), SLOT(clicked())) ;
}

/*  KBCtrlCheck								*/
/*  ~KBCtrlCheck: Destructor for check box wrapper			*/
/*  (returns)	:		:					*/

KBCtrlCheck::~KBCtrlCheck ()
{
}

/*  KBCtrlCheck								*/
/*  getValue	: Get value						*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBCtrlCheck::getValue ()
{
//	fprintf
//	(	stderr,
//		"KBCtrlCheck::getValue: [%s] c=[%d] [%s]\n",
//		(cchar *)m_check->getName(),
//		isChecked(),
//		(cchar *)m_check->getFieldType()->getDescrip()
//	)	;

	/* If the control is unchecked and the initial value was null	*/
	/* then return a null string.					*/
	if (!isChecked() && getIniValue().isNull())
		return	KBValue (m_check->getFieldType()) ;

	return	KBValue (isChecked(), m_check->getFieldType()) ;
}

/*  KBCtrlCheck								*/
/*  setValue	: Set value display row					*/
/*  value	: const KBValue &: Value to set				*/
/*  (returns)	: void		 :					*/

void	KBCtrlCheck::setValue
	(	const KBValue	&value
	)
{
//	fprintf
//	(	stderr,
//		"KBCtrlCheck::setValue: %s: %d: %s=%d\n",
//		m_check->getName().latin1(),
//		m_drow,
//		value.getRawText().latin1(),
//		value.isTrue()
//	)	;

	setChecked (value.isTrue()) ;
	KBControl::setValue (value) ;
}

/*  KBCtrlCheck								*/
/*  clearValue	: Clear value for specified query and display row	*/
/*  query	: bool		: Clearing for query			*/
/*  (returns)	: void		:					*/

void	KBCtrlCheck::clearValue
	(	bool
	)
{
	setChecked (false) ;
}

/*  KBCtrlCheck								*/
/*  changed	: Test if field value has been changed			*/
/*  qrow	: uint		: Query row to check			*/

bool	KBCtrlCheck::changed ()
{
	/* Special case: This handles the situation where the inital	*/
	/* value is null, otherwise we get empty != null.		*/
	if (!isChecked() && getIniValue().isNull())
		return false ;

	return	isChecked() != getIniValue().isTrue() ;
}

/*  KBCtrlCheck								*/
/*  isEmpty	: See if control is empty				*/
/*  (returns)	: bool		: Empty					*/

bool	KBCtrlCheck::isEmpty ()
{
	return	! isChecked () ;
}

/*  KBCtrlCheck								*/
/*  giveFocus	: Give focus to the control				*/
/*  (returns)	: void							*/

void	KBCtrlCheck::giveFocus ()
{
	QCheckBox::setFocus () ;
}

/*  KBCtrlCheck								*/
/*  clicked	: Checkbox clicked notification				*/
/*  (returns)	: void		:					*/

void	KBCtrlCheck::clicked ()
{
	if (startUpdate ())
		m_check->userChange
		(	m_check->getBlock()->getCurDRow() + m_drow,
			isChecked()
		)	;
}

/*  KBCtrlCheck								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlCheck::showAs
	(	KB::ShowAs	mode
	)
{
	KBControl::showAs (mode) ;
	setupProperties	() ;
}

/*  KBCtrlCheck								*/
/*  setupProperties							*/
/*		: Set properties after design change			*/
/*  (returns)	: void		:					*/

void	KBCtrlCheck::setupProperties ()
{
	setReadOnly ((m_showing == KB::ShowAsData) && m_check->isReadOnly()) ;
}

/*  KBCtrlCheck								*/
/*  write	: Write field 						*/
/*  writer	: KBWriter *	: Output writer				*/
/*  rect	: QRect		: Field area				*/
/*  value	: KBValue &	: Value for reports			*/
/*  extra	: int &		: Return extra space			*/
/*  fSubs	: bool		: Substitution flag			*/
/*  (returns)	: bool		: Success				*/

bool	KBCtrlCheck::write
	(	KBWriter	*writer,
		QRect		rect,
		const KBValue	&value,
		bool		fSubs,
		int		&extra
	)
{
	if (writer->asReport())
	{
		QRect	 ur (rect.left(), rect.top(), rect.height(), rect.height()) ;
		QPixmap	 pm (ur.size()) ;
		pm.fill  () ;

		if (value.isTrue())
		{
			QPainter p(&pm)	;

			style().drawPrimitive
			(	QStyle::PE_CheckMark,
				&p,
				QRect(QPoint(0,0), ur.size()),
				colorGroup(),
				QStyle::Style_Default|QStyle::Style_Down,
				QStyleOption::Default
			)	; 
		}

		new KBWriterPixmap
		(	writer,
			rect,
			pm
		)	;

		extra	= 0	;
		return	true	;
	}

	return	KBControl::write (writer, rect, value, fSubs, extra) ;
}

bool	KBCtrlCheck::isNavigationKey
	(	QKeyEvent	*k
	)
{
	return	KBNavigator::isNavigationKey (k) ;
}
