/***************************************************************************
    file	         : kb_qtfield.cpp
    copyright            : (C) 1999,2000,2001,2002,2003,2004 by Mike Richardson
			   (C) 2000,2001,2002,2003,2004 by theKompany.com
			   (C) 2001,2002,2003,2004 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qcursor.h>
#include	<qvalidator.h>

#if		__KB_AUTOMOC
#include 	"kb_qtfield.moc"
#else
#include 	"kb_qtfield.h"
#endif

#include	"kb_field.h"
#include	"kb_writer.h"
#include	"kb_block.h"
#include	"kb_display.h"
#include	"kb_docroot.h"
#include	"kb_helper.h"

#include	"rk_pushbutton.h"
#include	"rk_lineedit.h"

/*  KBCtrlLineEdit							*/
/*  --------------							*/
/*  Wrapper around a line edit so that we can trap the set-focus call.	*/

class	KBCtrlLineEdit : public RKLineEdit
{
	KBControl	*m_control	;

public	:

	KBCtrlLineEdit	(QWidget *, KBControl *) ;

	void		setFocus ()	;
}	;

/*  KBCtrlLineEdit							*/
/*  KBCtrlLineEdit							*/
/*		: Constructor for line edit wrapper			*/
/*  parent	: QWidget *	 : Parent widget			*/
/*  control	: KBControl *	 : Parent control			*/
/*  (returns)	: KBCtrlLineEdit :					*/

KBCtrlLineEdit::KBCtrlLineEdit
	(	QWidget		*parent,
		KBControl	*control
	)
	:
	RKLineEdit	(parent),
	m_control	(control)
{
}

/*  KBCtrlLineEdit							*/
/*  setFocus	: Trap set-focus call					*/
/*  (returns)	: void		:					*/

void	KBCtrlLineEdit::setFocus ()
{
	/* Pass this to the control which then decides whether to	*/
	/* allow this or not. See the comment in KBCtrlField::giveFocus	*/
	m_control->setFocus () ;
}

/*  ------------------------------------------------------------------	*/

/*  KBCtrlField								*/
/*  KBCtrlField	: Constructor for line edit wrapper			*/
/*  display	: KBDisplay *	: Parent display			*/
/*  field	: KBField *	: Parent field				*/
/*  drow	: uint		: Row number in field			*/
/*  (returns)	: KBCtrlField	:					*/

KBCtrlField::KBCtrlField
	(	KBDisplay	*display,
		KBField		*field,
		uint		drow
	)
	:
	RKHBox		(display->getDisplayWidget()),
	KBControl	(this, display, field, drow),
	m_field		(field)
{
//	fprintf
//	(	stderr,
//		"KBCtrlField::KBCtrlField: creating row %d\n",
//		drow
//	)	;

	m_lEdit		= new KBCtrlLineEdit (this, this) ;
	m_inputMask	= new KBMaskedInput  (m_lEdit) ;
	m_bHelp		= 0	;
	m_inSetValue	= false	;

	connect
	(	m_inputMask,
		SIGNAL	(textChangedTo(const QString &)),
		SLOT	(textChangedTo(const QString &))
	)	;
	connect
	(	m_lEdit,
		SIGNAL	(returnPressed()),
		SLOT	(returnPressed())
	)	;

	m_lEdit->installEventFilter (this) ;
	m_lEdit->setSizePolicy	    (QSizePolicy (QSizePolicy::Preferred, QSizePolicy::Preferred)) ;
}

/*  KBCtrlField								*/
/*  ~KBCtrlField: Desctructor for field class				*/
/*  (returns)	:		:					*/

KBCtrlField::~KBCtrlField ()
{
}

/*  KBCtrlField								*/
/*  showName	: Show control name					*/
/*  (returns)	: void		:					*/

void	KBCtrlField::showName ()
{
#if	! __KB_RUNTIME
	if ((m_drow == 0) && (m_showing == KB::ShowAsDesign))
		m_lEdit->setText (m_field->getName()) ;
#endif
}


/*  KBCtrlField								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlField::showAs
	(	KB::ShowAs	mode
	)
{
	KBControl::showAs (mode) ;

	DELOBJ	(m_bHelp) ;
	setupProperties() ;

	setCursor
	(	m_showing == KB::ShowAsDesign ?
				QCursor(Qt::arrowCursor) :
				QCursor(Qt::ibeamCursor)
	)	;

	if (m_showing == KB::ShowAsDesign)
		m_lEdit->setReadOnly (true) ;

	FIELD_NOCHANGE
	(
		if (m_showing == KB::ShowAsDesign)
			m_lEdit->setText (m_field->getName()) ;
		else	m_lEdit->setText ("") ;
	)
}

/*  KBCtrlField								*/
/*  setValue	: Set field value					*/
/*  value	: const KBValue &: Value				*/
/*  (returns)	: void		 :					*/

void	KBCtrlField::setValue
	(	const KBValue	&value
	)
{
//	fprintf
//	(	stderr,
//		"KBCtrlField::setValue: %s: [%s]->[%s]\n",
//		(cchar *)value.getType()->getDescrip(),
//		(cchar *)value.getRawText(),
//		(cchar *)value.getText(m_field->getFormat())
//	)	;

	FIELD_NOCHANGE
	(
		m_lEdit->setText (value.getText(m_field->getFormat())) ;
	)

	KBControl::setValue (value) ;
}

/*  KBCtrlField								*/
/*  getValue	: Get field value					*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBCtrlField::getValue ()
{
	QString	text	= m_lEdit->text() ;

	/* If the control is empty and the initial value was null (as	*/
	/* opposed to an empty string), then return a null string.	*/
	if (text.isEmpty() && getIniValue().isNull())
		return	KBValue (m_field->getFieldType()) ;


	/* If the "deformat" option is set then we try to undo the	*/
	/* formatting using the same string used to do the formatting.	*/
	if (m_field->getDeFormat ())
		return	KBValue
			(	text,
				m_field->getFieldType(),
				m_field->getFormat   ()
			)	;


	return	KBValue (text, m_field->getFieldType()) ;
}

/*  KBCtrlField								*/
/*  clearValue	: Clear value for specified query and display row	*/
/*  query	: bool		: Clearing for query			*/
/*  (returns)	: void		:					*/

void	KBCtrlField::clearValue
	(	bool	query
	)
{
	FIELD_NOCHANGE
	(
		m_lEdit->setText ("") ;
	)

	KBControl::clearValue (query) ;
}

/*  KBCtrlField								*/
/*  changed	: Test if field value has been changed			*/
/*  (returns)	: bool		: TRUE if changed			*/

bool	KBCtrlField::changed ()
{
	QString	text	= m_lEdit->text() ;

//	fprintf
//	(	stderr,
//		"KBCtrlField::changed: d=%d q=%d text=[%s][%d] ini=[%s][%d] inif=[%s]\n",
//		m_drow,
//		m_field->getBlock()->getCurDRow(),
//		(cchar *)text,
//		text.isEmpty(),
//		(cchar *)getIniValue().getRawText(),
//		getIniValue().getRawText().isEmpty(),
//		(cchar *)getIniValue().getText(m_field->getFormat())
//	)	;

	/* Special case: This handles the situation where the inital	*/
	/* value is null, otherwise we get empty != null.		*/
	if (text.isEmpty() && getIniValue().getRawText().isEmpty())
		return false ;

	return	text != getIniValue().getText(m_field->getFormat()) ;
}

/*  KBCtrlField								*/
/*  isEmpty	: See if control is empty				*/
/*  (returns)	: bool		: Empty					*/

bool	KBCtrlField::isEmpty ()
{
	return	m_lEdit->text().isEmpty() ;
}

/*  KBCtrlField								*/
/*  isValid	: See if control is valid				*/
/*  allowNull	: bool		: True to ignore not-null check		*/
/*  (returns)	: bool		: Empty					*/

bool	KBCtrlField::isValid
	(	bool		allowNull
	)
{
	QString	value	;
	QString	text	= m_lEdit->text() ;

	/* If the control is empty and the initial value was null (as	*/
	/* opposed to an empty string), then use a null value.		*/
	if (!text.isEmpty() || !getIniValue().isNull())
		if (m_field->getDeFormat ())
			value	= KBValue::deFormat
				  (	text,
					m_field->getFieldType(),
					m_field->getFormat()
				  )	;
		else	value	= text	;
		
	if (!m_field->checkValid (value, allowNull))
	{	setError (m_field->lastError()) ;
		return	 false ;
	}

	return	true ;
}


/*  KBCtrlField	:							*/
/*  eventFilter	: Event filter on edit control				*/
/*  o		: QObject *	: Object being filtered			*/
/*  e		: QEvent *	: Event					*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBCtrlField::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	if (o != m_lEdit)
		return	KBControl::eventFilter (o, e) ;

	switch (e->type())
	{
		case QEvent::FocusIn	:
		{
			QFocusEvent *fe = (QFocusEvent *)e ;

			/* Select all the text if focus is arriving on	*/
			/* account of something like tabbing.		*/
			if (fe->reason() == QFocusEvent::Other)
			{
				int l = m_lEdit->text().length () ;
				m_lEdit->setSelection      (0, l) ;
				m_lEdit->setCursorPosition (l)	  ;
			}

			/* See if we need to create a helper button.	*/
			/* It would be nice to do this in ::showAs,	*/
			/* but that seems to cause a crash in		*/
			/* QScrollView.					*/
			if ((m_showing == KB::ShowAsData) && (m_bHelp == 0) && !m_field->getHelper().isEmpty())
				if (KBHelperReg::helperExists (m_field->getHelper ()))
				{
					m_bHelp	= new RKPushButton(this);

					m_bHelp->hide		()	;
					m_bHelp->setText	("..")	;
					m_bHelp->setFixedWidth	(height()) ;
					m_bHelp->setFixedHeight	(height()) ;

					connect
					(	m_bHelp,
						SIGNAL(clicked    ()),
						SLOT  (helpClicked())
					)	;
				}

			if (m_bHelp != 0) m_bHelp->show() ;

			return	KBControl::eventFilter (o, e) ;
		}

		case QEvent::FocusOut	:
			/* On focus out, hide the helper button if one	*/
			/* is on display.				*/
			if (m_bHelp != 0) m_bHelp->hide() ;
			return	KBControl::eventFilter (o, e) ;

		case QEvent::KeyPress	:
		case QEvent::KeyRelease	:
		{
			/* Normally enter/return triggers field		*/
			/* navigation, the if there is on "onreturn"	*/
			/* event then skip navigation processing.	*/
			if (m_showing != KB::ShowAsData)
				break	;

			int	key = ((QKeyEvent *)e)->key() ;
			if ((key != Qt::Key_Enter) && (key != Qt::Key_Return))
				break	;
			
			if (m_field->getAttrVal("onreturn").isEmpty())
				break	;

			return	false	;
		}

		default	:
			break	;
	}

	return	KBControl::eventFilter (o, e) ;
}

/*  KBCtrlField								*/
/*  giveFocus	: Give focus to the control				*/
/*  (returns)	: void							*/

void	KBCtrlField::giveFocus ()
{
	/* Focus is to be given to the control. Note that we must NOT	*/
	/* call the KBCtrlLineEdit::setFocus or we get a loop!		*/
	m_lEdit->RKLineEdit::setFocus () ;
}


/*  KBCtrlField								*/
/*  helpClicked	: Helper button clicked					*/
/*  (returns)	: void		:					*/

void	KBCtrlField::helpClicked ()
{
	KBValue	value1	= KBValue
			  (	m_lEdit->text(),
				m_field->getFieldType(),
				m_field->getDeFormat () ?
					m_field->getFormat() :
					QString::null
			  )	;

	QString	resval	= KBHelperDlg::run
			  (	m_field->getHelper (),
				value1  .getRawText(),
				m_field->getDocRoot()->getLocation()
			  )	;
	if (resval.isNull()) return ;

	KBValue	value2	= KBValue (resval, m_field->getFieldType()) ;
	m_lEdit->setText    (value2.getText(m_field->getFormat ())) ;
	m_field->userChange (m_field->getBlock()->getCurDRow() + m_drow) ;
	m_field->onHelper   (m_field->getBlock()->getCurDRow() + m_drow, m_lEdit->text()) ;
}

/*  KBCtrlField								*/
/*  textChangedTo							*/
/*		: Text changed notification				*/
/*  text	: const QString & : New text value			*/
/*  (returns)	: void		  :					*/

void	KBCtrlField::textChangedTo
	(	const QString	&text
	)
{
//	fprintf
//	(	stderr,
//		"KBCtrlField::textChangedTo: [%s] %d/%d\n",
//		text.latin1(),
//		!m_inSetValue,
//		m_showing == KB::ShowAsData
//	)	;

	if (!m_inSetValue && (m_showing == KB::ShowAsData))
	{
		if (!startUpdate ())
			return	;

		if (text != getIniValue().getRawText())
			m_field->userChange (m_field->getBlock()->getCurDRow() + m_drow) ;
	}
}

/*  KBCtrlField								*/
/*  returnPressed: Used presses return/enter				*/
/*  (returns)	 : void		:					*/

void	KBCtrlField::returnPressed ()
{
//	fprintf
//	(	stderr,
//		"KBCtrlField::returnPressed: %d %d [%s]\n",
//		m_inSetValue,
//		m_showing == KB::ShowAsData,
//		m_lEdit->text().latin1()
//	)	;

	/* NOTE: See the code in ::eventFilter ref. return/enter being	*/
	/*	 normally a navigation event.				*/
	if (!m_inSetValue && (m_showing == KB::ShowAsData))
		m_field->returnPressed
		(	m_field->getBlock()->getCurDRow() + m_drow,
			m_lEdit->text()
		)	;
}



/*  KBCtrlField								*/
/*  write	: Write field 						*/
/*  writer	: KBWriter *	: Output writer				*/
/*  rect	: QRect		: Field area				*/
/*  value	: KBValue &	: Value for reports			*/
/*  extra	: int &		: Return extra space			*/
/*  fSubs	: bool		: Substitution flag			*/
/*  (returns)	: bool		: Success				*/

bool	KBCtrlField::write
	(	KBWriter	*writer,
		QRect		rect,
		const KBValue	&value,
		bool		fSubs,
		int		&extra
	)
{
	if (writer->asReport())
	{
		new KBWriterText
		(	writer,
			rect,
			m_field->getPalette(true),
			m_field->getFont   (true),
			value.isNull () ?
				QString::null :
				value.getText(m_field->getFormat()),
			m_field->getAlign  ()|Qt::AlignVCenter,
			fSubs
		)	;

		extra	= 0	;
		return	true	;
	}

	return	KBControl::write (writer, rect, value, fSubs, extra) ;
}

/*  KBCtrlField								*/
/*  morphText	: Get text for morphed control				*/
/*  (returns)	: QString	: Text					*/

QString	KBCtrlField::morphText ()
{
	return	m_lEdit->text()	;
}

/*  KBCtrlField								*/
/*  setupProperties							*/
/*		: Set properties after design change			*/
/*  (returns)	: void		:					*/

void	KBCtrlField::setupProperties ()
{
	m_lEdit	   ->setReadOnly  (m_field->isReadOnly() || (m_showing == KB::ShowAsDesign)) ;
	m_lEdit	   ->setAlignment (m_field->getAlign  ()) ;
	m_inputMask->setEnabled   (m_showing == KB::ShowAsData) ;
	m_inputMask->setInputMask (m_field->getAttrVal ("mask")) ;

#if	! __KB_RUNTIME
	if ((m_showing == KB::ShowAsDesign) && (m_drow == 0))
	{
		m_lEdit->setEchoMode (RKLineEdit::Normal) ;
		showName () ;
		return	    ;
	}
#endif

	m_lEdit->setEchoMode	  (m_field->isPasswd() ? RKLineEdit::Password : RKLineEdit::Normal) ;
}

/*  KBCtrlField								*/
/*  mainWidget	: Get the main widget					*/
/*  (returns)	: QWidget *	: Main widget				*/

QWidget	*KBCtrlField::mainWidget ()
{
	return	m_lEdit	;
}

/*  KBCtrlField								*/
/*  sizeHint	: Get size hint						*/
/*  (returns)	: QSize		: Size hint				*/

QSize	KBCtrlField::sizeHint () const
{
	/* See comments in header file ...				*/
	return	m_lEdit->sizeHint() ;
}

/*  KBCtrlField								*/
/*  sizePolicy	: Get size policy					*/
/*  (returns)	: QSizePolicy	: Size policy				*/

QSizePolicy
	KBCtrlField::sizePolicy () const
{
	/* See comments in header file ...				*/
	return	QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed) ;
}
