/***************************************************************************
    file	         : kb_qtlistbox.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_AUTOMOC
#include	"kb_qtlistbox.moc"
#else
#include	"kb_qtlistbox.h"
#endif

#include	"kb_listbox.h"
#include	"kb_display.h"
#include	"kb_block.h"

#include	"rk_listbox.h"

/*  KBCtrlListBox							*/
/*  KBCtrlListBox							*/
/*		: Constructor for list box wrapper			*/
/*  display	: KBDisplay *	: Parent display			*/
/*  listbox	: KBListBox *	: Parent listbox item			*/
/*  drow	: uint		: Row number in field			*/
/*  (returns)	: KBCtrlListBox	:					*/

KBCtrlListBox::KBCtrlListBox
	(	KBDisplay	*display,
		KBListBox	*listbox,
		uint		drow
	)
	:
	RKListBox	(display->getDisplayWidget()),
	KBControl	(this, display, listbox, drow),
	m_listbox	(listbox)
{
	connect
	(	this,
		SIGNAL	(highlighted(int)),
		SLOT	(slotChanged())
	)	;

	m_inSetValue = false ;
}

/*  KBCtrlListBox							*/
/*  ~KBCtrlListBox							*/
/*		: Destructor for list box class				*/
/*  (returns)	:		:					*/

KBCtrlListBox::~KBCtrlListBox ()
{
}

/*  KBCtrlListBox							*/
/*  setData	: Set value set						*/
/*  _valset	: void *	:					*/
/*  (returns)	: void		:					*/

void	KBCtrlListBox::setData
	(	void	*
	)
{
	LISTBOX_NOCHANGE
	(
	clear		 () ;
	insertStringList (m_listbox->getValues()) ;
	)
}

/*  KBCtrlListBox							*/
/*  slotChanged	: Selection changed notification			*/
/*  (returns)	: void		:					*/

void	KBCtrlListBox::slotChanged ()
{
	if (!m_inSetValue && startUpdate())
		m_listbox->userChange
		(	m_listbox->getBlock()->getCurDRow() + m_drow,
			currentText()
		)	;
}

/*  KBCtrlListBox								*/
/*  showName	: Show control name					*/
/*  (returns)	: void		:					*/

void	KBCtrlListBox::showName ()
{
#if	! __KB_RUNTIME
	if ((m_drow == 0) && (m_showing == KB::ShowAsDesign))
	{	clear	   () ;
		insertItem (m_listbox->getName()) ;
	}
#endif
}

/*  KBCtrlListBox							*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlListBox::showAs
	(	KB::ShowAs	mode
	)
{
	KBControl::showAs (mode) ;

	LISTBOX_NOCHANGE
	(
	clear	() ;
	)

	setupProperties () ;

#if	! __KB_RUNTIME
	if ((mode == KB::ShowAsDesign) && (m_drow == 0))
	{
		showName () ;
		return	    ;
	}
#endif

	if (m_showing == KB::ShowAsData)
	{
		LISTBOX_NOCHANGE
		(
		insertStringList (m_listbox->getValues()) ;
		)
	}
}

/*  KBCtrlListBox							*/
/*  setValue	: Set value						*/
/*  value	: const KBValue &: Value				*/
/*  (returns)	: void		 :					*/

void	KBCtrlListBox::setValue
	(	const KBValue	&value
	)
{
	QString	v	  = value.getRawText() ;
	int	slot	  = m_listbox->getValues().findIndex(v) ;

	if (slot < 0)
	{
		for (int p = v.length() - 1 ; p >= 0 ; p -= 1)
			if (v.at(p) != QChar(' '))
			{
				v = v.left (p + 1) ;
				break	;
			}

		slot	  = m_listbox->getValues().findIndex(v) ;

		fprintf
		(	stderr,
			"KBCtrlListBox::setValue: retry [%s]->[%s]->[%d]\n",
			(cchar *)value.getRawText(),
			(cchar *)v,
			slot
		)	;
	}

	LISTBOX_NOCHANGE
	(
		setCurrentItem (slot < 0 ? 0 : slot) ;
	)

	KBControl::setValue (value) 	;
}

/*  KBCtrlListBox							*/
/*  getValue	: Get value						*/
/*  (returns)	: KBValue 	: Value					*/

KBValue	KBCtrlListBox::getValue ()
{
	/* If the control is empty and the initial value was null (as	*/
	/* opposed to an empty string), then return a null string.	*/
	if (currentText().isEmpty() && getIniValue().isNull())
		return	KBValue (m_listbox->getFieldType()) ;

	return	KBValue (currentText(), m_listbox->getFieldType()) ;
}

/*  KBCtrlListBox							*/
/*  clearValue	: Clear value						*/
/*  query	: bool		: Clearing for query			*/
/*  (returns)	: KBValue 	: Value					*/

void	KBCtrlListBox::clearValue
	(	bool	
	)
{
	setCurrentItem (0) ;
}

/*  KBCtrlListBox							*/
/*  changed	: See if field has changed				*/
/*  (returns)	: bool 		: Changed				*/

bool	KBCtrlListBox::changed ()
{
	/* Special case: This handles the situation where the inital	*/
	/* value is null, otherwise we get empty != null.		*/
	if (currentText().isEmpty() && getIniValue().getRawText().isEmpty())
		return false ;

	return	currentText() != getIniValue().getRawText() ;
}

/*  KBCtrlListBox							*/
/*  isEmpty	: See if control is empty				*/
/*  (returns)	: bool		: Empty					*/

bool	KBCtrlListBox::isEmpty ()
{
	return	currentText().isEmpty() ;
}

/*  KBCtrlListBox								*/
/*  isValid	: See if control is valid				*/
/*  allowNull	: bool		: True to ignore not-null check		*/
/*  (returns)	: bool		: Empty					*/

bool	KBCtrlListBox::isValid
	(	bool		allowNull
	)
{
	if (!m_listbox->checkValid (currentText(), allowNull))
	{
		setError (m_listbox->lastError()) ;
		return	false	;
	}

	return	true	;
}

/*  KBCtrlListBox								*/
/*  giveFocus	: Give focus to the control				*/
/*  (returns)	: void							*/

void	KBCtrlListBox::giveFocus ()
{
	RKListBox::setFocus () ;
}

/*  KBCtrlListBox							*/
/*  setupProperties							*/
/*		: Set properties after design change			*/
/*  (returns)	: void		:					*/

void	KBCtrlListBox::setupProperties ()
{
	setReadOnly ((m_showing == KB::ShowAsData) && m_listbox->isReadOnly()) ;
}
