/***************************************************************************
    file	         : kb_qtmemo.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2000,2001,2002,2003 by theKompany.com
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qpopupmenu.h>

#if		__KB_AUTOMOC
#include 	"kb_qtmemo.moc"
#else
#include 	"kb_qtmemo.h"
#endif

#include	"kb_block.h"
#include	"kb_memo.h"
#include	"kb_display.h"
#include	"kb_writer.h"
#include	"kb_dialog.h"
#include	"kb_highlighter.h"
#include	"kb_file.h"

#include	"tk_filedialog.h"


/*  KBCtrlMemo								*/
/*  KBCtrlMemo	: Constructor for memo edit wrapper			*/
/*  display	: KBDisplay *	: Parent display			*/
/*  memo	: KBMemo *	: Parent memo				*/
/*  drow	: uint		: Row number in memo			*/
/*  (returns)	: KBCtrlMemo	:					*/

KBCtrlMemo::KBCtrlMemo
	(	KBDisplay	*display,
		KBMemo		*memo,
		uint		drow
	)
	:
	KBTextEdit	(display->getDisplayWidget()),
	KBControl	(this, display, memo, drow),
	m_memo		(memo)
{
	m_highlighter	= 0 ;
	setHilite () ;

	m_inSetValue= false ;
	m_modified  = false ;

	connect
	(	this,
		SIGNAL	(textChanged()),
		SLOT	(editChanged())
	)	;

	viewport()->installEventFilter (this) ;
	RKApplication::self()->installMousePressFilter (viewport(), &m_filter) ;
}

/*  KBCtrlMemo								*/
/*  ~KBCtrlMemo	: Desctructor for memo class				*/
/*  (returns)	:		:					*/

KBCtrlMemo::~KBCtrlMemo ()
{
	DELOBJ (m_highlighter) ;
}

QPopupMenu
	*KBCtrlMemo::createPopupMenu
	(	const QPoint 	&pos
	)
{
	QPopupMenu *p = KBTextEdit::createPopupMenu (pos) ;

	if (!m_memo->isReadOnly())
	{
		p->insertSeparator () ;
		p->insertItem	   (TR("Insert from file"), this, SLOT(loadFromFile())) ;
		p->insertItem	   (TR("Save to file"	 ), this, SLOT(saveToFile  ())) ;
	}

	return	p ;
}

/*  KBCtrlMemo								*/
/*  loadFromFile: Load control from a file				*/
/*  (returns)	: void		:					*/

void	KBCtrlMemo::loadFromFile ()
{
	fprintf
	(	stderr,
		"KBCtrlMemo::loadFromFile: called\n"
	)	;

	KBFileDialog fDlg 
		     (	".",
			QString::null,
			qApp->activeWindow(),
			"loadfile",
			true
		     )	;

	fDlg.setMode	  (KBFileDialog::ExistingFile) ;
	fDlg.setCaption   (TR("Load from file ...." )) ;
	if (!fDlg.exec ()) return ;

	QString	name	  = fDlg.selectedFile () ;
	if (name.isEmpty()) return ;

	KBFile	file(name)	;
	if (!file.open(IO_ReadOnly))
	{
		file.lastError().DISPLAY() ;
		return	;
	}

	setText	(file.readAll()) ;
}

/*  KBCtrlMemo								*/
/*  saveToFile	: Save contents to a file				*/
/*  (returns)	: void		:					*/

void	KBCtrlMemo::saveToFile ()
{
	fprintf
	(	stderr,
		"KBCtrlMemo::saveToFile: called\n"
	)	;

	KBFileDialog fDlg 
		     (	".",
			QString::null,
			qApp->activeWindow(),
			"loadfile",
			true
		     )	;

	fDlg.setMode	  (KBFileDialog::AnyFile  ) ;
	fDlg.setCaption   (TR("Save to file ....")) ;
	if (!fDlg.exec ()) return ;

	QString	name	  = fDlg.selectedFile () ;
	if (name.isEmpty()) return ;

	KBFile	file(name)	;
	if (!file.open(IO_WriteOnly|IO_Truncate))
	{
		file.lastError().DISPLAY() ;
		return	;
	}

	file.writeBlock (text().local8Bit()) ;
}

/*  KBCtrlMemo								*/
/*  eventFilter	: Widget event filter routine				*/
/*  o		: QObject *	: Object receiving event		*/
/*  e		: QEvent *	: Event in question			*/
/*  (returns)	: bool		: False to continue processing		*/

bool	KBCtrlMemo::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	if (e->type() == QEvent::KeyPress)
		return	false	;

	if (KBControl::eventFilter (o, e))
		return	true	;

	return	KBTextEdit::eventFilter (o, e) ;
}

/*  KBCtrlMemo								*/
/*  viewSetFocusOK							*/
/*		: Check if OK to set focus into the memo		*/
/*  (returns)	: bool		: True if OK				*/

bool	KBCtrlMemo::viewSetFocusOK()
{
	return	m_memo->moveFocusOK (m_drow) ;
}

/*  KBCtrlMemo								*/
/*  setHilite	: Set highlight mode					*/
/*  (returns)	: void		:					*/

void	KBCtrlMemo::setHilite ()
{
	setHighlight (m_memo->getAttrVal("hilite")) ;
}

/*  KBCtrlMemo								*/
/*  setPalette	: Set palette on widget					*/
/*  pal		: const QPalette *: Palette in question			*/
/*  (returns)	: void		  :					*/

void	KBCtrlMemo::setPalette
	(	const QPalette *pal
	)
{
	KBTextEdit::setPalette (*pal) ;
}

/*  KBCtrlMemo								*/
/*  setFont	: Set font on widget					*/
/*  font	: const QFont *	: Font in question			*/
/*  (returns)	: void		:					*/

void	KBCtrlMemo::setFont
	(	const QFont *font
	)
{
	KBTextEdit::setFont (*font) ;
}

/*  KBCtrlMemo								*/
/*  showName	: Show control name					*/
/*  (returns)	: void		:					*/

void	KBCtrlMemo::showName ()
{
#if	! __KB_RUNTIME
	if ((m_drow == 0) && (m_showing == KB::ShowAsDesign))
		setText (m_memo->getName()) ;
#endif
}

/*  KBCtrlMemo								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlMemo::showAs
	(	KB::ShowAs	mode
	)
{
	KBControl::showAs (mode) ;
	setupProperties ()	 ;

#if	! __KB_RUNTIME
	if ((mode == KB::ShowAsDesign) && (m_drow == 0))
	{
		showName	() ;
		setWordWrap	(RKTextEdit::NoWrap) ;
		return	 ;
	}
#endif
	if (m_memo->isWrapping())
	{
		setWordWrap	(RKTextEdit::WidgetWidth) ;
		setWrapPolicy	(RKTextEdit::AtWordBoundary) ;

//		QString	wchars	= m_memo->getAttrVal("wrapchars") ;
//		if (wchars.isEmpty()) wchars = " \t" ;
//		textDocument()->setWordBound(wchars) ;
	}
	else	setWordWrap	(RKTextEdit::NoWrap) ;

	MEMO_NOCHANGE
	(
		clear	() ;
	)
}

/*  KBCtrlMeno								*/
/*  giveFocus	: Give focus to the control				*/
/*  (returns)	: void							*/

void	KBCtrlMemo::giveFocus ()
{
	KBTextEdit::setFocus () ;
}


/*  KBCtrlMemo								*/
/*  setValue	: Set memo value					*/
/*  value	: const KBValue &: Value				*/
/*  (returns)	: void		 :					*/

void	KBCtrlMemo::setValue
	(	const KBValue	&value
	)
{
	MEMO_NOCHANGE
	(
		setText (value.getRawText()) ;
		m_modified  = false  ;
	)

	KBControl::setValue (value) ;
}

/*  KBCtrlMemo								*/
/*  editChanged	: Something changes in the editor			*/
/*  (returns)	: void		:					*/

void	KBCtrlMemo::editChanged ()
{
	if (!m_inSetValue && (m_showing == KB::ShowAsData))
		if (startUpdate())
		{
			m_memo->userChange (m_memo->getBlock()->getCurDRow() + m_drow) ;
			m_modified = true ;
		}
}

/*  KBCtrlMemo								*/
/*  getValue	: Get memo value					*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBCtrlMemo::getValue ()
{
	QString	_text	= text () ;

	if (_text.isEmpty() && getIniValue().isNull())
		return	KBValue (m_memo->getFieldType()) ;

	return	KBValue (_text,  m_memo->getFieldType()) ;
}

/*  KBCtrlMemo								*/
/*  clearValue	: Clear value for specified query and display row	*/
/*  query	: bool		: Clearing for query			*/
/*  (returns)	: void		:					*/

void	KBCtrlMemo::clearValue
	(	bool	query
	)
{
	MEMO_NOCHANGE
	(
		clear	()	;
	)

	KBControl::clearValue (query) ;
}

/*  KBCtrlMemo								*/
/*  changed	: Test if memo value has been changed			*/
/*  (returns)	: bool		: TRUE if changed			*/

bool	KBCtrlMemo::changed ()
{
	QString	val = text () ;
	QString	ini = getIniValue().getRawText() ;

	/* Special case: This handles the situation where the inital	*/
	/* value is null, otherwise we get empty != null.		*/
	if (val.isEmpty() && ini.isEmpty()) return false ;

	return	val != ini ;
}

/*  KBCtrlMemo								*/
/*  isEmpty	: See if control is empty				*/
/*  (returns)	: bool		: Empty					*/

bool	KBCtrlMemo::isEmpty ()
{
	return	text().isEmpty() ;
}

/*  KBCtrlMemo								*/
/*  isValid	: See if control is valid				*/
/*  allowNull	: bool		: True to ignore not-null check		*/
/*  (returns)	: bool		: Empty					*/

bool	KBCtrlMemo::isValid
	(	bool		allowNull
	)
{
	QString	value	;

	if (!text().isEmpty() || !getIniValue().isNull())
		value	= text() ;
		
	if (!m_memo->checkValid (value, allowNull))
	{	setError (m_memo->lastError()) ;
		return	 false ;
	}

	return	true ;
}

/*  KBCtrlMemo								*/
/*  mainWidget	: Get the main widget for the control			*/
/*  (returns)	: QWidget *	: Widget in question			*/

QWidget	*KBCtrlMemo::mainWidget ()
{
	return	viewport () ;
}

/*  KBCtrlMemo								*/
/*  write	: Write memo 						*/
/*  writer	: KBWriter *	: Output writer				*/
/*  rect	: QRect		: Memo area				*/
/*  value	: KBValue &	: Value for reports			*/
/*  fSubs	: bool		: Substitution flag			*/
/*  extra	: int &		: Return extra space			*/
/*  (returns)	: bool		: Success				*/

bool	KBCtrlMemo::write
	(	KBWriter	*writer,
		QRect		rect,
		const KBValue	&value,
		bool		fSubs,
		int		&extra
	)
{
	if (writer->asReport())
	{
		KBWriterText *w = new KBWriterText
				  (	writer,
					rect,
					m_memo->getPalette(true),
					m_memo->getFont   (true),
					value.getRawText  (),
					0x1001,
					fSubs
				  )	;
		extra	= w->extra()	;
		return	true	;
	}

	return	KBControl::write (writer, rect, value, fSubs, extra) ;
}

/*  KBCtrlMemo								*/
/*  setupProperties							*/
/*		: Set properties after design change			*/
/*  (returns)	: void		:					*/

void	KBCtrlMemo::setupProperties ()
{
	setHilite () ;
}

/*  KBCtrlMemo								*/
/*  notifyFilter: Notification filter					*/
/*  o		: QObject *	: Object to receive event		*/
/*  e		: QEvent *	: Event in question			*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBCtrlMemo::notifyFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	if (m_showing == KB::ShowAsData)
		if (e->type() == QEvent::MouseButtonPress)
		{
			fprintf
			(	stderr,
				"KBCtrlMemo::notifyFilter: called: hasFocus=%d\n",
				hasFocus()
			)	;

			if (hasFocus())
				return	false ;

			if (m_memo->moveFocusOK (m_drow))
			{
				m_memo->focusInEvent (m_drow, QFocusEvent::Mouse) ;
				return	false ;
			}

			return	true ;
		}

	return	false	;
}
