/***************************************************************************
    file	         : kb_qttabber.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<stdarg.h>

#include	<qapplication.h>
#include	<qptrlist.h>
#include	<qstring.h>

#include	"kb_classes.h"
#include	"kb_attr.h"
#include	"kb_type.h"
#include	"kb_value.h"

#include	"kb_event.h"
#include	"kb_tabber.h"
#include	"kb_display.h"
#include	"kb_writer.h"
#include	"kb_qttabber.h"
#include	"kb_ascii.h"



/*  KBCtrlTabberBar							*/
/*  KBCtrlTabberBar							*/
/*		: Constructor for tab bar wrapper			*/
/*  display	: KBDisplay *	: Parent display			*/
/*  atbber	: KBTabber *	: Parent tabber				*/
/*  design	: bool		: Design mode				*/
/*  (returns)	: KBCtrlTabber	:					*/

KBCtrlTabberBar::KBCtrlTabberBar
	(	KBDisplay	*display,
		KBTabberBar	*tabberBar
	)
	:
	QTabBar		(display->getDisplayWidget()),
	KBControl	(this, display, tabberBar),
	m_tabberBar	(tabberBar)
{
	QTabBar::connect
	(	(QTabBar *)this,
		SIGNAL(   selected(int)),
		m_tabberBar,
 		SLOT  (tabSelected(int))
	)	;
}

/*  KBCtrlTabberBar							*/
/*  ~KBCtrlTabberBar							*/
/*		: Destructor for tab bar widget				*/
/*  (returns)	:		:					*/

KBCtrlTabberBar::~KBCtrlTabberBar ()
{
}

/*  KBCtrlTabberBar							*/
/*  giveFocus	: Give focus to the control				*/
/*  (returns)	: void							*/

void	KBCtrlTabberBar::giveFocus ()
{
	QTabBar::setFocus () ;
}

/*  KBCtrlTabberBar							*/
/*  addTab	: Add a new tab						*/
/*  label	: const QString & : Tab text				*/
/*  (returns)	: int		  : Tab identifier			*/

int	KBCtrlTabberBar::addTab
	(	const QString	&label
	)
{
	return	QTabBar::addTab (new QTab (label)) ;
}

/*  KBCtrlTabberBar							*/
/*  setTabText	: Set tab text						*/
/*  id		: int		  : Tab identififer			*/
/*  text	: const QString & : Text				*/
/*  (returns)	: void							*/

void	KBCtrlTabberBar::setTabText
	(	int		id,
		const QString	&text
	)
{
	QTab *t = tab (id) ;
	if (t != 0) t->setText (text) ;
}

/*  KBCtrlTabberBar								*/
/*  getTabRect	: Get tab area for specified page			*/
/*  id		: int		: Page identifier			*/
/*  (returns)	: QRect		: Area					*/

QRect	KBCtrlTabberBar::getTabRect
	(	int		id
	)
{
	fprintf
	(	stderr,
		"KBCtrlTabberBar::getTabRect: WIDTH=%d\n",
		width()
	)	;

	QTab	*t = tab (id) ;
	return	t == 0 ? QRect() : t->rect() ;
}

/*  KBCtrlTabberBar							*/
/*  removeTab	: Remove a tab						*/
/*  id		: int		: Tab identififer			*/
/*  (returns)	: void							*/

void	KBCtrlTabberBar::removeTab
	(	int		id
	)
{
	QTab *t = tab (id) ;
	if (t != 0) QTabBar::removeTab (t) ;
}


/*  KBCtrlTabberBar							*/
/*  layoutTabs	: Fix up tab layout					*/
/*  (return)	: void		:					*/

void	KBCtrlTabberBar::layoutTabs ()
{
	QTabBar::layoutTabs() ;

	if (!m_tabberBar->wideTabs()) return ;

	uint	nTabs	= count() ;
	if (nTabs == 0) return	  ;
	uint	tWidth	= width()/nTabs	;

	for (uint idx = 0 ; idx < nTabs ; idx += 1)
	{
		QTab	*tab	= tabAt(idx)	;
		QRect	rect	= tab->rect()	;

		rect.setLeft	(tWidth * idx)	;
		rect.setWidth	(tWidth)	;

//		fprintf
//		(	stderr,
//			"KBCtrlTabberBar::layoutTabs: %d: %s  *****\n",
//			idx,
//			_TEXT(rect)
//		)	;

		tab->setRect	(rect)		;
	}
}

void	KBCtrlTabberBar::paint
	(	QPainter	*painter,
		QTab		*tab,
		bool		selected
	)
	const
{
	QTabBar::paint (painter, tab, selected) ;
}

/*  KBCtrlTabberBar							*/
/*  write	: Write field 						*/
/*  writer	: KBWriter *	: Output writer				*/
/*  rect	: QRect		: Field area				*/
/*  value	: KBValue &	: Value for reports			*/
/*  fSubs	: bool		: Substitution flag			*/
/*  extra	: int &		: Return extra space			*/
/*  (returns)	: bool		: Success				*/

bool	KBCtrlTabberBar::write
	(	KBWriter	*writer,
		QRect		rect,
		const KBValue	&value,
		bool		fSubs,
		int		&extra
	)
{
	if (writer->asReport())
	{
		extra	= 0  ;
		return	true ;
	}

	return	KBControl::write (writer, rect, value, fSubs, extra) ;
}
