/***************************************************************************
    file	         : kb_qttree.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<time.h>

#include	<qapplication.h>
#include	<qptrlist.h>
#include	<qstring.h>
#include	<qlistbox.h>
#include	<qtimer.h>
#include	<qptrstack.h>
#include	<qheader.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"

#include	"kb_writer.h"
#include	"kb_block.h"
#include	"kb_tree.h"
#include	"kb_display.h"

#include	"kb_multilistbox.h"

#if		__KB_AUTOMOC
#include 	"kb_qttree.moc"
#else
#include 	"kb_qttree.h"
#endif


/*  KBCtrlTreeItem							*/
/*  --------------							*/
/*  Class used for entries in the tree ...				*/

class	KBCtrlTreeItem : public QListViewItem
{
	KBCtrlTree		*m_ctrlTree	;
	const KBLTValueSet	&m_valset	;

	/* The index value is the index into the value set for this	*/
	/* item, with negative values for branch nodes that do not map	*/
	/* to actual values.						*/
	int			m_idx		;

	/* The from and use values specify the range of entries in the	*/
	/* value set that are covered by this item. The use value will	*/
	/* always be unity for leaf nodes.				*/
	int			m_from		;
	int			m_use		;

public	:

	KBCtrlTreeItem
	(	RKListView		*parent,
		QListViewItem		*after,
		KBCtrlTree		*ctrlTree,
		const KBLTValueSet	&valset,
		int			idx,
		int			from,
		int			use,
		uint			data,
		uint			width
	)	;
	KBCtrlTreeItem
	(	QListViewItem		*parent,
		QListViewItem		*after,
		KBCtrlTree		*ctrlTree,
		const KBLTValueSet	&valset,
		int			idx,
		int			from,
		int			use,
		uint			data,
		uint			width
	)	;


	bool			containsIndex	(int)	;
	const	QStringList	&entry		() 	;
	virtual	void		setOpen		(bool)	;

	inline	int	index	()
	{
		return	m_idx	;
	}
}	;


/*  KBCtrlTreeItem							*/
/*  KBCtrlTreeItem							*/
/*		: Constructor for tree tree item			*/
/*  parent	: RKListView *	  : Parent list view			*/
/*  after	: QListViewItem * : Insert after			*/
/*  ctrlTree	: KBCtrlTree *	  : Control object			*/
/*  valset	: KBLTValueSet &  : Data value set			*/
/*  idx		: int		  : Corresponding entry in value set	*/
/*  from	: int		  : Index of first entry in value set	*/
/*  use		: int		  : Number of value set entries covered	*/
/*  data	: int		  : Index for first display data value	*/
/*  width	: int		  : Number of values displayed		*/
/*  (returns)	: KBCtrlTreeItem  :					*/

KBCtrlTreeItem::KBCtrlTreeItem
	(	RKListView		*parent,
		QListViewItem		*after,
		KBCtrlTree		*ctrlTree,
		const KBLTValueSet	&valset,
		int			idx,
		int			from,
		int			use,
		uint			data,
		uint			width
	)
	:
	QListViewItem	(parent, after),
	m_ctrlTree	(ctrlTree),
	m_valset	(valset),
	m_idx		(idx),
	m_from		(from),
	m_use		(use)
{
//	fprintf
//	(	stderr,
//		"KBCtrlTreeItem::KBCtrlTreeItem: top    [%d:%d] at %d\n",
//		m_idx,
//		m_use,
//		time(0)
//	)	;

	const QStringList &e = entry() ;

	for (uint col = 0 ; col < width ; col += 1)
		setText	(col, e[data + col]) ;

	setExpandable (m_ctrlTree != 0) ;
}

/*  KBCtrlTreeItem							*/
/*  KBCtrlTreeItem							*/
/*		: Constructor for tree tree item			*/
/*  parent	: QListViewItem * : Parent list view item		*/
/*  after	: QListViewItem * : Insert after			*/
/*  ctrlTree	: KBCtrlTree *	  : Control object			*/
/*  valset	: KBLTValueSet &  : Data value set			*/
/*  idx		: int		  : Corresponding entry in value set	*/
/*  from	: int		  : Index of first entry in value set	*/
/*  use		: int		  : Number of value set entries covered	*/
/*  data	: int		  : Index for first display data value	*/
/*  width	: int		  : Number of values displayed		*/
/*  (returns)	: KBCtrlTreeItem  :					*/

KBCtrlTreeItem::KBCtrlTreeItem
	(	QListViewItem		*parent,
		QListViewItem		*after,
		KBCtrlTree		*ctrlTree,
		const KBLTValueSet	&valset,
		int			idx,
		int			from,
		int			use,
		uint			data,
		uint			width
	)
	:
	QListViewItem	(parent, after),
	m_ctrlTree	(ctrlTree),
	m_valset	(valset),
	m_idx		(idx),
	m_from		(from),
	m_use		(use)
{
//	fprintf
//	(	stderr,
//		"KBCtrlTreeItem::KBCtrlTreeItem: child [%d,%d]\n",
//		m_idx,
//		m_use
//	)	;

	const QStringList &e = entry() ;

	for (uint col = 0 ; col < width ; col += 1)
		setText	(col, e[data + col]) ;

	setExpandable (m_ctrlTree != 0) ;
}


/*  KBCtrlTreeItem							*/
/*  containsIndex: See if specified index is within this item		*/
/*  idx		 : int		: Required index			*/
/*  (returns)	 : bool		: True if so				*/

bool	KBCtrlTreeItem::containsIndex
	(	int		idx
	)
{
	return	(m_from <= idx) && (idx < m_from + m_use) ;

}

/*  KBCtrlTreeItem							*/
/*  entry	: Get data values entry for this item			*/
/*  (returns)	: QStringList &	: Values				*/

const QStringList &KBCtrlTreeItem::entry ()
{
	return	m_valset[m_from] ;
}

/*  KBCtrlTreeItem							*/
/*  setOpen	: Open or close item					*/
/*  open	: bool		: True to open				*/
/*  (returns)	: void		:					*/

void	KBCtrlTreeItem::setOpen
	(	bool		open
	)
{
	if (open)
		/* If the control tree is set (will not for known leaf	*/
		/* nodes) and there are no children then create the	*/
		/* children. We can then clear the tree pointer (saves	*/
		/* trying repeatedly if no children are forthcoming).	*/
		if ((m_ctrlTree != 0) && (firstChild() == 0))
		{
			m_ctrlTree->loadDataTree (this, m_valset, m_from, m_use, depth() + 1) ;
			m_ctrlTree = 0 ;
		}

	QListViewItem::setOpen (open) ;
}


/*  ------------------------------------------------------------------	*/

/*  KBCtrlTree								*/
/*  KBCtrlTree	: Constructor for combo box wrapper			*/
/*  display	: KBDisplay *	: Parent display			*/
/*  tree	: KBTree *	: Parent tree				*/
/*  drow	: uint		: Row number in field			*/
/*  (returns)	: KBCtrlTree	:					*/

KBCtrlTree::KBCtrlTree
	(	KBDisplay	*display,
		KBTree		*tree,
		uint		drow
	)
	:
	RKListView	(display->getDisplayWidget()),
	KBControl	(this, display, tree, drow),
	m_tree		(tree)
{
	setSorting		(-1)	;
	setRootIsDecorated	(true)	;
	header()->hide()		;

	connect
	(	this,
		SIGNAL	(currentChanged	   (QListViewItem *)),
		SLOT	(slotCurrentChanged(QListViewItem *))
	)	;

	m_deferFocus	= 0 ;
	m_deferMouse	= 0 ;
	m_keyset	= 0 ;
}

/*  KBCtrlTree								*/
/*  ~KBCtrlTree	: Desctructor for tree class				*/
/*  (returns)	:		:					*/

KBCtrlTree::~KBCtrlTree ()
{
	DELOBJ	(m_deferFocus)	;
	DELOBJ	(m_deferMouse)	;
}


/*  KBCtrlTree								*/
/*  loadDataTree: Load data as tree					*/
/*  parent	: KBCtrlTreeItem * : Parent item or null for top level	*/
/*  valset	: 		   : List of values			*/
/*  fromRow	: uint		   : Load data starting at row ...	*/
/*  forRows	: uint		   : ... for rows			*/
/*  offset	: uint		   : Grouping offset			*/
/*  (returns)	: void		   :					*/

void	KBCtrlTree::loadDataTree
	(	KBCtrlTreeItem		*parent,
		const KBLTValueSet	&valset,
		uint			fromRow,
		uint			forRows,
		uint			offset
	)
{
	uint		exprCount	= m_tree->exprCount   () ;
	uint		extraCount	= m_tree->extraCount  () ;
	uint		groupCount	= m_tree->groupCount  () ;
	uint		groupOffset	= exprCount + extraCount ;
	KBCtrlTreeItem	*last		= 0 ;

	if (offset >= groupCount)
	{
		fprintf
		(	stderr,
			"KBCtrlTree::loadDataTree: data: for=%d\n",
			forRows
		)	;

		for (uint idx = 0 ; idx < forRows ; idx += 1, fromRow += 1)
			last = new KBCtrlTreeItem
				   (	parent,
					last,
					0,
					valset,
					fromRow,
					fromRow,
					1,
					offset,
					exprCount - offset
				   )	;
		return	;
	}

	if (fromRow == 0)
	{
		last	= new KBCtrlTreeItem
			      (		this,
					last,
					this,
					valset,
					-1,
					0,
					1,
					0,
					0
			      )	;
		fromRow	+= 1	;
		forRows	-= 1	;
	}

	KBLTValueSet::const_iterator iter = valset.at(fromRow) ;

	while (forRows > 0)
	{
		const QString	&grpval	= (*iter)[groupOffset + offset] ;
		uint		useRows	= 1 ;

		iter++	;

		while (useRows < forRows)
			if ((*iter)[groupOffset + offset] == grpval)
			{
				useRows	+= 1 ;
				iter	++   ;
			}
			else	break	;

		fprintf
		(	stderr,
			"KBCtrlTree::loadDataTree: nest: from=%d for=%d grpval=[%s]\n",
			fromRow,
			useRows,
			(cchar *)grpval
		)	;

		if (parent == 0)
			last	= new KBCtrlTreeItem
				      (		this,
						last,
						this,
						valset,
						-1,
						fromRow,
						useRows,
						offset,
						1
				      )	;
		else	last	= new KBCtrlTreeItem
				      (		parent,
						last,
						this,
						valset,
						-1,
						fromRow,
						useRows,
						offset,
						1
				      )	;

//		loadDataTree (last, valset, fromRow, useRows, offset + 1) ;

		fromRow	+= useRows ;
		forRows	-= useRows ;
	}

}


/*  KBCtrlTree								*/
/*  loadDataFlat: Load data as flattened tree				*/
/*  valset	: 		: List of values			*/
/*  (returns)	: void		:					*/

void	KBCtrlTree::loadDataFlat
	(	const KBLTValueSet	&valset
	)
{
	uint	exprCount	= m_tree->exprCount   () ;
	uint	extraCount	= m_tree->extraCount  () ;
	uint	groupCount	= m_tree->groupCount  () ;
	uint	groupOffset	= exprCount + extraCount ;

	fprintf
	(	stderr,
		"KBCtrlTree::loadDataFlat: ec=%d gc=%d\n",
		exprCount,
		groupCount
	)	;

	/* The strategy is to insert an item into the list view for	*/
	/* each value. These are structured such that we get one top-	*/
	/* level item for each distinct first group value (strictly,	*/
	/* one for each change in first group value); under each there	*/
	/* is a second-level item for each distinct second group value	*/
	/* (with the same first group value) and so on.	We use a	*/
	/* stack to keep track of the last listview item at each level	*/
	/* (we don't actually need a stack, just a vector, but the	*/
	/* stack is easy to manipulate).				*/

	/* Prime the system with the first entry, setting all stack	*/
	/* entries to point to this first item. The add all following	*/
	/* entries.							*/
	QPtrStack<KBCtrlTreeItem> stack	;
	KBCtrlTreeItem *null	= new KBCtrlTreeItem (this,    0, 0, valset, -1, 0, 1, 0, exprCount) ;
	KBCtrlTreeItem *item	= new KBCtrlTreeItem (this, null, 0, valset,  1, 1, 1, 0, exprCount) ;

	while (stack.count() < groupCount)
		stack.push (item) ;


	for (uint idx = 2 ; idx < valset.count() ; idx += 1)
	{
		const QStringList &sl = valset[idx] ;

		/* Work back down the stack to pop'ing from it while	*/
		/* the group value differs from the group value for the	*/
		/* entry to be inserted....				*/
		for (int gc2 = groupCount - 1 ; gc2 >= 0 ; gc2 -= 1)
			if (stack.top()->entry()[groupOffset + gc2] != sl[groupOffset + gc2])
				stack.pop() ;
			else	break	;

		/* ....so that the stack is left with items that match	*/
		/* the group terms of the current entry (think of this	*/
		/* as prefix matching). We can then append this entry,	*/
		/* with the special case that if the stack is empty	*/
		/* then we append at the top level.			*/
		if (stack.count() > 0)
		{
			QListViewItem *child = stack.top()->firstChild() ;
			while ((child != 0) && (child->nextSibling() != 0))
				child = child->nextSibling() ;

			item	= new KBCtrlTreeItem (stack.top(), child, 0, valset, idx, idx, 1, 0, exprCount) ;
		}
		else
		{
			QListViewItem *child = firstChild() ;
			while ((child != 0) && (child->nextSibling() != 0))
				child = child->nextSibling() ;

			item	= new KBCtrlTreeItem (this,        child, 0, valset, idx, idx, 1, 0, exprCount) ;
		}

		/* Top up the stack with the new listview item.		*/
		while (stack.count() < groupCount)
			stack.push (item) ;
	}
}

/*  KBCtrlTree								*/
/*  loadDataValues							*/
/*		: Load values into control				*/
/*  valset	: 		: List of values			*/
/*  (returns)	: void		:					*/

void	KBCtrlTree::loadDataValues
	(	const KBLTValueSet	&valset
	)
{
	uint exprCount = m_tree->exprCount () ;
	if (exprCount == 0) return ;

	clear () ;

	while (columns() > (int)exprCount) removeColumn (0) ;
	while (columns() < (int)exprCount) addColumn    (QString::null) ;

	if (valset.count() > 1)
		if (m_tree->treeType() == 0)
			loadDataTree (0, valset, 0, valset.count(), 0) ;
		else	loadDataFlat (valset) ;
}

/*  KBCtrlTree								*/
/*  setData	: Set value set						*/
/*  data	: void *	: Set of values				*/
/*  (returns)	: void		:					*/

void	KBCtrlTree::setData
	(	void	*data
	)
{
	KBLinkTreeData	*ltd    = (KBLinkTreeData *)data ;
	DELOBJ		(m_keyset)      ;
	loadDataValues	(ltd->m_valset) ;
}

/*  KBCtrlTree								*/
/*  slotCurrentChanged							*/
/*		: Current item changed					*/
/*  item	: QListViewItem * : New current item			*/
/*  (returns)	: void		  :					*/

void	KBCtrlTree::slotCurrentChanged
	(	QListViewItem	*item
	)
{
	if ((item != 0) && (m_showing == KB::ShowAsData))
		if (startUpdate())
		{
			int	idx = ((KBCtrlTreeItem *)item)->index() ;

			m_tree->userChange
			(	m_tree->getBlock()->getCurDRow() + m_drow,
				idx < 0 ? 0 : idx
			)	;

			if (m_tree->clickOpen ())
				item->setOpen (true) ;
		}
}

/*  KBCtrlTree								*/
/*  showName	: Show control name					*/
/*  (returns)	: void		:					*/

void	KBCtrlTree::showName ()
{
#if	! __KB_RUNTIME
	if ((m_drow == 0) && (m_showing == KB::ShowAsDesign))
	{	
		clear () ;
		new QListViewItem (this, m_tree->getName()) ;
	}
#endif
}

/*  KBCtrlTree								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlTree::showAs
	(	KB::ShowAs	mode
	)
{
	KBControl::showAs (mode) ;

	clear		() ;
	setupProperties () ;

#if	! __KB_RUNTIME
	if ((mode == KB::ShowAsDesign) && (m_drow == 0))
	{	showName () ;
		return	    ;
	}
#endif
}

/*  KBCtrlTree								*/
/*  setCurrent	: Set current item					*/
/*  idx		: uint		   : Index required			*/
/*  child	: KBCtrlTreeItem * : Scan position in tree		*/
/*  (returns)	: bool		   : Item found				*/

bool	KBCtrlTree::setCurrent
	(	int		idx,
		KBCtrlTreeItem	*child
	)
{
	/* This is a depth-first recursive descend through the tree	*/
	/* looking for the item with the correct index. Once found we	*/
	/* unwind by returning true.					*/
	while (child != 0)
	{
		if (child->index() == idx)
		{
			setCurrentItem    (child) ;
			ensureItemVisible (child) ;
			return	true ;
		}

		/* If the child has no further chilren and it is not a	*/
		/* leaf then see if it contains the required index and	*/
		/* if so expand that branch.				*/
		if ((child->firstChild() == 0) && (child->index() < 0))
			if (child->containsIndex(idx))
				child->setOpen (true) ;


		if (setCurrent (idx, (KBCtrlTreeItem *)child->firstChild()))
		{
			return	true ;
		}

		child  = (KBCtrlTreeItem *)child->nextSibling() ;
	}

	return	false	;
}

/*  KBCtrlTree								*/
/*  closeAllTree: Close all tree branches				*/
/*  item	: QListViewItem * : Branch to close			*/
/*  (returns)	: void		  :					*/

void	KBCtrlTree::closeAllTree
	(	QListViewItem	*item
	)
{
	while (item != 0)
	{
		item->setOpen (false) ;
		closeAllTree  (item->firstChild()) ;
		item = item->nextSibling() ;
	}
}

/*  KBCtrlTree								*/
/*  setValue	: Set value						*/
/*  value	: const KBValue &: Value				*/
/*  (returns)	: void		 :					*/

void	KBCtrlTree::setValue
	(	const KBValue	&value
	)
{
	if (m_tree->setClose ()) closeAllTree (firstChild()) ;

	/* Set the current item, with a fallback on the first (null)	*/
	/* item if we cannot find the value.				*/
	if (!setCurrent
		(	m_tree->valueToItem (value, m_keyset),
			(KBCtrlTreeItem *)firstChild()
		))
	{
		setCurrentItem	 (firstChild()) ;
		ensureItemVisible(firstChild()) ;
	}

	KBControl::setValue (value) ;
}

/*  KBCtrlTree								*/
/*  getValue	: Get value						*/
/*  (returns)	: KBValue 	: Value					*/

KBValue	KBCtrlTree::getValue ()
{
	KBCtrlTreeItem *item = (KBCtrlTreeItem *)currentItem() ;

//	fprintf
//	(	stderr,
//		"KBCtrlTree::getValue: item=[%p] idx=[%d] value=[%s]\n",
//		(void *)item,
//		item == 0 ? 0  : item->index(),
//		item == 0 ? "" : item->index() < 0 ? "" : (cchar *)m_tree->itemToValue(item->index(), 0).getRawText()
//	)	;

	return	item 	      == 0 ? KBValue() :
		item->index() <  0 ? KBValue() :
		m_tree->itemToValue (item->index(), m_keyset) ;
}

/*  KBCtrlTree								*/
/*  clearValue	: Get value						*/
/*  query	: bool		: Clear for query			*/
/*  (returns)	: KBValue 	: Value					*/

void	KBCtrlTree::clearValue
	(	bool	query
	)
{
	setCurrentItem   (firstChild()) ;
	KBControl::clearValue (query) ;
}

/*  KBCtrlTree								*/
/*  changed	: See of field has changed				*/
/*  (returns)	: bool 		: Changed				*/

bool	KBCtrlTree::changed ()
{
	KBCtrlTreeItem *item = (KBCtrlTreeItem *)currentItem() ;
	return	item == 0 ?
			false :
			item->index() != (int)m_tree->valueToItem (getIniValue(), 0) ;
}

/*  KBCtrlTree								*/
/*  isEmpty	: See if control is empty				*/
/*  (returns)	: bool		: Empty					*/

bool	KBCtrlTree::isEmpty ()
{
	KBCtrlTreeItem *item = (KBCtrlTreeItem *)currentItem() ;

	fprintf
	(	stderr,
		"KBCtrlTree::isEmpty: item=[%p] idx=[%d]\n",
		(void *)item,
		item == 0 ? 0 : item->index()
	)	;

	return	(item == 0) || (item->index() <= 0) ;
}

/*  KBCtrlTree								*/
/*  isValid	: See if control is valid				*/
/*  allowNull	: bool		: True to ignore not-null check		*/
/*  (returns)	: bool		: Empty					*/

bool	KBCtrlTree::isValid
	(	bool		allowNull
	)
{
	/* Validity check based on index: all but the leading null	*/
	/* entry must be valid, and the validity of the leading null	*/
	/* depends on the not-null setting.				*/
//	if (!m_tree->checkValid (currentItem(), allowNull))
//	{
//		setError (m_tree->lastError()) ;
//		return	false	;
//	}

	return	true	;
}

/*  KBCtrlTree								*/
/*  giveFocus	: Give focus to the control				*/
/*  (returns)	: void							*/

void	KBCtrlTree::giveFocus ()
{
	RKListView::setFocus () ;
}

/*  KBCtrlTree								*/
/*  focusInEvent: Focus arrives at the control				*/
/*  e		: QFocusEvent *	: Focus event				*/
/*  (returns)	: void							*/

void	KBCtrlTree::focusInEvent
	(	QFocusEvent	*e
	)
{
	KBValue	curVal	;

	/* If in data mode and the tree is dynamic then we will need to	*/
	/* refresh the control. This is nasty ....			*/
	if ((m_showing == KB::ShowAsData) && m_tree->dynamic())
		switch (e->reason())
		{
			case QFocusEvent::Tab	   :
			case QFocusEvent::Shortcut :
			case QFocusEvent::Other    :
				/* These cases are easy, the one below	*/
				/* is not.				*/
				curVal	= getValue() ;
				m_tree->doRefresh (m_drow) ;
				setValue (curVal)    ;
				break ;

			case QFocusEvent::Mouse    :
				/* If focus arrives on account of the	*/
				/* mouse then we have to defer then	*/
				/* focus in and mouse button events or	*/
				/* we get an escape-only lockup.	*/
				curVal	= getValue() ;
				m_tree->doRefresh (m_drow) ;
				setValue (curVal)    ;

				m_deferFocus = new QFocusEvent(QEvent::FocusIn) ;
				QTimer::singleShot (250, this, SLOT(passFocus())) ;
				return	;

			default	:
				break	;
		}

	RKListView::focusInEvent (e) ;
}

/*  KBCtrlTree								*/
/*  event	: Event filter						*/
/*  e		: QEvent *	: Event					*/
/*  (returns)	: bool		: True if event consumed		*/

bool	KBCtrlTree::event
	(	QEvent	*e
	)
{
	/* If this is a mouse press and focus is being deferred then we	*/
	/* have to defer the mouse press as well.			*/
	if (e->type() == QEvent::MouseButtonPress)
		if (m_deferFocus != 0)
		{
			QMouseEvent *me = (QMouseEvent *)e ;
			m_deferMouse = new QMouseEvent (me->type(), me->pos(), me->globalPos(), me->button(), me->state()) ;
			return	true ;
		}

	return	RKListView::event (e) ;
}

/*  KBCtrlTree								*/
/*  passFocus	: Actually pass focus into the control			*/
/*  (returns)	: void		:					*/

void	KBCtrlTree::passFocus ()
{
//	fprintf
//	(	stderr,
//		"KBCtrlTree::passFocus: [%p] [%p]\n",
//		(void *)m_deferFocus,
//		(void *)m_deferMouse
//	)	;

	if (m_deferFocus != 0)
	{
		QFocusEvent::setReason	  (QFocusEvent::Mouse) ;
		RKListView ::focusInEvent (m_deferFocus) ;
		DELOBJ	(m_deferFocus) ;
	}

	if (m_deferMouse != 0)
	{
		RKListView::mousePressEvent (m_deferMouse) ;
		DELOBJ	(m_deferMouse) ;
	}
}

/*  KBCtrlTree								*/
/*  write	: Write link 						*/
/*  writer	: KBWriter *	: Output writer				*/
/*  rect	: QRect		: Field area				*/
/*  value	: KBValue &	: Value for reports			*/
/*  fSubs	: bool		: Substitution flag			*/
/*  extra	: int &		: Return extra space			*/
/*  (returns)	: bool		: Success				*/

bool	KBCtrlTree::write
	(	KBWriter	*writer,
		QRect		rect,
		const KBValue	&value,
		bool		fSubs,
		int		&extra
	)
{
	if (writer->asReport())
	{
		new KBWriterText
		(	writer,
			rect,
			m_tree->getPalette(true),
			m_tree->getFont   (true),
			value.getRawText  (),
			Qt::AlignLeft|Qt::AlignVCenter,
			false
		)	;

		extra	= 0	;
		return	true	;
	}

	return	KBControl::write (writer, rect, value, fSubs, extra) ;
}

/*  KBCtrlTree								*/
/*  setupProperties							*/
/*		: Set properties after design change			*/
/*  (returns)	: void		:					*/

void	KBCtrlTree::setupProperties ()
{
//	setReadOnly ((m_showing == KB::ShowAsData) && m_tree->isReadOnly()) ;
}

/*  KBCtrlTree								*/
/*  setUserFilter							*/
/*		: Set user filter for link query			*/
/*  filter	: const QString & : Filter expression			*/
/*  (returns)	: void		  :					*/

void	KBCtrlTree::setUserFilter
	(	const QString	&filter
	)
{
	m_userFilter	= filter ;
}

/*  KBCtrlTree								*/
/*  setUserSorting							*/
/*		: Set user sorting for link query			*/
/*  sorting	: const QString & : Sorting expression			*/
/*  (returns)	: void		  :					*/

void	KBCtrlTree::setUserSorting
	(	const QString	&sorting
	)
{
	m_userSorting	= sorting ;
}

/*  KBCtrlTree								*/
/*  reload	: Reload values list					*/
/*  (returns)	: void		  :					*/

void	KBCtrlTree::reload ()
{
	/* Note the current value. We will restore this after the	*/
	/* control has been updated.					*/
	KBValue	curVal	= m_curVal ;

	DELOBJ  (m_keyset) ;

	/* If either a user filter or sorting expression in set the we	*/
	/* do an explicit load for this control only, providing our	*/
	/* own keyset (which is retained).				*/
	if (!m_userFilter.isEmpty() || !m_userSorting.isEmpty())
	{
		KBLTValueSet	valset ;

		DELOBJ (m_keyset) ;
		m_keyset = new QStringList ;

		m_tree->loadValues
		(	m_userFilter,
			m_userSorting,
			*m_keyset,
			valset
		)	;

		loadDataValues (valset) ;		
	}
	else	m_tree->doRefresh (m_drow) ;

	setValue (curVal)    ;
}

/*  KBCtrlTree								*/
/*  getExtra	: Return "extra" value for current row			*/
/*  which	: bool		: Display or extra			*/
/*  index	: uint		: Extra value index			*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBCtrlTree::getExtra
	(	bool		which,
		uint		index
	)
{
	KBCtrlTreeItem	*item	= (KBCtrlTreeItem *)currentItem() ;

	if (item == 0)
		return	KBValue	() ;

	return	m_tree->itemToExtra (item->index(), which, index) ;
}
