/***************************************************************************
    file	         : kb_script.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	_KB_SCRIPT_H
#define	_KB_SCRIPT_H

#include	<stdarg.h>

#include	<qvaluelist.h>
#include	<qptrlist.h>
#include	<qdict.h>

#include	"kb_module.h"
#include	"kb_moduleitem.h"
#include	"kb_location.h"

#include	"libkbase_exports.h"


class	KBDebug		;
class	TKToggleAction	;
class	KBTextEdit	;
class	RKListBox	;


/*  KBScript								*/
/*  ------------							*/
/*  This node specifies information about database script modules used	*/
/*  by reports and forms.						*/

class 	LIBKBASE_API	KBScript : public KBModule
{
public	:

	KBScript	(KBNode *, const QDict<QString> &, bool * = 0) ;
	KBScript	(KBNode *, cchar    *) ;
	KBScript	(KBNode *, KBScript *) ;
	virtual 	~KBScript () ;

	virtual	KBNode		*replicate (KBNode *) ;

	virtual	KBScript	*isScript  () { return this ; }

	enum	ExeRC
	{
		ExeError,	/* Script error, caller to handle	*/
		ExeFail,	/* Script error, debugger handles	*/
		ExeAbort,	/* Aborted for nested error		*/
		ExeTrue,	/* Script returned true			*/
		ExeFalse	/* Script returned false		*/
	}	;

}	;


/*  KBScriptItem							*/
/*  ------------							*/
/*  Wrapper class round KBModuleItem which manufactures KBScript	*/
/*  instances.								*/

class 	LIBKBASE_API	KBScriptItem : public KBModuleItem
{
protected :

	virtual	KBModule *makeModule (KBNode *, const QString &) ;

public	:

	inline	KBScriptItem
		(	RKListBox	*listBox,
			KBScript	*script
		)
		:
		KBModuleItem (listBox, script)
	{
	}

	inline	KBScriptItem
		(	RKListBox	*listBox,
			const QString	&name
		)
		:
		KBModuleItem (listBox, name)
	{
	}

	virtual	~KBScriptItem () ;
}	;


/*  KBScriptDlg								*/
/*  -----------								*/
/*  This wraps the module dialog to implement the virtual methods that	*/
/*  instantiate module items (here as script items).			*/

class 	LIBKBASE_API	KBScriptDlg : public KBModuleDlg
{
protected :

	virtual	KBModuleItem	*makeModuleItem (RKListBox *, KBModule 	    *) ;
	virtual	KBModuleItem	*makeModuleItem (RKListBox *, const QString &) ;

public	:

	KBScriptDlg (QWidget *, KBNode *, QPtrList<KBModule> &) ;
	virtual ~KBScriptDlg () ;
}	;


/*  KBScriptCode							*/
/*  ------------							*/
/*  This class is a base class for chunks of script code which is	*/
/*  manipulated in-store (eg., embedded in forms or held in a database)	*/

class 	LIBKBASE_API	KBScriptCode
{
protected	:

	KBEvent		*m_inherit	;

public		:

	inline	KBScriptCode
		(	KBEvent		*inherit
		)
	{
		m_inherit = inherit	;

	}	;

	virtual~KBScriptCode ()
	{
	} 	;

	virtual	KBScript::ExeRC	execute
		(	KBNode		*,
			uint		,
			const KBValue	*,
			KBValue		&
		)	= 0 ;
	virtual	KBScript::ExeRC	execute
		(	KBNode		*,
			KBNode		*,
			const QString	&,
			uint		,
			const KBValue	*,
			KBValue		&
		)	= 0 ;

	virtual	void	setBreakpoints
		(	const QValueList<int>	&
		)	= 0 ;
}	;


/*  KBScriptObject							*/
/*  --------------							*/
/*  Instances of this class can be attached to a KBObject by a script	*/
/*  interpreter to provide object persistence. These instances are	*/
/*  destroyed on data/design transitions and on KBObject deletion.	*/

class	LIBKBASE_API	KBScriptObject
{
public	:

	KBScriptObject() ;
	virtual	~KBScriptObject	() ;
}	;


/*  KBScriptIF								*/
/*  ----------								*/
/*  An instance of this object is instantiated by each form in order	*/
/*  to access a scripting language. It contains methods to load and to	*/
/*  unload modules, and to executen a script function.			*/

class 	LIBKBASE_API	KBScriptIF : public QObject
{	
	Q_OBJECT

public	:

	KBScriptIF () ;
	virtual	~KBScriptIF () ;

	virtual	void	setApp  (KBNotifier *, KBCallback *) = 0 ;

	virtual	bool	compile
		(	KBLocation		&,
			QString			&,
			QString			&,
			KBError			&
		)	= 0 ;

	virtual	KBScriptCode   *compileFunc
		(	const QString		&,
			const QString		&,
			const QString		&,
			QString			&,
			QString			&,
			const QStringList	&,
			KBEvent			*,
			KBError			&
		)	= 0 ;

	virtual	KBScriptCode   *compileExpr
		(	const QString		&,
			const QString		&,
			QString			&,
			QString			&,
			const QStringList	&,
			KBError			&
		)	= 0 ;

	virtual	bool	load
		(	KBLocation		&,
			QString			&,
			QString			&,
			KBError			&
		)	= 0 ;

	virtual	KBScript::ExeRC	execute
		(	const QStringList	&,
			const QString		&,
			KBNode			*,
			uint,
			const KBValue		*,
			KBValue 		&
		)	= 0 ;

	virtual	QString	   ident	() = 0	;

	virtual	KBLocation exeError	(QString &, uint &, QString &) = 0 ;

	/* Deleting and renaming script files is done through the	*/
	/* interface so that we can handle cases such as EL, where	*/
	/* there may be a compiled code file as well as the source file	*/
	virtual	bool	unlink		(KBLocation   &, KBError &) = 0 ;
	virtual	bool	rename		(KBLocation   &, const QString &, KBError &) = 0 ;

	virtual	void	editorInit	(KBTextEdit     *) ;
	virtual	KBDebug	*showDebug	(TKToggleAction *) ;
	virtual	bool	debugScript	(KBLocation   &, KBError	&) ;

	static	void	addIdentString	(const QString &, const QString &) ;
	static	QString	getIdentStrings	() ;

	static	void	registerScript
		        (       const QString   &language,
        		        KBFactory       *factory,
                		const QString   &options
        		)	;

	static	uint	languageToCode
			(	const QString	&language
			) 	;
	static	uint	languageCount ()	;
}	;


/*  KBScriptError							*/
/*  -------------							*/
/*  This class is used to store script error information, and to fire	*/
/*  up the appropriate handling code.					*/

class	KBMacroExec	;
class	KBAttrExpr	;

class	LIBKBASE_API	KBScriptError
{
public	:

	enum	Handle
	{	Normal,
		Form,
		Report,
		Block,
		Hidden
	}	;

private	:

	enum	Type
	{	None,
		Event,
		Slot,
		Location,
		Macro,
		Error,
		Abort
	}	;


	Type		m_type		;	
	KBError		m_error		;
	KBEvent		*m_event	;
	KBSlot		*m_slot		;
	KBLocation	m_location	;
	KBAttrExpr	*m_expr		;

	KBObject	*m_object	;
	QString		m_errText	;
	int		m_errLno	;
	bool		m_debugged	;
	Handle		m_handle	;

	void		processError	(const QString &) ;

public	:

	KBScriptError	() ;
	KBScriptError	(const KBError &) ;
	KBScriptError	(const KBError &, KBEvent     *) ;
	KBScriptError	(const KBError &, KBSlot      *) ;
	KBScriptError	(const KBError &, KBObject    *, KBLocation  &, const QString &, int, bool) ;
	KBScriptError	(const KBError &, KBObject    *, KBMacroExec *) ;
	KBScriptError	(const KBError &, KBAttrExpr  *) ;

	inline	void	setHandler
		(	Handle	handle
		)
	{
		m_handle = handle ;
	}

	inline	const	KBError	&error	()
	{
		return	m_error	;
	}

	void		processError	() ;
	static	void	processError	(KBScriptError *, Handle) ;
}	;



extern LIBKBASE_API KBScriptIF *LinkKBScript (const QString &, KBError &) ;

#endif	// _KB_SCRIPT_H
