/***************************************************************************
    file	         : kb_formlist.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include 	<qcursor.h>
#include 	<qpopupmenu.h>


#include	"kb_classes.h"
#include	"kb_location.h"
#include	"kb_dbinfo.h"
#include	"kb_listitem.h"
#include	"kb_viewer.h"

#include	"kb_nodereg.h"
#include	"kb_parse.h"
#include	"kb_appptr.h"
#include	"kb_callback.h"
#include	"kb_file.h"
#include	"kb_errordlg.h"

#if		__KB_AUTOMOC
#include	"kb_formlist.moc"
#else
#include	"kb_formlist.h"
#endif

#include	"kb_formbase.h"
#include	"kb_formviewer.h"

#include	"tk_messagebox.h"

#if		! __KB_RUNTIME
#include	"kb_wizform.h"
#endif

#include	"rk_vbox.h"
#include	"tk_filedialog.h"


/*  KBFormList								*/
/*  KBFormList	: Constructor for form list widget			*/
/*  parent	: QWidget *	: Parent widget				*/
/*  dbInfo	: KBDBInfo *	: Database information object		*/
/*  (returns)	: KBFormList	:					*/

KBFormList::KBFormList
	(	QWidget		*parent,
		KBDBInfo	*dbInfo
	)
	:
	KBFileList
	(	parent,
		dbInfo,
		"Create new form",
		"form",
		"form",
		"Form name"
	)
{
	enableWizard () ;
	enableWeb    () ;
}

/*  KBFormList								*/
/*  ~KBFormList	: Destructor for form list widget			*/
/*  (returns)	:		:					*/

KBFormList::~KBFormList ()
{
}

/*  KBFormList								*/
/*  showServerMenu : Show popup menu for server entries			*/
/*  (returns)	   : void	:					*/

void	KBFormList::showServerMenu ()
{
	QPopupMenu pop ;
	pop.insertItem (TR("Cancel")) ;
	pop.insertSeparator () ;
	pop.insertItem (TR("&Reload form list"),      this, SLOT(reloadServer ())) ;
#if	! __KB_RUNTIME
	pop.insertItem (TR("&Copy to server ..."),    this, SLOT(copyToServer ())) ;
	pop.insertItem (TR("Save all to &files ..."), this, SLOT(saveAllToFile())) ;
	pop.insertItem (TR("Save all to &web ..."),   this, SLOT(saveAllToWeb ())) ;
#endif
	pop.exec       (QCursor::pos()) ;
	return	;
}

/*  KBFormList								*/
/*  showObjectMenu : Show popup menu for server entries			*/
/*  (returns)	   : void	:					*/

void	KBFormList::showObjectMenu ()
{
	QPopupMenu pop ;
	pop.insertItem (TR("Cancel")) ;
	pop.insertSeparator () ;
	pop.insertItem (TR("&Data view"),    this, SLOT(showAsData   ())) ;
#if	! __KB_RUNTIME
	pop.insertItem (TR("D&esign view"),  this, SLOT(showAsDesign ())) ;
	pop.insertItem (TR("&Rename form"),  this, SLOT(renameForm   ())) ;
	pop.insertItem (TR("De&lete form"),  this, SLOT(deleteForm   ())) ;
	pop.insertItem (TR("&Save to file"), this, SLOT(saveObjToFile())) ;
	pop.insertItem (TR("Save to &web"),  this, SLOT(saveObjToWeb ())) ;
#endif
	pop.exec       (QCursor::pos()) ;
}

#if	! __KB_RUNTIME

/*  KBFormList								*/
/*  createByWizard							*/
/*		: Handle request to create object using a wizard	*/
/*  item	: KBListItem *	: Selected item				*/
/*  (returns)	: void		:					*/

void	KBFormList::createByWizard
	(	KBListItem	*item
	)
{
	QString		server	= item->parent()->text(0) ;
	KBLocation	dummy	(m_dbInfo, "form", server, "") ;

	KBWizardForm	wizForm (dummy, server)	;
	if (!wizForm.exec()) return ;

	KB::ShowAs	showAs	;
	QString		name	;
	QString		form	= wizForm.create (name, showAs)	;

	if (form.isEmpty ()) return ;

	KBLocation location	(m_dbInfo, "form", server, name) ;
	KBError	   error	;

	if (!location.save
		(	QString::null,
			QString::null,
			form,
			error
		))
	{
		error.DISPLAY()	;
		return	;	;
	}

	QDict<QString>	empty	 ;
	KBCallback	*cb	= KBAppPtr::getCallback() ;
	if (cb->openObject (0, location, showAs, empty, error) == KB::ShowRCError)
		error.DISPLAY () ;
}
#endif

/*  KBFormList								*/
/*  renameForm	: Rename a form						*/
/*  (returns)	: void		:					*/

void	KBFormList::renameForm ()
{
#if	! __KB_RUNTIME
	rename	(m_curItem) ;
#endif
}

/*  KBFormList								*/
/*  deleteForm	: Delete a form						*/
/*  (returns)	: void		:					*/

void	KBFormList::deleteForm ()
{
#if	! __KB_RUNTIME
	delobj	(m_curItem) ;
#endif
}

#if	! __KB_RUNTIME

/*  KBFormList								*/
/*  saveObjToWeb: Save form for web access				*/
/*  location	: KBLocation &	  : Form location			*/
/*  dir		: const QString & : Target directory			*/
/*  actRC	: ActRC		  : Continutation code			*/
/*  (returns)	: ActRC		  : Continutation code			*/

KBMessageBoxYNAC::ActRC
	KBFormList::saveObjToWeb
	(	const KBLocation	&location,
		const QString		&dir,
		KBMessageBoxYNAC::ActRC	actRC
	)
{
	fprintf
	(	stderr,
		"KBFormList::saveObjToWeb: [%s]->[%s]\n",
		(cchar *)location.name(),
		(cchar *)dir
	)	;

	KBError	   error ;
	QByteArray doc   ;
	KBForm	   *form ;

	if (!location.contents (doc, error))
	{
		error.DISPLAY() ;
		return	KBMessageBoxYNAC::ActCancel ;
	}

	if ((form = KBOpenFormText (location, doc, error)) == 0)
	{
		error.DISPLAY() ;
		return	KBMessageBoxYNAC::ActCancel ;
	}

	QWidget	display	;
	RKVBox	layout	(&display) ;
	layout.setTracking () ;

	QSize	size	;
	if (form->showDesign (&layout, size) != KB::ShowRCDesign)
	{
		delete	form	;
		return	KBMessageBoxYNAC::ActCancel ;
	}

	display.resize (size) ;
	display.show() ;

	/* Pump the events so that all the resizes get done, else we	*/
	/* may not see all the controls correctly sized.		*/
	qApp->processEvents() ;

	
	QString	text   ;

	{
		KBErrorBlock eb (KBErrorBlock::Accrue) ;
		form->printNode(text, 0, true) ;
	}

	if (text.isNull())
	{
		TKMessageBox::sorry
		(	0,
			TR("Save to web"),
			TR("Document is null, not saving")
		)	;
		delete	form	;
		return	KBMessageBoxYNAC::ActCancel ;
	}

	QString	name	= dir + "/" + location.name() + ".rkl.frm" ;

	/* Check what to do if the file already exists. This depends	*/
	/* on the action return code passed in.				*/
	if (QFileInfo(name).exists())
		switch (actRC)
		{
			case KBMessageBoxYNAC::ActYes	:
			case KBMessageBoxYNAC::ActNo	:
				/* Already proceeding to the next	*/
				/* object, so repeat the options.	*/
				actRC = KBMessageBoxYNAC::query
			   		(	0,
						QString (TR("%1 already exists: overwrite?")).arg(name),
						TR("Save to file ....")
					)	;
				if (actRC == KBMessageBoxYNAC::ActCancel) return actRC ;
				if (actRC == KBMessageBoxYNAC::ActNo    ) return actRC ;
				break	;

			case KBMessageBoxYNAC::ActSingle:
				/* This is a single file case, so its a	*/
				/* simple yes/no decision.		*/
				if (TKMessageBox::questionYesNo
				   	(	0,
						QString (TR("%1 already exists: overwrite?")).arg(name),
						TR("Save to file ....")
					)
					!= TKMessageBox::Yes)
					return	KBMessageBoxYNAC::ActCancel ;
				break	;

			default	:
				/* Presumably allowing overwrite of all	*/
				/* objects, so just proceed.		*/
				break	;
		}


	KBFile	file	(name) ;
	if (!file.open (IO_WriteOnly|IO_Truncate))
	{
		file.lastError().DISPLAY() ;
		delete	form	;
		return	KBMessageBoxYNAC::ActCancel ;
	}

	QCString data = text.utf8() ;
	file.writeBlock (data.data(), data.length()) ;

	delete	form	;
	return	actRC	;
}
#endif
