/***************************************************************************
    file	         : kb_qryjoindlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qlabel.h>

#include	"kb_classes.h"

#if		__KB_AUTOMOC
#include	"kb_qryjoindlg.moc"
#else
#include 	"kb_qryjoindlg.h"
#endif

#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_gridbox.h"
#include	"rk_lineedit.h"
#include	"rk_combobox.h"
#include	"rk_pushbutton.h"


KBQryJoinDlg::KBQryJoinDlg
	(	const QString	&tblChildName,
		const QString	&fldChildName,
		const QString	&tblParentName,
		const QString	&fldParentName,
		const QString	&jtype,
		bool		canDelete,
		bool		canEdit
	)
	:
	KBDialog	("Join properties", true)
{
	RKVBox	  *layMain	= new RKVBox	(this) ;
	layMain->setTracking () ;

	RKGridBox *layProps	= new RKGridBox (2, layMain) ;
	RKHBox	  *layButt	= new RKHBox    (layMain)    ;

	new QLabel (TR("Many ..."   ), layProps) ;
	new QLabel (TR("... to one" ), layProps) ;

	RKLineEdit *tblParent	= new RKLineEdit(layProps) ;
	RKLineEdit *tblChild	= new RKLineEdit(layProps) ;

	m_fldParent		= new RKLineEdit(layProps) ;
	m_fldChild		= new RKLineEdit(layProps) ;

	new QLabel (TR("Join type"  ), layProps) ;

	m_cbJoin		= new RKComboBox(layProps) ;


	tblChild   ->setText	(tblChildName ) ;
	tblParent  ->setText	(tblParentName) ;
	m_fldChild ->setText	(fldChildName ) ;
	m_fldParent->setText	(fldParentName) ;
	tblChild   ->setReadOnly(true) ;
	tblParent  ->setReadOnly(true) ;
	m_fldChild ->setReadOnly(!canEdit) ;
	m_fldParent->setReadOnly(!canEdit) ;

	m_cbJoin ->insertItem	(TR("Inner"      )) ;
	m_cbJoin ->insertItem	(TR("Left Outer" )) ;
	m_cbJoin ->insertItem	(TR("Right Outer")) ;

	layButt  ->addFiller () ;

	RKPushButton *bOK	= new RKPushButton (TR("OK"),     layButt) ;
	RKPushButton *bCancel	= new RKPushButton (TR("Cancel"), layButt) ;
	connect	(bOK,     SIGNAL(clicked()), SLOT(accept     ())) ;
	connect	(bCancel, SIGNAL(clicked()), SLOT(reject     ())) ;

	if (canDelete)
	{
		RKPushButton *bDelete	= new RKPushButton (TR("Delete"), layButt) ;
		connect	(bDelete, SIGNAL(clicked()), SLOT(clickDelete())) ;
	}


//	fprintf
//	(	stderr,
//		"KBQryJoinDlg::KBQryJoinDlg: tblChildName=[%s]\n",
//		(cchar *)tblChildName
//	)	;
//	fprintf
//	(	stderr,
//		"KBQryJoinDlg::KBQryJoinDlg: tblParentName=[%s]\n",
//		(cchar *)tblParentName
//	)	;
//	fprintf
//	(	stderr,
//		"KBQryJoinDlg::KBQryJoinDlg: fldChildName=[%s]\n",
//		(cchar *)fldChildName
//	)	;
//	fprintf
//	(	stderr,
//		"KBQryJoinDlg::KBQryJoinDlg: fldParentName=[%s]\n",
//		(cchar *)fldParentName
//	)	;


	if	(jtype == "left" ) m_cbJoin->setCurrentItem (1) ;
	else if	(jtype == "right") m_cbJoin->setCurrentItem (2) ;
	else			   m_cbJoin->setCurrentItem (0) ;


	QPalette pal = tblChild->palette() ;
	pal.setColor (QColorGroup::Base,       QColor(192,192,192)) ;
	pal.setColor (QColorGroup::Background, QColor(192,192,192)) ;
 
	tblChild ->setPalette  (pal) ;
	tblParent->setPalette  (pal) ;

	if (!canEdit)
	{
		m_fldChild ->setPalette  (pal) ;
		m_fldParent->setPalette  (pal) ;
	}

	m_doDel	 = false ;

}

void	KBQryJoinDlg::clickDelete ()
{
	m_doDel	= true	;
	done	(1)	;
}

bool	KBQryJoinDlg::getResults
	(	QString		&jtype
	)
{
	switch (m_cbJoin->currentItem())
	{
		case 1  : jtype = "left"  ; break ;
		case 2  : jtype = "right" ; break ;
		default : jtype = ""	  ; break ;
	}

	return	m_doDel	;
}

QString	KBQryJoinDlg::childField  ()
{
	return	m_fldChild ->text () ;
}

QString	KBQryJoinDlg::parentField ()
{
	return	m_fldParent->text () ;
}
