/***************************************************************************
    file	         : kb_formproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>


#include	"kb_classes.h"
#include	"kb_form.h"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_itemproxy.h"
#include	"kb_blockproxy.h"
#include	"kb_formproxy.h"
#include	"kb_kjsregister.h"

#include	"kb_kjsscript.h"

#include	"kb_appptr.h"
#include	"kb_callback.h"



/*  KBKJSOpenInfo							*/
/*  -------------							*/
/*  Helper class. There is a bunch of common stuff needed to open a	*/
/*  form or report, gather this up here.				*/

class	KBKJSOpenInfo
{
public	:

	KBForm		*m_form		;
	QString		m_name		;
	QString		m_key		;
	QDict<QString>	m_dict		;
	QDict<QString>	m_results	;
	KBError		m_error		;

	KBKJSOpenInfo	(const char *, KJS::ExecState *, const KJS::List &, KBForm *) ;

	KB::ShowRC	exec	(KBLocation &) ;
}	;

/*  KBKJSOpenInfo							*/
/*  KBKJSOpenInfo							*/
/*		: Constructor for helper class				*/
/*  caller	: const char *	    : Called name for errors		*/
/*  exec	: KJS::ExecState *  : Execution state			*/
/*  args	: const KJS::List & : Argument state			*/
/*  form	: KBForm *	    : Invoking form			*/
/*  (returns)	: KBKJSOpenInfo	    : m_pyBase will be null on error	*/

KBKJSOpenInfo::KBKJSOpenInfo
	(	const char	*caller,
		KJS::ExecState	*exec,
		const KJS::List	&args,
		KBForm		*form
	)
{
	m_name	= kjsStringArg(exec, args, 0) ;
	m_key	= kjsStringArg(exec, args, 2) ;
	m_form	= form	;

	fprintf
	(	stderr,
		"KBKJSOpenInfo::KBKJSOpenInfo: al=%d name=[%s] key=[%s]\n",
		args.size(),
		(cchar *)m_name,
		(cchar *)m_key
	)	;

	if (args.size() >= 2)
	{
		KJS::Object params = KJS::Object::dynamicCast (args.at(1)) ;
		if (params.type() == KJS::ObjectType)
		{
			KJS::ReferenceList	   k = params.propList(exec) ;
			KJS::ReferenceListIterator i = k.begin() ;
			while (i != k.end())
			{
				KJS::Identifier ident = i->getPropertyName (exec) ;
				KJS::Value	value = i->getValue	   (exec) ;

				fprintf
				(	stderr,
					"KBKJSOpenInfo::KBKJSOpenInfo: [%s]->[%s]\n",
					(cchar *)ident.ustring ()    .qstring(),
					(cchar *)value.toString(exec).qstring()
				)	;

				m_dict.insert
				(	ident.ustring ().qstring(),
					new QString(value.toString(exec).qstring())
				)	;

				i++ ;
			}
		}
	}

	m_results.setAutoDelete (true) 	     ;
	m_dict   .setAutoDelete (true)	     ;
}

/*  KBKJSOpenInfo							*/
/*  exec	: Execute form or whatever				*/
/*  location	: KBLocation &	: Object to execute			*/
/*  (returns)	: KB::ShowRC	: Execution code			*/

KB::ShowRC
	KBKJSOpenInfo::exec
	(	KBLocation	&location
	)
{
	KBCallback *cb	= KBAppPtr::getCallback() ;

	if (cb == 0)
		return	KB::ShowRCError ;

	return	cb->openObject
			  (	m_form,
				location,
				KB::ShowAsData,
				m_dict,
				m_error,
				m_key,
				&m_results
			  )	;
}


/*  ------------------------------------------------------------------	*/

/*  KBFormProxy								*/
/*  KBFormProxy: Constructor for rekall form proxy			*/
/*  interp	: KJS::Interpreter *	: Intepreter object		*/
/*  form	: KBForm *		: Rekall form			*/
/*  (returns)	: KBFormProxy		:				*/

KBFormProxy::KBFormProxy
	(	KBKJSInterpreter	*interp,
		KBForm			*form
	)
	:
	KBBlockProxy	(interp, form),
	m_form		(form)
{
}

/*  KBFormProxy								*/
/* ~KBFormProxy	: Destructor for rekall form proxy			*/
/*  (returns)	:		:					*/

KBFormProxy::~KBFormProxy ()
{
}

/*  KBFormProxy								*/
/*  addBindings	: Add proxy bindings to specified object		*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  object	: KJS::Object &		: Object to which to add	*/
/*  (returns)	: void			:				*/

void	KBFormProxy::addBindings
	(	KJS::ExecState	*exec,
		KJS::Object	&object
	)
{
	static	KBMethodSpec methods	[] =
	{
	{	"openForm",	MethodImp::id_openForm,		"s|Os"	},
	{	"openReport",	MethodImp::id_openReport,	"s|Os"	},
	{	"close",	MethodImp::id_close,		"|n"	},
	{	"getOpener",	MethodImp::id_getOpener,	0	},
	{	"getParameter",	MethodImp::id_getParameter,	"s"	},
	{	"showMessage",	MethodImp::id_showMessage,	"s"	},
	{	0,		0,				0	}
	}	;

	KBBlockProxy::addBindings (exec, object) ;

	for (KBMethodSpec *ms = &methods[0] ; ms->m_method != 0 ; ms += 1)
		object.put
		(	exec,
			KJS::Identifier(ms->m_method),
			KJS::Object    (new MethodImp(ms, this))
		)	;
}

/*  KBFormProxy								*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBFormProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	QString	p = property.qstring() ;

	fprintf
	(	stderr,
		"KBFormProxy::get: [%s][%s] cs=%d\n",
		(cchar *)m_object->getName(),
		(cchar *)p,
		m_interp->clientSide()
	)	;

	return	KBBlockProxy::get (exec, property) ;
}

/*  KBFormProxy								*/
/*  put		: Set property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  value	: const KJS::Value &	: Value				*/
/*  attr	: int			:				*/
/*  (returns)	: void			:				*/


void	KBFormProxy::put
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property,
		const KJS::Value	&value,
		int			attr
	)
{
	KJS::ObjectImp::put (exec, property, value, attr) ;
}


/*  ------------------------------------------------------------------  */

/*  KBFormProxy								*/
/*  MethodImp	: 							*/
/*  MethodImp	: Object implementation					*/
/*  spec	: KBMethodSpec * : Specification			*/
/*  object	: KBFormProxy *  : Associated proxy			*/
/*  (return)	: MethodImp	 :					*/

KBFormProxy::MethodImp::MethodImp
	(	KBMethodSpec		*spec,
		const KBFormProxy 	*form
	)
	:
	KBBlockProxy::MethodImp	(spec, form),
	m_form			(form)
{
}


/*  MethodImp								*/
/*  call	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBFormProxy::MethodImp::call
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KBForm		*form	= m_form->m_form ;

	fprintf
	(	stderr,
		"KBFormProxy::MethodImp::call: id=%d name=%s\n",
		m_spec->m_id,
		m_spec->m_method
	)	;

	KJS::Value	error	= checkArgs (exec, args) ;
	if (error.isValid()) return error ;


	if (m_spec->m_id == id_openForm)
	{
		KBKJSOpenInfo info ("KBForm.openForm", exec, args, form) ;
		KB::ShowRC rc	;

		KBLocation location 
		(	form->getDocRoot()->getDBInfo  (),
			"form",
			form->getDocRoot()->getLocation().server(),
			info.m_name
		)	;

		rc	= info.exec (location) ;

		if (rc == KB::ShowRCError)
		{	info.m_error.DISPLAY()  ;
			return	KJS::Number (0) ;
		}

		// CONVERT RESULTS DICTIONARY
		return	KJS::Null()	;
	}

	if (m_spec->m_id == id_openReport)
	{
		KBKJSOpenInfo info ("KBForm.openReport", exec, args, form) ;
		KB::ShowRC rc	;

		KBLocation location 
		(	form->getDocRoot()->getDBInfo  (),
			"report",
			form->getDocRoot()->getLocation().server(),
			info.m_name
		)	;

		rc	= info.exec (location) ;

		if (rc == KB::ShowRCError)
		{	info.m_error.DISPLAY()  ;
			return	KJS::Number (0) ;
		}

		return	KJS::Number(0)	;
	}

	if (m_spec->m_id == id_close)
	{
		form->getDocRoot()->doRequestClose(kjsNumberArg(exec, args, 0)) ;
		return	KJS::Number(0)	;
	}

	if (m_spec->m_id == id_getOpener)
	{
		return	fromKBNode (exec, form->opener()) ;
	}

	if (m_spec->m_id == id_getParameter)
	{
		return	KJS::String (form->getDocRoot()->getParamValue (kjsStringArg(exec, args, 0))) ;
	}

	if (m_spec->m_id == id_showMessage)
	{
		form->getDocRoot()->doSetMessage(kjsStringArg(exec, args, 0)) ;
		return	KJS::Number(0)	;
	}
	
	return	KBBlockProxy::MethodImp::call (exec, object, args) ;
}


KBObjectProxy
	*makeFormProxy
	(	KBKJSInterpreter	*interp,
		KBNode			*node
	)
{
	return	new KBFormProxy (interp, (KBForm *)node) ;
}

void	registerKBForm	()
{
	registerClass	("KBForm", makeFormProxy) ;
}

