/***************************************************************************
    file	         : kb_kjdebug.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2000,2001,2002,2003 by theKompany.com
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qstring.h>

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>

#include	"kb_kjdebug.h"


extern	void	displayAllProperties
		(	KJS::Interpreter	*,
			KJS::Object		&
		)	;



KBKJSDebugger	*KBKJSDebugger::m_self		;


KBKJSDebugger::KBKJSDebugger ()
{
}

bool	KBKJSDebugger::sourceParsed
	(	KJS::ExecState		*exec,
		int			sourceId,
		const KJS::UString	&source,
		int			errorLine
	)
{
	fprintf
	(	stderr,
		"KBKJSDebugger::sourceParsed: id=%d eln=%d\n",
		sourceId,
		errorLine
	)	;

	m_lastSourceID	= sourceId	;
	m_lastLine	= errorLine	;

	return	true	;
}


bool	KBKJSDebugger::atStatement
	(	KJS::ExecState	*exec
	)
{
	m_lastSourceID	= exec->context().sourceId()	     ;
	m_lastLine	= exec->context().curStmtFirstLine() ;

	fprintf
	(	stderr,
		"KBKJSDebugger::atStatement: id=%d lno=%d\n",
		m_lastSourceID,
		m_lastLine
	)	;

	return	true	;
}

bool	KBKJSDebugger::enterContext
	(	KJS::ExecState	*exec
	)
{
	m_lastSourceID	= exec->context().sourceId()	     ;
	m_lastLine	= exec->context().curStmtFirstLine() ;

	fprintf
	(	stderr,
		"KBKJSDebugger::enterContext: id=%d lno=%d\n",
		m_lastSourceID,
		m_lastLine
		
	)	;


	return	true	;
}

bool	KBKJSDebugger::exitContext
	(	KJS::ExecState		*exec,
		const KJS::Completion	&completion
	)
{
//	m_lastSourceID	= exec->context().sourceId()	     ;
//	m_lastLine	= exec->context().curStmtFirstLine() ;

	fprintf
	(	stderr,
		"KBKJSDebugger::exitContext: id=%d lno=%d\n",
		exec->context().sourceId(),
		exec->context().curStmtFirstLine()
	)	;

	return	true	;
}

bool	KBKJSDebugger::exception
	(	KJS::ExecState		*exec,
		const KJS::Value	&value,
		bool			inTryCatch
	)
{
	KJS::Object exobj = KJS::Object::dynamicCast (value) ;
	KJS::Value  name  = exobj.get (exec, "name"   ) ;
	KJS::Value  msg   = exobj.get (exec, "message") ;

	displayAllProperties (exec->interpreter(), exobj) ;

	if (exec->context().imp() != 0)
	{
		m_lastSourceID	   = exec->context().sourceId()	        ;
		m_lastLine	   = exec->context().curStmtFirstLine() ;
	}

	m_exceptionName	   = name.toString(exec).qstring() ;
	m_exceptionMessage = msg .toString(exec).qstring() ;

	fprintf
	(	stderr,
		"KBKJSDebugger::exception: imp=%p id=%d lno=%d vt=%d name=%s msg=%s\n",
		(void *)exec->context().imp(),
		m_lastSourceID,
		m_lastLine,
		value.type(),
		m_exceptionName	  .latin1(),
		m_exceptionMessage.latin1()
	)	;

	return	true	;
}



KBKJSDebugger
	*KBKJSDebugger::self ()
{
	if (m_self == 0) m_self = new KBKJSDebugger() ;
	return	m_self ;
}
