/***************************************************************************
    file	         : kb_kjsscript.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>
#include	<math.h>

#include	<qintdict.h>

#include	"kb_classes.h"
#include	"kb_node.h"
#include	"kb_textedit.h"
#include	"kb_file.h"
#include	"kb_locator.h"

#include	"kb_kjsscript.moc"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_rekallmain.h"
#include	"kb_cookiejar.h"
#include	"kb_kjdebug.h"
#include	"kb_kjsregister.h"
#include	"kb_kjsfactory.h"

#include	"kb_libloader_kde.h"

#define	EXPRFUNC	"exprFunc"
#define	EXPRWRAP	"function " EXPRFUNC " () { return %1 ; }"



#define	__DISPLAYALLPROPERTIES(i,o)

#ifdef	__DISPLAYALLPROPERTIES
void	displayAllProperties
	(	KJS::ExecState		*exec,
		KJS::Object		&object
	)
{
	KJS::ReferenceList	   allProps = object.propList(exec, true) ;
	KJS::ReferenceListIterator iter	    = allProps.begin() ;

	fprintf
	(	stderr,
		"Identifier: %s: ic=%d: properties\n",
		object.className().qstring().latin1(),
		object.implementsCall()
	)	;

    	while (iter != allProps.end())
	{
		KJS::Reference  ref	= iter++ ;
		KJS::Identifier ident	= ref.getPropertyName(exec) ;
		KJS::Value	value	= object.get (exec, ident)  ;

		fprintf
		(	stderr,
			"Identifier: [%s] type=%d [%s]\n",
			ident.qstring().latin1(),
			value.type(),
			value.toString(exec).qstring().latin1()
		)	;
	}
}

void	displayAllProperties
	(	KJS::Interpreter	*interpreter,
		KJS::Object		&object
	)
{
	displayAllProperties (interpreter->globalExec(), object) ;
}

#endif

/*  kjsStringArg: Return argument as a string				*/
/*  exec	: ExecState *	  : Execution state			*/
/*  args	: const List &	  : Argument list			*/
/*  index	: int		  : Index into argument list		*/
/*  defval	: const QString & : Default value			*/
/*  (returns)	: QString	  : Argument as string			*/

QString	kjsStringArg
	(	ExecState	*exec,
		const List	&args,
		int		index,
		const QString	&defval
	)
{
	if (index >= args.size())
		return	defval	;

	/* KJS::Value::toString() throws an exception on objects (type	*/
	/* 6) so treat this specially. Not a usual case but might be	*/
	/* used for debugging.						*/
	const KJS::Value &v = args.at(index) ;
	if (v.type() == 6)
	{	KJS::Object o = KJS::Object::dynamicCast(v) ;
		if (o.implementsCall())
			return	"[function]" ;
		return	"[object]" ;
	}

	return	v.toString(exec).qstring() ;
}

/*  kjsBooleanArg							*/
/*		: Return argument as a boolean				*/
/*  exec	: ExecState *	: Execution state			*/
/*  args	: const List &	: Argument list				*/
/*  index	: int		: Index into argument list		*/
/*  defval	: bool		: Default value				*/
/*  (returns)	: bool		: Argument as boolean			*/

bool	kjsBooleanArg
	(	ExecState	*exec,
		const List	&args,
		int		index,
		bool		defval
	)
{
	if (index >= args.size())
		return	defval	;

	const KJS::Value &v = args.at(index) ;
	if (v.type() == 6)
		return	false	;

	return	v.toBoolean(exec) ;
}

/*  kjsNumberArg: Return argument as a number				*/
/*  exec	: ExecState *	: Execution state			*/
/*  args	: const List &	: Argument list				*/
/*  index	: int		: Index into argument list		*/
/*  defval	: int		: Default value				*/
/*  (returns)	: int		: Argument as number			*/

int	kjsNumberArg
	(	ExecState	*exec,
		const List	&args,
		int		index,
		int		defval
	)
{
	if (index >= args.size())
		return	defval	;

	const KJS::Value &v = args.at(index) ;
	if (v.type() == 6)
		return	0	;

	return	(int)v.toNumber(exec) ;
}


namespace KJS
{
	UString::UString
		(	const QString	&text
		)
	{
		uint	length	= text.length ()    ;
		UChar	*data	= new UChar[length] ;

		memcpy	(data, text.unicode(), length * sizeof(UChar)) ;
		rep	= UString::Rep::create(data, length) ;
	}

	QString	UString::qstring () const
	{
		return	QString	((QChar *)data(), size()) ;
	}
	QString	Identifier::qstring () const
	{
		return	QString	((QChar *)data(), size()) ;
	}
}


/*  ------------------------------------------------------------------  */


static	QIntDict<KBKJSScriptCode>	scriptCodeMap	;

static	QString				errMsg		;
static	uint				errLno		;
static	QString				errText		;
static	int				errSourceID	;

/*  ------------------------------------------------------------------  */



/*  KBKJSScriptObject							*/
/*  KBKJSScriptObject							*/
/*		: Constructor for persistent script object		*/
/*  kjsObject	: KBObjectProxy *   : Associated KJS objkect object	*/
/*  (returns)	: KBKJSScriptObject :					*/

KBKJSScriptObject::KBKJSScriptObject
	(	KBObjectProxy	*kjsObject
	)
	:
	m_kjsObject (kjsObject)
{
	m_kjsObject->ref () ;

//	fprintf
//	(	stderr,
//		"KBKJSScriptObject::KBKJSScriptObject: called for [%p] r=[%d]\n",
//		(void *)m_kjsObject,
//		m_kjsObject->refcount
//	)	;
}

/*  KBKJSScriptObject							*/
/*  KBKJSScriptObject							*/
/*		: Destructor for persistent script object		*/
/*  (returns)	:		:					*/

KBKJSScriptObject::~KBKJSScriptObject ()
{
//	fprintf
//	(	stderr,
//		"KBKJSScriptObject::~KBKJSScriptObject: called for [%p] r=[%d]\n",
//		(void *)m_kjsObject,
//		m_kjsObject->refcount
//	)	;

	if (m_kjsObject->deref ())
		delete	m_kjsObject ;
}

/*  KBKJSScriptObject							*/
/*  object	: Retrieve python object				*/
/*  (returns)	: PyObject *	: Python object				*/

KBObjectProxy
	*KBKJSScriptObject::object ()
{
	fprintf
	(	stderr,
		"KBKJSScriptObject::object: called for [%p] r=[%d]\n",
		(void *)m_kjsObject,
		m_kjsObject->refcount
	)	;

	/* Increment the reference count. The called must decrement it	*/
	/* later when the called no longer needs it.			*/
	m_kjsObject->ref ()	;
	return	m_kjsObject	;
}



/*  ------------------------------------------------------------------  */

/*  KBKJSScriptCode							*/
/*  KBKJSScriptCode							*/
/*		: Constructor for inline script object			*/
/*  interpreter	: KBKJSInterpreter *	: KJS interpreter object	*/
/*  script	: const QString &	: Script code			*/
/*  fname	: KJS::Identifier &	: Entry function name		*/
/*  location	: KBLocation &		: Nominal location		*/
/*  ok		: bool &		: Return success		*/
/*  (returns)	: KBKJSScriptCode	: Inline script object		*/

KBKJSScriptCode::KBKJSScriptCode
	(	KBKJSInterpreter	*interpreter,
		const QString		&script,
		KBEvent			*inherit,
		const KJS::Identifier	&fname,
		KBLocation		&location,
		bool			&ok
	)
	:
	KBScriptCode	(inherit),
	m_interpreter	(interpreter),
	m_location	(location)
{
	/* Evaluate the code. This should result in the named function	*/
	/* as a property of the interpreters global object. Check first	*/
	/* that the execution is successful.				*/
	KJS::Completion compVal	= m_interpreter->evaluate
				  (	script,
					m_interpreter->globalObject()
				  )	;

	fprintf
	(	stderr,
		"KBKJSScriptCode::KBKJSScriptCode: got ct=%d val=%d\n%s\n",
		(int)compVal.complType(),
		(int)compVal.value().isValid(),
		(cchar *)script
	)	;

	switch (compVal.complType())
	{
		case KJS::Normal	:
		case KJS::ReturnValue	:
			ok	= true	;
			break	;

		case KJS::Break		:
		case KJS::Continue	:
		case KJS::Throw		:
			ok	= false	;
			return	;

		default	:
			ok	= false	;
			return	;
	}

	if (!ok)
	{
		fprintf
		(	stderr,
			"KBKJSScriptCode::KBKJSScriptCode: unexpected return code\n"
		)	;
		return	;
	}

	__DISPLAYALLPROPERTIES (m_interpreter, m_interpreter->globalObject()) ;

	/* Retrieve the function; checking that we get a valid value,	*/
	/* that it is an object, and that it implements "call" (if not	*/
	/* then it is not a function).					*/
	KJS::Value	value	= m_interpreter->globalObject().get(m_interpreter->globalExec(), fname) ;
	if (!value.isValid())
	{
		fprintf
		(	stderr,
			"KBKJSScriptCode::KBKJSScriptCode: no function [%s]\n",
			fname.qstring().latin1()
		)	;
		ok	= false	;
		return	;
	}

	fprintf
	(	stderr,
		"KBKJSScriptCode::KBKJSScriptCode: %s: got vt=%d\n",
		(cchar *)fname.qstring(),
		(int    )value.type()
	)	;

	if (value.type() != KJS::ObjectType)
	{
		fprintf
		(	stderr,
			"KBKJSScriptCode::KBKJSScriptCode: not object type\n"
		)	;
		ok	= false	;
		return	;
	}

	m_kjsFunc = value.toObject(m_interpreter->globalExec()) ;

	if (!m_kjsFunc.implementsCall())
	{
		fprintf
		(	stderr,
			"KBKJSScriptCode::KBKJSScriptCode: call not implemented\n"
		)	;
		ok	= false	;
		return	;
	}

	m_sourceID = KBKJSDebugger::self()->lastSourceID() ;
	scriptCodeMap.insert (m_sourceID, this) ;

	ok	= true	;
}

/*  KBKJSScriptCode							*/
/*  ~KBKJSScriptCode							*/
/*		: Destructor for inline script object			*/
/*  (returns)	:		:					*/

KBKJSScriptCode::~KBKJSScriptCode ()
{
	scriptCodeMap.remove (m_sourceID) ;
}


/*  KBKJSScriptCode							*/
/*  execFunc	: Execute function					*/
/*  node	: KBNode *	  : Invoking rekall object		*/
/*  source	: KBNode *	  : Originator for slots		*/
/*  event	: const QString & : Event for slots			*/
/*  argc	: uint		  : Argument count			*/
/*  argv	: const KBValue * : Argument vector			*/
/*  resval	: KBValue *	  : Function return value		*/
/*  (return)	: KBScript::ExeRC : Execution return code		*/

KBScript::ExeRC
	KBKJSScriptCode::execFunc
	(	KBNode		*node,
		KBNode		*source,
		const QString	&event,
		uint		argc,
		const KBValue	*argv,
		KBValue		&resval
	)
{
	KJS::List   	args  	;
	KJS::ExecState	*exec	= m_interpreter->globalExec() ;
	KBObjectProxy	*proxy	= makeProxy (m_interpreter, node) ;

	if (proxy == 0)
	{
		KBError::EFault
		(	QString(TR("Failed to locate KJS class for %1")).arg(node->getElement()),
			QString::null,
			__ERRLOCN
		)	;
		return	KBScript::ExeError ;
	}

	KJS::Object object (proxy) 	  ;
	proxy->addBindings (exec, object) ;

	if (source != 0)
	{
		args.append (KBObjectProxy::fromKBValue (exec, source)) ;
		args.append (KBObjectProxy::fromKBValue (exec, event )) ;
	}

	/* Convert the KBValue argument values to KJS values and stack them	*/
	/* up in the argument list object.					*/
	for (uint idx = 0 ; idx < argc ; idx += 1)
		args.append (KBObjectProxy::fromKBValue (exec, argv[idx])) ;


	/* Invoke the function. The object is passed so that the function looks	*/
	/* like a method call with 'this' referring to the object.		*/
	KJS::Value 	value	= m_kjsFunc.call(exec, object, args) ;

	/* Decrement the reference count and delete it requested. I	*/
	/* *think* that this is right; actually, it will probably be	*/
	/* deleted only when the KJS::Object object disappears.		*/
	if (proxy->deref ()) delete proxy ;


	if (exec->hadException())
	{
		fprintf
		(	stderr,
			"KBKJSScriptCode::execute: Exception occurred\n"
		)	;

		errSourceID	= KBKJSDebugger::self()->lastSourceID	 () ;
		errLno		= KBKJSDebugger::self()->lastLine    	 () ;
		errText		= KBKJSDebugger::self()->exceptionMessage() ;
		errMsg		= "KJS Error:" +
				  KBKJSDebugger::self()->exceptionName	 () ;

		exec->clearException ()	   ;
		return	KBScript::ExeError ;
	}

	/* The value returned is converted back to a KBValue ..		*/
	switch (value.type())
	{
		case KJS::UnspecifiedType :
		case KJS::UndefinedType	  :
			fprintf
			(	stderr,
				"KBKJSScriptCode::execute: returned unspec/undef\n"
			)	;
			resval	= KBValue() ;
			break	;

		case KJS::NullType	  :
			fprintf
			(	stderr,
				"KBKJSScriptCode::execute: returned null\n"
			)	;
			resval	= KBValue() ;
			break	;

		case KJS::BooleanType	:
			fprintf
			(	stderr,
				"KBKJSScriptCode::execute: returned bool %d\n",
				value.toBoolean(exec)
			)	;
			resval	= KBValue
				  (	value.toBoolean(exec),
					&_kbBool
				  )	;
			break	;

		case KJS::StringType	:
			fprintf
			(	stderr,
				"KBKJSScriptCode::execute: returned string [%s]\n",
				(cchar *)value.toString (exec).qstring()
			)	;
			resval	= KBValue
				  (	value.toString (exec).qstring(),
					&_kbString
				  )	;
			break	;

		case KJS::NumberType	:
		{
			double	h ;
			double	d = value.toNumber (exec) ;
			double	f = modf (d, &h) ;

			fprintf
			(	stderr,
				"KBKJSScriptCode::execute: returned number [%f]\n",
				d
			)	;

			if (f == 0.0)
				resval	= KBValue ((int)h, &_kbFixed) ;
			else	resval	= KBValue (d,	   &_kbFloat) ;

			break	;
		}

		default	:
			fprintf
			(	stderr,
				"KBKJSScriptCode::execute: returned unknown\n"
			)	;
			resval	= KBValue() ;
			break	;
	}

	return	KBScript::ExeTrue ;
}

/*  KBKJSScriptCode							*/
/*  execute	: Execute function					*/
/*  node	: KBNode *	  : Invoking rekall object		*/
/*  argc	: uint		  : Argument count			*/
/*  argv	: const KBValue * : Argument vector			*/
/*  resval	: KBValue *	  : Function return value		*/
/*  (return)	: KBScript::ExeRC : Execution return code		*/

KBScript::ExeRC
	KBKJSScriptCode::execute
	(	KBNode		*node,
		uint		argc,
		const KBValue	*argv,
		KBValue		&resval
	)
{
	return	execFunc (node, 0, QString::null, argc, argv, resval) ;
}

KBScript::ExeRC
	KBKJSScriptCode::execute
	(	KBNode		*node,
		KBNode		*source,
		const QString	&event,
		uint		argc,
		const KBValue	*argv,
		KBValue		&resval
	)
{
	return	execFunc (node, source, event, argc, argv, resval) ;
}

void	KBKJSScriptCode::setBreakpoints
	(	const QValueList<int>	&
	)
{
}

/*  ------------------------------------------------------------------  */




KJS::Value
	KBKJSScriptIF::compileText
	(	KBLocation		&,
		const QString		&,
		QString			&,
		QString			&,
		KBError			&
	)
{
	return	KJS::Value() ;
}


KBScriptCode
	*KBKJSScriptIF::compileInline
	(	const QString		&,
		const QString		&,
		QString			&,
		QString			&,
		const QStringList	&,
		KBError			&
	)
{
	return	0 ;
}

KJS::Value
	KBKJSScriptIF::findFunction
	(	const QStringList	&,
		const QString		&
	)
{
	return	KJS::Value() ;
}


/*  KBKJSScriptIF							*/
/*  KBKJSScriptIF: Constructor for KJS scripting interface		*/
/*  options	 : const char *	 : Options				*/
/*  (returns)	 : KBKJSScriptIF :					*/

KBKJSScriptIF::KBKJSScriptIF
	(	const char	*options
	)
{
	bool	cs	= qstrcmp (options, "client") == 0 ;

	m_languageCode	= languageToCode (cs ? "kjs_cs" : "kjs") ;
	m_interpreter	= new KBKJSInterpreter (m_languageCode, cs) ;

	fprintf
	(	stderr,
		"KBPYScriptIF::KBPYScriptIF: m_languageCode=%d cs=%d\n",
		m_languageCode,
		cs
	)	;


	KJS::ExecState *exec = m_interpreter->globalExec  	    () ;
	KJS::Object    glob  = m_interpreter->globalObject	    () ;
	KJS::Object    objp  = m_interpreter->builtinObjectPrototype() ;

	glob.put
	(
		exec,
		"RekallMain",
		KJS::Object(new RekallMainObjectImp      (exec, static_cast<ObjectPrototypeImp *>(objp.imp()))),
		DontEnum
	)	;

	glob.put
	(
		exec,
		"RekallCookieJar",
		KJS::Object(new RekallCookieJarObjectImp (exec, static_cast<ObjectPrototypeImp *>(objp.imp()))),
		DontEnum
	)	;

	registerClasses () ;

	KBKJSDebugger::self()->attach(m_interpreter) ;

	QString	path    = locateFile ("appdata", QString("script/kjs/support.kjs")) ;
	if (path.isEmpty())
	{
		KBError::EError
		(	TR("Unable to locate kjs/support.kjs"),
			QString::null,
			__ERRLOCN
		)	;
		return	;
	}

	KBFile	support	(path) ;
	if (!support.open(IO_ReadOnly))
	{
		support.lastError().DISPLAY() ;
		return	;
	}

	QString		script	= support.readAll() ;
	KJS::Completion compVal	= m_interpreter->evaluate
				  (	script,
					m_interpreter->globalObject()
				  )	;

	fprintf
	(	stderr,
		"KBPYScriptIF::KBPYScriptIF:: got ct=%d val=%d\n",
		(int)compVal.complType(),
		(int)compVal.value().isValid()
	)	;
}

KBKJSScriptIF::~KBKJSScriptIF ()
{
//	scrIface 	= 0	;
	delete	m_interpreter	;
}



void	KBKJSScriptIF::setApp 
	(	KBNotifier	*,
		KBCallback	*
	)
{
}


bool	KBKJSScriptIF::compile
	(	KBLocation		&,
		QString			&,
		QString			&,
		KBError			&
	)
{
	return	false	;
}

/*  KBKJSScriptIF							*/
/*  compileFunc	: Compile an inline function				*/
/*  script	: const QString & : Function text			*/
/*  fname	: const QString & : Function name			*/
/*		: QString & 	  :					*/
/*		: QString & 	  :					*/
/*		: QStringList &   :					*/
/*  pError	: KBError &	  : Error return			*/
/*  (returns)	: KBScriptCode *  : Inl;ine script object		*/

KBScriptCode
	*KBKJSScriptIF::compileFunc
	(	const QString		&script,
		const QString		&,
		const QString		&fname,
		QString			&,
		QString			&,
		const QStringList	&,
		KBEvent			*inherit,
		KBError			&pError
	)
{
//	fprintf
//	(	stderr,
//		"KBKJSScriptIF::compileFunc: code\n%s\n",
//		(cchar *)script
//	)	;

	if (!m_interpreter->checkSyntax (script))
	{
		pError	= KBError
			  (	KBError::Error,
				"Syntax error",
				script,
				__ERRLOCN
			  )	;
		return	0 ;
	}

	KBLocation	inlineLoc
			(	0,
				"script",
				KBLocation::m_pInline,
				fname,
				script
			)	;

	bool	ok ;
	KBKJSScriptCode *code	= new KBKJSScriptCode
					(	m_interpreter,
						script,
						inherit,
						KJS::Identifier(fname.latin1()),
						inlineLoc,
						ok
					)	;

	if (!ok)
	{	delete	code	;
		return	0	;
	}

	return	code	;
}

KBScriptCode
	*KBKJSScriptIF::compileExpr
	(	const QString		&script,
		const QString		&,
		QString			&,
		QString			&,
		const QStringList	&,
		KBError			&pError
	)
{
	if (!m_interpreter->checkSyntax (script))
	{
		pError	= KBError
			  (	KBError::Error,
				"Syntax error",
				script,
				__ERRLOCN
			  )	;
		return	0 ;
	}

	KBLocation	inlineLoc
			(	0,
				"expr",
				KBLocation::m_pInline,
				EXPRFUNC,
				script
			)	;

	bool	ok ;
	KBKJSScriptCode *code	= new KBKJSScriptCode
				  (	m_interpreter,
					QString(EXPRWRAP).arg(script),
					0,
					EXPRFUNC,
					inlineLoc,
					ok
				  )	;

	if (!ok)
	{	delete	code	;
		return	0	;
	}

	return	code	;
}

bool	KBKJSScriptIF::load
	(	KBLocation		&location,
		QString			&eText,
		QString			&ePatt,
		KBError			&pError
	)
{
	QString	text	= location.contents  (pError) ;
	if (text.isNull())
	{
		return	false	;
	}

	KJS::Completion compVal	= m_interpreter->evaluate
				  (	text,
					m_interpreter->globalObject()
				  )	;

	fprintf
	(	stderr,
		"KBKJSScriptIF::load: got ct=%d val=%d\n%s\n",
		(int)compVal.complType(),
		(int)compVal.value().isValid(),
		(cchar *)text
	)	;

	switch (compVal.complType())
	{
		case KJS::Normal	:
		case KJS::ReturnValue	:
			return	true	;

		case KJS::Break		:
		case KJS::Continue	:
		case KJS::Throw		:
			return	false	;

		default	:
			break	;
	}

	fprintf
	(	stderr,
		"KBKJSScriptIF::load: unexpected return code\n"
	)	;
	return	false	;
}

KBScript::ExeRC
	KBKJSScriptIF::execute
	(	const QStringList	&,
		const QString		&,
		KBNode			*,
		uint,
		const KBValue		*,
		KBValue 		&
	)
{
	return	KBScript::ExeError ;
}

KBLocation
	KBKJSScriptIF::exeError
	(	QString		&_errMsg,
		uint		&_errLno,
		QString 	&_errText
	)
{
	_errMsg	 = errMsg  ;
	_errLno	 = errLno  ;
	_errText = errText ;

	/* If the error source is empty then it must be a failure	*/
	/* to locate the error. In this case we have nothing to go on.	*/
	if (errSourceID == 0)
		return	KBLocation
			(	0,
				"script",
				KBLocation::m_pInline,
				"[unknown]",
				"kjs"
			) ;

	/* If the error was in inline code then return just enough to	*/
	/* correspond to this. The original caller should know where	*/
	/* it is.							*/
	if (scriptCodeMap.find (errSourceID) != 0)
		return	KBLocation
			(	0,
				"script",
				KBLocation::m_pInline,
				KBLocation::m_pInline,
				"kjs"
			) ;

	/* OK, got a real looking identifier. It should exist in the	*/
	/* module map, in which case return the location for the map	*/
	/* entry.							*/
//	KBPYModule *module ;
//	if ((module = loadMap.find (errIdent)) != 0)
//	{	fprintf	(stderr, "KBPYScriptIF::exeError -> [%s]\n",
//				 (cchar *)module->location.ident()) ;
//		return	module->location ;
//	}

	/* Should not get here!						*/
	fprintf
	(	stderr,
		"KBKJSScriptIF::exeError failed for [%d]\n",
		errSourceID
	)	;
	return	KBLocation () ;
}

bool	KBKJSScriptIF::unlink
	(	KBLocation	&,
		KBError		&
	)
{
	return	false	;
}

bool	KBKJSScriptIF::rename
	(	KBLocation	&,
		const QString	&,
		KBError		&
	)
{
	return	false	;
}

void	KBKJSScriptIF::editorInit
	(	KBTextEdit	*editor
	)
{
	editor->setHighlight ("kjs") ;
}

KBDebug	*KBKJSScriptIF::showDebug
	(	TKToggleAction *
	)
{
	return	0 ;
}

bool	KBKJSScriptIF::debugScript
	(	KBLocation  	&,
		KBError 	&
	)
{
	return	false	;
}


QString	KBKJSScriptIF::ident ()
{
	return	QString("KJS: %1%2 %3")
			.arg(getBuildRelease())
			.arg(getBuildBuild  ())
			.arg(getBuildTime   ()) ;
}

KBFACTORYIMPL
(	KBKJSScriptFactory,
	script_kjs
)

/*  KBPYScriptFactory							*/
/*		: Get an instance of the appropriate interpreter	*/
/*  parent	: QObject *	: Parent object				*/
/*  (returns)	: QObject *	: Interpreter instance			*/

QObject	*KBKJSScriptFactory::create
	(	QObject		  *,
		cchar		  *,
		cchar		  *options,
		const QStringList &
	)
{
	return	new KBKJSScriptIF (options) ;
}

QString	KBKJSScriptFactory::ident ()
{
	return	QString("KJS: %1%2 %3")
			.arg(getBuildRelease())
			.arg(getBuildBuild  ())
			.arg(getBuildTime   ()) ;
}
