/***************************************************************************
    file	         : kb_linktreeproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>

#include	"kb_classes.h"
#include	"kb_linktree.h"
#include	"kb_block.h"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_itemproxy.h"
#include	"kb_linktreeproxy.h"
#include	"kb_kjsregister.h"

#include	"kb_kjsscript.h"



/*  KBLinkTreeProxy							*/
/*  KBLinkTreeProxy							*/
/*		: Constructor for rekall link/tree proxy		*/
/*  interp	: KJS::Interpreter *	: Intepreter object		*/
/*  linktree	: KBLinkTree *		: Rekall link/tree control	*/
/*  (returns)	: KBLinkTreeProxy	:				*/

KBLinkTreeProxy::KBLinkTreeProxy
	(	KBKJSInterpreter	*interp,
		KBLinkTree		*linktree
	)
	:
	KBItemProxy	(interp, linktree),
	m_linktree	(linktree)
{
}

/*  KBLinkTreeProxy							*/
/* ~KBLinkTreeProxy							*/
/*		: Destructor for rekall link/tree proxy			*/
/*  (returns)	:		:					*/

KBLinkTreeProxy::~KBLinkTreeProxy ()
{
}

/*  KBLinkTreeProxy							*/
/*  addBindings	: Add proxy bindings to specified link/tree		*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  object	: KJS::Object &		: Object to which to add	*/
/*  (returns)	: void			:				*/

void	KBLinkTreeProxy::addBindings
	(	KJS::ExecState	*exec,
		KJS::Object	&object
	)
{
	static	KBMethodSpec methods	[] =
	{
	{	"getExtra",		MethodImp::id_getExtra,	"nn"	},
	}	;

	KBItemProxy::addBindings (exec, object) ;

	for (KBMethodSpec *ms = &methods[0] ; ms->m_method != 0 ; ms += 1)
		object.put
		(	exec,
			KJS::Identifier(ms->m_method),
			KJS::Object    (new MethodImp(ms, this))
		)	;
}

/*  KBLinkTreeProxy							*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBLinkTreeProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	QString	p = property.qstring() ;

	fprintf
	(	stderr,
		"KBLinkTreeProxy::get: [%s] cs=%d\n",
		(cchar *)p,
		m_interp->clientSide()
	)	;

	return	KBItemProxy::get (exec, property) ;
}

/*  KBLinkTreeProxy							*/
/*  put		: Set property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  value	: const KJS::Value &	: Value				*/
/*  attr	: int			:				*/
/*  (returns)	: void			:				*/

void	KBLinkTreeProxy::put
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property,
		const KJS::Value	&value,
		int			attr
	)
{
	KBItemProxy::put (exec, property, value, attr) ;
}




/*  ------------------------------------------------------------------  */

KBLinkTreeProxy::MethodImp::MethodImp
	(	KBMethodSpec		*spec,
		const KBLinkTreeProxy	*linktree
	)
	:
	KBItemProxy::MethodImp		(spec, linktree),
	m_linktree			(linktree)
{
}

KJS::Value
	KBLinkTreeProxy::MethodImp::call
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KBLinkTree *linktree = m_linktree->m_linktree ;

	fprintf
	(	stderr,
		"KBLinkTreeProxy::MethodImp::call: [%s] for [%d]\n",
		(cchar *)linktree->getName(),
		m_spec->m_id
	)	;

	KJS::Value	error	= checkArgs (exec, args) ;
	if (error.isValid()) return error ;

	switch (m_spec->m_id)
	{
		case id_getExtra	:
			return	KJS::String
				(	linktree->getExtra
					(	getCurQRow(args.at(0).toInteger(exec)),
						false,
						args.at(1).toInteger(exec)
					)
					.getRawText()
				)	;

		default	:
			break	;
	}

	return	KBItemProxy::MethodImp::call (exec, object, args) ;
}



KBObjectProxy
	*makeLinkTreeProxy
	(	KBKJSInterpreter	*interp,
		KBNode			*node
	)
{
	return	new KBLinkTreeProxy (interp, (KBLinkTree *)node) ;
}

static	const char	*linktreeAlias[] =
{	
	"KBLink",
	"KBTree",
	0
}	;

void	registerKBLinkTree	()
{
	registerClass	("KBLinkTree", makeLinkTreeProxy, linktreeAlias) ;
}

