/***************************************************************************
    file	         : kb_choiceproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>

#include	"kb_classes.h"
#include	"kb_richtext.h"
#include	"kb_block.h"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_itemproxy.h"
#include	"kb_richtextproxy.h"
#include	"kb_kjsregister.h"

#include	"kb_kjsscript.h"



/*  KBRichTextProxy							*/
/*  KBRichTextProxy							*/
/*		: Constructor for rekall rich text proxy		*/
/*  interp	: KJS::Interpreter *	: Intepreter object		*/
/*  choice	: KBRichText *		: Rekall choice			*/
/*  (returns)	: KBRichTextProxy	:				*/

KBRichTextProxy::KBRichTextProxy
	(	KBKJSInterpreter	*interp,
		KBRichText		*richtext
	)
	:
	KBItemProxy	(interp, richtext),
	m_richtext	(richtext)
{
}

/*  KBRichTextProxy							*/
/* ~KBRichTextProxy							*/
/*		: Destructor for rekall rich text proxy			*/
/*  (returns)	:		:					*/

KBRichTextProxy::~KBRichTextProxy ()
{
}

/*  KBRichTextProxy							*/
/*  addBindings	: Add proxy bindings to specified richtext		*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  object	: KJS::Object &		: Object to which to add	*/
/*  (returns)	: void			:				*/

void	KBRichTextProxy::addBindings
	(	KJS::ExecState	*exec,
		KJS::Object	&object
	)
{
	static	KBMethodSpec methods	[] =
	{
	{	"makeSimpleLink",	MethodImp::id_makeSimpleLink,	0	},
	{	0,			0,				0	}
	}	;

	KBItemProxy::addBindings (exec, object) ;

	for (KBMethodSpec *ms = &methods[0] ; ms->m_method != 0 ; ms += 1)
		object.put
		(	exec,
			KJS::Identifier(ms->m_method),
			KJS::Object    (new MethodImp(ms, this))
		)	;
}

/*  KBRichTextProxy							*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBRichTextProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	QString	p = property.qstring() ;

	fprintf
	(	stderr,
		"KBRichTextProxy::get: [%s] cs=%d\n",
		(cchar *)p,
		m_interp->clientSide()
	)	;

	return	KBItemProxy::get (exec, property) ;
}

/*  KBRichTextProxy							*/
/*  put		: Set property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  value	: const KJS::Value &	: Value				*/
/*  attr	: int			:				*/
/*  (returns)	: void			:				*/

void	KBRichTextProxy::put
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property,
		const KJS::Value	&value,
		int			attr
	)
{
	KBItemProxy::put (exec, property, value, attr) ;
}




/*  ------------------------------------------------------------------  */

KBRichTextProxy::MethodImp::MethodImp
	(	KBMethodSpec		*spec,
		const KBRichTextProxy	*richtext
	)
	:
	KBItemProxy::MethodImp		(spec, richtext),
	m_richtext			(richtext)
{
}

KJS::Value
	KBRichTextProxy::MethodImp::call
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KBRichText *richtext = m_richtext->m_richtext ;

	fprintf
	(	stderr,
		"KBRichTextProxy::MethodImp::call: [%s] for [%d]\n",
		(cchar *)richtext->getName(),
		m_spec->m_id
	)	;

	switch (m_spec->m_id)
	{
		case id_makeSimpleLink :
			return	KJS::String
				(	QString("<a href='link:///%1'>%2</a>")
						.arg(kjsStringArg (exec, args, 1))
						.arg(kjsStringArg (exec, args, 0))
				)	;
		default	:
			break	;
	}

	return	KBItemProxy::MethodImp::call (exec, object, args) ;
}



KBObjectProxy
	*makeRichtextProxy
	(	KBKJSInterpreter	*interp,
		KBNode			*node
	)
{
	return	new KBRichTextProxy (interp, (KBRichText *)node) ;
}

void	registerKBRichText	()
{
	registerClass	("KBRichText", makeRichtextProxy) ;
}

