/***************************************************************************
    file	         : kb_stackproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qvariant.h>

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>
#include	<kjs/array_instance.h>


#include	"kb_classes.h"
#include	"kb_stack.h"
#include	"kb_config.h"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_stackproxy.h"
#include	"kb_kjsregister.h"

#include	"kb_kjsscript.h"




/*  ------------------------------------------------------------------	*/

/*  KBStackPageProxy							*/
/*  KBStackPageProxy							*/
/*		: Constructor for rekall stack page proxy		*/
/*  interp	: KBKJSInterpreter *	: Intepreter object		*/
/*  stackPage	: KBStackPage *		: Rekall stack page		*/
/*  (returns)	: KBStackPageProxy	:				*/

KBStackPageProxy::KBStackPageProxy
	(	KBKJSInterpreter	*interp,
		KBStackPage		*stackPage
	)
	:
	KBObjectProxy	(interp, stackPage),
	m_stackPage	(stackPage)
{
}

/*  KBStackPageProxy							*/
/* ~KBStackPageProxy							*/
/*		: Destructor for rekall object proxy			*/
/*  (returns)	:		:					*/

KBStackPageProxy::~KBStackPageProxy ()
{
}

/*  KBStackPageProxy							*/
/*  addBindings	: Add proxy bindings to specified object		*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  object	: KJS::Object &		: Object to which to add	*/
/*  (returns)	: void			:				*/

void	KBStackPageProxy::addBindings
	(	KJS::ExecState	*exec,
		KJS::Object	&object
	)
{
	static	KBMethodSpec methods [] =
	{
	{	"setCurrent",	MethodImp::id_setCurrent,	0	},
	{	0,		0,				0	}
	}	;

	KBObjectProxy::addBindings (exec, object) ;

	for (KBMethodSpec *ms = &methods[0] ; ms->m_method != 0 ; ms += 1)
		object.put
		(	exec,
			KJS::Identifier(ms->m_method),
			KJS::Object    (new MethodImp(ms, this))
		)	;
}

/*  KBStackPageProxy							*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBStackPageProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	return	KBObjectProxy::get (exec, property) ;
}

/*  KBStackPageProxy							*/
/*  put		: Set property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  value	: const KJS::Value &	: Value				*/
/*  attr	: int			:				*/
/*  (returns)	: void			:				*/

void	KBStackPageProxy::put
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property,
		const KJS::Value	&value,
		int			attr
	)
{
	KBObjectProxy::put (exec, property, value, attr) ;
}


/*  ------------------------------------------------------------------  */

/*  KBStackPageProxy							*/
/*  MethodImp	: 							*/
/*  MethodImp	: Object implementation					*/
/*  spec	: KBMethodSpec *     : Specification			*/
/*  stackPage	: KBStackPageProxy * : Associated proxy			*/
/*  (return)	: MethodImp	     :					*/

KBStackPageProxy::MethodImp::MethodImp
	(	KBMethodSpec		*spec,
		const KBStackPageProxy	*stackPage
	)
	:
	KBObjectProxy::MethodImp(spec, stackPage),
	m_stackPage		(stackPage)
{
}

/*  MethodImp								*/
/*  call	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBStackPageProxy::MethodImp::call
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KBStackPage *page = m_stackPage->m_stackPage ;

	fprintf
	(	stderr,
		"KBStackPageProxy::MethodImp::call: m_id=%d\n",
		m_spec->m_id
	)	;

	switch (m_spec->m_id)
	{
		case id_setCurrent	:
			fprintf
			(	stderr,
				"KBStackPageProxy::MethodImp::call: setCurrent: [%s.%s]\n",
				(cchar *)page->getElement(),
				(cchar *)page->getName   ()
			)	;
			page->setCurrent () ;
			return	KJS::Null() ;

		default	:
			break	;
	}

	return	KBObjectProxy::MethodImp::call (exec, object, args) ;
}


KBObjectProxy
	*makeStackPageProxy
	(	KBKJSInterpreter	*interp,
		KBNode			*node
	)
{
	return	new KBStackPageProxy (interp, (KBStackPage *)node) ;
}

void	registerKBStackPage	()
{
	registerClass	("KBStackPage", makeStackPageProxy) ;
}

