/***************************************************************************
    file	         : copier.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<errno.h>

#if		__KB_KDE
#include	<kapp.h>
#include	<klocale.h>
#include	<kaboutdata.h>
#include	<kcmdlineargs.h>
#endif

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_database.h"
#include	"kb_dbinfo.h"
#include	"kb_serverinfo.h"
#include	"kb_location.h"
#include	"kb_copybase.h"
#include	"kb_copyfile.h"
#include	"kb_copytable.h"
#include	"kb_copyxml.h"
#include	"kb_dom.h"

#define		SKELL	"copier"
#define		SKELU	"Copier"

static	FILE	*errfile	;

void	showError
	(	cchar		*where,
		KBError		&error
	)
{
	fprintf	(errfile, "Error: %s\n", where) ;
	fprintf	(errfile, "     : %s\n", (cchar *)error.getMessage()) ;
	fprintf	(errfile, "     : %s\n", (cchar *)error.getDetails()) ;
}


bool	execCopy
	(	KBCopyBase	*srce,
		KBCopyBase	*dest
	)
{
	KBError			error	;
	const QDict<QString>	pDict	;

	if (!srce->prepare (pDict, dest))
	{
		error = srce->lastError() ;
		showError ("Prepare source", error) ;
		return	false ;
	}
	if (!dest->prepare (pDict, srce))
	{
		error = dest->lastError() ;
		showError ("Prepare destination", error) ;
		return	false ;
	}

	int	srceCols = srce->getNumCols() ;
	int	destCols = dest->getNumCols() ;
	fprintf	(stderr, "Srce NumCols: %d\n", srceCols) ;
	fprintf	(stderr, "Dest NumCols: %d\n", destCols) ;

	int	numCols	 = srceCols > 0 ? srceCols : destCols > 0 ? destCols : 500 ;

	if (!dest->putRow (0, 0))
	{
		error = dest->lastError() ;
		showError ("Init destination", error) ;
		return	false	;
	}

	KBValue	values[numCols + 1] ;
	int	nRows = srce->execute (dest, values, numCols, 0) ;
	if (nRows < 0)
	{
		error = srce->lastError() ;
		showError ("Copy", error) ;
		return	false	;
	}

	QString	rep	;
	dest->finish	(rep)	;

	fprintf	(stderr, "Copy NumRows: %d\n", nRows) ;
	return	true	;
}

void	addXMLFields
	(	KBCopyXML		*copy,
		const QDomElement	&elem
	)
{
	for (QDomNode node = elem .firstChild () ;
			     !node.isNull     () ;
		      node = node .nextSibling() )
	{
		QDomElement elem = node.toElement () ;
		if (elem.isNull())
			continue ;

		if (elem.nodeName() == "field")
		{
			copy->addField (elem.attribute("name"), elem.attribute("asattr").toInt()) ;
			continue ;
		}
	}
}

bool	doCopy
	(	const QDomElement &elem
	)
{
	KBCopyBase	*srce	= 0	;
	KBCopyBase	*dest	= 0	;
	KBServerInfo    *svInfo	= 0	;
	KBServer        *server = 0	;
	KBDBInfo	dbInfo	("Copier")	;
	KBError         error   	;

	for (QDomNode node = elem .firstChild () ;
			     !node.isNull     () ;
		      node = node .nextSibling() )
	{
		QDomElement elem = node.toElement () ;
		if (elem.isNull())
			continue ;

		KBCopyBase *base = 0	;

		if	(elem.nodeName() == "serverinfo")
		{
			svInfo	= new KBServerInfo (0, elem) ;
			server	= svInfo->getServer (error)  ;
			if (server == 0)
			{	error.DISPLAY() ;
				return	false	;
			}
			dbInfo.insert (svInfo)	;
		}
		else if	(elem.nodeName() == "file")
		{
			KBLocation	locn	;
			KBCopyFile	*copy = new KBCopyFile (srce == 0, locn) ;
			copy->setFile	(elem.attribute("name")) ;
			copy->setDelim	("," )	;
			copy->setQualif	("\"")	;
			copy->setErrOpt	(0)	;
			copy->setHeader	(false, 0) ;
			base	= copy	;
		}
		else if	(elem.nodeName() == "xml")
		{
			KBLocation	locn	;
			KBCopyXML	*copy = new KBCopyXML (srce == 0, locn) ;
			copy->setFile	(elem.attribute("name")) ;
			copy->setMainTag("mainTag" ) ;
			copy->setRowTag ("rowTag"  ) ;
			addXMLFields    (copy, elem) ;
			copy->setErrOpt	(0)	;
			base	= copy	;
		}
		else if	(elem.nodeName() == "table")
		{
			KBLocation	locn
					(	&dbInfo,
						"table",
						svInfo->serverName(),
						elem.attribute("name")
					)	;
			KBCopyTable	*copy	= new KBCopyTable (srce == 0, locn) ;
			copy->setServer (svInfo->serverName  ()) ;
			copy->setTable	(elem.attribute("name")) ;

			DOMITER_BEGIN(elem,"field",field)
			{
				copy->addField(field.attribute("name")) ;
			}
			DOMITER_END
			base	= copy	;

		}

		if	(srce == 0) srce = base ;
		else if (dest == 0) dest = base ;
		else		    delete base	;
	}

	if ((srce == 0) || (dest == 0))
	{
		fprintf
		(	errfile,
			"xmlFile: %s: underspecified copier\n",
			elem.attribute("name").latin1()
		)	;
		return	false	;
	}

	bool	rc	= execCopy (srce, dest) ;

	delete	srce	;
	delete	dest	;
	if (svInfo != 0) delete svInfo	;
	return	rc	;
}

bool	xmlTest
	(	const QString	&xmlName
	)
{
	QFile	xmlFile	(xmlName) ;
	if (!xmlFile.open (IO_ReadOnly))
	{
		fprintf	(errfile, "xmlFile: %s: %s\n", (cchar *)xmlName, strerror(errno)) ;
		return	false	;
	}

	QDomDocument doc	;
	if (!doc.setContent (&xmlFile))
	{
		fprintf	(errfile, "xmlFile: %s: cannot parse\n", (cchar *)xmlName) ;
		return	false	;
	}

	QDomElement  docElem	= doc.documentElement() ;
	for (QDomNode node = docElem.firstChild	() ;
			     !node  .isNull	() ;
		      node = node   .nextSibling() )
	{
		QDomElement elem = node.toElement () ;
		if (elem.isNull())
			continue ;

		if (elem.nodeName() == "copy")
			doCopy (elem) ;
	}

	return	true	;
}
int	main
	(	int	argc,
		char	*argv[]
	)
{
	cchar	*sstde	= getenv ("REKALL_SHOWSTDERR") ;

	if ((sstde == 0) || (strcmp(sstde, "YES") != 0))
	{
		freopen	("/dev/null", "w", stderr) ;
		errfile	= fopen ("/dev/tty",  "w") ;
	}
	else	errfile	= stderr ;

#if	__KB_KDE
	char *dummy[2]	;

	dummy[0] = (char *)"copier" ;
	dummy[1] = 0	;

	KAboutData about (__KB_APPNAME,
			  I18N_NOOP("RekallCopierTest"),
			  "0.0.0",
			  I18N_NOOP("Rekall: Copier test harness"),
			  KAboutData::License_Unknown,
			  "(c) 2000,2001,2002 by theKompany.com",
			  0,
			  "http://www.rekall.a-i-s.co.uk",
			  "mike@quaking.demon.co.uk"
			 ) ;
	about.addAuthor  ("Mike Richardson", 0, "mike@quaking.demon.co.uk") ; 
	about.addAuthor  ("John Dean", 	     0, "john@rygannon.com") ; 

	KCmdLineArgs::init (2, dummy, &about) ;

	KApplication	app	;
#endif

	if (argc != 2)
	{
		fprintf	(errfile, "usage: " SKELL " xmlfile\n") ;
		exit	(1) ;
	}

	if (!xmlTest (argv[1]))
	{	fprintf	(errfile, SKELU " test exiting: failed\n") ;
		exit	(1) ;
	}
}

