/***************************************************************************
    file	         : script.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>
#include	<errno.h>
#include	<unistd.h>

#include	<qfile.h>
#include	<qdom.h>
#include	<qmainwindow.h>
#include	<qlibrary.h>
#include	<qapplication.h>
#include	<dlfcn.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_script.h"
#include	"kb_libloader.h"
#include	"kb_locator.h"
#include	"kb_desktop.h"
#include	"kb_form.h"
#include	"kb_display.h"
#include	"kb_options.h"



#if		__KB_KDE
#include	<kapp.h>
#include	<klocale.h>
#include	<kaboutdata.h>
#include	<kcmdlineargs.h>
#endif


typedef	void	*(*IFPROC)()	;

static	FILE	*errfile	;

uint	getArgs
	(	QDomElement	&parent,
		KBValue		*argv
	)
{
	uint	argc	= 0 ;

	for (QDomNode node = parent.firstChild	() ;
			     !node .isNull	() ;
		      node = node  .nextSibling() )
	{
		QDomElement elem = node.toElement () ;
		if (elem.isNull()) continue ;

		if (elem.nodeName() != "arg") continue ;

		QString	    typ	 = elem.attribute("type" ) ;
		QString	    val	 = elem.attribute("value") ;

		if (typ == "fixed")
		{
			argv[argc] = val.toInt() ;
			argc	  += 1 ;
			continue  ;
		}

		if (typ == "float")
		{
			argv[argc] = val.toDouble() ;
			argc	  += 1 ;
			continue  ;
		}

		argv[argc] = val ;
		argc	  += 1 ;
	}

	return	argc	;
}

bool	formTest
	(	QDomElement	&parent,
		KBForm		*form
	)
{
	QMainWindow	window	;
	QDict<QString>	pDict	;
	KBValue		key	;
	QSize		size	;

	form->showData (&window, pDict, key, size) ;
	QWidget	*top	= form->getDisplay()->getTopWidget() ;

	window.setCentralWidget (top)	;

	window .show     ()		;
	window .resize   (size.width(), size.height()) ;

	top   ->show	 () ;
	top   ->resize	 (size.width(), size.height()) ;

	for (QDomNode node = parent.firstChild	() ;
			     !node .isNull	() ;
		      node = node  .nextSibling() )
	{
		QDomElement elem = node.toElement () ;
		if (elem.isNull()) continue ;

		if (elem.nodeName() == "buttonOnClick")
		{
			KBObject *obj = form->getNamedObject (elem.attribute("name")) ;
			if ((obj == 0) || (obj->getElement() != "KBButton"))
			{
				fprintf
				(	stderr,
					"script: form: %s: no button\n",
					elem.attribute("name").latin1()
				)	;
				return	false	;
			}

			KBError	error	 ;
			KBValue	resval	 ;
			KBEvent	*onClick = obj->getAttr("onclick")->isEvent() ;

			onClick->doExecute
			(	resval,
				0,
				0,
				false
			)	;

			continue ;
		}
	}

	return	true	;
}

bool	languageTest
	(	QDomElement	&parent,
		KBScriptIF	*iface
	)
{
	for (QDomNode node = parent.firstChild	() ;
			     !node .isNull	() ;
		      node = node  .nextSibling() )
	{
		QDomElement elem = node.toElement () ;
		if (elem.isNull()) continue ;

		if (elem.nodeName() == "inline")
		{
			QString		tag	= elem.attribute("tag"  ) ;
			QString		fname	= elem.attribute("fname") ;
			QString		eText	;
			QString		ePatt	;
			KBError		error	;

			QString		text	= elem.text().stripWhiteSpace() + "\n" ;
			KBScriptCode	*code	= iface->compileFunc
						  (	text,
						  	QString::null,
							fname,
							eText,
							ePatt,
							QStringList(),
							0,
							error
			    			  )	;

			if (code == 0)
			{
				fprintf	(stderr, "Script: compile failed '%s'\n", (cchar *)tag) ;
				error.display	(stderr) ;
				return	false	;
			}

			KBValue		args[64] ;
			uint		argc	 = getArgs (elem, args) ;

			KBValue	        rv	 ;
			KBObject	node	 (0, "KBObject", QRect()) ;
			KBScript::ExeRC rc	 = code->execute (&node, argc, args, rv) ;

			printf
			(	"%-10s: %2d: %-8s: %s\n",
				(cchar *)tag,
				(int    )rc,
				(cchar *)rv.getType()->getDescrip(),
				(cchar *)rv.getRawText()
			)	;

			delete	code	;
			continue	;
		}

		if (elem.nodeName() == "expr")
		{
			QString		tag	= elem.attribute("tag"  ) ;
			QString		eText	;
			QString		ePatt	;
			KBError		error	;

			QString		text	= elem.text().stripWhiteSpace() ;
			KBScriptCode	*code	= iface->compileExpr
						  (	text,
						  	QString::null,
							eText,
							ePatt,
							QStringList(),
							error
			    			  )	;

			if (code == 0)
			{
				fprintf	(stderr, "Script: compile failed '%s'\n", (cchar *)tag) ;
				error.display	(stderr) ;
				return	false	;
			}

			KBValue	        rv	 ;
			KBNode		node	 (0, "KBBase") ;
			KBScript::ExeRC rc	 = code->execute (&node, 0, 0, rv) ;

			printf
			(	"%-10s: %2d: %-8s: %s\n",
				(cchar *)tag,
				(int    )rc,
				(cchar *)rv.getType()->getDescrip(),
				(cchar *)rv.getRawText()
			)	;

			delete	code	;
			continue	;
		}

		if (elem.nodeName() == "form")
		{
			QFile	formDef	(elem.attribute("name")) ;
			if (!formDef.open(IO_ReadOnly))
			{
				fprintf
				(	stderr,
					"script: form: %s: cannot open\n",
					elem.attribute("name").latin1()
				)	;
				exit	(1) ;
			}

			QByteArray formText = formDef.readAll() ;
			KBLocation formLocn
				   (	0,
					"form",
					KBLocation::m_pFile,
					elem.attribute("name")
				   )	;

			KBForm	   *form	;
			KBError	   error	;

			extern	KBForm	*KBOpenFormText (const KBLocation &, QByteArray &, KBError &) ;

			if ((form = KBOpenFormText (formLocn, formText, error)) == 0)
			{
				error.DISPLAY() ;
				exit	(1)	;
			}

			formTest (elem, form) ;
			continue ;
		}
	}

	return	true	;
}

bool	xmlTest
	(	const QString	&xmlName
	)
{
	QFile	xmlFile	(xmlName) ;
	if (!xmlFile.open (IO_ReadOnly))
	{
		fprintf	(stderr, "xmlFile: %s: %s\n", (cchar *)xmlName, strerror(errno)) ;
		return	false	;
	}

	QDomDocument doc	;
	if (!doc.setContent (&xmlFile))
	{
		fprintf	(stderr, "xmlFile: %s: cannot parse\n", (cchar *)xmlName) ;
		return	false	;
	}

	QDomElement  docElem	= doc.documentElement() ;
	for (QDomNode node = docElem.firstChild	() ;
			     !node  .isNull	() ;
		      node = node   .nextSibling() )
	{
		QDomElement elem = node.toElement () ;
		if (elem.isNull()) continue ;

		if (elem.nodeName() == "test")
		{
			QString	 lang = elem.attribute("language") ;
			KBError	   error    ;
			KBScriptIF *iface  = LinkKBScript (lang, error) ;
			if (iface == 0)
			{
				error.display	(stderr) ;
				return	false	;
			}

			if (!languageTest (elem, iface))
				return	false	;
		}

	}

	return	true	;
}


int	main
	(	int	argc,
		char	*argv[]
	)
{
	cchar	*sstde	= getenv ("REKALL_SHOWSTDERR") ;

	if ((sstde == 0) || (strcmp(sstde, "YES") != 0))
	{
		freopen	("/dev/null", "w", stderr) ;
		errfile	= fopen ("/dev/tty",  "w") ;
	}
	else	errfile	= stderr ;

#if	__KB_KDE
	char *dummy[2]	;

	dummy[0] = (char *)"script" ;
	dummy[1] = 0	;

	KAboutData about (__KB_APPNAME,
			  I18N_NOOP("RekallScriptTest"),
			  "0.0.0",
			  I18N_NOOP("Rekall: Script test harness"),
			  KAboutData::License_Unknown,
			  "(c) 2000,2001,2002 by theKompany.com",
			  0,
			  "http://www.rekall.a-i-s.co.uk",
			  "mike@quaking.demon.co.uk"
			 ) ;
	about.addAuthor  ("Mike Richardson", 0, "mike@quaking.demon.co.uk") ; 
	about.addAuthor  ("John Dean", 	     0, "john@rygannon.com") ; 

	KCmdLineArgs::init (2, dummy, &about) ;

	KApplication	app	;
#endif
#if	__KB_TKC
	QApplication	(argc, argv) ;
#endif

	KBError		error	;

	if (argc != 2)
	{
		fprintf	(stderr, "usage: script xmlfile\n") ;
		exit	(1) ;
	}

	extern	void    registerAllNodes() ;
	registerAllNodes() ;

	KBLibLoader	*loader	= KBLibLoader::self () ;

	/* Scan the local desktop files which descrive the rekall	*/
	/* "services". These are the same files as used for the KTrader	*/
	/* version, but the decoding in KBDesktop is much simplified.	*/
	QString		 dtDir	= locateDir
				  (	"appdata",
					"services/rekall_dummy.desktop"
				  )	;
	QPtrList<KBDesktop> dtDefs		;
	KBDesktop::scan (dtDir + "/services", "rekall_", dtDefs) ;

	for (unsigned int idx = 0 ; idx < dtDefs.count() ; idx += 1)
	{
		KBDesktop *desktop = dtDefs.at(idx) ;

		if (desktop->property ("ServiceTypes") != "Rekall/Script")
			continue	;

		QString	libname	 = desktop->property("X-KDE-Library") ;
		if (libname.isEmpty()) continue ;

		KBLibrary *lib 	 = loader->getGlobalLibrary (libname) ;
		if (lib == 0)
		{
			fprintf
			(	stderr,
				"script: %s: %s\n",
				(cchar *)libname,
				(cchar *)loader->lastErrorMessage()
			)	;
			continue;
		}

		KBFactory *factory = (KBFactory *)lib->factory () ;
		if (factory ==  0)
		{	
			fprintf	(stderr, "script: cannot create %s\n", (cchar *)libname) ;
			continue  ;
		}

		QString	lang	= desktop->property("X-KDE-RekallPart-Language") ;
		QString	options	= desktop->property("X-KDE-RekallPart-Options" ) ;

		fprintf
		(	stderr,
			"script: got language [%s] f=%p\n",
			(cchar *)lang,
			(void  *)factory
		)	;

		KBScriptIF::registerScript  (lang, factory, options) ;
	}

//	fprintf	(stderr, "sizeof(QByteArray) : %d\n", sizeof(QByteArray)) ;
//	fprintf	(stderr, "sizeof(KBValue   ) : %d\n", sizeof(KBValue   )) ;
//	fprintf	(stderr, "sizeof(KBRowSet  ) : %d\n", sizeof(KBRowSet  )) ;

	KBOptions::setOption ("pyencoding", "ISO8859-1") ;

	if (!xmlTest (argv[1]))
	{	fprintf	(stderr, "Script test exiting: failed\n") ;
		_exit	(1) ;
	}

	fprintf	(stderr, "Script test exiting\n") ;
	return	0 ;
}
